@echo off
rem ********************************************************************************
rem *** Command Line Utilities; Author: Sergei Kitaev ******************************
rem ********************************************************************************
if "%~0"=="$SQL.cmd" goto START

if not "%$cmd_plain%"=="1" cls
echo 
echo  SQL Command Line Utility (based on "SQLCMD"-utility) 
echo 
goto START

:SYNTAX
if "%~0"=="$SQL.cmd" goto INCORRECT_NESTED_CALL
echo.
echo  Variable number of parameters is supported: 
echo.
echo 0  $sql_server and $sql_database variables will be used
echo      instead of parameters
echo 1  $sql_server will be used as SQL-server name,
echo      the first parameter will be taken as SQL-database name
echo 2  The first parameters will be taken as SQL-server name,
echo      the second one will be taken as SQL-database name
echo.
echo  Examples: 
echo.
echo $sql
echo $sql SAPPHIRE_DB
echo $sql MERCURY_SRV EMERALD_DB
echo.
echo *** Use "?", "/?" or "-?" see this help. ***
goto FINISH

:START
setlocal
set errorlevel=0
set errorlevel=

if "%cmdextversion%"=="" goto INCORRECT_CMDEXT_VERSION
if not cmdextversion 2   goto INCORRECT_CMDEXT_VERSION

set SettingsBatchFileName=$sql_settings.cmd
set SettingsBatchFileXName=$SQL_SETTINGS.cmd
set SettingsBatchFile=%SettingsBatchFileXName%

if [%1]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%2]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%3]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%4]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%5]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%6]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%7]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%8]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%9]==[""] goto EMPTY_PARAMS_FORBIDDEN

if not [%2]==[] goto NOT_SYNTAX_PARAMS
if "%~1"=="?"   goto SYNTAX
if "%~1"=="/?"  goto SYNTAX
if "%~1"=="-?"  goto SYNTAX
:NOT_SYNTAX_PARAMS

if not [%3]==[] goto TOO_MANY_PARAMS
if not [%2]==[] goto TWO_PARAMS
if not [%1]==[] goto ONE_PARAM

if not defined $sql_server   goto SQL_SERVER_NOT_DEF
if not defined $sql_database goto SQL_DATABASE_NOT_DEF
goto RUN

:ONE_PARAM
set $sql_database=%~1
if not defined $sql_server goto SQL_SERVER_NOT_DEF
goto RUN

:TWO_PARAMS
if "%~1"=="" goto EMPTY_PARAMS
if "%~2"=="" goto EMPTY_PARAMS
set $sql_server=%~1
set $sql_database=%~2

:RUN
echo.
echo  SQL-Server  : %$sql_server%
echo  SQL-Database: %$sql_database%
echo.
echo 
sqlcmd -S"%$sql_server%" -d"%$sql_database%"
if %errorlevel% neq 0 goto FAILURE

:COMPLETED
if "%~0"=="$SQL.cmd" goto FINISH
echo.
echo  SQL COMMAND LINE  COMPLETED 
echo 
goto :EOF

:FAILURE
if "%~0"=="$SQL.cmd" goto ERROR
echo.
echo  ## SQL COMMAND LINE  FAILURE 
echo 
goto ERROR

:SQL_SERVER_NOT_DEF
echo.
echo ## Environment variable "$sql_server" is not defined.
goto ERROR

:SQL_DATABASE_NOT_DEF
echo.
echo ## Environment variable "$sql_database" is not defined.
goto ERROR

:TOO_MANY_PARAMS
echo.
echo ## Too many parameters. Maximum two supported -
echo ## SQL-server name (optionally) and database name (optionally).
goto ERROR

rem ================================================================================

:INCORRECT_CMDEXT_VERSION
echo.
echo ## Incorrect Version of CMD-Extensions. Version 2 or later is required.
goto ERROR

:INCORRECT_NESTED_CALL
echo.
echo ## Incorrect nested call.
goto ERROR

:EMPTY_PARAMS_FORBIDDEN
echo.
echo ## Empty parameters ("") are forbidden.
goto ERROR

:ERROR
if "%~0"=="$SQL.cmd" goto SET_ERROR
echo 
if not "%$cmd_plain%"=="1" pause

:SET_ERROR
endlocal
set errorlevel=1
goto :EOF

:FINISH
endlocal
set errorlevel=0
set errorlevel=
if "%~0"=="$SQL.cmd" goto :EOF
echo.
if not "%$cmd_plain%"=="1" pause
goto :EOF

rem ================================================================================
