@echo off
rem ********************************************************************************
rem *** Command Line Utilities; Author: Sergei Kitaev ******************************
rem ********************************************************************************
if "%~0"=="$SQLTRANS.cmd" goto START

if not "%$cmd_plain%"=="1" cls
echo 
echo  SQL-Script Translation Utility (based on "SQLCMD"-utility) 
echo 
goto START

:SYNTAX
if "%~0"=="$SQLTRANS.cmd" goto INCORRECT_NESTED_CALL
echo.
echo  Supported parameters: 
echo.
echo 1) Required: ^<SQL-script file/pattern^>
echo 2) Optional: ^<SQL-report file^> (message/result)
echo.
echo  Examples: 
echo.
echo $sqltrans "CreateStructure.sql"
echo $sqltrans "aux_*.sql"
echo $sqltrans "My Query.sql" "My Report.txt"
echo.
echo  Required Environment Variables: 
echo.
echo $sql_server, $sql_database, $oem_cp, TEMP
echo.
echo  Other Affecting Environment Variables (Optional): 
echo.
echo $sql_settings_script  alternative SQL-settings script file
echo                         (defaults to "$sql_settings.cmd")
echo.
echo $sql_script_cp  codepage for input script (defaults to ACP)
echo $sql_report_cp  codepage for report file  (defaults to ACP)
echo.
echo $sqltrans.AdditionalParams  additional parameters for SQLCMD
echo.
echo $sqltrans.SkipConfirmation  0/1 (defaults to "0")
echo $sqltrans.SkipCompletionPause  0/1 (defaults to "0")
echo $sqltrans.ShowConnectionInfo  0/1 (default value depends on the situation)
echo.
echo  You can define environment variables in the folowing command batch: 
echo.
echo %SettingsBatchFileName%  in the current execution folder
echo.
echo  You may use custom SQL-settings in the following script file: 
echo.
echo %SettingsScriptFileName%  in the current execution folder
echo.
echo *** Use "?", "/?", "-?" or no parameters to see this help. ***
goto FINISH

:START
setlocal
set errorlevel=0
set errorlevel=

if "%cmdextversion%"=="" goto INCORRECT_CMDEXT_VERSION
if not cmdextversion 2   goto INCORRECT_CMDEXT_VERSION

set SettingsBatchFileName=$sql_settings.cmd
set SettingsBatchFileXName=$SQL_SETTINGS.cmd
set SettingsScriptFileName=$sql_settings.sql

set SettingsBatchFile=%SettingsBatchFileXName%
set SettingsScriptFile=%~dp0%SettingsScriptFileName%

if [%1]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%2]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%3]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%4]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%5]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%6]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%7]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%8]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%9]==[""] goto EMPTY_PARAMS_FORBIDDEN

if not [%2]==[] goto NOT_SYNTAX_PARAMS
if "%~1"==""    goto SYNTAX
if "%~1"=="?"   goto SYNTAX
if "%~1"=="/?"  goto SYNTAX
if "%~1"=="-?"  goto SYNTAX
:NOT_SYNTAX_PARAMS

if not [%3]==[] goto TOO_MANY_PARAMS

set ScriptFilePattern=%~1
set ReportFile=%~2

set ReportFileSpecified=false
if not [%2]==[] set ReportFileSpecified=true

if not exist "%ScriptFilePattern%" goto SQL_FILES_NO_FOUND

if not exist "%SettingsBatchFile%" goto SKIP_SETTINGS_BATCH
echo.
echo * Applying settings from batch: "%SettingsBatchFileName%" ...
call %SettingsBatchFile%
if %errorlevel% neq 0 goto FAILURE
:SKIP_SETTINGS_BATCH

if defined $sql_settings_script goto USE_SETTINGS_SCRIPT
if exist ".\%SettingsScriptFileName%" set SettingsScriptFile=.\%SettingsScriptFileName%
goto CHECK_SETTINGS_SCRIPT

:USE_SETTINGS_SCRIPT
set SettingsScriptFile=%$sql_settings_script%

:CHECK_SETTINGS_SCRIPT
if not exist "%SettingsScriptFile%" goto SETTINGS_SCRIPT_NOT_FOUND

if not defined $sql_server   goto SQL_SERVER_NOT_DEF
if not defined $sql_database goto SQL_DATABASE_NOT_DEF
if not defined $oem_cp       goto OEM_CP_NOT_DEF
if not defined TEMP          goto TEMP_NOT_DEF

set TempScriptFile=%TEMP%\$sql_script.sql
set TempOutputFile=%TEMP%\$sql_output.txt

set ScriptCP=%$sql_script_cp%

if "%ReportFileSpecified%"=="false" set OutputCP=%$oem_cp%
if "%ReportFileSpecified%"=="true"  set OutputCP=%$sql_report_cp%

set ScriptCPSpecified=false
set OutputCPSpecified=false

if defined ScriptCP set ScriptCPSpecified=true
if defined OutputCP set OutputCPSpecified=true

set CPSpecifier=
if "%ScriptCPSpecified%|%OutputCPSpecified%"=="false|false" goto :END_CP_SPECIFIER
set CPSpecifier=-f
if "%ScriptCPSpecified%"=="true"                          set CPSpecifier=%CPSpecifier%i:%ScriptCP%
if "%ScriptCPSpecified%|%OutputCPSpecified%"=="true|true" set CPSpecifier=%CPSpecifier%,
if "%OutputCPSpecified%"=="true"                          set CPSpecifier=%CPSpecifier%o:%OutputCP%
:END_CP_SPECIFIER

call :DELETE_FILE_SILENT "%TempScriptFile%"
if %errorlevel% neq 0 goto FAILURE

call :DELETE_FILE_SILENT "%TempOutputFile%"
if %errorlevel% neq 0 goto FAILURE

if "%ReportFileSpecified%"=="false" set OutputFile=%TempOutputFile%
if "%ReportFileSpecified%"=="true"  set OutputFile=%ReportFile%

echo.
echo  SQL-Script File/Pattern: "%ScriptFilePattern%"
if "%ReportFileSpecified%"=="false" goto SKIP_OUT_FILE_NOTICE
echo  SQL-Message/Result File: "%OutputFile%"
:SKIP_OUT_FILE_NOTICE
echo 
echo * Preparing temporary SQL-script ...
copy/b "%SettingsScriptFile%"+"%ScriptFilePattern%" "%TempScriptFile%"
if %errorlevel% neq 0 goto FAILURE

if "%$sqltrans.SkipConfirmation%"=="1" goto HEADER_WITHOUT_CONFIRMATION

if not "%~0"=="$SQLTRANS.cmd" goto HEADER_WITH_CONFIRMATION
:HEADER_WITHOUT_CONFIRMATION
echo.
echo 
echo  STARTING THE SCRIPT(S) ON SQL-SERVER: 
if not "%$sqltrans.ShowConnectionInfo%"=="1" goto EXEC_SQL
call :PRINT_CONNECTION_INFO
echo.
echo 
goto EXEC_SQL

:HEADER_WITH_CONFIRMATION
echo.
echo 
echo  STARTING THE SCRIPT(S) ON SQL-SERVER:  ("Ctrl+C"  break off) 
if "%$sqltrans.ShowConnectionInfo%"=="0" goto CONFIRMATION_PAUSE
call :PRINT_CONNECTION_INFO
:CONFIRMATION_PAUSE
echo.
pause

if "%ReportFileSpecified%"=="false" goto SKIP_DELETE_OUTPUT_FILE
call :DELETE_FILE_SILENT "%OutputFile%"
if %errorlevel% neq 0 goto FAILURE
:SKIP_DELETE_OUTPUT_FILE

:EXEC_SQL
set Command=SqlCmd -S"%$sql_server%" -d"%$sql_database%" -i"%TempScriptFile%" -o"%OutputFile%" -b -w65535 %CPSpecifier% %$sqltrans.AdditionalParams%
echo.
echo  %Command%
echo.
echo * Executing SQL ...
%Command%
if %errorlevel% neq 0 goto SQL_ERROR
set RetLabel=SUCCESS
goto LISTING

:SQL_ERROR
set RetLabel=FAILURE
goto LISTING

:SUCCESS
echo.
echo  SQL-TRANSLATION  SUCCESS 
echo 
if "%ReportFileSpecified%"=="false" goto FINISH
if "%~0"=="$SQLTRANS.cmd" goto FINISH
call :PRINT_REFERENCE
goto FINISH

:FAILURE
echo.
echo  ## SQL-TRANSLATION  FAILURE 
echo 
call :PRINT_REFERENCE
goto ERROR

:LISTING
if "%ReportFileSpecified%"=="true" goto %RetLabel%
type "%OutputFile%"
if %errorlevel% neq 0 goto FAILURE
goto %RetLabel%

rem ================================================================================

:PRINT_CONNECTION_INFO
echo.
echo  SQL-Server  : %$sql_server%
echo  SQL-Database: %$sql_database%
goto :EOF

rem ================================================================================

:PRINT_REFERENCE
echo.
if not exist "%OutputFile%" goto :EOF
echo  SQL-Report File (message/result)  "%OutputFile%"
goto :EOF

rem ================================================================================

:SQL_SERVER_NOT_DEF
echo.
echo ## Environment variable "$sql_server" is not defined.
goto ERROR

:SQL_DATABASE_NOT_DEF
echo.
echo ## Environment variable "$sql_database" is not defined.
goto ERROR

:TEMP_NOT_DEF
echo.
echo ## Environment variable "TEMP" is not defined.
goto ERROR

:OEM_CP_NOT_DEF
echo.
echo ## Environment variable "$oem_cp" is not defined.
goto ERROR

:SQL_FILES_NO_FOUND
echo.
echo ## SQL-Script file(s) not found: "%ScriptFilePattern%".
goto ERROR

:SETTINGS_SCRIPT_NOT_FOUND
echo.
echo ## SQL-settings script file not found: "%SettingsScriptFile%".
goto ERROR

:TOO_MANY_PARAMS
echo.
echo ## Maximum two parameters supported 
echo ## SQL-script filename(pattern) and SQL-message file (optionally).
goto ERROR

rem ================================================================================

:DELETE_FILE_SILENT
if not exist "%~1" goto DELETE_FILE_SILENT_SUCCESS
del/q "%~1"
if not exist "%~1" goto DELETE_FILE_SILENT_SUCCESS
echo Can not delete file: "%~1".
set errorlevel=1
goto :EOF
:DELETE_FILE_SILENT_SUCCESS
set errorlevel=0
set errorlevel=
goto :EOF

rem ================================================================================

:INCORRECT_CMDEXT_VERSION
echo.
echo ## Incorrect Version of CMD-Extensions. Version 2 or later is required.
goto ERROR

:INCORRECT_NESTED_CALL
echo.
echo ## Incorrect nested call.
goto ERROR

:EMPTY_PARAMS_FORBIDDEN
echo.
echo ## Empty parameters ("") are forbidden.
goto ERROR

:ERROR
if "%~0"=="$SQLTRANS.cmd" goto SET_ERROR
if "%$sqltrans.SkipCompletionPause%"=="1" goto SET_ERROR
echo 
if not "%$cmd_plain%"=="1" pause

:SET_ERROR
endlocal
set errorlevel=1
goto :EOF

:FINISH
endlocal
set errorlevel=0
set errorlevel=
if "%~0"=="$SQLTRANS.cmd" goto :EOF
if "%$sqltrans.SkipCompletionPause%"=="1" goto :EOF
echo.
if not "%$cmd_plain%"=="1" pause
goto :EOF

rem ================================================================================
