@echo off
rem ********************************************************************************
rem *** Command Line Utilities; Author: Sergei Kitaev ******************************
rem ********************************************************************************
if "%~0"=="$SQLUPLOAD.cmd" goto START

if not "%$cmd_plain%"=="1" cls
echo 
echo  SQL Data Uploading Utility (based on "BCP"-utility) 
echo 
goto START

:SYNTAX
if "%~0"=="$SQLUPLOAD.cmd" goto INCORRECT_NESTED_CALL
echo.
echo  Supported parameters: 
echo.
echo ^<FileName^> ^<TableName^> [^<FileName^> ^<TableName^> ... ]
echo.
echo Maximum  8 parameters, to load up to 4 files / 4 datatables.
echo.
echo The following two SQL-files in the current directory supported for execution,
echo if present: "$pre_upload.sql", "$post_upload.sql"  to run before and after
echo uploading of text data accordingly.
echo.
echo  Examples: 
echo.
echo $sqlupload "xtable.txt" "XTABLE_UPLOAD"
echo $sqlupload data_1.txt IMPORT_1  data_2.txt IMPORT_2  data_3.txt IMPORT_3
echo.
echo  Required Environment Variables: 
echo.
echo $sql_server, $sql_database, $oem_cp, TEMP
echo.
echo  Other Affecting Environment Variables (Optional): 
echo.
echo $sql_script_cp, $sql_report_cp, $sql_uploading_cp, $sql_uploading_colsep
echo.
echo  You can define environment variables in the folowing command batch: 
echo.
echo %SettingsBatchFileName%  in the current execution folder
echo.
echo  You may use custom SQL-settings in the following script file: 
echo.
echo $sql_settings.sql  in the current execution folder
echo.
echo *** Use "?", "/?", "-?" or no parameters to see this help. ***
goto FINISH

:START
setlocal
set errorlevel=0
set errorlevel=

if "%cmdextversion%"=="" goto INCORRECT_CMDEXT_VERSION
if not cmdextversion 2   goto INCORRECT_CMDEXT_VERSION

set SettingsBatchFileName=$sql_settings.cmd
set SettingsBatchFileXName=$SQL_SETTINGS.cmd
set SettingsBatchFile=%SettingsBatchFileXName%

if [%1]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%2]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%3]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%4]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%5]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%6]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%7]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%8]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%9]==[""] goto EMPTY_PARAMS_FORBIDDEN

if not [%2]==[] goto NOT_SYNTAX_PARAMS
if "%~1"==""    goto SYNTAX
if "%~1"=="?"   goto SYNTAX
if "%~1"=="/?"  goto SYNTAX
if "%~1"=="-?"  goto SYNTAX
:NOT_SYNTAX_PARAMS

if not "%9"=="" goto TOO_MANY_PARAMS
if not "%8"=="" goto PARAMS_OK
if not "%7"=="" goto INCORRECT_PARAMS
if not "%6"=="" goto PARAMS_OK
if not "%5"=="" goto INCORRECT_PARAMS
if not "%4"=="" goto PARAMS_OK
if not "%3"=="" goto INCORRECT_PARAMS
if not "%2"=="" goto PARAMS_OK
goto INCORRECT_PARAMS
:PARAMS_OK

if not exist "%SettingsBatchFile%" goto SKIP_SETTINGS_BATCH
echo.
echo * Applying settings from batch: "%SettingsBatchFileName%" ...
call %SettingsBatchFile%
if %errorlevel% neq 0 goto FAILURE
:SKIP_SETTINGS_BATCH

if not defined $sql_server   goto SQL_SERVER_NOT_DEF
if not defined $sql_database goto SQL_DATABASE_NOT_DEF
if not defined TEMP          goto TEMP_NOT_DEF

set PreUploadScriptFile=$pre_upload.sql
set PostUploadScriptFile=$post_upload.sql

set UploadingCP=ACP
if defined $sql_uploading_cp set UploadingCP=%$sql_uploading_cp%

set UploadingColSep=^|
if defined $sql_uploading_colsep set UploadingColSep=%$sql_uploading_colsep%

if not "%~0"=="$SQLUPLOAD.cmd" goto DELAYED_HEADER
echo.
echo 
echo  STARTING OF LOADING DATA TO SQL-SERVER: 
goto PRE_LOAD

:DELAYED_HEADER
echo.
echo 
echo  STARTING OF LOADING DATA TO SQL-SERVER:  ("Ctrl+C"  break off) 
echo.
echo  SQL-Server  : %$sql_server%
echo  SQL-Database: %$sql_database%
echo.
echo  Parameters: %1 %2  %3 %4  %5 %6  %7 %8
echo.
pause

:PRE_LOAD
if not exist "%PreUploadScriptFile%" goto PROCESS_FILES
echo.
echo 
call $SQLTRANS.cmd %PreUploadScriptFile%
if %errorlevel% neq 0 goto FAILURE

:PROCESS_FILES
call :PERFORM_BCP "%~1" "%~2"
if %errorlevel% neq 0 goto FAILURE

if [%3]==[] goto POST_LOAD
call :PERFORM_BCP "%~3" "%~4"
if %errorlevel% neq 0 goto FAILURE

if [%5]==[] goto POST_LOAD
call :PERFORM_BCP "%~5" "%~6"
if %errorlevel% neq 0 goto FAILURE

if [%7]==[] goto POST_LOAD
call :PERFORM_BCP "%~7" "%~8"
if %errorlevel% neq 0 goto FAILURE

:POST_LOAD
echo.
echo 
if not exist "%PostUploadScriptFile%" goto SUCCESS
call $SQLTRANS.cmd %PostUploadScriptFile%
if %errorlevel% neq 0 goto FAILURE

:SUCCESS
echo.
echo  DATA UPLOADING  SUCCESS 
echo 
goto FINISH

:FAILURE
echo.
echo  ## DATA UPLOADING  FAILURE 
echo 
goto ERROR

rem ================================================================================

:PERFORM_BCP
echo.
echo 

set Command=BCP "%$sql_database%..[%~2]" in "%~1" -S"%$sql_server%" -T -m0 -c -C%UploadingCP% -t"%UploadingColSep%"
echo.
echo  %Command%
%Command%
goto :EOF

rem ================================================================================

:SQL_SERVER_NOT_DEF
echo.
echo ## Environment variable "$sql_server" is not defined.
goto ERROR

:SQL_DATABASE_NOT_DEF
echo.
echo ## Environment variable "$sql_database" is not defined.
goto ERROR

:TEMP_NOT_DEF
echo.
echo ## Environment variable "TEMP" is not defined.
goto ERROR

:INCORRECT_PARAMS
echo.
echo ## Incorrect number of parameters  even number expected.
goto ERROR

:TOO_MANY_PARAMS
echo.
echo ## Too many parameters. Maximum 8 parameters supported -
echo ## to load up to 4 files / 4 datatables.
goto ERROR

rem ================================================================================

:INCORRECT_CMDEXT_VERSION
echo.
echo ## Incorrect Version of CMD-Extensions. Version 2 or later is required.
goto ERROR

:INCORRECT_NESTED_CALL
echo.
echo ## Incorrect nested call.
goto ERROR

:EMPTY_PARAMS_FORBIDDEN
echo.
echo ## Empty parameters ("") are forbidden.
goto ERROR

:ERROR
if "%~0"=="$SQLUPLOAD.cmd" goto SET_ERROR
echo 
if not "%$cmd_plain%"=="1" pause

:SET_ERROR
endlocal
set errorlevel=1
goto :EOF

:FINISH
endlocal
set errorlevel=0
set errorlevel=
if "%~0"=="$SQLUPLOAD.cmd" goto :EOF
echo.
if not "%$cmd_plain%"=="1" pause
goto :EOF

rem ================================================================================
