@echo off
rem ********************************************************************************
rem *** Command Line Utilities; Author: Sergei Kitaev ******************************
rem ********************************************************************************
if "%~0"=="$WDKBUILD.cmd" goto START

if not "%$cmd_plain%"=="1" cls
echo 
echo  WDK-Build Utility (based on "build.exe", "setenv.bat" and "setwdf.bat") 
echo 
goto START

:SYNTAX
if "%~0"=="$WDKBUILD.cmd" goto INCORRECT_NESTED_CALL
echo.
echo  Required parameters (in any order), to be forwarded to "setenv.bat": 
echo.
echo 1) Configuration      : c, chk, checked, f, fre, free
echo 2) Target OS          : W2K, WXP, WNET, WLH
echo 3) Target architecture: x86, I386, IA32, amd64, IA64, 64, x86-64
echo.
echo Any additional parameters (if provided) will be forwarded to "build.exe"
echo instead of default parameters, which are the following: "%DefaultBuildParameters%" (re-build).
echo There is one parameter that is always active: "-g" (colored mode).
echo.
echo  Optional parameters: 
echo.
echo Sequence of any other parameters for "setenv.bat".
echo.
echo  Required Environment Variables: 
echo.
echo $wdk_root  path to WDK version directory (without backslash)
echo.
echo  The following special forms may be used for cleanup: 
echo.
echo * $wdkbuild Clean ^<CleanupKeyword^>
echo * $wdkbuild CleanAll
echo.
echo Such calls will remove output and intermediate files and subfolders
echo (including build logs), in current directory. The second one is
echo for the both configurations ("Checked" and "Free")
echo and for all known platforms/architectures.
echo.
echo  Examples: 
echo.
echo $wdkbuild x86 W2K checked
echo $wdkbuild f amd64 WLH -ceZ
echo.
echo $wdkbuild Clean chk_wxp_x86
echo $wdkbuild CleanAll
echo.
echo *** Use "?", "/?", "-?" or no parameters to see this help. ***
goto FINISH

:START
setlocal
set errorlevel=0
set errorlevel=

if "%cmdextversion%"=="" goto INCORRECT_CMDEXT_VERSION
if not cmdextversion 2   goto INCORRECT_CMDEXT_VERSION

set SaveCD=%CD%

set DefaultBuildParameters=-ce

if [%1]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%2]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%3]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%4]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%5]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%6]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%7]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%8]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%9]==[""] goto EMPTY_PARAMS_FORBIDDEN

if not [%2]==[] goto NOT_SYNTAX_PARAMS
if "%~1"==""    goto SYNTAX
if "%~1"=="?"   goto SYNTAX
if "%~1"=="/?"  goto SYNTAX
if "%~1"=="-?"  goto SYNTAX
:NOT_SYNTAX_PARAMS

if /i "%~1"=="Clean"    goto PREPARE_CLEAN_PARAMS
if /i "%~1"=="CleanAll" goto PREPARE_CLEAN_ALL_PARAMS
goto PREPARE_BUILD_PARAMS

:PREPARE_CLEAN_PARAMS
if /i     [%2]==[] goto INCORRECT_PARAMS
if /i not [%3]==[] goto INCORRECT_PARAMS
set CleanupKeyword=%~2
goto CLEAN

:PREPARE_CLEAN_ALL_PARAMS
if /i not [%2]==[] goto INCORRECT_PARAMS
goto CLEAN_ALL

:PREPARE_BUILD_PARAMS
if [%3]==[] goto INCORRECT_PARAMS

rem ================================================================================

set EnvironmentParameters=%~1 %~2 %~3

set BuildExeParameters=%DefaultBuildParameters%
if not [%4]==[] set BuildExeParameters=%4 %5 %6 %7 %8 %9
set BuildExeParameters=-g %BuildExeParameters%

if not defined $wdk_root              goto WDK_ROOT_NOT_DEF
if not defined PROCESSOR_ARCHITECTURE goto PROC_ARC_NOT_DEF

echo.
echo  Environment parameters: %EnvironmentParameters%
echo  "build.exe" parameters: %BuildExeParameters%
echo.
echo WDK root directory is "%$wdk_root%".
echo.
echo 
echo  BUILDING PROJECT: 

echo.
echo * Setting environment ...
call %$wdk_root%\bin\setenv.bat %$wdk_root% %EnvironmentParameters%
if %errorlevel% neq 0 goto FAILURE
set errorlevel=0
set errorlevel=

call :RESTORE_PATH_SILENT

echo.
echo 
echo.

%$wdk_root%\bin\%PROCESSOR_ARCHITECTURE%\build.exe %BuildExeParameters%
if %errorlevel% neq 0 goto FAILURE

:SUCCESS
echo.
echo  BUILD  SUCCESS 
echo 
goto FINISH

:FAILURE
echo.
echo  ## BUILD  FAILURE 
echo 
goto ERROR

rem ================================================================================

:CLEAN
if "%~0"=="$WDKBUILD.cmd" goto SKIP_CLEAN_HEADER
echo.
echo 
echo  REMOVING INTERMEDIATE AND OUTPUT FILES/DIRECTORIES: 
echo.
echo  CleanupKeyword = %CleanupKeyword%
echo.
echo 
:SKIP_CLEAN_HEADER

call :DELETE_FILE "build%CleanupKeyword%.log"
if %errorlevel% neq 0 goto FAILURE

call :DELETE_FILE "build%CleanupKeyword%.err"
if %errorlevel% neq 0 goto FAILURE

call :DELETE_FILE "build%CleanupKeyword%.wrn"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "obj%CleanupKeyword%"
if %errorlevel% neq 0 goto FAILURE

if "%~0"=="$WDKBUILD.cmd" goto SKIP_CLEAN_FOOTER
echo.
echo 
:SKIP_CLEAN_FOOTER

goto FINISH

rem ================================================================================

:CLEAN_ALL
if "%~0"=="$WDKBUILD.cmd" goto SKIP_CLEAN_ALL_HEADER
echo.
echo 
echo  REMOVING ALL INTERMEDIATE AND OUTPUT FILES/DIRECTORIES: 
:SKIP_CLEAN_ALL_HEADER


call :DELETE_FILES "buildchk_*_*.log"
if %errorlevel% neq 0 goto FAILURE

call :DELETE_FILES "buildchk_*_*.err"
if %errorlevel% neq 0 goto FAILURE

call :DELETE_FILES "buildchk_*_*.wrn"
if %errorlevel% neq 0 goto FAILURE


call :DELETE_FILES "buildfre_*_*.log"
if %errorlevel% neq 0 goto FAILURE

call :DELETE_FILES "buildfre_*_*.err"
if %errorlevel% neq 0 goto FAILURE

call :DELETE_FILES "buildfre_*_*.wrn"
if %errorlevel% neq 0 goto FAILURE


call :REMOVE_DIR "objchk_w2k_x86"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objchk_wxp_x86"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objchk_wnet_x86"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objchk_wlh_x86"
if %errorlevel% neq 0 goto FAILURE


call :REMOVE_DIR "objfre_w2k_x86"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objfre_wxp_x86"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objfre_wnet_x86"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objfre_wlh_x86"
if %errorlevel% neq 0 goto FAILURE


call :REMOVE_DIR "objchk_w2k_amd64"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objchk_wxp_amd64"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objchk_wnet_amd64"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objchk_wlh_amd64"
if %errorlevel% neq 0 goto FAILURE


call :REMOVE_DIR "objfre_w2k_amd64"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objfre_wxp_amd64"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objfre_wnet_amd64"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objfre_wlh_amd64"
if %errorlevel% neq 0 goto FAILURE


call :REMOVE_DIR "objchk_w2k_ia64"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objchk_wxp_ia64"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objchk_wnet_ia64"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objchk_wlh_ia64"
if %errorlevel% neq 0 goto FAILURE


call :REMOVE_DIR "objfre_w2k_ia64"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objfre_wxp_ia64"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objfre_wnet_ia64"
if %errorlevel% neq 0 goto FAILURE

call :REMOVE_DIR "objfre_wlh_ia64"
if %errorlevel% neq 0 goto FAILURE


if "%~0"=="$WDKBUILD.cmd" goto SKIP_CLEAN_ALL_FOOTER
echo.
echo 
:SKIP_CLEAN_ALL_FOOTER

goto FINISH

rem ================================================================================

:INCORRECT_PARAMS
echo.
echo ## Incorrect parameters for $wdkbuild.
echo ## (For syntax explanation call: $wdkbuild ?)
goto ERROR

:WDK_ROOT_NOT_DEF
echo.
echo ## Environment variable "$wdk_root" is not defined.
goto ERROR

:PROC_ARC_NOT_DEF
echo.
echo ## Environment variable "PROCESSOR_ARCHITECTURE" is not defined.
goto ERROR

rem ================================================================================

:RESTORE_PATH
if "%CD%"=="%SaveCD%" goto :EOF
echo.
echo * Restoring path to: "%SaveCD%" ...
cd /d "%SaveCD%"
if "%CD%"=="%SaveCD%" goto :EOF
goto CANT_RESTORE_PATH

:RESTORE_PATH_SILENT
if "%CD%"=="%SaveCD%" goto :EOF
cd /d "%SaveCD%"
if "%CD%"=="%SaveCD%" goto :EOF

:CANT_RESTORE_PATH
echo.
echo ## FATAL ERROR:
echo Can not restore path to: "%SaveCD%".
echo.
echo Batch processing terminated!
echo 
pause
exit

rem ================================================================================

:DELETE_FILE
if not exist "%~1" goto :EOF
echo.
echo * Deleting file: "%~1" ...
del/q "%~1"
if not exist "%~1" goto :EOF
set errorlevel=1
goto :EOF

:DELETE_FILES
if not exist "%~1" goto :EOF
echo.
echo * Deleting files: "%~1" ...
del/q "%~1"
if not exist "%~1" goto :EOF
set errorlevel=1
goto :EOF

:REMOVE_DIR
if not exist "%~1" goto :EOF
echo.
echo * Removing directory: "%~1" ...
rmdir/s/q "%~1"
if not exist "%~1" goto :EOF
set errorlevel=1
goto :EOF

rem ================================================================================

:INCORRECT_CMDEXT_VERSION
echo.
echo ## Incorrect Version of CMD-Extensions. Version 2 or later is required.
goto ERROR

:INCORRECT_NESTED_CALL
echo.
echo ## Incorrect nested call.
goto ERROR

:EMPTY_PARAMS_FORBIDDEN
echo.
echo ## Empty parameters ("") are forbidden.
goto ERROR

rem ================================================================================

:ERROR
if "%~0"=="$WDKBUILD.cmd" goto SKIP_ERROR_PAUSE
echo 
if not "%$cmd_plain%"=="1" pause

:SKIP_ERROR_PAUSE
call :RESTORE_PATH

:SET_ERROR
endlocal
set errorlevel=1
goto :EOF

:FINISH
endlocal
set errorlevel=0
set errorlevel=
if "%~0"=="$WDKBUILD.cmd" goto :EOF
echo.
if not "%$cmd_plain%"=="1" pause
goto :EOF

rem ================================================================================
