@echo off
rem ********************************************************************************
rem *** Command Line Utilities; Author: Sergei Kitaev ******************************
rem ********************************************************************************
if /i "%~1"=="(SpecialSingleCall)" goto SPECIAL_SINGLE_CALL
if "%~0"=="$XAPPDISTR.cmd" goto START

if not "%$cmd_plain%"=="1" cls
echo 
echo  XCopy App.Distribution Utility (based on XCOPY, TASKKILL and OPENFILES) 
echo 
goto START

:SYNTAX
if "%~0"=="$XAPPDISTR.cmd" goto INCORRECT_NESTED_CALL
echo.
echo  Parameters: 
echo.
echo 1) Required: ^<Application Folder Name^> (simple application name)
echo 2) Required: ^<Common Inner Path Under App.Server^> (without app.folder name)
echo 3) Optional: ^<App.Servers List File^> 
echo       distribution scope (application servers);
echo       default scope is a list of the following files:
echo          "~Output\<FailedServerName>.Errors.txt"
echo          (errors from previous distribution)
echo 4) Optional: ^<Action^> 
echo       "Distribute" (default) / "Update" / "Remove" (case insensitive)
echo.
echo  Environment Variables (Optional): 
echo.
echo $xappdistr.TaskKillExeName  instead of "<Application Folder Name>.exe"
echo $xappdistr.OpenFilesSubstring  instead of "<Application Folder Name>"
echo    Empty (undefined) variable causes default substitution (how just above).
echo    Specify "None" keyword (case insensitive) to avoid
echo    "Task Kill" and "Open Files / Disconnect".
echo.
echo $xappdistr.PostUploadScriptFile 
echo    to invoke special CMD-batch after every successuful upload;
echo    the following parameters will be substituted for the script batch
echo    (in quotation marks):
echo       1) ^<Application Folder Name^>;
echo       2) ^<Application Server Name^>;
echo       3) ^<Destination Folder Path^>.
echo.
echo $xappdistr.PostUploadScriptErrorAffectsFailedServersList  0/1
echo    Default value is "1" (affects).
echo.
echo $xappdistr.AskConfirmation  0/1
echo    Default behaviour of this flag depends on call mode of the utility:
echo       "$xappdistr ..." (direct call)  defaults to "1";
echo       "call $XAPPDISTR.cmd ..." (nested call)  defaults to "0".
echo.
echo  Examples: 
echo.
echo $XAppDistr "XProgram" "SoftShare\CompanySoft" "%%AppServersListFile%%"
echo $XAppDistr "XProgram" "SoftShare\CompanySoft" "%%AppServersListFile%%" "Update"
echo $XAppDistr "XProgram" "SoftShare\CompanySoft" "%%AppServersListFile%%" "Remove"
echo.
echo $XAppDistr "XProgram" "SoftShare\CompanySoft"
echo $XAppDistr "XProgram" "SoftShare\CompanySoft" "" "Update"
echo $XAppDistr "XProgram" "SoftShare\CompanySoft" "" "Remove"
echo.
echo *** Use "?", "/?", "-?" or no parameters to see this help. ***
goto FINISH

:START
setlocal enableextensions enabledelayedexpansion

if "%cmdextversion%"=="" goto INCORRECT_CMDEXT_VERSION
if not cmdextversion 2   goto INCORRECT_CMDEXT_VERSION

if [%1]==[""] goto EMPTY_PARAM_FORBIDDEN_1
if [%2]==[""] goto EMPTY_PARAM_FORBIDDEN_2

if not [%2]==[] goto NOT_SYNTAX_PARAMS
if "%~1"==""    goto SYNTAX
if "%~1"=="?"   goto SYNTAX
if "%~1"=="/?"  goto SYNTAX
if "%~1"=="-?"  goto SYNTAX
:NOT_SYNTAX_PARAMS

if     [%2]==[] goto INCORRECT_PARAMS
if not [%5]==[] goto INCORRECT_PARAMS

rem ================================================================================

set AppFolderName=%~1
set OnServerCommonInnerPath=%~2
set ServersListFile=%~3
set Action=%~4

rem --------------------------------------------------------------------------------

call :ASSIGN_COMMON_VARIABLES

rem --------------------------------------------------------------------------------

set ReportFileName=~AppDistribReport.%AppFolderName%.txt
set ReportFile=%OutputFolder%\%ReportFileName%

set ErrorFilePattern=%OutputFolder%\*.Errors.txt
set LogFilePattern=%OutputFolder%\*.Log.txt

set BaseWindowTitle=%AppFolderName% /%Action%

rem --------------------------------------------------------------------------------

set VerboseMode=1
if "%~0"=="$XAPPDISTR.cmd" set VerboseMode=0
if "%$xappdistr.AskConfirmation%"=="1" set VerboseMode=1
if "%$xappdistr.AskConfirmation%"=="0" set VerboseMode=0

rem ================================================================================

if /i "%Action%"=="REMOVE" goto SKIP_APP_SORCE_FOLDER_CHECKING
if not exist "%AppSourceFolder%\" goto APP_SORCE_FOLDER_NOT_FOUND
:SKIP_APP_SORCE_FOLDER_CHECKING

if "%ServersListFile%"=="" goto SKIP_SERVERS_LIST_FILE_CHECKING
if not exist "%ServersListFile%" goto SERVERS_LIST_FILE_NOT_FOUND
:SKIP_SERVERS_LIST_FILE_CHECKING

rem ================================================================================

echo.
echo  Application Folder Name     : %AppFolderName%
echo -------------------------------------------------------------------------------
echo  On Server Common Inner Path : %OnServerCommonInnerPath%
echo  App.Servers List File       : %ServersListFile%
echo -------------------------------------------------------------------------------
echo  Action                      : %Action%
echo -------------------------------------------------------------------------------
echo  Task Kill Exe-Name          : %TaskKillExeName%
echo  Open Files Substring        : %OpenFilesSubstring%
echo -------------------------------------------------------------------------------
echo  Report File                 : %ReportFile%
if /i not "%Action%"=="REMOVE" (
if /i not "%PostUploadScriptFile%"=="" (
echo -------------------------------------------------------------------------------
echo  Post Upload Script File     : %PostUploadScriptFile%
echo  Post Upload Script Error
echo   Affects Failed Servers List : %PostUploadScriptErrorAffectsFailedServersList%
) else (
rem
)
) else (
rem
)

if "%$xappdistr.AskConfirmation%"=="1" goto CONFIRMATION_PROMPT
if "%$xappdistr.AskConfirmation%"=="0" goto NO_CONFIRMATION_PROMPT
if "%~0"=="$XAPPDISTR.cmd"             goto NO_CONFIRMATION_PROMPT

:CONFIRMATION_PROMPT
echo.
echo 
echo  CONFIRM FORCED APP.DISTRIBUTION MULTI-START:  ("Ctrl+C"  break off) 
echo.
pause
goto SKIP_HEADER

:NO_CONFIRMATION_PROMPT
echo.
echo 
echo  FORCED APPLICATION DISTRIBUTION MULTI-START: 
:SKIP_HEADER

rem ================================================================================

set ReportFileCreated=0

rem --------------------------------------------------------------------------------

if "%ServersListFile%"=="" goto PREVIOUS_ERRORS_SCOPE

echo.
echo * Preparing output files ...

call :RECREATE_OUTPUT_FOLDER
if %errorlevel% neq 0 goto FAILURE

call :RESET_ERROR
for /f "usebackq" %%S in ("%ServersListFile%") do (
rem -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

set /a DistributionItemNo+=1

set ServerName=%%S

set ErrorFile=!OutputFolder!\!ServerName!.Errors.txt

call :CREATE_OPERATION_LOG_FILE "!ErrorFile!"
if %errorlevel% neq 0 goto FAILURE

call :RESET_ERROR
rem -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
)
if !errorlevel! neq 0 goto FAILURE

goto START_DISTRIBUTION

rem ================================================================================

:PREVIOUS_ERRORS_SCOPE

set ErrorFilesList=

call :RESET_ERROR
for %%F in ("%ErrorFilePattern%") do (
rem -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
set ErrorFilesList=!ErrorFilesList!;%%F
rem -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
)
if %errorlevel% neq 0 goto FAILURE

rem --------------------------------------------------------------------------------

echo.
echo * Recreating output files ...

call :RECREATE_OUTPUT_FOLDER
if %errorlevel% neq 0 goto FAILURE

call :RESET_ERROR
for %%F in (%ErrorFilesList%) do (
rem -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
call :RESET_ERROR
call :CREATE_OPERATION_LOG_FILE "%%F"
if !errorlevel! neq 0 goto FAILURE
rem -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
)
if %errorlevel% neq 0 goto FAILURE

rem ================================================================================

:START_DISTRIBUTION

call :PRINT_REPORT_HEADER > "%ReportFile%"
if %errorlevel% neq 0 goto FAILURE

set ReportFileCreated=1

echo.
echo * Starting "%Action%" operations for "%AppFolderName%" app. ...

echo.>> "!ReportFile!"
echo === Started "%Action%" operations for "%AppFolderName%" app.: ===>> "!ReportFile!"

set DistributionItemNo=0

call :RESET_ERROR
for %%F in ("%ErrorFilePattern%") do (
rem --------------------------------------------------------------------------------

set /a DistributionItemNo+=1

call :EXTRACT_SERVER_NAME "%%F"
set DestinationFolder=\\!ServerName!\!OnServerCommonInnerPath!\!AppFolderName!

if "%VerboseMode%"=="1" (
rem -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
echo.
echo !DistributionItemNo!^)
echo  !DestinationFolder! ...
rem -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
) else (
rem
)

echo.>> "!ReportFile!"
echo !DistributionItemNo!^)>> "!ReportFile!"
echo ==^> !DestinationFolder!;>> "!ReportFile!"

set Command="%~dp0$xappdistr.cmd" "(SpecialSingleCall)" "!AppFolderName!" "!OnServerCommonInnerPath!" "!ServerName!" "!Action!"

call :RESET_ERROR
start /min "[!ServerName!] - !BaseWindowTitle!" /D"!CD!" CMD /C "!Command!"

if !errorlevel! neq 0 (
rem -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
echo ## Can not start command batch.>> "!ReportFile!"
goto FAILURE
rem -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
) else (
rem
)

call :RESET_ERROR
rem --------------------------------------------------------------------------------
)
if %errorlevel% neq 0 goto FAILURE

echo.
echo 

call :PRINT_REPORT_SPLITTER >> "!ReportFile!"

echo.
echo * Waiting for completion of all started operations ...

:WAIT_COMPLETION
call :RESET_ERROR
for /f %%T in ('"TaskList /v | Find /i "!BaseWindowTitle!""') do (
rem -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Sleep 1
if %errorlevel% neq 0 goto FAILURE
goto WAIT_COMPLETION
rem -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
)
if %errorlevel% neq 0 goto FAILURE

if not exist "%ErrorFilePattern%" goto SUCCESS

call :PRINT_REPORT_OPERATIONS_FAILED
call :PRINT_REPORT_OPERATIONS_FAILED >> "!ReportFile!"

goto FAILURE

rem ================================================================================

:SUCCESS
call :PRINT_REPORT_OPERATIONS_SUCCESSED
call :PRINT_REPORT_OPERATIONS_SUCCESSED >> "%ReportFile%"

call :PRINT_REPORT_SUCCESS_FOOTER >> "%ReportFile%"

echo.
echo  APPLICATION DISTRIBUTION  SUCCESS 
echo 
goto FINISH

rem --------------------------------------------------------------------------------

:FAILURE
if "%ReportFileCreated%"=="1" call :PRINT_REPORT_FAILURE_FOOTER >> "%ReportFile%"

echo.
echo  ## APPLICATION DISTRIBUTION  FAILURE 
echo 
goto ERROR

rem ********************************************************************************


rem ********************************************************************************
:ASSIGN_COMMON_VARIABLES
rem --------------------------------------------------------------------------------

set AppSourceFolder=%AppFolderName%

set OutputFolderName=~Output
set OutputFolder=%OutputFolderName%

rem --------------------------------------------------------------------------------

if "%Action%"=="" set Action=Distribute
if /i "%Action%"=="DISTRIBUTE" (
set Action=Distribute
) else (
if /i "%Action%"=="UPDATE" (
set Action=Update
) else (
if /i not "%Action%"=="REMOVE" goto INCORRECT_PARAMS
set Action=Remove
)
)

rem --------------------------------------------------------------------------------

set TaskKillExeName=%AppFolderName%.exe
if not "%$xappdistr.TaskKillExeName%"==""     set TaskKillExeName=%$xappdistr.TaskKillExeName%
if /i  "%$xappdistr.TaskKillExeName%"=="NONE" set TaskKillExeName=

set OpenFilesSubstring=%AppFolderName%
if not "%$xappdistr.OpenFilesSubstring%"==""     set OpenFilesSubstring=%$xappdistr.OpenFilesSubstring%
if /i  "%$xappdistr.OpenFilesSubstring%"=="NONE" set OpenFilesSubstring=

set PostUploadScriptFile=%$xappdistr.PostUploadScriptFile%

if "%$xappdistr.PostUploadScriptErrorAffectsFailedServersList%"=="" (
set PostUploadScriptErrorAffectsFailedServersList=1
) else (
set PostUploadScriptErrorAffectsFailedServersList=%$xappdistr.PostUploadScriptErrorAffectsFailedServersList%
)

rem --------------------------------------------------------------------------------
goto :EOF
rem ********************************************************************************


rem ********************************************************************************
:RECREATE_OUTPUT_FOLDER
rem --------------------------------------------------------------------------------

call :REMOVE_FOLDER "%OutputFolder%"
if %errorlevel% neq 0 goto RETURN_ERROR

mkdir "%OutputFolder%"
if %errorlevel% neq 0 goto RETURN_ERROR

rem --------------------------------------------------------------------------------
goto RETURN_SUCCESS
rem ********************************************************************************


rem ********************************************************************************
:PRINT_REPORT_HEADER
rem --------------------------------------------------------------------------------
echo *******************************************************************************
echo *** APPLICATION DISTRIBUTION: *************************************************
echo *** ------------------------- *************************************************
echo.
time /t
echo.
echo # Application Folder Name    : %AppFolderName%
echo # On Server Common Inner Path: %OnServerCommonInnerPath%
echo # App.Servers List File      : %ServersListFile%
echo # Action                     : %Action%
echo # Task Kill Exe-Name         : %TaskKillExeName%
echo # Open Files Substring       : %OpenFilesSubstring%
if /i not "%Action%"=="REMOVE" (
if /i not "%PostUploadScriptFile%"=="" (
echo # Post Upload Script File    : %PostUploadScriptFile%
echo # Post Upload Script Error
echo   Affects Failed Servers List: %PostUploadScriptErrorAffectsFailedServersList%
) else (
rem
)
) else (
rem
)
rem --------------------------------------------------------------------------------
goto RETURN_SUCCESS
rem ********************************************************************************


rem ********************************************************************************
:PRINT_REPORT_SPLITTER
echo.
echo ===============================================================================
goto RETURN_SUCCESS
rem ********************************************************************************


rem ********************************************************************************
:PRINT_REPORT_OPERATIONS_SUCCESSED
rem --------------------------------------------------------------------------------
echo.
echo All operations completed successfully.
echo See log files for details: "%LogFilePattern%".
rem --------------------------------------------------------------------------------
goto RETURN_SUCCESS
rem ********************************************************************************


rem ********************************************************************************
:PRINT_REPORT_OPERATIONS_FAILED
rem --------------------------------------------------------------------------------
echo.
echo ## One ore more distribution operations have failed.
echo ## See error files for details: "%ErrorFilePattern%".
echo ## See log files for details: "%LogFilePattern%".
rem --------------------------------------------------------------------------------
goto RETURN_SUCCESS
rem ********************************************************************************


rem ********************************************************************************
:PRINT_REPORT_SUCCESS_FOOTER
rem --------------------------------------------------------------------------------
echo.
time /t
echo.
echo *** ----------------------------------- ***************************************
echo *** APPLICATION DISTRIBUTION -- SUCCESS ***************************************
echo *******************************************************************************
rem --------------------------------------------------------------------------------
goto RETURN_SUCCESS
rem ********************************************************************************


rem ********************************************************************************
:PRINT_REPORT_FAILURE_FOOTER
rem --------------------------------------------------------------------------------
echo.
time /t
echo.
echo *** -------------------------------------- ************************************
echo *** ## APPLICATION DISTRIBUTION -- FAILURE ************************************
echo *******************************************************************************
rem --------------------------------------------------------------------------------
goto RETURN_SUCCESS
rem ********************************************************************************


rem ********************************************************************************
:CREATE_OPERATION_LOG_FILE
rem --------------------------------------------------------------------------------
call :PRINT_OPERATION_LOG_HEADER "%~n1" > %~1
rem --------------------------------------------------------------------------------
goto RETURN_SUCCESS
rem ********************************************************************************


rem ********************************************************************************
:PRINT_OPERATION_LOG_HEADER
rem --------------------------------------------------------------------------------
echo *******************************************************************************
echo *** SINGLE APP.DISTRIBUTION OPERATION: ****************************************
echo *** ---------------------------------- ****************************************
echo.
time /t
echo.
echo # Application Folder Name    : %AppFolderName%
echo # On Server Common Inner Path: %OnServerCommonInnerPath%
echo # Application Server Name    : %~n1
echo # Action                     : %Action%
echo # Task Kill Exe-Name         : %TaskKillExeName%
echo # Open Files Substring       : %OpenFilesSubstring%
if /i not "%Action%"=="REMOVE" (
if /i not "%PostUploadScriptFile%"=="" (
echo # Post Upload Script File    : %PostUploadScriptFile%
echo # Post Upload Script Error
echo   Affects Failed Servers List: %PostUploadScriptErrorAffectsFailedServersList%
) else (
rem
)
) else (
rem
)
rem --------------------------------------------------------------------------------
goto RETURN_SUCCESS
rem ********************************************************************************


rem ********************************************************************************
:PRINT_OPERATION_LOG_SUCCESS
rem --------------------------------------------------------------------------------
echo.
time /t
echo.
echo *** -------------------------------------------- ******************************
echo *** SINGLE APP.DISTRIBUTION OPERATION -- SUCCESS ******************************
echo *******************************************************************************
rem --------------------------------------------------------------------------------
goto RETURN_SUCCESS
rem ********************************************************************************


rem ********************************************************************************
:PRINT_OPERATION_LOG_FAILURE
rem --------------------------------------------------------------------------------
echo.
time /t
echo.
echo *** ----------------------------------------------- ***************************
echo *** ## SINGLE APP.DISTRIBUTION OPERATION -- FAILURE ***************************
echo *******************************************************************************
rem --------------------------------------------------------------------------------
goto RETURN_SUCCESS
rem ********************************************************************************


rem ********************************************************************************
:EXTRACT_SERVER_NAME
call :EXTRACT_SERVER_NAME_2 "%~n1"
call :RESET_ERROR
goto :EOF
rem ================================================================================
:EXTRACT_SERVER_NAME_2
set ServerName=%~n1
goto :EOF
rem ********************************************************************************


rem ********************************************************************************
:SPECIAL_SINGLE_CALL
rem --------------------------------------------------------------------------------
setlocal enableextensions enabledelayedexpansion
rem ================================================================================

set AppFolderName=%~2
set OnServerCommonInnerPath=%~3
set ServerName=%~4
set Action=%~5

rem --------------------------------------------------------------------------------

call :ASSIGN_COMMON_VARIABLES

rem --------------------------------------------------------------------------------

set ErrorFileName=%ServerName%.Errors.txt
set LogFileName=%ServerName%.Log.txt

set ErrorFile=%OutputFolder%\%ErrorFileName%
set LogFile=%OutputFolder%\%LogFileName%

set ReportFile=%ErrorFile%

set UploadTempFolderName=$AppDistribution.%AppFolderName%.Temp

set RenameToLogFileOnFailure=0

rem --------------------------------------------------------------------------------

echo 
echo  SINGLE APP.DISTRIBUTION OPERATION: 
echo.
echo  Application Folder Name     : %AppFolderName%
echo -------------------------------------------------------------------------------
echo  On Server Common Inner Path : %OnServerCommonInnerPath%
echo  Application Server Name     : %ServerName%
echo -------------------------------------------------------------------------------
echo  Action                      : %Action%
echo -------------------------------------------------------------------------------
echo  Task Kill Exe-Name          : %TaskKillExeName%
echo  Open Files Substring        : %OpenFilesSubstring%
echo -------------------------------------------------------------------------------
echo  Error File                  : %ErrorFile%
echo  Log File                    : %LogFile%
if /i not "%Action%"=="REMOVE" (
if /i not "%PostUploadScriptFile%"=="" (
echo -------------------------------------------------------------------------------
echo  Post Upload Script File     : %PostUploadScriptFile%
echo  Post Upload Script Error
echo   Affects Failed Servers List : %PostUploadScriptErrorAffectsFailedServersList%
) else (
rem
)
) else (
rem
)

rem --------------------------------------------------------------------------------

echo.
if /i "%Action%"=="DISTRIBUTE" echo  Distributing "%AppFolderName%" application: 
if /i "%Action%"=="UPDATE"     echo  Updating "%AppFolderName%" application: 
if /i "%Action%"=="REMOVE"     echo  Removing "%AppFolderName%" application: 

rem --------------------------------------------------------------------------------

set DestinationFolder=\\%ServerName%\%OnServerCommonInnerPath%\%AppFolderName%
set UploadTempFolder=\\%ServerName%\%OnServerCommonInnerPath%\%UploadTempFolderName%

echo.
echo  %DestinationFolder% ...

call :UPLOAD_APPLICATION >> "%ReportFile%" 2>&1
if %errorlevel% equ 0 goto SINGLE_COMPLETE

echo.
echo # ERROR (see error file for details^)

if not "!PostUploadScriptError!"=="1" goto SINGLE_FAILURE
if not "!PostUploadScriptErrorAffectsFailedServersList!"=="1" set RenameToLogFileOnFailure=1
goto SINGLE_FAILURE

rem --------------------------------------------------------------------------------

:SINGLE_COMPLETE
call :PRINT_OPERATION_LOG_SUCCESS >> "%ReportFile%"

ren "%ReportFile%" "%LogFileName%"
if %errorlevel% neq 0 goto SINGLE_PRINT_FAILURE

echo.
echo  SINGLE APP.DISTRIBUTION OPERATION  SUCCESS 
echo 
goto END_LOCAL_RETURN_SUCCESS

rem --------------------------------------------------------------------------------

:SINGLE_FAILURE
call :PRINT_OPERATION_LOG_FAILURE >> "%ReportFile%"

if "%RenameToLogFileOnFailure%"=="1" ren "%ReportFile%" "%LogFileName%"

:SINGLE_PRINT_FAILURE
echo.
echo  ## SINGLE APP.DISTRIBUTION OPERATION  FAILURE 
echo 
goto END_LOCAL_RETURN_ERROR

rem ********************************************************************************


rem ********************************************************************************
:UPLOAD_APPLICATION
rem --------------------------------------------------------------------------------
set PostUploadScriptError=0
rem ================================================================================

echo.
if /i "%Action%"=="DISTRIBUTE" echo === Distributing "%AppFolderName%" application: ===
if /i "%Action%"=="UPDATE"     echo === Updating "%AppFolderName%" application: ===
if /i "%Action%"=="REMOVE"     echo === Removing "%AppFolderName%" application: ===

echo.
time /t
echo ==^> "%DestinationFolder%":

rem ================================================================================

if not exist "%DestinationFolder%\" goto UPLOAD_APPLICATION_PREPARE_TO_COPY

rem ================================================================================

if "%TaskKillExeName%"=="" goto UPLOAD_APPLICATION_SKIP_TASK_KILL

call :RESET_ERROR
for /f %%T in ('"TaskList /s "%ServerName%" | Find /i "%TaskKillExeName%""') do (
TaskKill /s "%ServerName%" /im "%TaskKillExeName%" > nul
goto UPLOAD_APPLICATION_BREAK_TASK_KILL
)
:UPLOAD_APPLICATION_BREAK_TASK_KILL
rem if %errorlevel% neq 0 goto UPLOAD_APPLICATION_ERROR

:UPLOAD_APPLICATION_SKIP_TASK_KILL

rem --------------------------------------------------------------------------------

if "%OpenFilesSubstring%"=="" goto UPLOAD_APPLICATION_SKIP_DISCONNECT_FILES

call :RESET_ERROR
for /f "tokens=2 delims=," %%I in ('"OpenFiles /query /v /fo CSV /s "%ServerName%" | Find /i "%OpenFilesSubstring%""') do (
OpenFiles /disconnect /s "%ServerName%" /o * /id "%%I" > nul
rem if %errorlevel% neq 0 goto UPLOAD_APPLICATION_ERROR
)
rem if %errorlevel% neq 0 goto UPLOAD_APPLICATION_ERROR

:UPLOAD_APPLICATION_SKIP_DISCONNECT_FILES

rem ================================================================================

call :REMOVE_FOLDER "%UploadTempFolder%"
if %errorlevel% neq 0 goto UPLOAD_APPLICATION_ERROR

CMD /C "ren "%DestinationFolder%\" "%UploadTempFolderName%""
if %errorlevel% neq 0 goto UPLOAD_APPLICATION_ERROR

if /i not "%Action%"=="UPDATE" goto UPLOAD_APPLICATION_PREPARE_TO_COPY

call :RESET_ERROR
xcopy /q /i /e /k /d /y "%AppSourceFolder%" "%UploadTempFolder%"
if %errorlevel% neq 0 goto UPLOAD_APPLICATION_ERROR

goto UPLOAD_APPLICATION_RENAME_BACK

rem --------------------------------------------------------------------------------

:UPLOAD_APPLICATION_PREPARE_TO_COPY
call :REMOVE_FOLDER_PERSISTENT "%UploadTempFolder%"
if %errorlevel% neq 0 goto UPLOAD_APPLICATION_ERROR

if /i "%Action%"=="REMOVE" goto UPLOAD_APPLICATION_COMPLETE

rem --------------------------------------------------------------------------------

call :RESET_ERROR
xcopy /q /i /e /k "%AppSourceFolder%" "%UploadTempFolder%"
if %errorlevel% neq 0 goto UPLOAD_APPLICATION_ERROR

:UPLOAD_APPLICATION_RENAME_BACK
call :RENAME_FOLDER_PERSISTENT "%UploadTempFolder%" "%AppFolderName%"
if %errorlevel% neq 0 goto UPLOAD_APPLICATION_ERROR

rem --------------------------------------------------------------------------------

if /i "%Action%"=="REMOVE"      goto UPLOAD_APPLICATION_COMPLETE
if "%PostUploadScriptFile%"=="" goto UPLOAD_APPLICATION_COMPLETE

echo * Executing post-upload script ...
call :RESET_ERROR
call %PostUploadScriptFile% "%AppFolderName%" "%ServerName%" "%DestinationFolder%"
if %errorlevel% neq 0 goto UPLOAD_APPLICATION_POST_UPLOAD_COMMAND_ERROR

rem ================================================================================
:UPLOAD_APPLICATION_COMPLETE
if /i "%Action%"=="REMOVE"          goto UPLOAD_APPLICATION_COMPLETE_WITH_OK
if not "%PostUploadScriptFile%"=="" goto UPLOAD_APPLICATION_COMPLETE_WITH_OK
call :RESET_ERROR
goto :EOF
rem ================================================================================
:UPLOAD_APPLICATION_COMPLETE_WITH_OK
echo # OK
call :RESET_ERROR
goto :EOF
rem ================================================================================
:UPLOAD_APPLICATION_ERROR
echo # *** ERROR ***
set errorlevel=1
goto :EOF
rem ================================================================================
:UPLOAD_APPLICATION_POST_UPLOAD_COMMAND_ERROR
set PostUploadScriptError=1
goto UPLOAD_APPLICATION_ERROR
rem ********************************************************************************


rem ********************************************************************************

:INCORRECT_PARAMS
echo.
echo ## Incorrect parameters.
echo ## Run with "?", "/?", "-?" or without parameters for syntax.
goto ERROR

:APP_SORCE_FOLDER_NOT_FOUND
echo.
echo ## Application source folder not found: "%AppSourceFolder%".
goto ERROR

:SERVERS_LIST_FILE_NOT_FOUND
echo.
echo ## Application servers list file not found: "%ServersListFile%".
goto ERROR

rem ********************************************************************************


rem ********************************************************************************
:RESET_ERROR
set errorlevel=0
set errorlevel=
goto :EOF
rem ********************************************************************************


rem ********************************************************************************

:RETURN_SUCCESS
call :RESET_ERROR
goto :EOF

:RETURN_ERROR
set errorlevel=1
goto :EOF

rem ================================================================================

:END_LOCAL_RETURN_SUCCESS
endlocal
call :RESET_ERROR
goto :EOF

:END_LOCAL_RETURN_ERROR
endlocal
set errorlevel=1
goto :EOF

rem ********************************************************************************


rem ********************************************************************************
:DELETE_FILE_VERBOSE
rem --------------------------------------------------------------------------------
if "%~1"=="" goto DELETE_FILE_VERBOSE_INCORRECT_CALL
if not exist "%~1" goto RETURN_SUCCESS
echo.
echo * Deleting file: "%~1" ...
call :RESET_ERROR
del/q "%~1"
if exist "%~1" goto RETURN_ERROR
rem --------------------------------------------------------------------------------
goto RETURN_SUCCESS
rem ================================================================================
:DELETE_FILE_VERBOSE_INCORRECT_CALL
echo.
echo ## Incorrect subroutine call: "DELETE_FILE_VERBOSE".
goto RETURN_ERROR
rem ********************************************************************************


rem ********************************************************************************
:REMOVE_FOLDER
rem --------------------------------------------------------------------------------
if "%~1"=="" goto REMOVE_FOLDER_INCORRECT_CALL
if not exist "%~1\" goto RETURN_SUCCESS
call :RESET_ERROR
rmdir /s /q "%~1"
if exist "%~1\" goto RETURN_ERROR
rem --------------------------------------------------------------------------------
goto RETURN_SUCCESS
rem ================================================================================
:REMOVE_FOLDER_INCORRECT_CALL
echo.
echo ## Incorrect subroutine call: "REMOVE_FOLDER".
goto RETURN_ERROR
rem ********************************************************************************


rem ********************************************************************************
:REMOVE_FOLDER_PERSISTENT
setlocal
rem --------------------------------------------------------------------------------

if "%~1"=="" goto REMOVE_FOLDER_PERSISTENT_INCORRECT_CALL

if not exist "%~1\" goto END_LOCAL_RETURN_SUCCESS

set TotalDelay=60
set SingleDelay=5

set DelaySum=0
rem --------------------------------------------------------------------------------
:REMOVE_FOLDER_PERSISTENT_LOOP
call :RESET_ERROR

if %DelaySum% neq 0 Sleep "%SingleDelay%"
if %errorlevel% neq 0 goto END_LOCAL_RETURN_ERROR

rmdir /s /q "%~1" > nul 2>&1
if not exist "%~1\" goto END_LOCAL_RETURN_SUCCESS

if %DelaySum% geq %TotalDelay% goto REMOVE_FOLDER_PERSISTENT_BREAK
set /a DelaySum+=%SingleDelay%

goto REMOVE_FOLDER_PERSISTENT_LOOP
rem --------------------------------------------------------------------------------
:REMOVE_FOLDER_PERSISTENT_BREAK

call :RESET_ERROR
rmdir /s /q "%~1"
if exist "%~1\" goto END_LOCAL_RETURN_ERROR

goto END_LOCAL_RETURN_SUCCESS
rem ================================================================================
:REMOVE_FOLDER_PERSISTENT_INCORRECT_CALL
echo.
echo ## Incorrect subroutine call: "REMOVE_FOLDER_PERSISTENT".
goto REMOVE_FOLDER_PERSISTENT_ERROR
rem ********************************************************************************


rem ********************************************************************************
:RENAME_FOLDER_PERSISTENT
setlocal
rem --------------------------------------------------------------------------------

if "%~1"=="" goto RENAME_FOLDER_PERSISTENT_INCORRECT_CALL
if "%~2"=="" goto RENAME_FOLDER_PERSISTENT_INCORRECT_CALL

if not exist "%~1\" goto END_LOCAL_RETURN_SUCCESS

set TotalDelay=60
set SingleDelay=5

set DelaySum=0
rem --------------------------------------------------------------------------------
:RENAME_FOLDER_PERSISTENT_LOOP
call :RESET_ERROR

if %DelaySum% neq 0 Sleep "%SingleDelay%"
if %errorlevel% neq 0 goto END_LOCAL_RETURN_ERROR

ren "%~1\" "%~2" > nul 2>&1
if %errorlevel% equ 0 goto END_LOCAL_RETURN_SUCCESS

if %DelaySum% geq %TotalDelay% goto RENAME_FOLDER_PERSISTENT_BREAK
set /a DelaySum+=%SingleDelay%

goto RENAME_FOLDER_PERSISTENT_LOOP
rem --------------------------------------------------------------------------------
:RENAME_FOLDER_PERSISTENT_BREAK

call :RESET_ERROR
ren "%~1\" "%~2"
if %errorlevel% neq 0 goto END_LOCAL_RETURN_ERROR

goto END_LOCAL_RETURN_SUCCESS
rem ================================================================================
:RENAME_FOLDER_PERSISTENT_INCORRECT_CALL
echo.
echo ## Incorrect subroutine call: "RENAME_FOLDER_PERSISTENT".
goto RENAME_FOLDER_PERSISTENT_ERROR
rem ********************************************************************************


rem ********************************************************************************

:INCORRECT_CMDEXT_VERSION
echo.
echo ## Incorrect Version of CMD-Extensions. Version 2 or later is required.
goto ERROR

:INCORRECT_NESTED_CALL
echo.
echo ## Incorrect nested call.
goto ERROR

rem ================================================================================

:EMPTY_PARAM_FORBIDDEN_1
echo.
echo ## Empty parameter ("") is forbidden for this position.
echo ## Parameter position No (1-based): 1
echo ## Parameter meaning: ^<Application Folder Name^>
goto ERROR

:EMPTY_PARAM_FORBIDDEN_2
echo.
echo ## Empty parameter ("") is forbidden for this position.
echo ## Parameter position No (1-based): 2
echo ## Parameter meaning: ^<Common Inner Path Under App.Server^>
goto ERROR

rem ================================================================================

:ERROR
if "%~0"=="$XAPPDISTR.cmd" goto SET_ERROR
echo 
if not "%$cmd_plain%"=="1" pause

:SET_ERROR
endlocal
set errorlevel=1
goto :EOF

:FINISH
endlocal
set errorlevel=0
set errorlevel=
if "%~0"=="$XAPPDISTR.cmd" goto :EOF
echo.
if not "%$cmd_plain%"=="1" pause
goto :EOF

rem ********************************************************************************
