﻿//**************************************************************************************************
//
// KEY DEFINITIONS -- PREPROCESSOR.
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#pragma once

#ifndef $KEY_DEFS_PP_H
#define $KEY_DEFS_PP_H

//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Universal debug flag (user/kernel mode):
//
//==================================================================================================

#ifdef $DEBUG
   #error "'$DEBUG' flag must not be defined explicitly."
#endif
#if $KERNEL_MODE
   #if DBG
      #define $DEBUG    1
   #endif
#else
   #if _DEBUG
      #define $DEBUG    1
   #endif
#endif


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Stringizing helpers:
//
//==================================================================================================

#define $STRINGIZE(stringBody)         $STRINGIZE_AUX(stringBody)
#define $STRINGIZE_AUX(stringBody)     #stringBody

#define $STRINGIZE_W(stringBody)       $STRINGIZE_W_AUX(stringBody)
#define $STRINGIZE_W_AUX(stringBody)   L#stringBody


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Abstract keywords:
//
//==================================================================================================

#define $VOID_VALUE  ((void)0)
#define $UNREACHABLE


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Code block operators:
//
//==================================================================================================

#define $BLOCK

#define $LOOP  for (;;)


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Fixed array helpers:
//
//==================================================================================================

#define $COUNT_OF(fixedArrayVar) \
   (sizeof(fixedArrayVar)/sizeof((fixedArrayVar)[0]))

#define $LAST_ELEMENT_OF(fixArrayVar) \
   ((fixArrayVar)[$COUNT_OF(fixArrayVar)-1])


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Typified memory cleanup:
//
//==================================================================================================

#define $CLEAR_(pValue) \
   $CLEAR_ELEMENTS_((pValue),1)

#if $KERNEL_MODE
   #define $CLEAR_ELEMENTS_(pElements,count) \
      (RtlZeroMemory((pElements),sizeof(*(pElements))*(count)))
   #define $CLEAR_ARRAY_VARIABLE(fixedArrayVar) \
      (RtlZeroMemory((fixedArrayVar),sizeof(fixedArrayVar)))
#else
   #define $CLEAR_ELEMENTS_(pElements,count) \
      (ZeroMemory((pElements),sizeof(*(pElements))*(count)))
   #define $CLEAR_ARRAY_VARIABLE(fixedArrayVar) \
      (ZeroMemory((fixedArrayVar),sizeof(fixedArrayVar)))
#endif


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Logic helpers:
//
//==================================================================================================

#define $AS_BOOL(condition)         ((condition) ? TRUE : FALSE)

#define $EVEN(number)               (!$ODD(number))
#define $ODD(number)                ($AS_BOOL((number)%2))

#define $BETWEEN_(value,from,to)    ((value)>=(from) && (value)<=(to))
#define $EXBETWEEN_(value,from,to)  ((value)>(from) && (value)<(to))

#define $IS_ONE_OF_2_(v,a1,a2) \
   ( (v)==(a1) || (v)==(a2) )
#define $IS_ONE_OF_3_(v,a1,a2,a3) \
   ( (v)==(a1) || (v)==(a2) || (v)==(a3) )
#define $IS_ONE_OF_4_(v,a1,a2,a3,a4) \
   ( (v)==(a1) || (v)==(a2) || (v)==(a3) || (v)==(a4) )
#define $IS_ONE_OF_5_(v,a1,a2,a3,a4,a5) \
   ( (v)==(a1) || (v)==(a2) || (v)==(a3) || (v)==(a4) || (v)==(a5) )
#define $IS_ONE_OF_6_(v,a1,a2,a3,a4,a5,a6) \
   ( (v)==(a1) || (v)==(a2) || (v)==(a3) || (v)==(a4) || (v)==(a5) || (v)==(a6) )
#define $IS_ONE_OF_7_(v,a1,a2,a3,a4,a5,a6,a7) \
   ( (v)==(a1) || (v)==(a2) || (v)==(a3) || (v)==(a4) || (v)==(a5) || (v)==(a6) || (v)==(a7) )
#define $IS_ONE_OF_8_(v,a1,a2,a3,a4,a5,a6,a7,a8) \
   ( (v)==(a1) || (v)==(a2) || (v)==(a3) || (v)==(a4) || (v)==(a5) || \
     (v)==(a6) || (v)==(a7) || (v)==(a8) )
#define $IS_ONE_OF_9_(v,a1,a2,a3,a4,a5,a6,a7,a8,a9) \
   ( (v)==(a1) || (v)==(a2) || (v)==(a3) || (v)==(a4) || (v)==(a5) || \
     (v)==(a6) || (v)==(a7) || (v)==(a8) || (v)==(a9) )
#define $IS_ONE_OF_10_(v,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10) \
   ( (v)==(a1) || (v)==(a2) || (v)==(a3) || (v)==(a4) || (v)==(a5) || \
     (v)==(a6) || (v)==(a7) || (v)==(a8) || (v)==(a9) || (v)==(a10) )


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// String helpers (zero terminated):
//
//==================================================================================================

#define $IS_EMPTY_SZ(szString)            (!*(szString))
#define $ZERO_LAST_CHAR_OF(szBuffVar)     ($LAST_ELEMENT_OF(szBuffVar)=0,(szBuffVar))


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Substitutions ("if zero","if null"):
//
//==================================================================================================

#define $ZERO_SUBST_(subj,subst)          ((subj)!=0 ? (subj) : (subst))
#define $NULL_SUBST_(subj,subst)          ((subj)!=NULL ? (subj) : (subst))

#define $ASSIGN_IF_ZERO_(target,value)    ((target)==0 ? (target)=(value) : (target))
#define $ASSIGN_IF_NULL_(target,value)    ((target)==NULL ? (target)=(value) : (target))


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Arguments substitution:
//
//==================================================================================================

#define $ARRAY_N_SIZE_OF(fixedArrayVar)   (fixedArrayVar), sizeof(fixedArrayVar)
#define $ARRAY_N_COUNT_OF(fixedArrayVar)  (fixedArrayVar), $COUNT_OF(fixedArrayVar)

#define $ADDRESS_N_SIZE_OF(variable)      (&(variable)), sizeof(variable)
#define $POINTER_N_OBSIZE_(pValue)        (pValue), sizeof(*(pValue))

#define $VA_LIST(lastFixedArg,argPtrVar) \
   (va_start((argPtrVar),(lastFixedArg)),(argPtrVar))
#define $VA_COMPLEMENT(lastFixedArg,argPtrVar) \
   (lastFixedArg), $VA_LIST((lastFixedArg),(argPtrVar))


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Bit flags helpers:
//
//==================================================================================================

#define $FLAG(bitNo)                      $FLAG_32(bitNo)
#define $CHECK_BIT(bitWord,bitNo)         $CHECK_BIT_32((bitWord),(bitNo))

#define $FLAG_8(bitNo)                    (($UINT8 )1 << (bitNo))
#define $FLAG_16(bitNo)                   (($UINT16)1 << (bitNo))
#define $FLAG_32(bitNo)                   (($UINT32)1 << (bitNo))
#define $FLAG_64(bitNo)                   (($UINT64)1 << (bitNo))

#define $BYTE_FLAG(bitNo)                 $FLAG_8(bitNo)
#define $WORD_FLAG(bitNo)                 $FLAG_16(bitNo)
#define $DWORD_FLAG(bitNo)                $FLAG_32(bitNo)
#define $QWORD_FLAG(bitNo)                $FLAG_64(bitNo)

#define $CHECK_BIT_8(bitWord,bitNo)       (( ($UINT8 )(bitWord) & ($UINT8 )1 << (bitNo) )!=0)
#define $CHECK_BIT_16(bitWord,bitNo)      (( ($UINT16)(bitWord) & ($UINT16)1 << (bitNo) )!=0)
#define $CHECK_BIT_32(bitWord,bitNo)      (( ($UINT32)(bitWord) & ($UINT32)1 << (bitNo) )!=0)
#define $CHECK_BIT_64(bitWord,bitNo)      (( ($UINT64)(bitWord) & ($UINT64)1 << (bitNo) )!=0)

#define $CHECK_BYTE_BIT(bitWord,bitNo)    $CHECK_BIT_8(bitWord,bitNo) 
#define $CHECK_WORD_BIT(bitWord,bitNo)    $CHECK_BIT_16(bitWord,bitNo)
#define $CHECK_DWORD_BIT(bitWord,bitNo)   $CHECK_BIT_32(bitWord,bitNo)
#define $CHECK_QWORD_BIT(bitWord,bitNo)   $CHECK_BIT_64(bitWord,bitNo)


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// "IOCTL"-code component extraction:
//
//==================================================================================================

#define $IOCTL_REQUIRED_ACCESS(ctlCode)   ( ( ((ULONG)(ctlCode))>>14 &  0x0000FFFFu )      )
#define $IOCTL_FUNCTION_CODE(ctlCode)     ( ( ((ULONG)(ctlCode))     &  0x00003FFFu ) >> 2 )
#define $IOCTL_TRANSFER_TYPE(ctlCode)     ( ( ((ULONG)(ctlCode))     &  0x00000003u )      )
#define $IOCTL_COMMON_BIT(ctlCode)        ( ( ((ULONG)(ctlCode))>>31                )      )
#define $IOCTL_CUSTOM_BIT(ctlCode)        ( ( ((ULONG)(ctlCode))>>13 & ~0x00000001u )      )


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// String function implementation (centralized message support):
//
//==================================================================================================

#define $IMPLEMENT_STR_FUNCTION(functionName,stringText) \
   EXTERN_C $PCSZ __stdcall functionName() \
   { return stringText; }

#define $IMPLEMENT_STR_FUNCTION_2(functionName,stringText_1,stringText_2) \
   EXTERN_C $PCSZ __stdcall functionName() \
   { return stringText_1 stringText_2; }

#define $IMPLEMENT_STR_FUNCTION_3(functionName,stringText_1,stringText_2,stringText_3) \
   EXTERN_C $PCSZ __stdcall functionName() \
   { return stringText_1 stringText_2 stringText_3; }

#define $IMPLEMENT_STR_FUNCTION_4( functionName, \
   stringText_1, stringText_2, stringText_3, stringText_4) \
   EXTERN_C $PCSZ __stdcall functionName() \
   { return stringText_1 stringText_2 stringText_3 stringText_4; }

#define $IMPLEMENT_STR_FUNCTION_5( functionName, \
   stringText_1, stringText_2, stringText_3, stringText_4, stringText_5 ) \
   EXTERN_C $PCSZ __stdcall functionName() \
   { return stringText_1 stringText_2 stringText_3 stringText_4 stringText_5; }

//--------------------------------------------------------------------------------------------------

#define $IMPLEMENT_ASTR_FUNCTION(functionName,stringText) \
   EXTERN_C $PCASZ __stdcall functionName() \
   { return stringText; }

#define $IMPLEMENT_ASTR_FUNCTION_2(functionName,stringText_1,stringText_2) \
   EXTERN_C $PCASZ __stdcall functionName() \
   { return stringText_1 stringText_2; }

#define $IMPLEMENT_ASTR_FUNCTION_3(functionName,stringText_1,stringText_2,stringText_3) \
   EXTERN_C $PCASZ __stdcall functionName() \
   { return stringText_1 stringText_2 stringText_3; }

#define $IMPLEMENT_ASTR_FUNCTION_4( functionName, \
   stringText_1, stringText_2, stringText_3, stringText_4) \
   EXTERN_C $PCASZ __stdcall functionName() \
   { return stringText_1 stringText_2 stringText_3 stringText_4; }

#define $IMPLEMENT_ASTR_FUNCTION_5( functionName, \
   stringText_1, stringText_2, stringText_3, stringText_4, stringText_5 ) \
   EXTERN_C $PCASZ __stdcall functionName() \
   { return stringText_1 stringText_2 stringText_3 stringText_4 stringText_5; }

//==================================================================================================

#endif // $KEY_DEFS_PP_H
