﻿#include "Precompiled.h"
#include "SharedIncludes.h"

//==================================================================================================

#define FATAL_ERROR_FMT_CAPTION \
   "## === Fatal error in \"%s\": ===\r\n"
#define FATAL_ERROR_FMT_MESSAGE \
   "## %s\r\n"
#define FATAL_ERROR_FMT_IRQL \
   "## Current IRQL: %s (%d).\r\n"
#define FATAL_ERROR_FMT_STATUS \
   "## Error status: 0x%08X (%s).\r\n"
#define FATAL_ERROR_FMT_SOURCE \
   "## Source file --> \"%s\"\r\n" \
   "## Line No ------> %d\r\n"

//--------------------------------------------------------------------------------------------------

#define FATAL_ERROR_FMT \
   "\r\n\r\n" \
   FATAL_ERROR_FMT_CAPTION \
   "##\r\n" \
   FATAL_ERROR_FMT_MESSAGE \
   "##\r\n" \
   FATAL_ERROR_FMT_IRQL \
   "\r\n"
#define FATAL_ERROR_FMT_WITH_STATUS \
   "\r\n\r\n" \
   FATAL_ERROR_FMT_CAPTION \
   "##\r\n" \
   FATAL_ERROR_FMT_MESSAGE \
   "##\r\n" \
   FATAL_ERROR_FMT_IRQL \
   FATAL_ERROR_FMT_STATUS \
   "\r\n"

#define FATAL_ERROR_FMT_EX \
   "\r\n\r\n" \
   FATAL_ERROR_FMT_CAPTION \
   "##\r\n" \
   FATAL_ERROR_FMT_MESSAGE \
   "##\r\n" \
   FATAL_ERROR_FMT_IRQL \
   "##\r\n" \
   FATAL_ERROR_FMT_SOURCE \
   "\r\n"
#define FATAL_ERROR_FMT_WITH_STATUS_EX \
   "\r\n\r\n" \
   FATAL_ERROR_FMT_CAPTION \
   "##\r\n" \
   FATAL_ERROR_FMT_MESSAGE \
   "##\r\n" \
   FATAL_ERROR_FMT_IRQL \
   FATAL_ERROR_FMT_STATUS \
   "##\r\n" \
   FATAL_ERROR_FMT_SOURCE \
   "\r\n"

//==================================================================================================

static $CPCASZ
   c_pcaszFunctionFailureFmt          = "Function \"s\" has failed.",
   c_pcaszFunctionFailureWithStatusFmt= "Function \"s\" has failed with status: 0x%08X (%s).",
   c_pcaszAssertionFailure            = "Assertion has failed in function \"%s\".",
   c_pcaszAssertionFailureDbg         = "Assertion has failed: << %s >>.";

//==================================================================================================

void $RaiseFatalError($PCASZ pcaszMessage)
{
   $RaiseFatalErrorDbg(NULL,0,pcaszMessage);
}

void $RaiseFatalErrorWithStatus(NTSTATUS status,$PCASZ pcaszMessage)
{
   $RaiseFatalErrorWithStatusDbg(NULL,0,status,pcaszMessage);
}

//--------------------------------------------------------------------------------------------------

void $RaisefFatalError($PCASZ pcaszMessageFmt, ... )
{
   va_list argList;

   $VRaisefFatalError($VA_COMPLEMENT(pcaszMessageFmt,argList));
}

void $RaisefFatalErrorWithStatus(NTSTATUS status,$PCASZ pcaszMessageFmt, ... )
{
   va_list argList;

   $VRaisefFatalErrorWithStatus(status,$VA_COMPLEMENT(pcaszMessageFmt,argList));
}

//--------------------------------------------------------------------------------------------------

void $VRaisefFatalError($PCASZ pcaszMessageFmt,va_list argList)
{
   $VRaisefFatalErrorDbg(NULL,0,pcaszMessageFmt,argList);
}

void $VRaisefFatalErrorWithStatus(NTSTATUS status,$PCASZ pcaszMessageFmt,va_list argList)
{
   $VRaisefFatalErrorWithStatusDbg(NULL,0,status,pcaszMessageFmt,argList);
}

//==================================================================================================

void $RaiseFatalErrorDbg( $PCASZ pcaszSourceFile OPTIONAL,
                         int nLineNo OPTIONAL,
                         $PCASZ pcaszMessage )
{
   KIRQL irql=KeGetCurrentIrql();

   DbgPrint(
      pcaszSourceFile && nLineNo ? FATAL_ERROR_FMT_EX : FATAL_ERROR_FMT,
      $DriverBinaryName(), pcaszMessage, $IrqlIdentifier(irql), irql,
      pcaszSourceFile, nLineNo );
   DbgBreakPoint();
}

void $RaiseFatalErrorWithStatusDbg( $PCASZ pcaszSourceFile OPTIONAL, int nLineNo OPTIONAL,
                                    NTSTATUS status, $PCASZ pcaszMessage )
{
   KIRQL irql=KeGetCurrentIrql();

   DbgPrint(
      pcaszSourceFile && nLineNo ? FATAL_ERROR_FMT_WITH_STATUS_EX : FATAL_ERROR_FMT_WITH_STATUS,
      $DriverBinaryName(), pcaszMessage, $IrqlIdentifier(irql), irql,
      status, $StatusSeverityName(status),
      pcaszSourceFile, nLineNo );
   DbgBreakPointWithStatus(status);
}

//--------------------------------------------------------------------------------------------------

void $RaisefFatalErrorDbg( $PCASZ pcaszSourceFile OPTIONAL, int nLineNo OPTIONAL,
                          $PCASZ pcaszMessageFmt, ... )
{
   va_list argList;

   $VRaisefFatalErrorDbg(
      pcaszSourceFile, nLineNo,
      $VA_COMPLEMENT(pcaszMessageFmt,argList) );
}

void $RaisefFatalErrorWithStatusDbg( $PCASZ pcaszSourceFile OPTIONAL, int nLineNo OPTIONAL,
                                     NTSTATUS status, $PCASZ pcaszMessageFmt, ... )
{
   va_list argList;

   $VRaisefFatalErrorWithStatusDbg(
      pcaszSourceFile, nLineNo,
      status, $VA_COMPLEMENT(pcaszMessageFmt,argList) );
}

//--------------------------------------------------------------------------------------------------

void $VRaisefFatalErrorDbg( $PCASZ pcaszSourceFile OPTIONAL, int nLineNo OPTIONAL,
                           $PCASZ pcaszMessageFmt, va_list argList )
{
   $ACH aszMessage[$DBG_PRINT_BUFFER_CCH];

   $StrVPrintfA( $ARRAY_N_COUNT_OF(aszMessage), pcaszMessageFmt, argList );

   $RaiseFatalErrorDbg(pcaszSourceFile,nLineNo,aszMessage);
}

void $VRaisefFatalErrorWithStatusDbg( $PCASZ pcaszSourceFile OPTIONAL, int nLineNo OPTIONAL,
                                      NTSTATUS status, $PCASZ pcaszMessageFmt, va_list argList )
{
   $ACH aszMessage[$DBG_PRINT_BUFFER_CCH];

   $StrVPrintfA( $ARRAY_N_COUNT_OF(aszMessage), pcaszMessageFmt, argList );

   $RaiseFatalErrorWithStatusDbg(pcaszSourceFile,nLineNo,status,aszMessage);
}

//==================================================================================================

void $RaiseFatalError_FunctionFailure($PCASZ pcaszFunctionName)
{
   $RaiseFatalErrorDbg_FunctionFailure(NULL,0,pcaszFunctionName);
}

void $RaiseFatalError_FunctionFailureWithStatus($PCASZ pcaszFunctionName,NTSTATUS status)
{
   $RaiseFatalErrorDbg_FunctionFailureWithStatus(NULL,0,pcaszFunctionName,status);
}

//--------------------------------------------------------------------------------------------------

void $RaiseFatalErrorDbg_FunctionFailure( $PCASZ pcaszSourceFile OPTIONAL,
                                         int nLineNo OPTIONAL,
                                         $PCASZ pcaszFunctionName )
{
   $RaisefFatalErrorDbg(pcaszSourceFile,nLineNo,c_pcaszFunctionFailureFmt,pcaszFunctionName);
}

void $RaiseFatalErrorDbg_FunctionFailureWithStatus
(
   $PCASZ pcaszSourceFile OPTIONAL, int nLineNo OPTIONAL,
   $PCASZ pcaszFunctionName, NTSTATUS status
)
{
   $RaisefFatalErrorWithStatusDbg(
      pcaszSourceFile, nLineNo, status,
      c_pcaszFunctionFailureWithStatusFmt,
      pcaszFunctionName, status, $StatusSeverityName(status) );
}

//==================================================================================================

void $RaiseFatalError_AssertionFailure($PCASZ pcaszCallingFunctionName)
{
   $RaisefFatalError(c_pcaszAssertionFailure,pcaszCallingFunctionName);
}

//--------------------------------------------------------------------------------------------------

void $RaiseFatalErrorDbg_AssertionFailure( $PCASZ pcaszSourceFile OPTIONAL,
                                           int nLineNo OPTIONAL,
                                           $PCASZ pcaszFailedExpression )
{
   $RaisefFatalErrorDbg(pcaszSourceFile,nLineNo,c_pcaszAssertionFailureDbg,pcaszFailedExpression);
}

//==================================================================================================
