﻿#include "Precompiled.h"
#include "SharedIncludes.h"

//==================================================================================================

BOOLEAN $RetrieveInputBuffer
(
   OUT NTSTATUS* pStatus, WDFREQUEST hRequest, size_t minimumRequiredSize,
   OUT $PPCVOID ppcvBuffer, OUT size_t* pLength OPTIONAL
)
{
   if (NT_SUCCESS(
      *pStatus = WdfRequestRetrieveInputBuffer( hRequest, minimumRequiredSize,
                                                ($PPVOID)ppcvBuffer, pLength )
   ))
      return TRUE;
   $DBG_PRINT_FUNCTION_FAILURE_WITH_STATUS("WdfRequestRetrieveInputBuffer",*pStatus);
   return FALSE;
}

//--------------------------------------------------------------------------------------------------

BOOLEAN $RetrieveOutputBuffer
(
   OUT NTSTATUS* pStatus, WDFREQUEST hRequest, size_t minimumRequiredSize,
   OUT $PPVOID ppvBuffer, OUT size_t* pLength OPTIONAL
)
{
   if (NT_SUCCESS(
      *pStatus = WdfRequestRetrieveOutputBuffer(hRequest,minimumRequiredSize,ppvBuffer,pLength)
   ))
      return TRUE;
   $DBG_PRINT_FUNCTION_FAILURE_WITH_STATUS("WdfRequestRetrieveOutputBuffer",*pStatus);
   return FALSE;
}

//==================================================================================================

BOOLEAN $RetrieveUserInputBuffer
(
   OUT NTSTATUS* pStatus, WDFREQUEST hRequest, size_t minimumRequiredSize,
   OUT $PPCVOID ppcvSafeBuffer, OUT size_t* pLength OPTIONAL
)
{
   NTSTATUS                status;
   PVOID                   pvUnsafeBuffer;
   WDFMEMORY               hMemory;
   WDF_OBJECT_ATTRIBUTES   memoryObjectAttributes;
   size_t                  length;

   PAGED_CODE();

   if (!NT_SUCCESS(
      *pStatus = WdfRequestRetrieveUnsafeUserInputBuffer( hRequest, minimumRequiredSize,
                                                          &pvUnsafeBuffer, &length )
   ))
   {
      $DBG_PRINT_FUNCTION_FAILURE_WITH_STATUS("WdfRequestRetrieveUnsafeUserInputBuffer",*pStatus);
      return FALSE;
   }

   WDF_OBJECT_ATTRIBUTES_INIT(&memoryObjectAttributes);
   memoryObjectAttributes.ParentObject=hRequest;

   if (!NT_SUCCESS(
      *pStatus = WdfMemoryCreate( &memoryObjectAttributes, PagedPool, $DriverPoolTag(),
                                  length, &hMemory, NULL )
   ))
   {
      $DBG_PRINT_FUNCTION_FAILURE_WITH_STATUS("WdfMemoryCreate",*pStatus);
      return FALSE;
   }

   __try
   {
      ProbeForRead(pvUnsafeBuffer,length,1);
      status = WdfMemoryCopyFromBuffer(hMemory,0,pvUnsafeBuffer,length);
   }
   __except( GetExceptionCode()==STATUS_ACCESS_VIOLATION ?
             EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH )
   {
      $DBG_PRINT_ERROR("Access violation when reading data from user input buffer.");
      *pStatus = STATUS_ACCESS_VIOLATION;
      return FALSE;
   }

   if (!NT_SUCCESS(*pStatus=status))
   {
      $DBG_PRINT_FUNCTION_FAILURE_WITH_STATUS("WdfMemoryCopyFromBuffer",*pStatus);
      return FALSE;
   }

   *ppcvSafeBuffer=WdfMemoryGetBuffer(hMemory,pLength);

   return TRUE;
}

//--------------------------------------------------------------------------------------------------

BOOLEAN $RetrieveUserOutputBuffer
(
   OUT NTSTATUS* pStatus, WDFREQUEST hRequest, size_t minimumRequiredSize,
   OPTIONAL OUT $PPVOID ppvSafeBuffer, OUT size_t* pLength OPTIONAL,
   OUT WDFMEMORY* phMemory, OUT $PPVOID ppvUnsafeBuffer
)
{
   WDF_OBJECT_ATTRIBUTES   memoryObjectAttributes;
   size_t                  length;

   PAGED_CODE();

   if (!NT_SUCCESS(
      *pStatus = WdfRequestRetrieveUnsafeUserOutputBuffer( hRequest, minimumRequiredSize,
                                                           ppvUnsafeBuffer, &length )
   ))
   {
      $DBG_PRINT_FUNCTION_FAILURE_WITH_STATUS("WdfRequestRetrieveUnsafeUserOutputBuffer",*pStatus);
      return FALSE;
   }

   if (pLength)
      *pLength=length;

   WDF_OBJECT_ATTRIBUTES_INIT(&memoryObjectAttributes);
   memoryObjectAttributes.ParentObject=hRequest;

   if (!NT_SUCCESS(
      *pStatus = WdfMemoryCreate( &memoryObjectAttributes, PagedPool, $DriverPoolTag(),
                                  length, phMemory, ppvSafeBuffer )
   ))
   {
      $DBG_PRINT_FUNCTION_FAILURE_WITH_STATUS("WdfMemoryCreate",*pStatus);
      return FALSE;
   }

   if (ppvSafeBuffer)
      *ppvSafeBuffer=WdfMemoryGetBuffer(*phMemory,NULL);

   return TRUE;
}

//==================================================================================================

BOOLEAN $CopyOutputMemoryToUnsafeUserBuffer( OUT NTSTATUS* pStatus,
                                             WDFMEMORY hMemory,
                                             PVOID pvUnsafeBuffer )
{
   NTSTATUS    status;
   size_t      bufferSize;

   PAGED_CODE();

   WdfMemoryGetBuffer(hMemory,&bufferSize);

   __try
   {
      ProbeForWrite(pvUnsafeBuffer,bufferSize,1);
      status = WdfMemoryCopyToBuffer(hMemory,0,pvUnsafeBuffer,bufferSize);
   }
   __except( GetExceptionCode()==STATUS_ACCESS_VIOLATION ?
             EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH )
   {
      $DBG_PRINT_ERROR("Access violation when writing data to user output buffer.");
      *pStatus = STATUS_ACCESS_VIOLATION;
      return FALSE;
   }

   if (!NT_SUCCESS(*pStatus=status))
   {
      $DBG_PRINT_FUNCTION_FAILURE_WITH_STATUS("WdfMemoryCopyToBuffer",*pStatus);
      return FALSE;
   }

   return TRUE;
}

//==================================================================================================
