﻿#include "Precompiled.h"
#include "SharedIncludes.h"

//==================================================================================================

static BOOLEAN CheckCondition(OUT NTSTATUS* pStatus,BOOLEAN bCondition);
static BOOLEAN CheckConditionEx(OUT NTSTATUS* pStatus,BOOLEAN bCondition,NTSTATUS failureStatus);

//==================================================================================================

BOOLEAN $CheckInputLength(OUT NTSTATUS* pStatus,BOOLEAN bCondition)
{
   if (CheckCondition(pStatus,bCondition))
      return TRUE;
   $DBG_PRINT_INCORRECT_INPUT_LENGTH_E();
   return FALSE;
}

BOOLEAN $CheckOutputLength(OUT NTSTATUS* pStatus,BOOLEAN bCondition)
{
   if (CheckCondition(pStatus,bCondition))
      return TRUE;
   $DBG_PRINT_INCORRECT_OUTPUT_LENGTH_E();
   return FALSE;
}

//--------------------------------------------------------------------------------------------------

BOOLEAN $CheckInputLengthIsZero(IN OUT NTSTATUS* pStatus,size_t length)
{
   if (CheckCondition(pStatus,!length))
      return TRUE;
   $DBG_PRINT_NONZERO_INPUT_LENGTH_E();
   return FALSE;
}

BOOLEAN $CheckOutputLengthIsZero(IN OUT NTSTATUS* pStatus,size_t length)
{
   if (CheckCondition(pStatus,!length))
      return TRUE;
   $DBG_PRINT_NONZERO_OUTPUT_LENGTH_E();
   return FALSE;
}

//--------------------------------------------------------------------------------------------------

BOOLEAN $CheckInputLengthNotZero(IN OUT NTSTATUS* pStatus,size_t length)
{
   if (CheckCondition(pStatus,length!=0))
      return TRUE;
   $DBG_PRINT_ZERO_INPUT_LENGTH_E();
   return FALSE;
}

BOOLEAN $CheckOutputLengthNotZero(IN OUT NTSTATUS* pStatus,size_t length)
{
   if (CheckCondition(pStatus,length!=0))
      return TRUE;
   $DBG_PRINT_ZERO_OUTPUT_LENGTH_E();
   return FALSE;
}

//--------------------------------------------------------------------------------------------------

BOOLEAN $CheckInputOutputLengthsAreZero( OUT NTSTATUS* pStatus,
                                         size_t inputLength,
                                         size_t ouputLength )
{
   return
      $CheckInputLengthIsZero(pStatus,inputLength) &&
      $CheckOutputLengthIsZero(pStatus,ouputLength);
}

BOOLEAN $CheckInputOutputLengthsNotZero( OUT NTSTATUS* pStatus,
                                         size_t inputLength,
                                         size_t ouputLength )
{
   return
      $CheckInputLengthNotZero(pStatus,inputLength) &&
      $CheckOutputLengthNotZero(pStatus,ouputLength);
}

//--------------------------------------------------------------------------------------------------

BOOLEAN $CheckInputLengthIs(OUT NTSTATUS* pStatus,size_t length,size_t expectedLength)
{
   return $CheckInputLength(pStatus,length==expectedLength);
}

BOOLEAN $CheckOutputLengthIs(OUT NTSTATUS* pStatus,size_t length,size_t expectedLength)
{
   return $CheckOutputLength(pStatus,length==expectedLength);
}

//--------------------------------------------------------------------------------------------------

BOOLEAN $CheckSufficientInputLength(OUT NTSTATUS* pStatus,size_t length,size_t minimumLength)
{
   if (CheckConditionEx(pStatus,length>=minimumLength,STATUS_BUFFER_TOO_SMALL))
      return TRUE;
   $DBG_PRINT_INSUFFICIENT_INPUT_LENGTH_E();
   return FALSE;
}

BOOLEAN $CheckSufficientOutputLength(OUT NTSTATUS* pStatus,size_t length,size_t minimumLength)
{
   if (CheckConditionEx(pStatus,length>=minimumLength,STATUS_BUFFER_TOO_SMALL))
      return TRUE;
   $DBG_PRINT_INSUFFICIENT_OUTPUT_LENGTH_E();
   return FALSE;
}

//==================================================================================================

static BOOLEAN CheckCondition(OUT NTSTATUS* pStatus,BOOLEAN bCondition)
{
   return CheckConditionEx(pStatus,bCondition,STATUS_INVALID_BUFFER_SIZE);
}

static BOOLEAN CheckConditionEx(OUT NTSTATUS* pStatus,BOOLEAN bCondition,NTSTATUS failureStatus)
{
   *pStatus = bCondition ? STATUS_SUCCESS : failureStatus;
   return $AS_BOOL(bCondition);
}

//==================================================================================================
