﻿#include "Precompiled.h"
#include "SharedIncludes.h"

//=================================================================================================

static void VerifyStringRoutineParameter($PCSZ pcszString);
static void VerifyStringRoutineParameters($PCSZ pcszDest,size_t cchDest,$PCSZ pcszSource);
static BOOLEAN CheckStringSafeRoutineStatus(NTSTATUS status);

//-------------------------------------------------------------------------------------------------

static $CPCASZ
   c_pcaszInvalidParametersForStringRoutine="Invalid parameter(s) for string routine.";

//=================================================================================================

void VerifyStringRoutineParameterX(PVOID pvString)
{
   if (!pvString)
      $RaiseFatalErrorWithStatus(
         STATUS_INVALID_PARAMETER,
         c_pcaszInvalidParametersForStringRoutine );
}

void VerifyStringRoutineParametersX(PVOID pvDest,size_t cchDest,PVOID pvSrcOrFmt)
{
   if (!(pvDest && cchDest && pvSrcOrFmt))
      $RaiseFatalErrorWithStatus(
         STATUS_INVALID_PARAMETER,
         c_pcaszInvalidParametersForStringRoutine );
}

//=================================================================================================

size_t $StrLength($PCSZ pcsz)
{
   $PCCHAR pcch;

   VerifyStringRoutineParameter(pcsz);

   for (pcch=pcsz;;pcch++)
      if (!*pcch)
         return pcch-pcsz;
}

//-------------------------------------------------------------------------------------------------

BOOLEAN $StrCopy(OUT $PSZ pszDest,size_t cchDest,$PCSZ pcszSource)
{
   VerifyStringRoutineParameters(pszDest,cchDest,pcszSource);
   return CheckStringSafeRoutineStatus(
      RtlStringCchCopyW(pszDest,cchDest,pcszSource) );
}

//-------------------------------------------------------------------------------------------------

BOOLEAN $StrPrintf(OUT $PSZ pszDest,size_t cchDest,$PCSZ pcszFormat, ... )
{
   va_list argList;
   VerifyStringRoutineParameters(pszDest,cchDest,pcszFormat);
   return $StrVPrintf(
      pszDest, cchDest, $VA_COMPLEMENT(pcszFormat,argList) );
}

BOOLEAN $StrVPrintf(OUT $PSZ pszDest,size_t cchDest,$PCSZ pcszFormat,va_list argList)
{
   VerifyStringRoutineParameters(pszDest,cchDest,pcszFormat);
   return CheckStringSafeRoutineStatus(
      RtlStringCchVPrintfW(pszDest,cchDest,pcszFormat,argList) );
}

//=================================================================================================

BOOLEAN $StrUsCopy( OUT PUNICODE_STRING pustr,
                    $PSZ pszDest, size_t cchDest,
                    $PCSZ pcszSource )
{
   BOOLEAN rslt=$StrCopy(pszDest,cchDest,pcszSource);
   RtlInitUnicodeString(pustr,pszDest);
   return rslt;
}

//-------------------------------------------------------------------------------------------------

BOOLEAN $StrUsPrintf( OUT PUNICODE_STRING pustr,
                      $PSZ pszDest, size_t cchDest,
                      $PCSZ pcszFormat, ... )
{
   va_list argList;
   return $StrUsVPrintf(
      pustr, pszDest, cchDest, $VA_COMPLEMENT(pcszFormat,argList) );
}

BOOLEAN $StrUsVPrintf( OUT PUNICODE_STRING pustr,
                       $PSZ pszDest, size_t cchDest,
                       $PCSZ pcszFormat, va_list argList )
{
   BOOLEAN rslt=$StrVPrintf(pszDest,cchDest,pcszFormat,argList);
   RtlInitUnicodeString(pustr,pszDest);
   return rslt;
}

//=================================================================================================

static void VerifyStringRoutineParameter($PCSZ pcszString)
{
   VerifyStringRoutineParameterX((PVOID)pcszString);
}

static void VerifyStringRoutineParameters($PCSZ pcszDest,size_t cchDest,$PCSZ pcszSource)
{
   VerifyStringRoutineParametersX((PVOID)pcszDest,cchDest,(PVOID)pcszSource);
}

//-------------------------------------------------------------------------------------------------

BOOLEAN CheckStringSafeRoutineStatus(NTSTATUS status)
{
   BOOLEAN bBufferOverlfow =
      status==STATUS_BUFFER_OVERFLOW;
   if (!NT_SUCCESS(status) && !bBufferOverlfow)
      $RaiseFatalErrorWithStatus( status,
         status==STATUS_INVALID_PARAMETER ?
            "Invalid parameter for string safe routine." :
            "Failure of string safe routine." );
   return
      !bBufferOverlfow;
}

//=================================================================================================
