﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// КОМАНДЫ УСТРОЙСТВА:
//
//==================================================================================================

#include "Precompiled.h"
#include "SharedIncludes.h"

//==================================================================================================

static BOOLEAN ClearDeviceMemory( OUT NTSTATUS* pStatus, PDRIVER_FDO_DATA pFdoData,
                                  $UINT uBankNo, $UINT uStartAddress, $UINT uLength );

static BOOLEAN TestDeviceMemory( PDRIVER_FDO_DATA pFdoData, $UINT uBitPattern, $UINT uBankNo,
                                 $UINT uStartAddress, $UINT uAddressIncrement,
                                 OUT $PUINT puInfoValues );

//==================================================================================================

void Driver_DeviceDeactivate(PDRIVER_FDO_DATA pFdoData)
{
   //if ( Driver_DeviceReadRegister(pFdoData,SHAPER_RGM) != SHAPER_RGM_INACTIVE )
   {
      Driver_DeviceWriteRegister(pFdoData,SHAPER_RGM,0x80600000u);
      Driver_DeviceWriteRegister(pFdoData,SHAPER_RGM,SHAPER_RGM_INACTIVE);

      $DBG_PRINT_INFORMATION("Устройство переведено в неактивное состояние.");
   }
}

//--------------------------------------------------------------------------------------------------

BOOLEAN Driver_DeviceLoadRegisters( OUT NTSTATUS* pStatus, PDRIVER_FDO_DATA pFdoData,
                                    $PCUINT pcuAddressValuePairs, $UINT uPairCount )
{
   $UINT uIdx;

   for (uIdx=0; uIdx<uPairCount; uIdx++)
      if (!Driver_DeviceCheckRegAddress(pStatus,pcuAddressValuePairs[uIdx*2u]))
      {
         $DBG_PRINT_ERROR("Некорректные данные для загрузки регистров устройства.");
         *pStatus = STATUS_INVALID_PARAMETER;
         return FALSE;
      }

   for (uIdx=0; uIdx<uPairCount; uIdx++)
      Driver_DeviceWriteRegister( pFdoData,
                                  pcuAddressValuePairs[uIdx*2u],
                                  pcuAddressValuePairs[uIdx*2u+1u] );

   *pStatus = STATUS_SUCCESS;
   return TRUE;

}

//--------------------------------------------------------------------------------------------------

BOOLEAN Driver_DeviceLoadProfile( OUT NTSTATUS* pStatus, PDRIVER_FDO_DATA pFdoData,
                                  $UINT uBankNo, $UINT uStartAddress,
                                  $PCUINT pcuFreqMagCodes, $UINT uSampleCount )
{
   DRIVER_MEMREG_PRESERVATION
      memoryRegisters;
   $UINT
      uIdx;

   if (!(
      $IS_ONE_OF_2_(uBankNo,0u,1u) &&
      uStartAddress<=SHAPER_MAX_CELL_ADDRESS &&
      uStartAddress+uSampleCount<=SHAPER_MAX_CELL_ADDRESS+1u &&
      uSampleCount
   ))
      goto INCORRECT_DATA_TO_LOAD;

   for (uIdx=0; uIdx<uSampleCount; uIdx++)
      if ( (pcuFreqMagCodes[uIdx] & SHAPER_FREQMAG_CODE_MASK) != pcuFreqMagCodes[uIdx] )
         goto INCORRECT_DATA_TO_LOAD;

   Driver_DeviceSwitchToMemoryMode(pFdoData,&memoryRegisters);

   if (!ClearDeviceMemory( pStatus, pFdoData, uBankNo,
                           0u, uStartAddress ))
      goto RESTORE;

   for (uIdx=0; uIdx<uSampleCount; uIdx++)
      if (!Driver_DeviceWriteMemoryAndCheck( pStatus, pFdoData, uBankNo,
                                             uStartAddress+uIdx, pcuFreqMagCodes[uIdx] ))
         goto RESTORE;

   if (!ClearDeviceMemory( pStatus, pFdoData, uBankNo,
                           uStartAddress+uSampleCount,
                           SHAPER_MEMORY_BANK_CAPACITY-uStartAddress-uSampleCount ))
      goto RESTORE;

   *pStatus = STATUS_SUCCESS;

RESTORE:
   Driver_DeviceRestoreFromMemoryMode(pFdoData,&memoryRegisters);
   return NT_SUCCESS(*pStatus);

INCORRECT_DATA_TO_LOAD:
   $DBG_PRINT_ERROR("Некорректные данные для загрузки профиля сигнала.");
   *pStatus = STATUS_INVALID_PARAMETER;
   return FALSE;
}

//--------------------------------------------------------------------------------------------------

BOOLEAN Driver_DeviceTestMemory(PDRIVER_FDO_DATA pFdoData,OUT $PUINT puInfoValues)
{
   DRIVER_MEMREG_PRESERVATION
      memoryRegisters;
   $UINT
      auStartAddresses   []={0,0,1},
      auAddressIncrements[]={1,2,2},
      uRGMPreviousValue,
      uBankNo;
   BOOLEAN
      rslt = TRUE;
   int
      i;

   Driver_DeviceSwitchToMemoryMode(pFdoData,&memoryRegisters);

   for (uBankNo=0; uBankNo<=1; uBankNo++)
      for (i=0; i<3; i++)
      {
         $UINT
            uStart    = auStartAddresses   [i],
            uIncrement= auAddressIncrements[i];
         if (!(
            TestDeviceMemory(pFdoData,0x00000000u,uBankNo,uStart,uIncrement,puInfoValues) &&
            TestDeviceMemory(pFdoData,0xFFFFFFFFu,uBankNo,uStart,uIncrement,puInfoValues) &&
            TestDeviceMemory(pFdoData,0x55555555u,uBankNo,uStart,uIncrement,puInfoValues) &&
            TestDeviceMemory(pFdoData,0xAAAAAAAAu,uBankNo,uStart,uIncrement,puInfoValues)
         ))
         {
            rslt = FALSE;
            goto RESTORE;
         }
      }

RESTORE:
   Driver_DeviceRestoreFromMemoryMode(pFdoData,&memoryRegisters);

   return rslt;
}

//--------------------------------------------------------------------------------------------------

BOOLEAN Driver_DeviceGenerateSawTooth( OUT NTSTATUS* pStatus, PDRIVER_FDO_DATA pFdoData,
                                       BOOLEAN bIsFixedParameterMagnitude,
                                       $UINT uFixedFrequencyOrMagnitude,
                                       $UINT uSampleDelay )
{
   $UINT
      uFixedParameterMaxValue,
      uVariableParameterValue,
      uVariableParameterMaxValue,
      uRGMPreviousValue;
   int
      uFixedParameterFirstBit,
      uVariableParameterFirstBit;

   if (!bIsFixedParameterMagnitude)
   {
      uFixedParameterMaxValue   = SHAPER_MAX_SAWTEST_FREQUENCY_CODE;
      uVariableParameterMaxValue= SHAPER_MAX_SAWTEST_MAGNITUDE_CODE;
      uFixedParameterFirstBit   = SHAPER_FREQUENCY_FIRST_BIT;
      uVariableParameterFirstBit= SHAPER_MAGNITUDE_FIRST_BIT;
   }
   else
   {
      uFixedParameterMaxValue   = SHAPER_MAX_SAWTEST_MAGNITUDE_CODE;
      uVariableParameterMaxValue= SHAPER_MAX_SAWTEST_FREQUENCY_CODE;
      uFixedParameterFirstBit   = SHAPER_MAGNITUDE_FIRST_BIT;
      uVariableParameterFirstBit= SHAPER_FREQUENCY_FIRST_BIT;
   }

   if (!(
      uFixedFrequencyOrMagnitude<=uFixedParameterMaxValue &&
      uSampleDelay<=50u
   ))
   {
      $DBG_PRINT_ERROR("Некорректные параметры для воспроизведения пилозуба.");
      *pStatus = STATUS_INVALID_PARAMETER;
      return FALSE;
   }

   uRGMPreviousValue = Driver_DeviceReadRegister(pFdoData,SHAPER_RGM);

   Driver_DeviceWriteRegister(pFdoData,SHAPER_RGM,SHAPER_RGM_SYNTHESIZER);

   for ( uVariableParameterValue=0u;
         uVariableParameterValue<uVariableParameterMaxValue;
         uVariableParameterValue++ )
   {
      Driver_DeviceWriteRegister( pFdoData, SHAPER_RD,
         uFixedFrequencyOrMagnitude << uFixedParameterFirstBit |
         uVariableParameterValue    << uVariableParameterFirstBit );
      if (uSampleDelay)
         KeStallExecutionProcessor(uSampleDelay);
   }

   Driver_DeviceWriteRegister(pFdoData,SHAPER_RGM,uRGMPreviousValue);

   return TRUE;
}

//==================================================================================================

static BOOLEAN ClearDeviceMemory( OUT NTSTATUS* pStatus, PDRIVER_FDO_DATA pFdoData,
                                  $UINT uBankNo, $UINT uStartAddress, $UINT uLength )
{
   $UINT uAddress;

   for ( uAddress=uStartAddress;
         uAddress<uStartAddress+uLength;
         uAddress++ )
      if (!Driver_DeviceWriteMemoryAndCheck(pStatus,pFdoData,uBankNo,uAddress,0u))
         return FALSE;

   *pStatus = STATUS_SUCCESS;
   return TRUE;
}

//--------------------------------------------------------------------------------------------------

static BOOLEAN TestDeviceMemory( PDRIVER_FDO_DATA pFdoData, $UINT uBitPattern, $UINT uBankNo,
                                 $UINT uStartAddress, $UINT uAddressIncrement,
                                 OUT $PUINT puInfoValues )
{
   $UINT
      uTestValue = uBitPattern & SHAPER_CELL_VALUE_MASK,
      uAddress;

   for ( uAddress=uStartAddress;
         uAddress<=SHAPER_MAX_CELL_ADDRESS;
         uAddress+=uAddressIncrement )
      Driver_DeviceWriteMemory(pFdoData,uBankNo,uAddress,uTestValue);

   for ( uAddress=uStartAddress;
         uAddress<=SHAPER_MAX_CELL_ADDRESS;
         uAddress+=uAddressIncrement )
   {
      $UINT
         uReadValue = Driver_DeviceReadMemory(pFdoData,uBankNo,uAddress);

      if ( uReadValue != uTestValue )
      {
         Driver_DeviceReportMemoryWriteReadDifference( uBankNo, uAddress,
                                                       uTestValue, uReadValue );
         puInfoValues[0] = uBankNo;
         puInfoValues[1] = uAddress;
         puInfoValues[2] = uTestValue;
         puInfoValues[3] = uReadValue;

         return FALSE;
      }
   }

   return TRUE;
}

//==================================================================================================
