﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// ОПЕРАЦИИ ЧТЕНИЯ/ЗАПИСИ В ПАМЯТЬ УСТРОЙСТВА:
//
//==================================================================================================

#include "Precompiled.h"
#include "SharedIncludes.h"

//==================================================================================================

static void VerifyInternalAddress($UINT uBankNo,$UINT uAddress);

//==================================================================================================

void Driver_DeviceSwitchToMemoryMode( PDRIVER_FDO_DATA pFdoData,
                                      OUT PDRIVER_MEMREG_PRESERVATION pRegisters )
{
   pRegisters->RGM     = Driver_DeviceReadRegisterFromCache( pFdoData, SHAPER_RGM     );
   pRegisters->MCR1    = Driver_DeviceReadRegisterFromCache( pFdoData, SHAPER_MCR1    );
   pRegisters->MCR2    = Driver_DeviceReadRegisterFromCache( pFdoData, SHAPER_MCR2    );
   pRegisters->RTHRESH = Driver_DeviceReadRegisterFromCache( pFdoData, SHAPER_RTHRESH );
   pRegisters->RDEL    = Driver_DeviceReadRegisterFromCache( pFdoData, SHAPER_RDEL    );
   pRegisters->RPERIOD = Driver_DeviceReadRegisterFromCache( pFdoData, SHAPER_RPERIOD );

   Driver_DeviceWriteRegister( pFdoData, SHAPER_RGM, SHAPER_RGM_MEMORY );
}

//--------------------------------------------------------------------------------------------------

void Driver_DeviceRestoreFromMemoryMode( PDRIVER_FDO_DATA pFdoData,
                                         PCDRIVER_MEMREG_PRESERVATION pcRegisters )
{
   Driver_DeviceWriteRegister( pFdoData, SHAPER_RGM,
                               SHAPER_RGM_REG_RESTORATION_MASK & pcRegisters->RGM );

   Driver_DeviceWriteRegister( pFdoData, SHAPER_MCR1   , pcRegisters->MCR1    );
   Driver_DeviceWriteRegister( pFdoData, SHAPER_MCR2   , pcRegisters->MCR2    );
   Driver_DeviceWriteRegister( pFdoData, SHAPER_RTHRESH, pcRegisters->RTHRESH );
   Driver_DeviceWriteRegister( pFdoData, SHAPER_RDEL   , pcRegisters->RDEL    );
   Driver_DeviceWriteRegister( pFdoData, SHAPER_RPERIOD, pcRegisters->RPERIOD );
   Driver_DeviceWriteRegister( pFdoData, SHAPER_RGM    , pcRegisters->RGM     );
}

//--------------------------------------------------------------------------------------------------

$UINT Driver_DeviceReadMemory(PDRIVER_FDO_DATA pFdoData,$UINT uBankNo,$UINT uAddress)
{
   VerifyInternalAddress(uBankNo,uAddress);

   Driver_DeviceWriteRegister( pFdoData, SHAPER_MCR1,
      (uBankNo ? SHAPER_BANK_1 : SHAPER_BANK_0) | SHAPER_READ_OPERATION | uAddress );

   return Driver_DeviceReadRegister(pFdoData,SHAPER_RD);
}

//--------------------------------------------------------------------------------------------------

void Driver_DeviceWriteMemory( PDRIVER_FDO_DATA pFdoData, $UINT uBankNo,
                               $UINT uAddress, $UINT uValue )
{
   VerifyInternalAddress(uBankNo,uAddress);

   if ( (uValue & SHAPER_CELL_VALUE_MASK) != uValue )
      $RaisefFatalErrorWithStatus( STATUS_INVALID_PARAMETER,
         "Попытка записи некорректного значения в память устройства: 0x%08X.",
         uValue );

   Driver_DeviceWriteRegister(pFdoData,SHAPER_RD,uValue);

   Driver_DeviceWriteRegister( pFdoData, SHAPER_MCR1,
      (uBankNo ? SHAPER_BANK_1 : SHAPER_BANK_0) | SHAPER_WRITE_OPERATION | uAddress );
}

//--------------------------------------------------------------------------------------------------

BOOLEAN Driver_DeviceWriteMemoryAndCheck( OUT NTSTATUS* pStatus, PDRIVER_FDO_DATA pFdoData,
                                          $UINT uBankNo, $UINT uAddress, $UINT uValue )
{
   $UINT uReadValue;

   Driver_DeviceWriteMemory(pFdoData,uBankNo,uAddress,uValue);

   uReadValue = Driver_DeviceReadMemory(pFdoData,uBankNo,uAddress);

   if (uReadValue==uValue)
   {
      *pStatus = STATUS_SUCCESS;
      return TRUE;
   }

   Driver_DeviceReportMemoryWriteReadDifference( uBankNo, uAddress,
                                                 uValue, uReadValue );

   *pStatus = STATUS_IO_DEVICE_ERROR;
   return FALSE;
}

//--------------------------------------------------------------------------------------------------

void Driver_DeviceReportMemoryWriteReadDifference( $UINT uBankNo, $UINT uAddress,
                                                   $UINT uWrittenValue, $UINT uReadValue )
{
   VerifyInternalAddress(uBankNo,uAddress);

   $DBG_PRINT_ERROR(
      "___ Считанное значение ячейки памяти отличается от ранее записанного: ___" );

   $DBG_PRINTF_ERROR(( "## Номер банка (0/1) ----> %u"    , uBankNo       ));
   $DBG_PRINTF_ERROR(( "## Адрес ячейки ---------> 0x%03X", uAddress      ));
   $DBG_PRINTF_ERROR(( "## Записанное значение --> 0x%08X", uWrittenValue ));
   $DBG_PRINTF_ERROR(( "## Считанное значение ---> 0x%08X", uReadValue    ));
}

//==================================================================================================

static void VerifyInternalAddress($UINT uBankNo,$UINT uAddress)
{
   if (!$IS_ONE_OF_2_(uBankNo,0u,1u))
      $RaisefFatalErrorWithStatus( STATUS_INVALID_PARAMETER,
         "Попытка обращения к некорректному банку памяти устройства: %u.",
         uBankNo );
   if (uAddress>SHAPER_MAX_CELL_ADDRESS)
      $RaisefFatalErrorWithStatus( STATUS_INVALID_PARAMETER,
         "Попытка обращения по некорректному адресу памяти устройства: 0x%08X.",
         uAddress );
}

//==================================================================================================
