﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// ОПЕРАЦИИ ЧТЕНИЯ/ЗАПИСИ В РЕГИСТРЫ УСТРОЙСТВА:
//
//==================================================================================================

#include "Precompiled.h"
#include "SharedIncludes.h"

//==================================================================================================

static void VerifyRegAddress($UINT uRelativeAddress);
static int RegIndexByAddress($UINT uRelativeAddress);

//==================================================================================================

BOOLEAN Driver_DeviceIsValidRegAddress($UINT uRelativeAddress)
{
   return
      uRelativeAddress % ($UINT)sizeof($UINT) == 0u &&
      $BETWEEN_( uRelativeAddress,
         SHAPER_MIN_REG_ADDRESS, SHAPER_MAX_REG_ADDRESS );
}

//--------------------------------------------------------------------------------------------------

BOOLEAN Driver_DeviceCheckRegAddress(OUT NTSTATUS* pStatus,$UINT uRelativeAddress)
{
   if (!(Driver_DeviceIsValidRegAddress(uRelativeAddress)))
   {
      $DBG_PRINTF_ERROR((
         "Некорректный адрес регистра устройства: 0x%08X (относительный).",
         uRelativeAddress ));
      *pStatus = STATUS_INVALID_PARAMETER;
      return FALSE;
   }
   *pStatus = STATUS_SUCCESS;
   return TRUE;
}

//--------------------------------------------------------------------------------------------------

$UINT Driver_DeviceReadRegister(PDRIVER_FDO_DATA pFdoData,$UINT uRelativeAddress)
{
   return Driver_DeviceReadRegisterEx(pFdoData,uRelativeAddress,FALSE,NULL);
}

//--------------------------------------------------------------------------------------------------

$UINT Driver_DeviceReadRegisterFromCache(PDRIVER_FDO_DATA pFdoData,$UINT uRelativeAddress)
{
   return Driver_DeviceReadRegisterEx(pFdoData,uRelativeAddress,TRUE,NULL);
}

//--------------------------------------------------------------------------------------------------

$UINT Driver_DeviceReadRegisterEx( PDRIVER_FDO_DATA pFdoData,
                                   $UINT uRelativeAddress, BOOLEAN bReadFromCache,
                                   OUT PBOOLEAN pbReallyCachedValueReturned OPTIONAL )
{
   PULONG puIoAddress;

   VerifyRegAddress(uRelativeAddress);

   if (bReadFromCache)
   {
      int nRegIdx = RegIndexByAddress(uRelativeAddress);

      if (pbReallyCachedValueReturned)
         *pbReallyCachedValueReturned = pFdoData->RegPresenceInCache[nRegIdx];

      return pFdoData->CachedRegValues[nRegIdx];
   }

   if (pbReallyCachedValueReturned)
      *pbReallyCachedValueReturned = FALSE;

   if (pFdoData->IsInDummyMode)
      return 0u;

   puIoAddress = (PULONG)
      ( ($PBYTE)pFdoData->BaseAddressPointer +
        (size_t)uRelativeAddress );

   return pFdoData->MemoryMappedPorts ?
      *puIoAddress : (READ_PORT_ULONG(puIoAddress));
}

//--------------------------------------------------------------------------------------------------

void Driver_DeviceWriteRegister(PDRIVER_FDO_DATA pFdoData,$UINT uRelativeAddress,$UINT uValue)
{
   int
      nRegIdx = RegIndexByAddress(uRelativeAddress);
   PULONG
      puIoAddress;

   VerifyRegAddress(uRelativeAddress);

   pFdoData->CachedRegValues   [ nRegIdx ] = uValue;
   pFdoData->RegPresenceInCache[ nRegIdx ] = TRUE;

   if (pFdoData->IsInDummyMode)
      return;

   puIoAddress = (PULONG)
      ( ($PBYTE)pFdoData->BaseAddressPointer +
        (size_t)uRelativeAddress );

   if (pFdoData->MemoryMappedPorts)
      *puIoAddress = uValue;
   else
      (WRITE_PORT_ULONG(puIoAddress,uValue));
}

//==================================================================================================

static void VerifyRegAddress($UINT uRelativeAddress)
{
   NTSTATUS status;

   if (!Driver_DeviceCheckRegAddress(&status,uRelativeAddress))
      $RaisefFatalErrorWithStatus( status,
         "Попытка обращения по некорректному адресу регистра устройства: 0x%08X (относительный).",
         uRelativeAddress );
}

static int RegIndexByAddress($UINT uRelativeAddress)
{
   VerifyRegAddress(uRelativeAddress);

   return (int)((uRelativeAddress-SHAPER_MIN_REG_ADDRESS)/sizeof($UINT));
}

//==================================================================================================
