﻿//**************************************************************************************************
//
// ОПРЕДЕЛЕНИЯ ДРАЙВЕРА.
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#pragma once

//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
//  Константы:
//
//==================================================================================================

#define DRIVER_LOAD_PROFILE_HEADER_ELEMENTS  (2u)
#define DRIVER_TEST_MEMORY_INFO_ELEMENTS     (4u)


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
//  Общие структуры:
//
//==================================================================================================

// Структура контекста устройства:
typedef struct _DRIVER_FDO_DATA
{
   BOOLEAN        CreatedSuccessfully;
   BOOLEAN        HardwarePreparedSuccessfully;
   BOOLEAN        IsInDummyMode;
   BOOLEAN        WasInD0;
   BOOLEAN        FileCreated;
   BOOLEAN        FileInitialized;
   BOOLEAN        MemoryMappedPorts;
   PVOID          BaseAddressPointer;
   $UINT          IoRange;
   $UINT          RGMValueD0;
   $UINT          CachedRegValues   [SHAPER_TOTAL_REGISTERS];
   BOOLEAN        RegPresenceInCache[SHAPER_TOTAL_REGISTERS];
   WDFSPINLOCK    DispatchLevelSpinLockHandle;
}
DRIVER_FDO_DATA, *PDRIVER_FDO_DATA;

// Генерация аксессора получения указателя на структуру контекста устройства:
WDF_DECLARE_CONTEXT_TYPE_WITH_NAME( DRIVER_FDO_DATA, Driver_FdoGetData );

//--------------------------------------------------------------------------------------------------

// Структура сохранения регистров памяти устройства:
typedef struct _DRIVER_MEMREG_PRESERVATION
{
   $UINT
      RGM, MCR1, MCR2,
      RTHRESH, RDEL, RPERIOD;
}
DRIVER_MEMREG_PRESERVATION, *PDRIVER_MEMREG_PRESERVATION;
typedef const DRIVER_MEMREG_PRESERVATION* PCDRIVER_MEMREG_PRESERVATION;


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Глобальные переменные:
//
//==================================================================================================

extern BOOLEAN
   Driver_AllowDummyMode;
extern LONG volatile
   Driver_DeviceCount;


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
//  Обработчики WDF-событий:
//
//==================================================================================================

// Точка входа драйвера:
NTSTATUS DriverEntry
(
   PDRIVER_OBJECT    pDriverObject,
   PUNICODE_STRING   pRegistryPath
);

//--------------------------------------------------------------------------------------------------

// Действия перед выгрузкой драйвера:
void Driver_EvtDriverUnload
(
   WDFDRIVER hDriver
);

//==================================================================================================

// Создание объекта устройства:
NTSTATUS Driver_EvtDriverDeviceAdd
(
   WDFDRIVER         hDriver,
   PWDFDEVICE_INIT   pDeviceInit
);

//--------------------------------------------------------------------------------------------------

// Очистка дополнительных связанных с устройством ресурсов:
void Driver_EvtDeviceContextCleanup
(
   WDFOBJECT hDeviceObject
);

//==================================================================================================

// Выделение аппаратных ресурсов ресурсов (порты ввода/вывода, память, прерывания):
NTSTATUS Driver_EvtDevicePrepareHardware
(
   WDFDEVICE      hDevice,
   WDFCMRESLIST   hRawResources,
   WDFCMRESLIST   hTranslatedResources
);

// Освобождение аппаратных ресурсов (порты ввода/вывода, память, прерывания):
NTSTATUS Driver_EvtDeviceReleaseHardware
(
   WDFDEVICE      hDevice,
   WDFCMRESLIST   hTranslatedResources
);

//--------------------------------------------------------------------------------------------------

// Введение устройства в рабочее состояние:
NTSTATUS Driver_EvtDeviceD0Entry
(
   WDFDEVICE               hDevice,
   WDF_POWER_DEVICE_STATE  powerState
);

// Выведение устройства из рабочего состояния (спящий режим и т.п.):
NTSTATUS Driver_EvtDeviceD0Exit
(
   WDFDEVICE               hDevice,
   WDF_POWER_DEVICE_STATE  powerState
);

//==================================================================================================

// Создание файла устройства (при открытии из приложения):
void Driver_EvtDeviceFileCreate
(
   WDFDEVICE      hDevice,
   WDFREQUEST     hRequest,
   WDFFILEOBJECT  hFileObject
);

// Закрытие файла устройства:
void Driver_EvtFileClose
(
   WDFFILEOBJECT hFileObject
);

//--------------------------------------------------------------------------------------------------

// Обработка специализированных запросов ввода/вывода:
void Driver_EvtIoDeviceControl
(
   WDFQUEUE    hQueue,
   WDFREQUEST  hRequest,
   size_t      outputLength,
   size_t      inputLength,
   ULONG       uIoControlCode
);


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Операции чтения/записи непосредственно в регистры устройства:
//
//==================================================================================================

BOOLEAN Driver_DeviceIsValidRegAddress($UINT uRelativeAddress);
BOOLEAN Driver_DeviceCheckRegAddress(OUT NTSTATUS* pStatus,$UINT uRelativeAddress);

//--------------------------------------------------------------------------------------------------

$UINT Driver_DeviceReadRegister(PDRIVER_FDO_DATA pFdoData,$UINT uRelativeAddress);
$UINT Driver_DeviceReadRegisterFromCache(PDRIVER_FDO_DATA pFdoData,$UINT uRelativeAddress);
$UINT Driver_DeviceReadRegisterEx( PDRIVER_FDO_DATA pFdoData,
                                   $UINT uRelativeAddress, BOOLEAN bReadFromCache,
                                   OUT PBOOLEAN pbReallyCachedValueReturned OPTIONAL );

void Driver_DeviceWriteRegister(PDRIVER_FDO_DATA pFdoData,$UINT uRelativeAddress,$UINT uValue);


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Операции устройства -- чтение/запись во внутреннюю память:
//
//==================================================================================================

void Driver_DeviceSwitchToMemoryMode( PDRIVER_FDO_DATA pFdoData,
                                      OUT PDRIVER_MEMREG_PRESERVATION pMemoryRegisters );
void Driver_DeviceRestoreFromMemoryMode( PDRIVER_FDO_DATA pFdoData,
                                         PCDRIVER_MEMREG_PRESERVATION pcMemoryRegisters );

//--------------------------------------------------------------------------------------------------

$UINT Driver_DeviceReadMemory(PDRIVER_FDO_DATA pFdoData,$UINT uBankNo,$UINT uAddress);

void Driver_DeviceWriteMemory( PDRIVER_FDO_DATA pFdoData, $UINT uBankNo,
                               $UINT uAddress, $UINT uValue );
BOOLEAN Driver_DeviceWriteMemoryAndCheck( OUT NTSTATUS* pStatus, PDRIVER_FDO_DATA pFdoData,
                                          $UINT uBankNo, $UINT uAddress, $UINT uValue );

//--------------------------------------------------------------------------------------------------

void Driver_DeviceReportMemoryWriteReadDifference( $UINT uBankNo, $UINT uAddress,
                                                   $UINT uWrittenValue, $UINT uReadValue );


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Операции устройства -- реализация команд:
//
//==================================================================================================

void Driver_DeviceDeactivate(PDRIVER_FDO_DATA pFdoData);

//--------------------------------------------------------------------------------------------------

BOOLEAN Driver_DeviceLoadRegisters( OUT NTSTATUS* pStatus, PDRIVER_FDO_DATA pFdoData,
                                    $PCUINT pcuAddressValuePairs, $UINT uPairCount );

//--------------------------------------------------------------------------------------------------

BOOLEAN Driver_DeviceLoadProfile( OUT NTSTATUS* pStatus, PDRIVER_FDO_DATA pFdoData,
                                  $UINT uBankNo, $UINT uStartAddress,
                                  $PCUINT pcuFreqMagCodes, $UINT uSampleCount );

//--------------------------------------------------------------------------------------------------

BOOLEAN Driver_DeviceTestMemory(PDRIVER_FDO_DATA pFdoData,OUT $PUINT puInfoValues);

BOOLEAN Driver_DeviceGenerateSawTooth( OUT NTSTATUS* pStatus, PDRIVER_FDO_DATA pFdoData,
                                       BOOLEAN bIsFixedParameterMagnitude,
                                       $UINT uFixedFrequencyOrMagnitude,
                                       $UINT uSampleDelay );


//**************************************************************************************************
