﻿#include "Precompiled.h"
#include "SharedIncludes.h"

#pragma code_seg("INIT")
#pragma data_seg("PAGEDATA")

//==================================================================================================
// Точка входа драйвера:
//--------------------------------------------------------------------------------------------------
NTSTATUS DriverEntry
(
   PDRIVER_OBJECT    pDriverObject,
   PUNICODE_STRING   pRegistryPath
)
{
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   NTSTATUS             status = STATUS_SUCCESS;
   WDF_DRIVER_CONFIG    config;
   WDFKEY               hParametersKey = NULL;
   UNICODE_STRING       ustrRegValueName;
   ULONG                uRegValue;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   $SetDriverBinaryName("Shaper.sys");
   $SetDriverPoolTag(($UINT)'pahS');
   $DBG_PRINT_IN_FUNCTION();
   PAGED_CODE();
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   $DBG_PRINT_INFORMATION  ( "##" );
   $DBG_PRINTF_INFORMATION(( "## %s", $VER_FILE_DESCRIPTION ));
   $DBG_PRINTF_INFORMATION(( "## Версия: %s", $VER_VERSION_STRING ));
   $DBG_PRINTF_INFORMATION(( "## Скомпилирован: %s %s (%s)",
                             __DATE__, __TIME__, $DriverBinaryName() ));
   $DBG_PRINTF_INFORMATION(( "## %s", $VER_COMMENTS ));
   $DBG_PRINT_INFORMATION  ( "##" );

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Создание WDF-объекта драйвера:

   WDF_DRIVER_CONFIG_INIT(&config,Driver_EvtDriverDeviceAdd);
   config.EvtDriverUnload = Driver_EvtDriverUnload;

   status = WdfDriverCreate(
      pDriverObject, pRegistryPath,
      WDF_NO_OBJECT_ATTRIBUTES,
      &config, WDF_NO_HANDLE );

   if (!NT_SUCCESS(status))
   {
      $DBG_PRINT_FUNCTION_FAILURE_WITH_STATUS("WdfDriverCreate",status);
      goto EXIT;
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Открытие ключа параметров в реестре:

   status = WdfDriverOpenParametersRegistryKey( WdfGetDriver(), STANDARD_RIGHTS_READ,
                                                WDF_NO_OBJECT_ATTRIBUTES, &hParametersKey );
   if (!NT_SUCCESS(status))
   {
      $DBG_PRINT_FUNCTION_FAILURE_WITH_STATUS("WdfDriverOpenParametersRegistryKey",status);
      hParametersKey = NULL;
      goto EXIT;
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Получение параметра возможности работы в режиме "бездействующей имитации":

   RtlInitUnicodeString(&ustrRegValueName,L"AllowDummyMode");

   status = WdfRegistryQueryULong(hParametersKey,&ustrRegValueName,&uRegValue);

   if (status==STATUS_OBJECT_NAME_NOT_FOUND)
   {
      $DBG_PRINT_INFORMATION("Параметер \"AllowDummyMode\" в реестре не задан.");
      status = STATUS_SUCCESS;
   }
   else
   {
      if (!NT_SUCCESS(status))
      {
         $DBG_PRINT_FUNCTION_FAILURE_WITH_STATUS("WdfRegistryQueryULong",status);
         goto EXIT;
      }

      if (!$IS_ONE_OF_2_(uRegValue,0u,1u))
      {
         status = $TracefError(
            "Некорректное значение параметра \"AllowDummyMode\" (в реестре): 0x%08X.",
            uRegValue );
         if (!NT_SUCCESS(status))
            goto EXIT;
         status = STATUS_INVALID_PARAMETER;
         goto EXIT;
      }

      if (Driver_AllowDummyMode=$AS_BOOL(uRegValue))
      {
         status = $TraceWarning("Включен параметер \"AllowDummyMode\" (в реестре).");
         if (!NT_SUCCESS(status))
            goto EXIT;
      }
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
EXIT:
   if (hParametersKey)
      WdfRegistryClose(hParametersKey);
   $DBG_PRINT_OUT_OF_FUNCTION_WITH_STATUS(status);
   return status;
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}
//==================================================================================================
