﻿#include "Precompiled.h"
#include "SharedIncludes.h"

#include <KernelUtils.PagedSegmentsOn.h>

//==================================================================================================
// Выделение аппаратных ресурсов ресурсов (порты ввода/вывода, память, прерывания):
//--------------------------------------------------------------------------------------------------
NTSTATUS Driver_EvtDevicePrepareHardware
(
   WDFDEVICE      hDevice,
   WDFCMRESLIST   hRawResources,
   WDFCMRESLIST   hTranslatedResources
)
{
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   NTSTATUS                         status = STATUS_SUCCESS;
   PDRIVER_FDO_DATA                 pFdoData;
   $UINT                            uResourcesCount, uIdx;
   PCM_PARTIAL_RESOURCE_DESCRIPTOR  pDescriptor;
   $UINT                            uBaseAddress, uIoRange;
   PHYSICAL_ADDRESS                 paBaseAddress;
   size_t                           stBaseAddress;
   PVOID                            pvVirtualAddress;
   BOOLEAN                          bIoResourceSupplied = FALSE;

   UNREFERENCED_PARAMETER(hRawResources);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   $DBG_PRINT_IN_FUNCTION();
   PAGED_CODE();
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   pFdoData = Driver_FdoGetData(hDevice);

// Получение числа предоставленных ресурсов:
   uResourcesCount = WdfCmResourceListGetCount(hTranslatedResources);
   if (!uResourcesCount)
   {
      $DBG_PRINT_ERROR("Аппаратные ресурсы не предоставлены.");
      if (Driver_AllowDummyMode)
      {
         pFdoData->IsInDummyMode = TRUE;
         goto EXIT;
      }
      goto CONFIGURATION_ERROR;
   }

// Перебор ресурсов устройства; в данном случае обрабатывается только один (порты/память):
   for (uIdx=0; uIdx<uResourcesCount; uIdx++)
   {
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//    Получение дескриптора ресурса ввода/вывода (порты/память):

      if (!( pDescriptor =
             WdfCmResourceListGetDescriptor(hTranslatedResources,uIdx) ))
      {
         $DBG_PRINT_FUNCTION_FAILURE("WdfCmResourceListGetDescriptor");
         goto CONFIGURATION_ERROR;
      }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//    Обработка ресурса ввода/вывода:

      switch (pDescriptor->Type)
      {
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//    Регистры в пространстве портов ввода/вывода:

      case CmResourceTypePort:

         $DBG_PRINT_IDENTIFIER("CmResourceTypePort");

         uBaseAddress= pDescriptor->u.Port.Start.LowPart;
         uIoRange    = pDescriptor->u.Port.Length;

         $DBG_PRINTF_INFORMATION((
            "Физические адреса портов (транслированные): 0x%08X..0x%08X.",
            uBaseAddress, uBaseAddress+uIoRange-1u ));

         if (bIoResourceSupplied)
            goto IO_RESOURCE_ALREADY_SUPPLIED;

         bIoResourceSupplied = TRUE;

         if (uIoRange!=SHAPER_IO_RANGE)
            goto INCORRECT_IO_RANGE;

         pFdoData->BaseAddressPointer= ULongToPtr(uBaseAddress);
         pFdoData->IoRange           = uIoRange;

         break;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//    Регистры в пространстве памяти ввода/вывода:

      case CmResourceTypeMemory:

         $DBG_PRINT_IDENTIFIER("CmResourceTypeMemory");

         paBaseAddress= pDescriptor->u.Memory.Start;
         uIoRange     = pDescriptor->u.Memory.Length;

      #if _WIN64
         stBaseAddress = paBaseAddress.QuadPart;
      #else
         stBaseAddress = paBaseAddress.LowPart;
      #endif

         $DBG_PRINTF_INFORMATION((
            "Физические адреса памяти (транслированные): 0x%P..0x%P.",
            stBaseAddress, stBaseAddress+(size_t)uIoRange-(size_t)1 ));

         if (bIoResourceSupplied)
            goto IO_RESOURCE_ALREADY_SUPPLIED;

         bIoResourceSupplied = TRUE;

         if (uIoRange!=SHAPER_IO_RANGE)
            goto INCORRECT_IO_RANGE;

         $DBG_PRINT_OPERATION("Отображение в виртуальную память");

         if (!( pvVirtualAddress =
                MmMapIoSpace( paBaseAddress, uIoRange, MmNonCached ) ))
         {
            $DBG_PRINT_FUNCTION_FAILURE("MmMapIoSpace");
            status = STATUS_INSUFFICIENT_RESOURCES;
            goto EXIT;
         }

         pFdoData->MemoryMappedPorts = TRUE;
         pFdoData->BaseAddressPointer= pvVirtualAddress;
         pFdoData->IoRange           = uIoRange;

         break;

   #if DBG
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//    Неподдерживаемые ресурсы (известные типы):

      case CmResourceTypeInterrupt     : $TraceIdentifier("CmResourceTypeInterrupt"     ); break;
      case CmResourceTypeMemoryLarge   : $TraceIdentifier("CmResourceTypeMemoryLarge"   ); break;
      case CmResourceTypeDma           : $TraceIdentifier("CmResourceTypeDma"           ); break;
      case CmResourceTypeDevicePrivate : $TraceIdentifier("CmResourceTypeDevicePrivate" ); break;
      case CmResourceTypeBusNumber     : $TraceIdentifier("CmResourceTypeBusNumber"     ); break;
      case CmResourceTypeDeviceSpecific: $TraceIdentifier("CmResourceTypeDeviceSpecific"); break;
      case CmResourceTypePcCardConfig  : $TraceIdentifier("CmResourceTypePcCardConfig"  ); break;
      case CmResourceTypeMfCardConfig  : $TraceIdentifier("CmResourceTypeMfCardConfig"  ); break;
      case CmResourceTypeConfigData    : $TraceIdentifier("CmResourceTypeConfigData"    ); break;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   #endif

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//    Неподдерживаемый ресурс (нераспознанный тип):

      default:
         $DBG_PRINT_UNSUPPORTED_CM_RESOURCE_TYPE(pDescriptor->Type);

      } // switch
   } // for

// Проверка на предоставленный ресурс ввода/вывода:
   if (!bIoResourceSupplied)
   {
      $DBG_PRINT_ERROR("Ресурс ввода/вывода не предоставлен.");
      goto CONFIGURATION_ERROR;
   }

   pFdoData->HardwarePreparedSuccessfully = TRUE;

   $DBG_PRINT_INFORMATION("Аппаратные ресурсы подключены успешно.");

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
EXIT:
   if (pFdoData->IsInDummyMode && NT_SUCCESS(status))
      status = $TraceWarning( "Фиктивное устройство обслуживается драйвером "
                              "в режиме \"бездействующей имитации\"." );
   $DBG_PRINT_OUT_OF_FUNCTION_WITH_STATUS(status);
   return status;
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CONFIGURATION_ERROR:
   status = STATUS_DEVICE_CONFIGURATION_ERROR;
   goto EXIT;
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
IO_RESOURCE_ALREADY_SUPPLIED:
   $DBG_PRINT_ERROR("Ресурс ввода/вывода уже был 1 раз предоставлен.");
   goto CONFIGURATION_ERROR;
INCORRECT_IO_RANGE:
   $DBG_PRINT_ERROR("Неверный диапазон ввода/вывода.");
   goto CONFIGURATION_ERROR;
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}
//==================================================================================================
