﻿#include "Precompiled.h"
#include "SharedIncludes.h"

#include <KernelUtils.PagedSegmentsOn.h>

//==================================================================================================
// Освобождение аппаратных ресурсов (порты ввода/вывода, память, прерывания):
//--------------------------------------------------------------------------------------------------
NTSTATUS Driver_EvtDeviceReleaseHardware
(
   WDFDEVICE      hDevice,
   WDFCMRESLIST   hTranslatedResources
)
{
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   NTSTATUS          status = STATUS_SUCCESS;
   PDRIVER_FDO_DATA  pFdoData;
   BOOLEAN           bTraceResourcesRelased = FALSE;

   UNREFERENCED_PARAMETER(hTranslatedResources);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   $DBG_PRINT_IN_FUNCTION();
   PAGED_CODE();
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   pFdoData = Driver_FdoGetData(hDevice);

   bTraceResourcesRelased = pFdoData->HardwarePreparedSuccessfully;
   pFdoData->HardwarePreparedSuccessfully = FALSE;

   if (pFdoData->MemoryMappedPorts)
   {
      $DBG_PRINT_OPERATION("Освобождение виртуальной памяти");

      bTraceResourcesRelased = TRUE;

      MmUnmapIoSpace( pFdoData->BaseAddressPointer,
                      pFdoData->IoRange );
   }

   pFdoData->MemoryMappedPorts = FALSE;
   pFdoData->BaseAddressPointer= NULL;
   pFdoData->IoRange           = 0;

   if (bTraceResourcesRelased)
      $DBG_PRINT_INFORMATION("Аппаратные ресурсы отключены.");

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
EXIT:
   $DBG_PRINT_OUT_OF_FUNCTION_WITH_STATUS(status);
   return status;
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}
//==================================================================================================
