﻿#include "Precompiled.h"
#include "SharedIncludes.h"

//==================================================================================================
// Обработка специализированных запросов ввода/вывода:
//--------------------------------------------------------------------------------------------------
void Driver_EvtIoDeviceControl
(
   WDFQUEUE    hQueue,
   WDFREQUEST  hRequest,
   size_t      outputLength,
   size_t      inputLength,
   ULONG       uIoControlCode
)
{
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   NTSTATUS          status = STATUS_SUCCESS;
   $UINT_PTR         infoLength = 0;
   $PCVOID           pcvInputBuffer;
   PVOID             pvOutputBuffer;
   $PCUINT           pcuInputBuffer;
   $PUINT            puOutputBuffer;
   PDRIVER_FDO_DATA  pFdoData;
   $UINT             uRelativeAddress, uIoValue;
   size_t            elementSize, elementCount;
   BOOLEAN           bSuppressTraceOutput =
                     uIoControlCode == IOCTL_SHAPER_GENERATE_SAW_TOOTH;

   UNREFERENCED_PARAMETER(hQueue);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   if (!bSuppressTraceOutput)
      $DBG_PRINT_IN_FUNCTION();
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   pFdoData = Driver_FdoGetData( $WdfRequestGetDevice(hRequest) );

   if ( uIoControlCode!=IOCTL_SHAPER_INITIALIZE &&
        !pFdoData->FileInitialized )
   {
      $DBG_PRINT_ERROR("Устройство не было проинициализировано (на данном файле).");
      status = STATUS_INVALID_DEVICE_STATE;
      goto COMPLETE;
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

// Обработка "IOCTL"-запросов:
   switch (uIoControlCode)
   {
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Инициализация (проверка версии и деактивация):
   case IOCTL_SHAPER_INITIALIZE:

      $DBG_PRINT_IDENTIFIER("IOCTL_SHAPER_INITIALIZE");

      if (pFdoData->FileInitialized)
      {
         $DBG_PRINT_ERROR("Устройство уже было проинициализировано (на данном файле).");
         status = STATUS_INVALID_DEVICE_STATE;
         break;
      }

      if (!(
         $CheckInputLengthIs(&status,inputLength,sizeof($UINT)*4) &&
         $CheckOutputLengthIs(&status,outputLength,sizeof($UINT)*4) &&
         $RetrieveInputBuffer(&status,hRequest,0,&pcvInputBuffer,NULL) &&
         $RetrieveOutputBuffer(&status,hRequest,0,&pvOutputBuffer,NULL)
      ))
         break;

      pcuInputBuffer=($PCUINT)pcvInputBuffer;
      puOutputBuffer=($PUINT)pvOutputBuffer;

      $DBG_PRINTF_INFORMATION((
         "Версия библиотеки поддержки: %u.%u.%u.%u.",
         pcuInputBuffer[0], pcuInputBuffer[1],
         pcuInputBuffer[2], pcuInputBuffer[3] ));

      if ( pcuInputBuffer[0] == $VER_MAJOR_VERSION   &&
           pcuInputBuffer[1] == $VER_MINOR_VERSION   &&
           pcuInputBuffer[2] == $VER_BUILD_NUMBER    &&
           pcuInputBuffer[3] == $VER_REVISION_NUMBER )
      {
         Driver_DeviceDeactivate(pFdoData);
         pFdoData->FileInitialized = TRUE;
      }
      else
      {
         $DBG_PRINT_ERROR("Версии драйвера и библиотеки поддержки не совпадают.");

         puOutputBuffer[0] = $VER_MAJOR_VERSION;
         puOutputBuffer[1] = $VER_MINOR_VERSION;
         puOutputBuffer[2] = $VER_BUILD_NUMBER;
         puOutputBuffer[3] = $VER_REVISION_NUMBER;
      }

      $DBG_PRINT_INPUT_TRANSFER(inputLength);

      if (!pFdoData->FileInitialized)
      {
         infoLength = outputLength;
         goto REPORT_OUTPUT_TRANSFER;
      }

      $DBG_PRINT_INFORMATION("Устройство проинициализировано успешно.");
      break;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Деактивация устройства (перевод в бездействующее состояние):
   case IOCTL_SHAPER_DEACTIVATE:

      $DBG_PRINT_IDENTIFIER("IOCTL_SHAPER_DEACTIVATE");

      if (!$CheckInputOutputLengthsAreZero(&status,inputLength,outputLength))
         break;

      Driver_DeviceDeactivate(pFdoData);
      break;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Ститывание 32-разрядного слова непосредственно из регистра:
   case IOCTL_SHAPER_READ_REGISTER:

      $DBG_PRINT_IDENTIFIER("IOCTL_SHAPER_READ_REGISTER");

      if (!(
         $CheckInputLengthIs(&status,inputLength,sizeof($UINT)*2) &&
         $CheckOutputLengthIs(&status,outputLength,sizeof($UINT)*2) &&
         $RetrieveInputBuffer(&status,hRequest,0,&pcvInputBuffer,NULL) &&
         $RetrieveOutputBuffer(&status,hRequest,0,&pvOutputBuffer,NULL)
      ))
         break;

      pcuInputBuffer=($PCUINT)pcvInputBuffer;
      puOutputBuffer=($PUINT)pvOutputBuffer;

      uRelativeAddress = *pcuInputBuffer;
      if (!(Driver_DeviceCheckRegAddress(&status,uRelativeAddress)))
         break;

      $BLOCK
      {
         BOOLEAN
            bReadFromCache        = $AS_BOOL(pcuInputBuffer[1]),
            bReallyCachedValueRead= FALSE;

         if (bReadFromCache)
            $DBG_PRINT_INFORMATION("Чтение из кэша.");

         $DBG_PRINTF_INFORMATION((
            "Адрес чтения --------> 0x%02X (относительный)",
            uRelativeAddress ));

         uIoValue = Driver_DeviceReadRegisterEx( pFdoData, uRelativeAddress,
                                                 bReadFromCache, &bReallyCachedValueRead );

         if (!bReadFromCache || bReallyCachedValueRead)
            $DBG_PRINTF_INFORMATION((
               "Считанное значение --> 0x%08X%s", uIoValue,
               bReallyCachedValueRead ? " (из кэша)" : "" ));
         else
            $DBG_PRINT_ERROR("Данный регистр в кэше не содержится (ещё ни разу не записывался).");

         puOutputBuffer[0] = uIoValue;
         puOutputBuffer[1] = bReallyCachedValueRead;
      }

      $DBG_PRINT_INPUT_TRANSFER(inputLength);

      infoLength = outputLength;
      goto REPORT_OUTPUT_TRANSFER;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Запись 32-разрядного слова непосредственно в регистр:
   case IOCTL_SHAPER_WRITE_REGISTER:

      $DBG_PRINT_IDENTIFIER("IOCTL_SHAPER_WRITE_REGISTER");

      if (!(
         $CheckInputLengthIs(&status,inputLength,sizeof($UINT)*2) &&
         $CheckOutputLengthIsZero(&status,outputLength) &&
         $RetrieveInputBuffer(&status,hRequest,0,&pcvInputBuffer,NULL)
      ))
         break;

      pcuInputBuffer=($PCUINT)pcvInputBuffer;

      uRelativeAddress= pcuInputBuffer[0];
      uIoValue        = pcuInputBuffer[1];

      if (!(Driver_DeviceCheckRegAddress(&status,uRelativeAddress)))
         break;

      $DBG_PRINTF_INFORMATION((
         "Адрес записи -----------> 0x%02X (относительный)",
         uRelativeAddress));
      $DBG_PRINTF_INFORMATION((
         "Записываемое значение --> 0x%08X",
         uIoValue ));

      Driver_DeviceWriteRegister(pFdoData,uRelativeAddress,uIoValue);

      infoLength = inputLength;
      goto REPORT_INPUT_TRANSFER;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Загрузка последовательности регистров:
   case IOCTL_SHAPER_LOAD_REGISTERS:

      $DBG_PRINT_IDENTIFIER("IOCTL_SHAPER_LOAD_REGISTERS");

      elementCount = inputLength/(elementSize=sizeof($UINT)*2);

      if (!(
         $CheckInputLength( &status,
                            inputLength % elementSize == 0 &&
                            elementCount>=1 ) &&
         $CheckOutputLengthIsZero(&status,outputLength) &&
         $RetrieveInputBuffer(&status,hRequest,0,&pcvInputBuffer,NULL)
      ))
         break;

      if (!Driver_DeviceLoadRegisters( &status, pFdoData,
                                       ($PCUINT)pcvInputBuffer,
                                       ($UINT)elementCount ))
         break;

      infoLength = inputLength;
      goto REPORT_INPUT_TRANSFER;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Загрузка профиля в память устройства:
   case IOCTL_SHAPER_LOAD_PROFILE:

      $DBG_PRINT_IDENTIFIER("IOCTL_SHAPER_LOAD_PROFILE");

      elementCount = inputLength/(elementSize=sizeof($UINT));

      if (!(
         $CheckInputLength( &status,
            inputLength % elementSize == 0 &&
            $BETWEEN_( elementCount,
               DRIVER_LOAD_PROFILE_HEADER_ELEMENTS+1u,
               DRIVER_LOAD_PROFILE_HEADER_ELEMENTS+SHAPER_MEMORY_BANK_CAPACITY ) ) &&
         $CheckOutputLengthIsZero(&status,outputLength) &&
         $RetrieveInputBuffer(&status,hRequest,0,&pcvInputBuffer,NULL)
      ))
         break;

      pcuInputBuffer = ($PCUINT)pcvInputBuffer;

      $BLOCK
      {
         $UINT
            uBankNo      = pcuInputBuffer[0],
            uStartAddress= pcuInputBuffer[1],
            uSampleCount = elementCount-DRIVER_LOAD_PROFILE_HEADER_ELEMENTS;
         $PCUINT
            pcuFreqMagCodes = pcuInputBuffer+DRIVER_LOAD_PROFILE_HEADER_ELEMENTS;

         if (!Driver_DeviceLoadProfile( &status, pFdoData,
                                        uBankNo, uStartAddress,
                                        pcuFreqMagCodes, uSampleCount ))
            break;
      }

      infoLength = inputLength;
      goto REPORT_INPUT_TRANSFER;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Тест банков памяти устройства:
   case IOCTL_SHAPER_TEST_MEMORY:

      $DBG_PRINT_IDENTIFIER("IOCTL_SHAPER_TEST_MEMORY");

      if (!(
         $CheckInputLengthIsZero(&status,inputLength) &&
         $CheckOutputLengthIs( &status, outputLength,
                               DRIVER_TEST_MEMORY_INFO_ELEMENTS*sizeof($UINT) ) &&
         $RetrieveOutputBuffer(&status,hRequest,0,&pvOutputBuffer,NULL)
      ))
         break;

      if (!Driver_DeviceTestMemory(pFdoData,($PUINT)pvOutputBuffer))
      {
         infoLength = outputLength;
         goto REPORT_OUTPUT_TRANSFER;
      }

      $DBG_PRINT_INFORMATION("Тест памяти прошёл успешно.");
      break;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Генерация одного пилозуба (для теста):
   case IOCTL_SHAPER_GENERATE_SAW_TOOTH:

      elementCount = inputLength/(elementSize=sizeof($UINT));

      if (!(
         $CheckInputLengthIs(&status,inputLength,3u*elementSize) &&
         $CheckOutputLengthIsZero(&status,outputLength) &&
         $RetrieveInputBuffer(&status,hRequest,0,&pcvInputBuffer,NULL)
      ))
         break;

      pcuInputBuffer = ($PCUINT)pcvInputBuffer;

      WdfSpinLockAcquire(pFdoData->DispatchLevelSpinLockHandle);

      $BLOCK
      {
         $UINT
            uIsFixedParameterMagnitude= pcuInputBuffer[0],
            uFixedFrequencyOrMagnitude= pcuInputBuffer[1],
            uSampleDelay              = pcuInputBuffer[2];

         if (!Driver_DeviceGenerateSawTooth( &status, pFdoData,
                                             $AS_BOOL(uIsFixedParameterMagnitude),
                                             uFixedFrequencyOrMagnitude,
                                             uSampleDelay ))
         {
            WdfSpinLockRelease(pFdoData->DispatchLevelSpinLockHandle);
            break;
         }
      }

      WdfSpinLockRelease(pFdoData->DispatchLevelSpinLockHandle);

      infoLength = inputLength;
      break;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Неподдерживаемые "IOCTL"-коды:
   default:
      $DBG_PRINT_UNSUPPORTED_CTL_CODE(uIoControlCode);
      status = STATUS_INVALID_DEVICE_REQUEST;
   } // switch

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
COMPLETE:
   WdfRequestCompleteWithInformation(hRequest,status,infoLength);
   goto EXIT;
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
COMPLETE_NO_INFO:
   WdfRequestComplete(hRequest,status);
   goto EXIT;
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
REPORT_INPUT_TRANSFER:
   $DBG_PRINT_INPUT_TRANSFER(infoLength);
   goto COMPLETE;
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
REPORT_OUTPUT_TRANSFER:
   $DBG_PRINT_OUTPUT_TRANSFER(infoLength);
   goto COMPLETE;
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
NOT_IMPLEMENTED:
   $DBG_PRINT_ERROR("Данная процедура в настоящее время не реализована.");
   status = STATUS_INVALID_DEVICE_REQUEST;
   goto COMPLETE_NO_INFO;
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
EXIT:
   if (!bSuppressTraceOutput)
      $DBG_PRINT_OUT_OF_FUNCTION();
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}
//==================================================================================================
