﻿//**************************************************************************************************
//
// ЗАГОЛОВОЧНЫЙ ФАЙЛ УСТРОЙСТВА ШЕЙПЕРА (ДЛЯ РЕЖИМОВ ПОЛЬЗОВАТЕЛЯ И ЯДРА).
//
// === H-файлы необходимые для подключения к проекту пользовательского режима: ===
//
// * "InitGuid.h" -- макрос "DEFINE_GUID"
// * "WinIoCtl.h" -- макрос "CTL_CODE"
// * "KeyDefs.h"  -- "$"-определения (для режимов пользователя и ядра)
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#pragma once


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// "GUID"-идентификатор(ы) устройства:
//
//==================================================================================================

// Интерфейс устройства ({FE74BA15-D68F-4749-9687-73894CBC7678}):
DEFINE_GUID( GUID_DEVINTERFACE_SHAPER,
   0xfe74ba15, 0xd68f, 0x4749, 0x96, 0x87, 0x73, 0x89, 0x4c, 0xbc, 0x76, 0x78 );


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// "IOCTL"-коды управления устройством (команды устройства):
//
//==================================================================================================

// Инициализация (проверка версии и деактивация):
#define IOCTL_SHAPER_INITIALIZE \
   CTL_CODE(FILE_DEVICE_UNKNOWN,0x800u,METHOD_BUFFERED,FILE_ANY_ACCESS)

// Деактивация устройства (перевод в бездействующее состояние):
#define IOCTL_SHAPER_DEACTIVATE \
   CTL_CODE(FILE_DEVICE_UNKNOWN,0x801u,METHOD_NEITHER,FILE_ANY_ACCESS)

// Ститывание 32-разрядного слова (непосредственно из регистра либо из кэша):
#define IOCTL_SHAPER_READ_REGISTER \
   CTL_CODE(FILE_DEVICE_UNKNOWN,0x802u,METHOD_BUFFERED,FILE_READ_DATA)

// Запись 32-разрядного слова (непосредственно в регистр):
#define IOCTL_SHAPER_WRITE_REGISTER \
   CTL_CODE(FILE_DEVICE_UNKNOWN,0x803u,METHOD_BUFFERED,FILE_WRITE_DATA)

// Загрузка последовательности регистров:
#define IOCTL_SHAPER_LOAD_REGISTERS \
   CTL_CODE(FILE_DEVICE_UNKNOWN,0x804u,METHOD_BUFFERED,FILE_WRITE_DATA)

// Загрузка профиля в память устройства:
#define IOCTL_SHAPER_LOAD_PROFILE \
   CTL_CODE(FILE_DEVICE_UNKNOWN,0x805u,METHOD_BUFFERED,FILE_WRITE_DATA)

// Тест банков памяти устройства:
#define IOCTL_SHAPER_TEST_MEMORY \
   CTL_CODE(FILE_DEVICE_UNKNOWN,0x806u,METHOD_BUFFERED,FILE_ANY_ACCESS)

// Генерация одного пилозуба (для теста):
#define IOCTL_SHAPER_GENERATE_SAW_TOOTH \
   CTL_CODE(FILE_DEVICE_UNKNOWN,0x807u,METHOD_BUFFERED,FILE_WRITE_DATA)


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Значения ввода/вывода (адреса относительные):
//
//==================================================================================================

#define SHAPER_TOTAL_REGISTERS            (8)

//--------------------------------------------------------------------------------------------------

#define SHAPER_IO_RANGE                   (64u)

#define SHAPER_MIN_REG_ADDRESS            (($UINT)(sizeof($UINT)*1))                      // 0x04
#define SHAPER_MAX_REG_ADDRESS            (($UINT)(sizeof($UINT)*SHAPER_TOTAL_REGISTERS)) // 0x20


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Регистры устройства (относительные адреса):
//
//==================================================================================================

#define SHAPER_MCR1                       (0x04u)
#define SHAPER_MCR2                       (0x08u)
#define SHAPER_RD                         (0x0Cu)
#define SHAPER_RTHRESH                    (0x10u)
#define SHAPER_RDEL                       (0x14u)
#define SHAPER_RPERIOD                    (0x18u)
#define SHAPER_RGM                        (0x1Cu)
#define SHAPER_DUMMY                      (0x20u)


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Организация внутренней памяти устройства (адресуютя 32-разрядные слова):
//
//==================================================================================================

#define SHAPER_MEMORY_BANK_CAPACITY       (0x1000u) // 4096

//--------------------------------------------------------------------------------------------------

#define SHAPER_MAX_CELL_ADDRESS           (SHAPER_MEMORY_BANK_CAPACITY-1u) // 0xFFF (4095)
#define SHAPER_CELL_VALUE_MASK            (0xFFFFFFFFu)

//--------------------------------------------------------------------------------------------------

#define SHAPER_FREQMAG_CODE_MASK          (0xFFFFFFFFu) // биты: 31..16 (частота), 15..0 (амплитуда)

#define SHAPER_FREQUENCY_BITS             (16)
#define SHAPER_MAGNITUDE_BITS             (16)

#define SHAPER_FREQUENCY_FIRST_BIT        (0)
#define SHAPER_MAGNITUDE_FIRST_BIT        (16)

#define SHAPER_MAX_FREQUENCY_CODE         ($FLAG(SHAPER_FREQUENCY_BITS)-1u) // (0xFFFF) 65535
#define SHAPER_MAX_MAGNITUDE_CODE         ($FLAG(SHAPER_MAGNITUDE_BITS)-1u) // (0xFFFF) 65535

#define SHAPER_MAX_SAWTEST_FREQUENCY_CODE (0x3FFFu) // 16383
#define SHAPER_MAX_SAWTEST_MAGNITUDE_CODE (0x3FFFu) // 16383

//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Режимы устройства (некоторые из них):
//
//==================================================================================================

#define SHAPER_RGM_INACTIVE               $FLAG(31)
#define SHAPER_RGM_MEMORY                 $FLAG(1)
#define SHAPER_RGM_SYNTHESIZER            $FLAG(3)
#define SHAPER_RGM_REG_RESTORATION_MASK   (0xFFFFFFF0u)


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Доступ к банкам памяти:
//
//==================================================================================================

#define SHAPER_READ_OPERATION             (0x00000000u)
#define SHAPER_WRITE_OPERATION            (0x00004000u)

#define SHAPER_BANK_0                     (0x00000000u)
#define SHAPER_BANK_1                     (0x00008000u)


//**************************************************************************************************
