﻿//**************************************************************************************************
//
// ОТКРЫТЫЙ КЛАСС ДОСТУПА К УСТРОЙСТВУ ШЕЙПЕРА (СТАТИЧЕСКИЙ).
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#pragma once

namespace ShaperAccess {

//**************************************************************************************************
public ref class ShaperDevice
{
   static ShaperDevice() {}

public:
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Константы:

   literal int
      TotalRegisters = SHAPER_TOTAL_REGISTERS;

   literal UInt32
      MinRegAddress = SHAPER_MIN_REG_ADDRESS,
      MaxRegAddress = SHAPER_MAX_REG_ADDRESS,

      MCR1   = SHAPER_MCR1,
      MCR2   = SHAPER_MCR2,
      RD     = SHAPER_RD,
      RTHRESH= SHAPER_RTHRESH,
      RDEL   = SHAPER_RDEL,
      RPERIOD= SHAPER_RPERIOD,
      RGM    = SHAPER_RGM,
      DUMMY  = SHAPER_DUMMY,

      MemoryBankCapacity = SHAPER_MEMORY_BANK_CAPACITY,

      MaxCellAddress= SHAPER_MAX_CELL_ADDRESS,
      CellValueMask = SHAPER_CELL_VALUE_MASK,

      FreqMagCodeMask  = SHAPER_FREQMAG_CODE_MASK,

      MaxFrequencyCode = SHAPER_MAX_FREQUENCY_CODE,
      MaxMagnitudeCode = SHAPER_MAX_MAGNITUDE_CODE,

      MaxSawTestFrequencyCode = SHAPER_MAX_SAWTEST_MAGNITUDE_CODE,
      MaxSawTestMagnitudeCode = SHAPER_MAX_SAWTEST_MAGNITUDE_CODE;

   literal int
      FrequencyBits    = SHAPER_FREQUENCY_BITS,
      MagnitudeBits    = SHAPER_MAGNITUDE_BITS,

      FrequencyFirstBit= SHAPER_FREQUENCY_FIRST_BIT,
      MagnitudeFirstBit= SHAPER_MAGNITUDE_FIRST_BIT;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Открытие/закрытие устройства:

   static property bool IsOpen { bool get(); };

   /// <Exception cref="System::InvalidOperationException" />
   static property String^ SymbolicLink { String^ get(); }

   /// <Exception cref="System::InvalidOperationException" />
   /// <Exception cref="System::IO::FileNotFoundException" />
   /// <Exception cref="System::ComponentModel::Win32Exception" />
   /// <Exception cref="ShaperAccess::ShaperException" />
   static void Open();

   /// <Exception cref="System::ArgumentException" />
   /// <Exception cref="System::InvalidOperationException" />
   /// <Exception cref="System::IO::FileNotFoundException" />
   /// <Exception cref="System::ComponentModel::Win32Exception" />
   /// <Exception cref="ShaperAccess::ShaperException" />
   static void Open([Optional] String^ strHardwareIdentifier);

   /// <Exception cref="System::ComponentModel::Win32Exception" />
   static void Close();

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Команды устройства:

   /// <Exception cref="System::InvalidOperationException" />
   /// <Exception cref="System::ComponentModel::Win32Exception" />
   static void Deactivate();

   /// <Exception cref="System::InvalidOperationException" />
   /// <Exception cref="System::ComponentModel::Win32Exception" />
   /// <Exception cref="ShaperAccess::ShaperException" />
   static UInt32 ReadRegister(UInt32 uRelativeAddress,bool bReadFromCache);

   /// <Exception cref="System::InvalidOperationException" />
   /// <Exception cref="System::ComponentModel::Win32Exception" />
   static void WriteRegister(UInt32 uRelativeAddress,UInt32 uValue);

   /// <Exception cref="System::ArgumentNullException" />
   /// <Exception cref="System::ArgumentException" />
   /// <Exception cref="System::InvalidOperationException" />
   /// <Exception cref="System::ComponentModel::Win32Exception" />
   static void LoadRegisters(array<UInt32>^ auAddressValuePairs);

   /// <Exception cref="System::ArgumentNullException" />
   /// <Exception cref="System::ArgumentException" />
   /// <Exception cref="System::InvalidOperationException" />
   /// <Exception cref="System::ComponentModel::Win32Exception" />
   static void LoadProfile(UInt32 uBankNo,UInt32 uStartAddress,array<UInt32>^ auFreqMagCodes);

   /// <Exception cref="System::InvalidOperationException" />
   /// <Exception cref="System::ComponentModel::Win32Exception" />
   static bool TestMemory( [Out] UInt32% uProblemBankNo,
                           [Out] UInt32% uErrorCellAddress,
                           [Out] UInt32% uWrittenValue,
                           [Out] UInt32% uReadValue );

   /// <Exception cref="System::InvalidOperationException" />
   /// <Exception cref="System::ComponentModel::Win32Exception" />
   static void GenerateSawTooth( bool bIsFixedParameterMagnitude,
                                 UInt32 uFixedFrequencyOrMagnitude,
                                 UInt32 uSampleDelay );

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Хелперы устройства:

   static bool IsValidRegAddress(UInt32 uRelativeAddress);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
private:

   literal int
      c_nMaxRegLayoutLength=100;

   static String^ ObtainLink(String^ strHardwareIdentifier);
   static String^ ExtractDevicePathFromInterfaceData
   (
      HDEVINFO                   hDeviceInfoSet,
      SP_DEVICE_INTERFACE_DATA%  deviceInterfaceData
   );
   static void VerifyIsOpen();
   static InvalidOperationException^ CreateNotOpenException();

   static volatile HANDLE
      sv_hDevice = INVALID_HANDLE_VALUE;
   static String
      ^volatile sv_strSymbolicLink;
   static Object
      ^s_obSyncObject = gcnew Object();
};
//**************************************************************************************************

} // ShaperAccess
