﻿#include "Precompiled.h"
#include "SharedIncludes.h"

namespace ShaperAccess {

//==================================================================================================

void ShaperDevice::Deactivate()
{
DEVICE_COMMAND_BEGIN

   DWORD dwBytesReturned;

   if (!::DeviceIoControl( sv_hDevice, IOCTL_SHAPER_DEACTIVATE,
                           NULL, 0, NULL, 0,
                           &dwBytesReturned, NULL ))
      throw gcnew Win32Exception();

DEVICE_COMMAND_END
}

//--------------------------------------------------------------------------------------------------

UInt32 ShaperDevice::ReadRegister(UInt32 uRelativeAddress,bool bReadFromCache)
{
DEVICE_COMMAND_BEGIN

   UInt32
      auInputBuffer [2] = { uRelativeAddress, bReadFromCache },
      auOutputBuffer[2];
   DWORD
      dwBytesReturned;

   if (!::DeviceIoControl( sv_hDevice, IOCTL_SHAPER_READ_REGISTER,
                           $ADDRESS_N_SIZE_OF(auInputBuffer),
                           $ADDRESS_N_SIZE_OF(auOutputBuffer),
                           &dwBytesReturned, NULL ))
      throw gcnew Win32Exception();

   UInt32
      uReadValue = auOutputBuffer[0];
   bool
      bReallyCachedValueRead = $AS_BOOL( auOutputBuffer[1] );

   if (bReadFromCache && !bReallyCachedValueRead)
      throw ShaperException::Create(
         "Регистр в кэше не содержится (ещё ни разу не записывался): 0x{0:X2} "
         "(относительный адрес).", uRelativeAddress );

   return uReadValue;
   
DEVICE_COMMAND_END
}

//--------------------------------------------------------------------------------------------------

void ShaperDevice::WriteRegister(UInt32 uRelativeAddress,UInt32 uValue)
{
DEVICE_COMMAND_BEGIN

   UInt32
      auInputBuffer[] = { uRelativeAddress, uValue };
   DWORD
      dwBytesReturned;

   if (!::DeviceIoControl( sv_hDevice, IOCTL_SHAPER_WRITE_REGISTER,
                           $ARRAY_N_SIZE_OF(auInputBuffer),
                           NULL, 0,
                           &dwBytesReturned, NULL ))
      throw gcnew Win32Exception();
   
DEVICE_COMMAND_END
}

//--------------------------------------------------------------------------------------------------

void ShaperDevice::LoadRegisters(array<UInt32>^ auAddressValuePairs)
{
   if (!auAddressValuePairs)
      throw gcnew ArgumentNullException();
   if (!auAddressValuePairs->Length)
      throw gcnew ArgumentException(
         "Пустой массив значений для загрузки в регистры устройства." );
   if (auAddressValuePairs->Length>c_nMaxRegLayoutLength)
      throw gcnew ArgumentException(
         "Слишком большой массив значений для загрузки в регистры устройства." );

DEVICE_COMMAND_BEGIN

   pin_ptr<UInt32>
      puInputBuffer = &auAddressValuePairs[0];
   DWORD
      dwBytesReturned;

   if (!::DeviceIoControl( sv_hDevice, IOCTL_SHAPER_LOAD_REGISTERS,
                           puInputBuffer, (DWORD)(auAddressValuePairs->Length*sizeof(UInt32)),
                           NULL, 0, &dwBytesReturned, NULL ))
      throw gcnew Win32Exception();

DEVICE_COMMAND_END
}

//--------------------------------------------------------------------------------------------------

void ShaperDevice::LoadProfile(UInt32 uBankNo,UInt32 uStartAddress,array<UInt32>^ auFreqMagCodes)
{
   if (!auFreqMagCodes)
      throw gcnew ArgumentNullException();
   if (!auFreqMagCodes->Length)
      throw gcnew ArgumentException(
         "Пустой массив значений для загрузки в память устройства." );
   if (auFreqMagCodes->Length>SHAPER_MEMORY_BANK_CAPACITY)
      throw gcnew ArgumentException(
         "Слишком большой массив значений для загрузки в память устройства." );

DEVICE_COMMAND_BEGIN

   array<UInt32>
      ^auInputArray=gcnew array<UInt32>(auFreqMagCodes->Length+2);

   auInputArray[0] = uBankNo;
   auInputArray[1] = uStartAddress;

   auFreqMagCodes->CopyTo(auInputArray,2);

   pin_ptr<UInt32>
      puInputBuffer = &auInputArray[0];
   DWORD
      dwBytesReturned;

   if (!::DeviceIoControl( sv_hDevice, IOCTL_SHAPER_LOAD_PROFILE,
                           puInputBuffer, (DWORD)((auInputArray->Length)*sizeof(UInt32)),
                           NULL, 0, &dwBytesReturned, NULL ))
      throw gcnew Win32Exception();

DEVICE_COMMAND_END
}

//--------------------------------------------------------------------------------------------------

bool ShaperDevice::TestMemory( [Out] UInt32% uProblemBankNo,
                               [Out] UInt32% uErrorCellAddress,
                               [Out] UInt32% uWrittenValue,
                               [Out] UInt32% uReadValue )
{
DEVICE_COMMAND_BEGIN

   UInt32
      auOutputBuffer[4];
   DWORD
      dwBytesReturned;

   if (!::DeviceIoControl( sv_hDevice, IOCTL_SHAPER_TEST_MEMORY,
                           NULL, 0, $ARRAY_N_SIZE_OF(auOutputBuffer),
                           &dwBytesReturned, NULL ))
      throw gcnew Win32Exception();

   if (!dwBytesReturned)
      $CLEAR_ARRAY_VARIABLE(auOutputBuffer);

   uProblemBankNo   = auOutputBuffer[0];
   uErrorCellAddress= auOutputBuffer[1];
   uWrittenValue    = auOutputBuffer[2];
   uReadValue       = auOutputBuffer[3];

   return !dwBytesReturned;

DEVICE_COMMAND_END
}

//--------------------------------------------------------------------------------------------------

void ShaperDevice::GenerateSawTooth( bool bIsFixedParameterMagnitude,
                                     UInt32 uFixedFrequencyOrMagnitude,
                                     UInt32 uSampleDelay )
{
DEVICE_COMMAND_BEGIN

   UInt32
      auInputBuffer[] = { bIsFixedParameterMagnitude, uFixedFrequencyOrMagnitude, uSampleDelay };
   DWORD
      dwBytesReturned;

   if (!::DeviceIoControl( sv_hDevice, IOCTL_SHAPER_GENERATE_SAW_TOOTH,
                           $ARRAY_N_SIZE_OF(auInputBuffer), NULL, 0,
                           &dwBytesReturned, NULL ))
      throw gcnew Win32Exception();

DEVICE_COMMAND_END
}

//==================================================================================================

} // ShaperAccess
