﻿#include "Precompiled.h"
#include "SharedIncludes.h"

namespace ShaperAccess {

//==================================================================================================

static HANDLE InterlockedExchangeHandle(IN OUT HANDLE volatile* pHandle,HANDLE value);

//==================================================================================================

bool ShaperDevice::IsOpen::get()
{
   return sv_hDevice!=INVALID_HANDLE_VALUE;
}

//--------------------------------------------------------------------------------------------------

void ShaperDevice::Open()
{
   Open(nullptr);
}

//--------------------------------------------------------------------------------------------------

void ShaperDevice::Open(String^ strHardwareIdentifier)
{
DEVICE_OPERATION_LOCK_BEGIN

   if (IsOpen)
      throw gcnew InvalidOperationException(
         "Попытка открытия незакрытого устройства шейпера." );

   String
      ^strDeviceLink = ObtainLink(strHardwareIdentifier);

   if (!strDeviceLink)
      throw gcnew FileNotFoundException(
         "Устройство шейпера в системе не обнаружено." );

   HANDLE
      hDevice;
   $PSZ
      pszDeviceLink = ($PSZ)(PVOID)Marshal::StringToHGlobalUni(strDeviceLink);
   try
   {
      hDevice=::CreateFile( pszDeviceLink, GENERIC_READ | GENERIC_WRITE,
                            0, NULL, OPEN_EXISTING, 0, NULL );
      if (hDevice==INVALID_HANDLE_VALUE)
         throw gcnew Win32Exception();
   }
   finally
   {
      Marshal::FreeHGlobal((IntPtr)pszDeviceLink);
   }

   try
   {
      UInt32
         auInputBuffer [4] = { $VER_MAJOR_VERSION, $VER_MINOR_VERSION,
                               $VER_BUILD_NUMBER, $VER_REVISION_NUMBER },
         auOutputBuffer[4];
      DWORD
         dwBytesReturned;

      if (!::DeviceIoControl( hDevice, IOCTL_SHAPER_INITIALIZE,
                              $ADDRESS_N_SIZE_OF(auInputBuffer),
                              $ADDRESS_N_SIZE_OF(auOutputBuffer),
                              &dwBytesReturned, NULL ))
         throw gcnew Win32Exception();

      if (dwBytesReturned)
      {
         if (dwBytesReturned!=sizeof(auOutputBuffer))
            throw gcnew ShaperException(
               "Неопознанная версия драйвера \"Shaper.sys\"." );
         throw ShaperException::Create(
            "Версия драйвера не совпадает с версией библиотеки поддержки.\r\n"
            "Драйвер-версия: {0}.{1}.{2}.{3} (\"Shaper.sys\").\r\n"
            "DLL-версия: {4}.{5}.{6}.{7} (\"ShaperAccess.dll\").",
            auOutputBuffer[0], auOutputBuffer[1], auOutputBuffer[2], auOutputBuffer[3],
            auInputBuffer [0], auInputBuffer [1], auInputBuffer [2], auInputBuffer [3] );
      }
   }
   catch (Exception^)
   {
      ::CloseHandle(hDevice);
      throw;
   }

   sv_strSymbolicLink=strDeviceLink;

   pin_ptr<volatile HANDLE>
      pDeviceHandle = &sv_hDevice;

   InterlockedExchangeHandle(pDeviceHandle,hDevice);

DEVICE_OPERATION_LOCK_END
}

//--------------------------------------------------------------------------------------------------

void ShaperDevice::Close()
{
DEVICE_OPERATION_LOCK_BEGIN

   if (IsOpen)
   {
      pin_ptr<volatile HANDLE>
         pDeviceHandle = &sv_hDevice;
      HANDLE
         hDevice = InterlockedExchangeHandle(pDeviceHandle,INVALID_HANDLE_VALUE);

      sv_strSymbolicLink=nullptr;

      if (!::CloseHandle(hDevice))
         throw gcnew Win32Exception();
   }

DEVICE_OPERATION_LOCK_END
}

//==================================================================================================

void ShaperDevice::VerifyIsOpen()
{
   if (!IsOpen)
      throw CreateNotOpenException();
}

//--------------------------------------------------------------------------------------------------

InvalidOperationException^ ShaperDevice::CreateNotOpenException()
{
   return
      gcnew InvalidOperationException(
         "Попытка обращения к неоткрытому устройству шейпера." );
}

//==================================================================================================

#pragma unmanaged

static HANDLE InterlockedExchangeHandle(IN OUT HANDLE volatile* pHandle,HANDLE value)
{
   return InterlockedExchangePointer(pHandle,value);
}

#pragma managed

//==================================================================================================

} // ShaperAccess
