﻿#include "Precompiled.h"
#include "SharedIncludes.h"

namespace ShaperAccess {

//==================================================================================================

String^ ShaperDevice::SymbolicLink::get()
{
   VerifyIsOpen();
   String
      ^rslt=sv_strSymbolicLink;
   if (!rslt)
      throw CreateNotOpenException();
   return rslt;
}

//--------------------------------------------------------------------------------------------------

String^ ShaperDevice::ObtainLink(String^ strHardwareIdentifier)
{
   if ( strHardwareIdentifier &&
        ( strHardwareIdentifier=="" ||
          strHardwareIdentifier->Trim()!=strHardwareIdentifier ) )
      throw gcnew ArgumentException(
         "Аппаратный идентификатор устройства специфицирован некорректно." );

   HDEVINFO
      hDeviceInfoSet=::SetupDiGetClassDevs( &GUID_DEVINTERFACE_SHAPER, NULL, NULL,
                                            DIGCF_DEVICEINTERFACE | DIGCF_PRESENT );
   if (hDeviceInfoSet==INVALID_HANDLE_VALUE)
      throw gcnew Win32Exception();
   try
   {
      String ^strDeviceLink=nullptr;

      for (DWORD dwMemberIndex=0;;dwMemberIndex++)
      {
         SP_DEVICE_INTERFACE_DATA
            deviceInterfaceData;
         deviceInterfaceData.cbSize=
            sizeof(SP_DEVICE_INTERFACE_DATA);
         if (!::SetupDiEnumDeviceInterfaces( hDeviceInfoSet, NULL,
                                             &GUID_DEVINTERFACE_SHAPER,
                                             dwMemberIndex, &deviceInterfaceData ))
            if (::GetLastError()==ERROR_NO_MORE_ITEMS)
               return strDeviceLink;
            else
               throw gcnew Win32Exception();

         String ^strDevLink=
            ExtractDevicePathFromInterfaceData(hDeviceInfoSet,deviceInterfaceData);
         if ( !strHardwareIdentifier ||
              strDevLink->IndexOf( strHardwareIdentifier,
                                   StringComparison::InvariantCultureIgnoreCase ) ==
              strDevLink->Length - strHardwareIdentifier->Length )
         {
            if (strDeviceLink)
               throw gcnew ShaperException(
                  strHardwareIdentifier ?
                     String::Format(
                        "По заданному аппаратному идентификатору обнаружено "
                        "несколько экземпляров устройства шейпера.\r\n"
                        "(Hardware-ID: \"{0}\".)",
                        strHardwareIdentifier ) :
                     "Обнаружено несколько экземпляров устройства шейпера.\r\n"
                     "Необходимо специфицировать аппаратный идентификатор." );
            strDeviceLink=strDevLink;
         }
      }
   }
   finally
   {
      if (!::SetupDiDestroyDeviceInfoList(hDeviceInfoSet))
         throw gcnew Win32Exception();
   }
}

//--------------------------------------------------------------------------------------------------

String^ ShaperDevice::ExtractDevicePathFromInterfaceData
(
   HDEVINFO                   hDeviceInfoSet,
   SP_DEVICE_INTERFACE_DATA%  deviceInterfaceData
)
{
   DWORD dwRequiredSize;

   if (!::SetupDiGetDeviceInterfaceDetail( hDeviceInfoSet, &deviceInterfaceData,
                                           NULL, 0, &dwRequiredSize, NULL ))
      if (::GetLastError()!=ERROR_INSUFFICIENT_BUFFER)
         throw gcnew Win32Exception();

   PSP_DEVICE_INTERFACE_DETAIL_DATA
      pDeviceInterfaceDetailData =
         (PSP_DEVICE_INTERFACE_DETAIL_DATA)
         (void*)Marshal::AllocHGlobal(dwRequiredSize);
   try
   {
      pDeviceInterfaceDetailData->cbSize=sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

      if (!::SetupDiGetDeviceInterfaceDetail(
            hDeviceInfoSet, &deviceInterfaceData,
            pDeviceInterfaceDetailData, dwRequiredSize,
            0, NULL ))
         throw gcnew Win32Exception();

      return gcnew String(pDeviceInterfaceDetailData->DevicePath);
   }
   finally
   {
      Marshal::FreeHGlobal((IntPtr)pDeviceInterfaceDetailData);
   }
}

//==================================================================================================

} // ShaperAccess
