﻿using System;
using System.IO;
using System.Threading;

using ElementaryExtensions;
using ShaperAccess;

namespace ShaperTerminal {

//**************************************************************************************************
class BinaryProfileException: ЄException {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public BinaryProfileException(string strMessage)
   :base(strMessage)
{}

//--------------------------------------------------------------------------------------------------

public static new BinaryProfileException Create(string strMessageFmt,params object[] aobParams)
{
   return new BinaryProfileException(string.Format(strMessageFmt,aobParams));
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // BinaryProfileException
//**************************************************************************************************


//**************************************************************************************************
class BinaryProfile {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public UInt32[]
   FreqMagCodes;

//==================================================================================================

public void Load(string strFileName)
{
   long
      nFixedSize=ShaperDevice.MemoryBankCapacity*4;
   string
      strFilePath=strFileName;

   FreqMagCodes = new UInt32[ ShaperDevice.MemoryBankCapacity ];

   try
   {
      Interlocked.Exchange( ref strFilePath, Path.GetFullPath(strFileName) );

      using (FileStream fileStream=File.OpenRead(strFilePath))
      {
         fileStream.Lock(0L,fileStream.Length);

         if (fileStream.Length!=nFixedSize)
            throw new ApplicationException(string.Format(
               "Неверный размер файла бинарного профиля: {0} байта(ов).\n"+
               "Должно быть ровно {1} байта(ов).",
               fileStream.Length, nFixedSize ));

         ReadHalf(fileStream,FrequencyOrMagnitude.Frequency);
         ReadHalf(fileStream,FrequencyOrMagnitude.Magnitude);
      }
   }
   catch (FileNotFoundException e)
   {
      throw new BinaryProfileException(e.Message);
   }
   catch (Exception e)
   {
      throw
         BinaryProfileException.Create( "{0}\nФайл: \"{1}\".",
                                        e.Message, strFilePath );
   }
}

//==================================================================================================

void ReadHalf(FileStream fs,FrequencyOrMagnitude frequencyOrMagnitude)
{
   for (int i=0; i<ShaperDevice.MemoryBankCapacity; i++)
   {
      bool
         bFrequency = frequencyOrMagnitude == FrequencyOrMagnitude.Frequency;
      int
         nReadByte_1,
         nReadByte_2;

      if ( (nReadByte_1=fs.ReadByte())<0 ||
           (nReadByte_2=fs.ReadByte())<0 )
         throw
            new ApplicationException("Попытка чтения за концом файла.");

      ushort
         usCode = (ushort)( (ushort)nReadByte_2<<8 |
                            (ushort)nReadByte_1 );

      if ( usCode > ( bFrequency ?
                      ShaperDevice.MaxFrequencyCode :
                      ShaperDevice.MaxMagnitudeCode ) )
         throw
            new ApplicationException(string.Format(
               "Недопустимый код {0}: {1}.\n"+
               "Индекс элемента слова в подмассиве: {2} (абсолютный: {3}).",
               bFrequency ? "частоты" : "амплитуды", usCode, i,
               i+(bFrequency ? 0 : ShaperDevice.MemoryBankCapacity) ));

      FreqMagCodes[i] |=
         (UInt32)usCode << ( bFrequency ?
            ShaperDevice.FrequencyFirstBit :
            ShaperDevice.MagnitudeFirstBit );
   }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // BinaryProfile
//**************************************************************************************************

} // ShaperTerminal
