﻿using System;
using System.Xml;
using System.Globalization;

using ElementaryExtensions;
using ShaperAccess;

namespace ShaperTerminal {

using Properties;

//**************************************************************************************************
class DeviceConfiguration: XmlConfiguration {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public DeviceConfiguration(): base(Resources.ShaperDevice_XSD)
{}

//--------------------------------------------------------------------------------------------------

public int
   RegLayouts_Count,
   Profiles_Count;
public string
   HardwareIdentifier,
   RegLayouts_DirectoryPath="",
   Profiles_DirectoryPath="";
public string[]
   RegLayouts_XmlFilePaths,
   Profiles_XmlFilePaths,
   Profiles_BinFilePaths;
public UInt32
   SawTest_SampleDelay;

//==================================================================================================

public void Load(string strFileName)
{
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   LoadDocument(strFileName);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   XmlNode[]
      childElements;
   XmlAttribute
      attribute;

   attribute = CurrentNode.Attributes["HardwareIdentifier"];
   if (attribute!=null)
   {
      HardwareIdentifier = attribute.InnerText;
      if (HardwareIdentifier.Equals("single",StringComparison.InvariantCultureIgnoreCase))
         HardwareIdentifier=null;
      else
         if ( HardwareIdentifier=="" ||
              HardwareIdentifier.Trim()!=HardwareIdentifier )
            throw CreateExceptionWithPath( attribute,
               "Аппаратный идентификатор устройства специфицирован некорректно.");
   }

   if (NavigateToNode("x:RegLayouts"))
   {
      RegLayouts_DirectoryPath = CurrentNode.Attributes["Directory"].InnerText;

      childElements = GetChildElements(CurrentNode);
      RegLayouts_Count = childElements.Length;

      RegLayouts_XmlFilePaths=new string[RegLayouts_Count];

      for (int i=0; i<RegLayouts_Count; i++)
         RegLayouts_XmlFilePaths[i] =
            Program.ComposeRegLayoutXmlFilePath(
               childElements[i].Attributes["BaseName"].InnerText );
   }

   if (NavigateToNode("x:Profiles"))
   {
      Profiles_DirectoryPath = CurrentNode.Attributes["Directory"].InnerText;

      childElements = GetChildElements(CurrentNode);
      Profiles_Count = childElements.Length;

      Profiles_BinFilePaths=new string[Profiles_Count];
      Profiles_XmlFilePaths=new string[Profiles_Count];

      for (int i=0; i<Profiles_Count; i++)
      {
         string strProfileFileBaseName =
            childElements[i].Attributes["BaseName"].InnerText;
         Profiles_XmlFilePaths[i] =
            Program.ComposeProfileXmlFilePath(strProfileFileBaseName);
         Profiles_BinFilePaths[i] =
            Program.ComposeProfileBinFilePath(strProfileFileBaseName);
      }
   }

   if (NavigateToNode("x:SawTest/x:SampleDelay"))
   {
      attribute = CurrentNode.Attributes["Microseconds"];

      try
      {
         SawTest_SampleDelay =
            ЄValueParser.ParseUInt32Invariant(
               attribute.InnerText, NumberStyles.None );
      }
      catch (ЄValueParsingException e)
      {
         throw CreateExceptionWithPath(attribute,e.Message);
      }
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // DeviceConfiguration
//**************************************************************************************************

} // ShaperTerminal
