﻿using System;
using System.Xml;

using ElementaryExtensions;
using ShaperAccess;

namespace ShaperTerminal {

using Properties;

//**************************************************************************************************
class RegistersConfiguration: XmlConfiguration {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public RegistersConfiguration(): base(sr_strSchemaText)
{}

//--------------------------------------------------------------------------------------------------

public string
   DisplayName;
public UInt32[]
   AddressValuePairs;

//==================================================================================================

public void Load(string strFileName)
{
   Load(strFileName,true);
}

public void Load(string strFileName,bool bLoadAllContent)
{
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   LoadDocument(strFileName);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   XmlNode[]
      elements=GetChildElements(CurrentNode);

   DisplayName = CurrentNode.Attributes["DisplayName"].InnerText;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   if (!bLoadAllContent)
      return;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   try
      { AddressValuePairs=new UInt32[elements.Length*2]; }
   catch (OverflowException e)
      { throw CreateExceptionWithPath(e.Message); }

   for (int i=0; i<elements.Length; i++)
   {
      AddressValuePairs[i*2] = sr_anRegAddresses[
         ЄString.IndexOf_Strict( elements[i].Name, sr_astrRegNames ) ];
      AddressValuePairs[i*2+1] =
         ЄHexParser.ParseUInt32( elements[i].Attributes["SetValue"].InnerText );
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}

//==================================================================================================

static readonly string
   sr_strSchemaText = Resources.ShaperDevice_Registers_XSD;

static readonly string[]
   sr_astrRegNames =
   {
      "MCR1", "MCR2", "RD", "RTHRESH", "RDEL", "RPERIOD", "RGM", "DUMMY"
   };

static readonly UInt32[]
   sr_anRegAddresses =
   {
      ShaperDevice.MCR1,
      ShaperDevice.MCR2,
      ShaperDevice.RD,
      ShaperDevice.RTHRESH,
      ShaperDevice.RDEL,
      ShaperDevice.RPERIOD,
      ShaperDevice.RGM,
      ShaperDevice.DUMMY
   };

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // RegistersConfiguration
//**************************************************************************************************

} // ShaperTerminal
