using System;
using System.IO;
using System.Threading;

using ElementaryExtensions;
using ShaperAccess;

namespace ShaperTerminal {

//**************************************************************************************************
static partial class Program {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static void BreakdownFinalCallback( string strErrorText,
                                           string strBriefErrorText,
                                           string strConsoleErrorText )
{
   ShaperDevice.Close();
}

//--------------------------------------------------------------------------------------------------

public static void SayGoodbye()
{
   if (s_bLogoInformationPrinted)
   {
      Console.PrintInformation( true,
         "{0}  ! {0}",
         new string(Unicode.BlackSquare,3) );

      Console.PrintTitleFooter( true,
         "\"{0}\" {1} {2}",
         Application.Name,
         Console.LongDashString,
         "ب" );
   }

   if (s_bStartupHasPlayed)
      Sound.PlaySync(SoundSignal.Exit);

   Console.PrintOperation(true," ");
}

//==================================================================================================

static void PrintLogoInformation()
{
   Console.Clear();
   Console.PrintTitleHeader( false, true,
      "    ( .. {0}  ..):",
      Console.LongDashString );
   s_bLogoInformationPrinted=true;
}

static void PrintProcedureTitle(string strCaption)
{
   Console.Clear();
   Console.PrintTitleHeader(false,true,strCaption.ToUpper()+":");
}

//==================================================================================================

static void CompletionDialog_Return()
{
   Console.PrintDoubleSplitter(true,true);
   Console.PressEnterOrEscapeToReturnDialog();
}

static void CompletionDialog_Continue()
{
   Console.PrintDoubleSplitter(true,true);
   Console.PressEnterOrEscapeToContinueDialog();
}

//==================================================================================================

static bool ConfirmRefusalDialog()
{
   Console.Print();
   return Console.OkCancelDialog(" :");
}

//==================================================================================================

public static void PauseUI()
{
   Thread.Sleep(c_nPauseUIInterval);
}

//==================================================================================================

static bool InputValidationCallback_FileBaseName(string strInput)
{
   if (strInput=="" || strInput.Trim()!=strInput)
      return false;
   char? prevChar=null;
   foreach (char ch in strInput)
   {
      if( !( Char.IsAsciiLetterOrDigit(ch) ||
             Ops.IsOneOf(ch,'_','-',' ') ) ||
          ch == ' ' && prevChar == ' ' )
         return false;
      prevChar=ch;
   }
   return true;
}

//==================================================================================================

public static string ComposeRegLayoutXmlFilePath(string strFileBaseName)
{
   return
      Path.Combine(s_DeviceConfiguration.RegLayouts_DirectoryPath,strFileBaseName)+
      c_strRegLayoutXmlFileExt;
}

//--------------------------------------------------------------------------------------------------

public static string ComposeProfileXmlFilePath(string strFileBaseName)
{
   return
      Path.Combine(s_DeviceConfiguration.Profiles_DirectoryPath,strFileBaseName)+
      c_strProfileXmlFileExt;
}

public static string ComposeProfileBinFilePath(string strFileBaseName)
{
   return
      Path.Combine(s_DeviceConfiguration.Profiles_DirectoryPath,strFileBaseName)+
      c_strProfileBinFileExt;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // Program
//**************************************************************************************************

} // ShaperTerminal
