﻿using System;
using System.IO;
using System.ComponentModel;

using ElementaryExtensions;
using ShaperAccess;

namespace ShaperTerminal {

using Properties;

//**************************************************************************************************
static partial class Program {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

static void LoadProfileProcedure()
{
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   bool
      bTooManyItems = s_astrProfileDisplayNames.Length>9;
   string[]
      astrDialogItems = bTooManyItems ? new string[10] : s_astrProfileDisplayNames;
   if (bTooManyItems)
   {
      Array.Copy(s_astrProfileDisplayNames,astrDialogItems,9);
      astrDialogItems[9]=c_strDialogItem_SpecifyFileName;
   }
   bool[]
      abItemsEnabling=new bool[astrDialogItems.Length+1];

   for (int i=0; i<abItemsEnabling.Length; i++)
      if (i<Math.Min(s_astrProfileDisplayNames.Length,9))
         abItemsEnabling[i]=s_abProfileAvailabilities[i];
      else
         abItemsEnabling[i]=true;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   for (;;)
   {
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      PrintProcedureTitle(c_strLoadProfileProcedureCaption);

      if (s_strActiveProfileDisplayName!=null)
      {
         ЄConsole.PrintFeature(s_strActiveProfileDisplayName);
         ЄConsole.Print();
      }

      int
         nDialogItemIdx = ЄConsole.NumberedListDialogX(
            "Выберите профиль сигнала (двойной массив кодов):",
            astrDialogItems,
            bTooManyItems ?
               ЄConsoleDialogListNumeration.EndWithZero : ЄConsoleDialogListNumeration.Normal,
             c_strDialogItem_RefusalAndReturn,
            abItemsEnabling );
      if (nDialogItemIdx<0)
         return;

      string
         strProfileXmlFilePath,
         strProfileBinFilePath;
      if (nDialogItemIdx<9)
      {
         strProfileXmlFilePath=s_DeviceConfiguration.Profiles_XmlFilePaths[nDialogItemIdx];
         strProfileBinFilePath=s_DeviceConfiguration.Profiles_BinFilePaths[nDialogItemIdx];
      }
      else
      {
         ЄConsole.PrintDialogCaption( true,
            "Введите базовое имя файлов профиля (без пути и расширения):" );
         ЄConsole.Print();
         string strFileBaseName =
            ЄConsole.InputString( "Базовое имя профиля",
                                  InputValidationCallback_FileBaseName );
         if (strFileBaseName==null)
            continue;
         strProfileXmlFilePath = ComposeProfileXmlFilePath(strFileBaseName);
         strProfileBinFilePath = ComposeProfileBinFilePath(strFileBaseName);
      }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      ЄConsole.PrintDoubleSplitter(true);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      ЄConsole.PrintOperation( true,
         "Чтение описания профиля: \"{0}\"",
         strProfileXmlFilePath );

      ProfileConfiguration
         profileConfiguration=new ProfileConfiguration();
      try
      {
         profileConfiguration.Load(strProfileXmlFilePath);
      }
      catch (XmlConfigurationException e)
      {
         ReportError(e);
         CompletionDialog_Continue();
         continue;
      }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      ЄConsole.PrintOperation( true,
         "Чтение данных профиля: \"{0}\"",
         strProfileBinFilePath );

      BinaryProfile
         binaryProfile=new BinaryProfile();
      try
      {
         binaryProfile.Load(strProfileBinFilePath);
      }
      catch (BinaryProfileException e)
      {
         ReportError(e);
         CompletionDialog_Continue();
         continue;
      }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      s_strActiveProfileDisplayName=null;

      ЄConsole.PrintOperation( true,
         "Загрузка значений в память устройства" );
      try
      {
         ShaperDevice.LoadProfile(
            profileConfiguration.LoadTo_BankNo,
            0u, binaryProfile.FreqMagCodes );
      }
      catch (Win32Exception e)
      {
         ReportError(e);
         CompletionDialog_Continue();
         continue;
      }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      s_strActiveProfileDisplayName = profileConfiguration.DisplayName;

      ЄConsole.PrintDoubleSplitter(true);

      ReportSuccess(true,"Профиль загружен успешно.");

      PauseUI();

      return;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   } // for
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // Program
//**************************************************************************************************

} // ShaperTerminal
