﻿using System;
using System.IO;
using System.ComponentModel;

using ElementaryExtensions;
using ShaperAccess;

namespace ShaperTerminal {

using Properties;

//**************************************************************************************************
static partial class Program {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

static void LoadRegistersProcedure()
{
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   bool
      bTooManyItems = s_astrRegLayoutDisplayNames.Length>9;
   string[]
      astrDialogItems = bTooManyItems ? new string[10] : s_astrRegLayoutDisplayNames;
   if (bTooManyItems)
   {
      Array.Copy(s_astrRegLayoutDisplayNames,astrDialogItems,9);
      astrDialogItems[9]=c_strDialogItem_SpecifyFileName;
   }
   bool[]
      abItemsEnabling=new bool[astrDialogItems.Length+1];

   for (int i=0; i<abItemsEnabling.Length; i++)
      if (i<Math.Min(s_astrRegLayoutDisplayNames.Length,9))
         abItemsEnabling[i]=s_abRegLayoutAvailabilities[i];
      else
         abItemsEnabling[i]=true;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   for (;;)
   {
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      PrintProcedureTitle(c_strLoadRegistersProcedureCaption);

      if (s_strActiveRegLayoutDisplayName!=null)
      {
         ЄConsole.PrintFeature(s_strActiveRegLayoutDisplayName);
         ЄConsole.Print();
      }

      int
         nDialogItemIdx = ЄConsole.NumberedListDialogX(
            "Выберите раскладку (регистровую конфигурацию):",
            astrDialogItems,
            bTooManyItems ?
               ЄConsoleDialogListNumeration.EndWithZero : ЄConsoleDialogListNumeration.Normal,
             c_strDialogItem_RefusalAndReturn,
            abItemsEnabling );
      if (nDialogItemIdx<0)
         return;

      string
         strRegLayoutXmlFilePath;
      if (nDialogItemIdx<9)
         strRegLayoutXmlFilePath=s_DeviceConfiguration.RegLayouts_XmlFilePaths[nDialogItemIdx];
      else
      {
         ЄConsole.PrintDialogCaption( true,
            "Введите базовое имя файла раскладки (без пути и расширения):" );
         ЄConsole.Print();
         string strFileBaseName =
            ЄConsole.InputString( "Базовое имя раскладки",
                                  InputValidationCallback_FileBaseName );
         if (strFileBaseName==null)
            continue;
         strRegLayoutXmlFilePath =
            ComposeRegLayoutXmlFilePath(strFileBaseName);
      }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      ЄConsole.PrintDoubleSplitter(true);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      ЄConsole.PrintOperation( true,
         "Чтение раскладки: \"{0}\"",
         strRegLayoutXmlFilePath );

      RegistersConfiguration
         registersConfiguration=new RegistersConfiguration();
      try
      {
         registersConfiguration.Load(strRegLayoutXmlFilePath);
      }
      catch (XmlConfigurationException e)
      {
         ReportError(e);
         CompletionDialog_Continue();
         continue;
      }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      s_strActiveRegLayoutDisplayName=null;

      ЄConsole.PrintOperation( true,
         "Загрузка значений в регистры устройства" );
      try
      {
         ShaperDevice.LoadRegisters(
            registersConfiguration.AddressValuePairs );
      }
      catch (Win32Exception e)
      {
         ReportError(e);
         CompletionDialog_Continue();
         continue;
      }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      s_strActiveRegLayoutDisplayName = registersConfiguration.DisplayName;

      ЄConsole.PrintDoubleSplitter(true);

      ReportSuccess(true,"Раскладка загружена успешно.");

      PauseUI();

      return;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   } // for
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // Program
//**************************************************************************************************

} // ShaperTerminal
