﻿using System;
using System.ComponentModel;

using ElementaryExtensions;
using ShaperAccess;

namespace ShaperTerminal {

//**************************************************************************************************
static partial class Program {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

static void ReadWriteRegisterProcedure()
{
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   PrintProcedureTitle(c_strReadWriteRegisterProcedureCaption);

   ЄConsole.PrintInfoCaption("Клавиши операции:");
   ЄConsole.PrintDialogItems
   (
      new string[]{ "Чтение", "Запись", "Чтение из кэша" },
      new ConsoleKey[]{ ConsoleKey.R, ConsoleKey.W, ConsoleKey.C },
      null
   );

   ЄConsole.PrintInfoCaption(true,"Формат ввода:");
   ЄConsole.PrintDashAlignedList
   (
      false,
      new string[]
      {
         c_strRegAddressCaption,
         c_strRegValueCaption
      },
      ЄConsole.DialogKeyColor,
      new string[]
      {
         "HEX (без префикса), 2 знака ровно",
         "HEX (без префикса), 1,2,4,6,8 знаков"
      },
      ЄConsole.IntensiveColor
   );

   ЄConsole.PrintInfoCaption(true,"Везде при вводе:");
   ЄConsole.PrintDialogItems
   (
      new string[]{ "сброс ввода (текущей операции либо всей процедуры)" }, true,
      new ConsoleKey[]{ ConsoleKey.Escape }, null
   );

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   for (bool bTheFirstLoop=true;;bTheFirstLoop=false)
   {
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      if (bTheFirstLoop)
         ЄConsole.PrintDoubleSplitter(true,true);
      else
         ЄConsole.PrintSplitter(true,true);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      UInt32 uAddress,uValue;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      ЄConsole.TypeArrowAlignedDialogPrompt(
         c_strReadWriteOperationCaption, sr_nMaxReadWriteCaptionLength );
      int nOpIdx=ЄConsole.Dialog(
         new ConsoleKey[]{ ConsoleKey.R, ConsoleKey.W, ConsoleKey.C },
         null, true );
      if (nOpIdx<0)
      {
         ЄConsole.Print();
         if (ЄConsole.ConfirmRefusalDialog(true))
            break;
         continue;
      }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      if (ЄOps.IsOneOf(nOpIdx,0,2))
      {
         bool bReadFromCache = nOpIdx == 2;

         ЄConsole.Print( ЄConsole.DialogKeyColor,
            bReadFromCache ? "<Читать из кэша>" : "<Читать>" );

         if (InputRegAddress(out uAddress))
         {
            try
            {
               uValue = ShaperDevice.ReadRegister(uAddress,bReadFromCache);
            }
            catch (Exception e)
            {
               if (!(e is Win32Exception || e is ShaperException))
                  throw;
               ReportError(true,e);
               continue;
            }
            ЄConsole.TypeArrowAlignedCaption(c_strRegValueCaption,sr_nMaxReadWriteCaptionLength);
            ReportSuccess("0x{0:X8}",uValue);
         }
      }
      else
      if (nOpIdx==1)
      {
         ЄConsole.Print(ЄConsole.DialogKeyColor,"<Писать>");

         if (InputRegAddress(out uAddress) && InputRegValue(out uValue))
         {
            try
            {
               ShaperDevice.WriteRegister(uAddress,uValue);
            }
            catch (Win32Exception e)
            {
               ReportError(true,e);
               continue;
            }
            ReportSuccess();
         }
      }
      else
         throw new ЄDeadCodeBranchException();

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      } // for
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}

//==================================================================================================

static bool InputRegAddress(out UInt32 uAddress)
{
   return InputRegAddressOrValue(c_strRegAddressCaption,AddressOrValue.Address,out uAddress);
}

static bool InputRegValue(out UInt32 uValue)
{
   return InputRegAddressOrValue(c_strRegValueCaption,AddressOrValue.Value,out uValue);
}

//--------------------------------------------------------------------------------------------------

static bool InputRegAddressOrValue( string strCaption,
                                    AddressOrValue addressOrValue,
                                    out UInt32 uParameter )
{
   string strOperand=
      ЄConsole.InputString
      (
         strCaption,
         sr_nMaxReadWriteCaptionLength,
         delegate (string strInput)
         {
            bool
               bAddress= addressOrValue==AddressOrValue.Address,
               bValue  = !bAddress;
            UInt32
               uOperand;
            if (bValue && !ЄOps.IsOneOf(strInput.Length,1,2,4,6,8))
               return false;
            return
               (bAddress ? strInput.Length==2 : true) &&
               ЄHexParser.TryParseUInt32(strInput,out uOperand) &&
               (bAddress ? ShaperDevice.IsValidRegAddress(uOperand) : true);
         }
      );
   if (strOperand==null)
   {
      uParameter=0; return false;
   }
   uParameter=ЄHexParser.ParseUInt32(strOperand);
   return true;
}

//==================================================================================================

const string
   c_strReadWriteOperationCaption= "Операция (R/W/C)",
   c_strRegAddressCaption        = "Адрес регистра (относительный)",
   c_strRegValueCaption          = "Значение (32-разрядное слово)";

//--------------------------------------------------------------------------------------------------

static readonly int
   sr_nMaxReadWriteCaptionLength=ЄString.MaxLength(
      c_strReadWriteOperationCaption, c_strRegAddressCaption, c_strRegValueCaption );

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // Program
//**************************************************************************************************

} // ShaperTerminal
