﻿using System;
using System.Threading;
using System.Globalization;

using ElementaryExtensions;
using ShaperAccess;

namespace ShaperTerminal {

//**************************************************************************************************
static partial class Program {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

static void SawTestProcedure()
{
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   for (;;)
   {
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      PrintProcedureTitle(c_strSawTestProcedureCaption);

      ЄConsole.PrintInfoCaption("Клавиши выбора фиксированного параметра:");
      ЄConsole.PrintDialogItems
      (
         new string[]{ "Частота", "Амплитуда" },
         new ConsoleKey[]{ ConsoleKey.F, ConsoleKey.M },
         null
      );

      ЄConsole.PrintInfoCaption(true,"Формат ввода:");
      ЄConsole.PrintDashAlignedList
      (
         false,
         new string[]
         {
            "Частота",
            "Амплитуда"
         },
         ЄConsole.DialogKeyColor,
         new string[]
         {
            string.Format("десятичный, 0..{0}",ShaperDevice.MaxSawTestFrequencyCode),
            string.Format("десятичный, 0..{0}",ShaperDevice.MaxSawTestMagnitudeCode)
         },
         ЄConsole.IntensiveColor
      );

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      ЄConsole.PrintDoubleSplitter(true,true);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      FrequencyOrMagnitude
         frequencyOrMagnitude;
      UInt32
         uFixedParameterValue;

      ЄConsole.TypeArrowAlignedDialogPrompt(
         c_strFixedParameterCaption, sr_nMaxFreqMagCaptionLength );
      int nFixedParameterIdx=ЄConsole.Dialog(
         new ConsoleKey[]{ ConsoleKey.F, ConsoleKey.M },
         null, true );
      switch (nFixedParameterIdx)
      {
      case -1:
         return;
      case 0:
         ЄConsole.Print(ЄConsole.DialogKeyColor,"<Частота>");
         frequencyOrMagnitude=FrequencyOrMagnitude.Frequency;
         if (!InputFrequency(out uFixedParameterValue))
            continue;
         break;
      case 1:
         ЄConsole.Print(ЄConsole.DialogKeyColor,"<Амплитуда>");
         frequencyOrMagnitude=FrequencyOrMagnitude.Magnitude;
         if (!InputMagnitude(out uFixedParameterValue))
            continue;
         break;
      default:
         throw new ЄDeadCodeBranchException();
      }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      ЄConsole.PrintDoubleSplitter(true);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      ЄConsole.PrintOperation(true,"Запуск теста");

      SawTestThreadExchange
         threadExchange = new SawTestThreadExchange(
            frequencyOrMagnitude, uFixedParameterValue );
      Thread
         thread=new Thread(new ParameterizedThreadStart(SawTestThreadRoutine));

      thread.Start(threadExchange);

      ЄConsole.PrintHeader(true,"ПИЛООБРАЗНЫЙ ТЕСТ:");

      UInt32 uSampleDelay =
         s_DeviceConfiguration.SawTest_SampleDelay;
      if (uSampleDelay>0)
         ЄConsole.PrintFeature( true, ЄConsole.ExclamationColor,
            "Задержка сигнала: {0} микросекунд(ы).", uSampleDelay );

      try
      {
         DateTime? dtPrevious=null;

         ЄConsole.InputIdlePollingCallback =
            delegate
            {
               if (threadExchange.Exception!=null)
                  throw new ЄConsoleInputPollingEscapeException();
               if (uSampleDelay==0)
               {
                  DateTime dtCurrent=DateTime.Now;

                  if ( dtPrevious==null ||
                       ЄDateTime.ElapsedMilliseconds(dtPrevious.Value,dtCurrent) >= 500 )
                  {
                     dtPrevious=dtCurrent;
                     ЄBeeper.SoundSync(1760,50);
                  }
               }
            };
         ЄConsole.InputIdlePollingInterval = 100;

         ЄConsole.PressKeyToDialog( true,
            "Для останова нажмите {0}",
            ConsoleKey.Escape );
      }
      catch (ЄConsoleInputPollingEscapeException)
      {}
      finally
      {
         ЄConsole.ResetInputIdlePolling();
      }

      if (threadExchange.Exception==null)
      {
         threadExchange.StopRequest=true;
         ЄConsole.PrintShallowDoubleSplitter(true);
         ЄConsole.PrintOperation(true,"Сброс теста");
      }
      else
      {
         ЄConsole.PrintShallowDoubleSplitter(true);
         ReportError(true,threadExchange.Exception);
      }

      thread.Join();

      ЄConsole.PrintFooter(true,"КОНЕЦ ТЕСТА.");

      if (threadExchange.Exception!=null)
         ЄConsole.PressEnterOrEscapeToContinueDialog(true);
      else
      {
         ReportSuccess(true,"Тест завершён.");
         PauseUI();
      }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   } // for
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}

//==================================================================================================

static bool InputFrequency(out UInt32 uFrequency)
{
   return InputFrequencyOrMagnitude( sr_strFrequencyCaption,
                                     FrequencyOrMagnitude.Frequency,
                                     out uFrequency );
}

static bool InputMagnitude(out UInt32 uMagnitude)
{
   return InputFrequencyOrMagnitude( sr_strMagnitudeCaption,
                                     FrequencyOrMagnitude.Magnitude,
                                     out uMagnitude );
}

//--------------------------------------------------------------------------------------------------

static bool InputFrequencyOrMagnitude( string strCaption,
                                       FrequencyOrMagnitude frequencyOrMagnitude,
                                       out UInt32 uParameter )
{
   string strOperand=
      ЄConsole.InputString
      (
         strCaption,
         sr_nMaxFreqMagCaptionLength,
         delegate (string strInput)
         {
            UInt32
               uOperand;
            return
               ЄValueParser.TryParseUInt32Invariant(strInput,NumberStyles.None,out uOperand) &&
               uOperand <= ( frequencyOrMagnitude==FrequencyOrMagnitude.Frequency ?
                             ShaperDevice.MaxSawTestFrequencyCode : ShaperDevice.MaxSawTestMagnitudeCode );
         }
      );
   if (strOperand==null)
   {
      uParameter=0; return false;
   }
   uParameter=ЄValueParser.ParseUInt32Invariant(strOperand);
   return true;
}

//==================================================================================================

const string
   c_strFixedParameterCaption =
      "Фиксированный параметр (F/M)";

//--------------------------------------------------------------------------------------------------

static readonly string
   sr_strFrequencyCaption = string.Format(
      "Значение кода частоты (0..{0})",
      ShaperDevice.MaxSawTestFrequencyCode ),
   sr_strMagnitudeCaption = string.Format(
      "Значение кода амплитуды (0..{0})",
      ShaperDevice.MaxSawTestMagnitudeCode );

static readonly int
   sr_nMaxFreqMagCaptionLength=ЄString.MaxLength(
      c_strFixedParameterCaption, sr_strFrequencyCaption, sr_strMagnitudeCaption );

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // Program
//**************************************************************************************************

} // ShaperTerminal
