﻿using System;
using System.IO;
using System.ComponentModel;

using ElementaryExtensions;
using ShaperAccess;

namespace ShaperTerminal {

//**************************************************************************************************
static partial class Program {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static void Run()
{
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   PrintLogoInformation();

   switch (ЄApplication.EntryPointArgumentsCount)
   {
   case 0:
      break;
   case 1:
      s_strConfigurationFileName=ЄApplication.GetEntryPointArgument(0);
      break;
   default:
      throw new ЄAppFailureException(
         "Допустим только один параметр командной строки -- "+
         "имя XML-файла конфигурации устройства." );
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   ЄConsole.PrintOperation("Загрузка конфигурации приложения");
   try
      { AppSettings.LoadConfiguration(); }
   catch (Exception e)
      { throw new ЄAppFailureException(e.Message); }

   using (new ЄRuntimeParametersAltering())
   {
      ЄBeeper.Mute = AppSettings.Mute;

      ЄRuntimeParametersAltering.ApplyChanges();
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   ЄConsole.PrintOperation( true,
      "Загрузка конфигурации устройства: \"{0}\"",
      s_strConfigurationFileName );
   try
      { s_DeviceConfiguration.Load(s_strConfigurationFileName); }
   catch (XmlConfigurationException e)
      { throw new ЄAppFailureException(e.Message); }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   ЄConsole.PrintOperation(true,"Препросмотр регистровых раскладок");

   s_astrRegLayoutDisplayNames = new string[s_DeviceConfiguration.RegLayouts_Count];
   s_abRegLayoutAvailabilities = new bool  [s_DeviceConfiguration.RegLayouts_Count];

   for (int i=0; i<s_DeviceConfiguration.RegLayouts_Count; i++)
   {
      string strRegLayoutXmlFilePath =
         s_DeviceConfiguration.RegLayouts_XmlFilePaths[i];

      ЄConsole.PrintExpressive(strRegLayoutXmlFilePath);

      try
      {
         s_abRegLayoutAvailabilities[i]=File.Exists(strRegLayoutXmlFilePath);
      }
      catch
      {}

      try
      {
         if (!s_abRegLayoutAvailabilities[i])
            throw new XmlConfigurationException(null);
         RegistersConfiguration
            registersConfiguration=new RegistersConfiguration();
         registersConfiguration.Load(strRegLayoutXmlFilePath,false);
         s_astrRegLayoutDisplayNames[i]=registersConfiguration.DisplayName;
      }
      catch (XmlConfigurationException)
      {
         s_astrRegLayoutDisplayNames[i]="file:///"+strRegLayoutXmlFilePath;
      }
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   ЄConsole.PrintOperation(true,"Препросмотр профилей сигнала");

   s_astrProfileDisplayNames = new string[s_DeviceConfiguration.Profiles_Count];
   s_abProfileAvailabilities = new bool  [s_DeviceConfiguration.Profiles_Count];

   for (int i=0; i<s_DeviceConfiguration.Profiles_Count; i++)
   {
      string
         strProfileXmlFilePath = s_DeviceConfiguration.Profiles_XmlFilePaths[i],
         strProfileBinFilePath = s_DeviceConfiguration.Profiles_BinFilePaths[i];
      bool
         bXmlFileExists=false,
         bBinFileExists=false;

      ЄConsole.PrintExpressive(strProfileXmlFilePath);
      try
         { bXmlFileExists=File.Exists(strProfileXmlFilePath); }
      catch
         {}

      if (bXmlFileExists)
      {
         ЄConsole.PrintExpressive(strProfileBinFilePath);
         try
            { bBinFileExists=File.Exists(strProfileXmlFilePath); }
         catch
            {}
      }

      s_abProfileAvailabilities[i] = bXmlFileExists && bBinFileExists;

      try
      {
         if (!bXmlFileExists)
            throw new XmlConfigurationException(null);
         ProfileConfiguration
            profileConfiguration=new ProfileConfiguration();
         profileConfiguration.Load(strProfileXmlFilePath,false);
         s_astrProfileDisplayNames[i]=profileConfiguration.DisplayName;
      }
      catch (XmlConfigurationException)
      {
         s_astrProfileDisplayNames[i]="file:///"+strProfileBinFilePath;
      }
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   ЄConsole.PrintShallowSplitter(true);

   if (s_DeviceConfiguration.HardwareIdentifier!=null)
      ЄConsole.PrintFaint( true,
         "Hardware-ID = \"{0}\".",
         s_DeviceConfiguration.HardwareIdentifier );
   ЄConsole.PrintOperation( true,
      "Открытие устройства (установка связи с драйвером \"Shaper.sys\")" );
   string
      strDeviceStatusMessage=null;
   try
   {
      ShaperDevice.Open(s_DeviceConfiguration.HardwareIdentifier);
   }
   catch (FileNotFoundException e)
   {
      strDeviceStatusMessage=e.Message;
   }
   catch (Exception e)
   {
      if (!(e is Win32Exception || e is ShaperException))
         throw;
      throw new ЄAppFailureException(e.Message);
   }
   s_bCleanupRequest=true;

   if (ShaperDevice.IsOpen)
      ЄConsole.PrintExpressive(ShaperDevice.SymbolicLink);
   else
      ЄConsole.PrintFaint(strDeviceStatusMessage);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   bool[] abItemsEnabling=new bool[c_nTotalProcedures+1];

   abItemsEnabling[c_nKeysUsageProcedureIdx] =
   abItemsEnabling[c_nAboutProcedureIdx    ] =
   abItemsEnabling[c_nTotalProcedures      ] = true;

   if (ShaperDevice.IsOpen)
   {
      abItemsEnabling[c_nReadWriteRegisterProcedureIdx] =
      abItemsEnabling[c_nDeactivateDeviceProcedureIdx ] =
      abItemsEnabling[c_nSawTestProcedureIdx          ] =
      abItemsEnabling[c_nTestMemoryProcedureIdx       ] = true;
      abItemsEnabling[c_nLoadRegistersProcedureIdx    ] = s_DeviceConfiguration.RegLayouts_Count>0;
      abItemsEnabling[c_nLoadProfileProcedureIdx      ] = s_DeviceConfiguration.Profiles_Count>0;
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   for (bool bTheFirstLoop=true;;bTheFirstLoop=false)
   {
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      PrintLogoInformation();

      ЄConsole.PrintDashAlignedList
      (
         new string[]
         {
            "Конфигурация программы",
            "Каталог раскладок",
            "Каталог профилей",
         },
         new string[]
         {
            s_strConfigurationFileName,
            s_DeviceConfiguration.RegLayouts_DirectoryPath != "" ?
               s_DeviceConfiguration.RegLayouts_DirectoryPath : ".\\",
            s_DeviceConfiguration.Profiles_DirectoryPath != "" ?
               s_DeviceConfiguration.Profiles_DirectoryPath : ".\\"
         }
      );

      ЄConsole.PrintSplitter(true);

      ЄConsole.PrintFeature( true,
         ShaperDevice.IsOpen ? ЄConsole.SuccessColor : ЄConsole.FaintColor,
         ShaperDevice.IsOpen ? "УСТРОЙСТВО ПОДКЛЮЧЕНО" : "УСТРОЙСТВО В СИСТЕМЕ НЕ ОБНАРУЖЕНО" );

      if (ShaperDevice.IsOpen)
      {
         ЄConsole.PrintFaint(true,ShaperDevice.SymbolicLink);

         ЄConsole.Print();
         ЄConsole.PrintDashAlignedList
         (
            new string[]
            {
               "Регистровая раскладка",
               "Профиль сигнала"
            },
            new string[]
            {
               s_strActiveRegLayoutDisplayName ?? "<не загружена>",
               s_strActiveProfileDisplayName   ?? "<не загружен>"
            },
            ЄConsole.ExclamationColor
         );
      }
      else
         if (s_DeviceConfiguration.HardwareIdentifier!=null)
            ЄConsole.PrintFaint( true,
               "Hardware-ID: {0}",
               s_DeviceConfiguration.HardwareIdentifier );

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      ЄConsole.PrintDoubleSplitter(true,true);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      if (bTheFirstLoop)
         if (ShaperDevice.IsOpen)
         {
            Sound.Play(SoundSignal.Statrup);
            s_bStartupHasPlayed=true;
         }
         else
            Sound.Play(SoundSignal.NotSuccess);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      int nItemIdx=ЄConsole.DialogX
      (
         "ВЫБЕРИТЕ ПРОЦЕДУРУ:",
         sr_astrProcedureCaptions,
         sr_MainMenuKeys,
         sr_MainMenuKeyModifiers,
         "Отключиться и выйти",
         ConsoleKey.X, ConsoleModifiers.Alt,
         abItemsEnabling
      );
      if (nItemIdx<0)
         break;

      switch (nItemIdx)
      {
      case c_nKeysUsageProcedureIdx        : KeysUsageProcedure();         break;
      case c_nReadWriteRegisterProcedureIdx: ReadWriteRegisterProcedure(); break;
      case c_nLoadRegistersProcedureIdx    : LoadRegistersProcedure();     break;
      case c_nLoadProfileProcedureIdx      : LoadProfileProcedure();       break;
      case c_nDeactivateDeviceProcedureIdx : DeactivateDeviceProcedure();  break;
      case c_nSawTestProcedureIdx          : SawTestProcedure();           break;
      case c_nTestMemoryProcedureIdx       : TestMemoryProcedure();        break;
      case c_nAboutProcedureIdx            : AboutProcedure();             break;
      default:
         throw new ЄDeadCodeBranchException();
      }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   } // for
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}

//==================================================================================================

public static void PerformCleanup()
{
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   if (!s_bCleanupRequest)
      return;
   s_bCleanupRequest=false;

   if (!ShaperDevice.IsOpen)
      return;
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   ЄConsole.PrintDoubleSplitter(true);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   ЄConsole.PrintOperation(true,"Закрытие устройства (дескриптора ОС)");
   try
   {
      ShaperDevice.Close();
   }
   catch (Win32Exception e)
   {
      throw new ЄAppFailureException(e.Message);
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   ЄConsole.PrintDoubleSplitter(true);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // Program
//**************************************************************************************************

} // ShaperTerminal
