﻿//**************************************************************************************************
//
// ОПРЕДЕЛЕНИЯ ДРАЙВЕРА.
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#pragma once

//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
//  Общие структуры:
//
//==================================================================================================

// Структура контекста устройства:
typedef struct _DRIVER_FDO_DATA
{
   BOOLEAN  CreatedSuccessfully;
   int      OpenFilesCount;
}
DRIVER_FDO_DATA, *PDRIVER_FDO_DATA;

// Генерация аксессора получения указателя на структуру контекста устройства:
WDF_DECLARE_CONTEXT_TYPE_WITH_NAME( DRIVER_FDO_DATA, Driver_FdoGetData );


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Общеразделяемые глобальные переменные:
//
//==================================================================================================

extern LONG volatile
   Driver_DeviceCount;


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
//  Обработчики WDF-событий:
//
//==================================================================================================

// Точка входа драйвера:
NTSTATUS DriverEntry
(
   PDRIVER_OBJECT    pDriverObject,
   PUNICODE_STRING   pRegistryPath
);

//--------------------------------------------------------------------------------------------------

// Действия перед выгрузкой драйвера:
void Driver_EvtDriverUnload
(
   WDFDRIVER hDriver
);

//==================================================================================================

// Создание объекта устройства:
NTSTATUS Driver_EvtDriverDeviceAdd
(
   WDFDRIVER         hDriver,
   PWDFDEVICE_INIT   pDeviceInit
);

//--------------------------------------------------------------------------------------------------

// Очистка дополнительных связанных с устройством ресурсов:
void Driver_EvtDeviceContextCleanup
(
   WDFOBJECT hDeviceObject
);

//==================================================================================================

// Выделение аппаратных ресурсов ресурсов (порты ввода/вывода, память, прерывания):
NTSTATUS Driver_EvtDevicePrepareHardware
(
   WDFDEVICE      hDevice,
   WDFCMRESLIST   hRawResources,
   WDFCMRESLIST   hTranslatedResources
);

// Освобождение аппаратных ресурсов (порты ввода/вывода, память, прерывания):
NTSTATUS Driver_EvtDeviceReleaseHardware
(
   WDFDEVICE      hDevice,
   WDFCMRESLIST   hTranslatedResources
);

//--------------------------------------------------------------------------------------------------

// Введение устройства в рабочее состояние:
NTSTATUS Driver_EvtDeviceD0Entry
(
   WDFDEVICE               hDevice,
   WDF_POWER_DEVICE_STATE  powerState
);

// Выведение устройства из рабочего состояния (спящий режим и т.п.):
NTSTATUS Driver_EvtDeviceD0Exit
(
   WDFDEVICE               hDevice,
   WDF_POWER_DEVICE_STATE  powerState
);

//==================================================================================================

// Создание файла устройства (при открытии из приложения):
void Driver_EvtDeviceFileCreate
(
   WDFDEVICE      hDevice,
   WDFREQUEST     hRequest,
   WDFFILEOBJECT  hFileObject
);

// Закрытие файла устройства:
void Driver_EvtFileClose
(
   WDFFILEOBJECT hFileObject
);

//--------------------------------------------------------------------------------------------------

// Обработка общего запроса чтения из устройства (буферизованный вывод):
void Driver_EvtIoRead
(
   WDFQUEUE    hQueue,
   WDFREQUEST  hRequest,
   size_t      outputLength
);

// Обработка общего запроса записи в устройство (буферизованный ввод):
void Driver_EvtIoWrite
(
   WDFQUEUE    hQueue,
   WDFREQUEST  hRequest,
   size_t      inputLength
);

//--------------------------------------------------------------------------------------------------

// Обработка специализированных запросов ввода/вывода:
void Driver_EvtIoDeviceControl
(
   WDFQUEUE    hQueue,
   WDFREQUEST  hRequest,
   size_t      outputLength,
   size_t      inputLength,
   ULONG       uIoControlCode
);


//**************************************************************************************************
