﻿#include "Precompiled.h"
#include "SharedIncludes.h"

#include <KernelUtils.PagedSegmentsOn.h>

//==================================================================================================
// Создание объекта устройства:
//--------------------------------------------------------------------------------------------------
NTSTATUS Driver_EvtDriverDeviceAdd
(
   WDFDRIVER         hDriver,
   PWDFDEVICE_INIT   pDeviceInit
)
{
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   NTSTATUS                      status = STATUS_SUCCESS;
   PDRIVER_FDO_DATA              pFdoData;
   WDF_PNPPOWER_EVENT_CALLBACKS  pnpPowerCallbacks;
   WDF_FILEOBJECT_CONFIG         fileConfig;
   WDF_IO_QUEUE_CONFIG           queueConfig;
   WDF_OBJECT_ATTRIBUTES         fdoAttributes;
   WDFDEVICE                     hDevice;
   WDFQUEUE                      hQueue;
   int                           nDevices;

   UNREFERENCED_PARAMETER(hDriver);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   $DBG_PRINT_IN_FUNCTION();
   PAGED_CODE();
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Подключение обработчиков "PNP/Power":

   WDF_PNPPOWER_EVENT_CALLBACKS_INIT(&pnpPowerCallbacks);

// === Обработчики "PNP": ===
   pnpPowerCallbacks.EvtDevicePrepareHardware = Driver_EvtDevicePrepareHardware;
   pnpPowerCallbacks.EvtDeviceReleaseHardware = Driver_EvtDeviceReleaseHardware;

// === Обработчики "Power": ===
   pnpPowerCallbacks.EvtDeviceD0Entry= Driver_EvtDeviceD0Entry;
   pnpPowerCallbacks.EvtDeviceD0Exit = Driver_EvtDeviceD0Exit;

   WdfDeviceInitSetPnpPowerEventCallbacks(pDeviceInit,&pnpPowerCallbacks);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Конфигурирование файлового объекта:

   WDF_FILEOBJECT_CONFIG_INIT(
      &fileConfig,
      Driver_EvtDeviceFileCreate,
      Driver_EvtFileClose,
      WDF_NO_EVENT_CALLBACK );

   WdfDeviceInitSetFileObjectConfig(
      pDeviceInit,
      &fileConfig,
      WDF_NO_OBJECT_ATTRIBUTES );

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Создание объекта устройства:

   WDF_OBJECT_ATTRIBUTES_INIT_CONTEXT_TYPE(&fdoAttributes,DRIVER_FDO_DATA);

   fdoAttributes.SynchronizationScope = WdfSynchronizationScopeDevice;
   // -- синхронизация на уровне устройства

   fdoAttributes.ExecutionLevel = WdfExecutionLevelPassive;
   // -- для возможности в режиме "METHOD_NEITHER" обращаться к буферам пользовательского процесса

   fdoAttributes.EvtCleanupCallback = Driver_EvtDeviceContextCleanup;
   // -- триггер отключения устройства

   status = WdfDeviceCreate( &pDeviceInit, &fdoAttributes, &hDevice );

   if (!NT_SUCCESS(status))
   {
      $DBG_PRINT_FUNCTION_FAILURE_WITH_STATUS("WdfDeviceCreate",status);
      goto EXIT;
   }

   // Получение указателя на контекст:
   pFdoData = Driver_FdoGetData(hDevice);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Создание интерфейса устройства:

   status = WdfDeviceCreateDeviceInterface(
               hDevice,
               &GUID_DEVINTERFACE_XDEVICE,
               NULL );
   if (!NT_SUCCESS(status))
   {
      $DBG_PRINT_FUNCTION_FAILURE_WITH_STATUS("WdfDeviceCreateDeviceInterface",status);
      goto EXIT;
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Создание основной очереди запросов ввода/вывода:

   WDF_IO_QUEUE_CONFIG_INIT_DEFAULT_QUEUE(
      &queueConfig,
      WdfIoQueueDispatchSequential ); // <-- последовательная обработка запросов

   // Не связывать ввод/вывод с состоянием физического устройства (software only driver):
   queueConfig.PowerManaged=WdfFalse;

   // Подписка на запросы ввода/вывода нулевой длины:
   queueConfig.AllowZeroLengthRequests=TRUE;

   queueConfig.EvtIoRead         = Driver_EvtIoRead;
   queueConfig.EvtIoWrite        = Driver_EvtIoWrite;
   queueConfig.EvtIoDeviceControl= Driver_EvtIoDeviceControl;

   status = WdfIoQueueCreate(
               hDevice,
               &queueConfig,
               WDF_NO_OBJECT_ATTRIBUTES,
               &hQueue );
   if (!NT_SUCCESS(status))
   {
      $DBG_PRINT_FUNCTION_FAILURE_WITH_STATUS("WdfIoQueueCreate",status);
      goto EXIT;
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Успешное завершение создания устройства:

   pFdoData->CreatedSuccessfully=TRUE;

   nDevices = InterlockedIncrement(&Driver_DeviceCount);

   $DBG_PRINT_INFORMATION("Новый объект устройства (и все вспомогательные) создан(ы) успешно.");
   $DBG_PRINTF_INFORMATION(("Всего устройств теперь: %d.",nDevices));

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
EXIT:
   $DBG_PRINT_OUT_OF_FUNCTION_WITH_STATUS(status);
   return status;
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}
//==================================================================================================
