﻿#include "Precompiled.h"
#include "SharedIncludes.h"

//==================================================================================================
// Обработка специализированных запросов ввода/вывода:
//--------------------------------------------------------------------------------------------------
void Driver_EvtIoDeviceControl
(
   WDFQUEUE    hQueue,
   WDFREQUEST  hRequest,
   size_t      outputLength,
   size_t      inputLength,
   ULONG       uIoControlCode
)
{
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   NTSTATUS                status = STATUS_SUCCESS;
   PDRIVER_FDO_DATA        pFdoData;
   $UINT_PTR               infoLength = 0;
   $PCVOID                 pcvInputBuffer;
   PVOID                   pvOutputBuffer, pvUnsafeOutputBuffer;
   WDFMEMORY               hOutputMemory;
   WDFSPINLOCK             hSpinLock;
   WDF_OBJECT_ATTRIBUTES   lockAttributes;
   $ACH                    aszStringFromApp[$DBG_PRINT_BUFFER_CCH], aszStringForApp[256];

   UNREFERENCED_PARAMETER(hQueue);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   $DBG_PRINT_IN_FUNCTION();
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   pFdoData = Driver_FdoGetData( $WdfRequestGetDevice(hRequest) );

// Обработка "IOCTL"-запросов:
   switch (uIoControlCode)
   {
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Трассировка сообщения приветствия:
   case IOCTL_XDEVICE_TRACE_HELLO:

      $DBG_PRINT_IDENTIFIER("IOCTL_XDEVICE_TRACE_HELLO");

      if (!$CheckInputOutputLengthsAreZero(&status,inputLength,outputLength))
         break;

      $TraceInformation("Привет из драйвера устройства \"XDevice\" (software only)!");
      break;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Получение даты-времени копмиляции драйвера:
   case IOCTL_XDEVICE_RETRIEVE_DRIVER_BUILD:

      $DBG_PRINT_IDENTIFIER("IOCTL_XDEVICE_RETRIEVE_DRIVER_BUILD");

      if (!$CheckInputLengthIsZero(&status,inputLength))
         break;

      WDF_OBJECT_ATTRIBUTES_INIT(&lockAttributes);
      lockAttributes.ParentObject = hRequest;

      status = WdfSpinLockCreate(&lockAttributes,&hSpinLock);
      if (!NT_SUCCESS(status))
      {
         $DBG_PRINT_FUNCTION_FAILURE_WITH_STATUS("WdfSpinLockCreate",status);
         goto COMPLETE;
      }

      $DBG_PRINT_OPERATION(
         "Поднятие уровня исполнения до \"DISPATCH_LEVEL\" (вызов \"WdfSpinLockAcquire\")" );
      WdfSpinLockAcquire(hSpinLock);

      $DBG_PRINT_CURRENT_IRQL();

      if (!$RetrieveOutputBuffer(&status,hRequest,1,&pvOutputBuffer,NULL))
         goto RELEASE_SPIN_LOCK;

      $StrPrintfA(
         $ARRAY_N_COUNT_OF(aszStringForApp),
         "%s: скомпилирован %s %s.",
         $DriverBinaryName(), __DATE__, __TIME__ );

      infoLength = $StrLengthA(aszStringForApp)+1;

      if (!$CheckSufficientOutputLength(&status,outputLength,infoLength))
         goto RELEASE_SPIN_LOCK;

      RtlCopyMemory(pvOutputBuffer,aszStringForApp,infoLength);
      $DBG_PRINT_OUTPUT_TRANSFER(infoLength);

   RELEASE_SPIN_LOCK:
      $DBG_PRINT_OPERATION(
         "Понижение уровня исполнения до прежнего значения (вызов \"WdfSpinLockAcquire\")" );
      WdfSpinLockRelease(hSpinLock);
      $DBG_PRINT_CURRENT_IRQL();
      break;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Обмен сообщениями для трассировки:
   case IOCTL_XDEVICE_EXCHANGE_TRACE_MESSAGES:

      $DBG_PRINT_IDENTIFIER("IOCTL_XDEVICE_EXCHANGE_TRACE_MESSAGES");

   // Проверка длин ввода и вывода:
      if (!inputLength)
         $DBG_PRINT_ZERO_INPUT_LENGTH_W();
      if (!outputLength)
         $DBG_PRINT_ZERO_OUTPUT_LENGTH_W();

   // Получение пользовательских буферов ввода и вывода:
      if ( inputLength &&
           !$RetrieveUserInputBuffer(&status,hRequest,1,&pcvInputBuffer,NULL) ||
           outputLength &&
           !$RetrieveUserOutputBuffer( &status, hRequest, 1, &pvOutputBuffer, NULL,
                                       &hOutputMemory, &pvUnsafeOutputBuffer ) )
         break;

   // Трассировка находящегося в буфере ввода сообщения (подготовленного приложением):
      if (inputLength)
      {
         if (!$StrCopyA($ARRAY_N_COUNT_OF(aszStringFromApp),($PCASZ)pcvInputBuffer))
            $DBG_PRINT_WARNING("Слишком длинное сообщение получено из приложения. Усечено!");
         if ($StrLengthA(aszStringFromApp))
            $TraceInformation(aszStringFromApp);
         else
            $DBG_PRINT_WARNING("Пустое сообщение получено из приложения.");
      }

   // Заполнение буфера вывода информацией (для приложения):
      if (outputLength)
      {
         if (!$StrPrintfA( ($PASZ)pvOutputBuffer, outputLength,
                           "IOCTL_XDEVICE_EXCHANGE_TRACE_MESSAGES" ))
            $DBG_PRINT_INSUFFICIENT_OUTPUT_LENGTH_W();
         if (!$CopyOutputMemoryToUnsafeUserBuffer(&status,hOutputMemory,pvUnsafeOutputBuffer))
            break;
      }

   // Отчёт о трасфере (в отладчик):
      if (inputLength)
         $DBG_PRINT_INPUT_TRANSFER($StrLengthA(aszStringFromApp)+1);
      if (outputLength)
         $DBG_PRINT_OUTPUT_TRANSFER($StrLengthA(($PCASZ)pvOutputBuffer)+1);
      goto COMPLETE_NO_INFO;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Неподдерживаемые "IOCTL"-коды:

   default:

      $DBG_PRINT_UNSUPPORTED_CTL_CODE(uIoControlCode);

      status = STATUS_INVALID_DEVICE_REQUEST;

   } // switch

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
COMPLETE:
   WdfRequestCompleteWithInformation(hRequest,status,infoLength);
   $DBG_PRINT_OUT_OF_FUNCTION();
   return;
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
COMPLETE_NO_INFO:
   WdfRequestComplete(hRequest,status);
   $DBG_PRINT_OUT_OF_FUNCTION();
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}
//==================================================================================================
