﻿#include "Precompiled.h"
#include "SharedIncludes.h"

namespace XDeviceTest {

//==================================================================================================

static HANDLE InterlockedExchangeHandle(IN OUT HANDLE volatile* pHandle,HANDLE value);

//==================================================================================================

HANDLE Handle::Exchange([In][Out] HANDLE% handle,HANDLE value)
{
   HANDLE previous=handle; handle=value; return previous;
}

HANDLE Handle::VolatileExchange([In][Out] HANDLE volatile* pHandle,HANDLE value)
{
   if (!pHandle)
      throw gcnew ArgumentNullException();
   return
      InterlockedExchangeHandle(pHandle,value);
}

//==================================================================================================

HANDLE Handle::InactiveValue(bool bIsOfInvalidHandleValueCategory)
{
   return bIsOfInvalidHandleValueCategory ? INVALID_HANDLE_VALUE : NULL;
}

bool Handle::IsActiveValue(HANDLE handle,bool bIsOfInvalidHandleValueCategory)
{
   return handle!=InactiveValue(bIsOfInvalidHandleValueCategory);
}

//==================================================================================================

void Handle::Close([Optional] HANDLE handle)
{
   Close(handle,false);
}

void Handle::Close([Optional] HANDLE handle,bool bIsOfInvalidHandleValueCategory)
{
   if (IsActiveValue(handle,bIsOfInvalidHandleValueCategory) && !::CloseHandle(handle))
      throw gcnew Win32Exception();
}

//==================================================================================================

void Handle::ZeroAndClose([In][Out][Optional] HANDLE% handle)
   { ResetAndClose(handle,false); }

void Handle::InvalidateAndClose([In][Out][Optional] HANDLE% handle)
   { ResetAndClose(handle,true); }

//--------------------------------------------------------------------------------------------------

void Handle::ResetAndClose([In][Out][Optional] HANDLE% handle,bool bIsOfInvalidHandleValueCategory)
{
   HANDLE hInactiveValue=
      InactiveValue(bIsOfInvalidHandleValueCategory);
   if ( handle!=hInactiveValue &&
        !::CloseHandle(Exchange(handle,hInactiveValue)) )
      throw gcnew Win32Exception();
}

//==================================================================================================

void Handle::VolatileZeroAndClose([In][Out] HANDLE volatile* pHandle)
   { VolatileResetAndClose(pHandle,false); }

void Handle::VolatileInvalidateAndClose([In][Out] HANDLE volatile* pHandle)
   { VolatileResetAndClose(pHandle,true); }

//--------------------------------------------------------------------------------------------------

void Handle::VolatileResetAndClose([In][Out] HANDLE volatile* pHandle,bool bIsOfInvalidHandleValueCategory)
{
   if (!pHandle)
      throw gcnew ArgumentNullException();
   HANDLE hInactiveValue=
      InactiveValue(bIsOfInvalidHandleValueCategory);
   if ( *pHandle!=hInactiveValue &&
        !::CloseHandle(VolatileExchange(pHandle,hInactiveValue)) )
      throw gcnew Win32Exception();
}

//==================================================================================================

#pragma unmanaged

static HANDLE InterlockedExchangeHandle(IN OUT HANDLE volatile* pHandle,HANDLE value)
{
   return InterlockedExchangePointer(pHandle,value);
}

#pragma managed

//==================================================================================================

} // XDeviceTest
