﻿#include "Precompiled.h"
#include "SharedIncludes.h"

using namespace XDeviceTest;

//==================================================================================================

static void ProcessUnhandledException(Exception^ e);

//==================================================================================================

int main(array<String ^>^ aobParams)
{
   Console::Title="Тест устройства \"XDevice\"";

   XConsole::PrintIntensive(
      "*** ТЕСТ УСТРОЙСТВА \"X-DEVICE\" (SOFTWARE ONLY DRIVER): ***" );
   XConsole::Print();

   int
      nExitCode=0;
   bool
      bWaitKeyForExit=false, bTestStarted=false, bSuccess=true;

   try
   {
      if (aobParams->Length!=0)
      {
         if (!(
            aobParams->Length==1 &&
            String::Compare(aobParams[0],"wait",true)==0
         ))
            throw gcnew AppFailureException(
               "Некорректные параметры командной строки. "
               "Допустим только один параметр -- ключевое слово \"wait\" "
               "(не чувствительно к регистру), для паузы в конце выполнения программы." );
         bWaitKeyForExit=true;
      }
      bTestStarted=true;
      Program::Run();
   }
   catch (AppFailureException^ ae)
   {
      if (nExitCode=ae->ExitCode)
         XConsole::PrintError(ae->Message);
      else
         XConsole::PrintWarning(ae->Message);
      bSuccess=false;
   }
   catch (Win32Exception^ we)
   {
      XConsole::PrintError(we->Message);
      bSuccess=false;
      nExitCode=1;
   }
   catch (Exception^ e)
   {
      ProcessUnhandledException(e);
      bSuccess=false;
      nExitCode=-1;
   }
   finally
   {
      try
      {
         Program::PerformCleanup();
      }
      catch (Exception^ e)
      {
         XConsole::PrintError(e->Message);
         ProcessUnhandledException(e);
         bSuccess=false;
         nExitCode=-1;
      }
   }

   if (bTestStarted)
   {
      XConsole::Print();
      if (bSuccess)
         XConsole::PrintSuccess("ТЕСТ ПРОШЁЛ УСПЕШНО!");
      else
         if (nExitCode==0)
            XConsole::PrintFaint("ТЕСТ НЕ ПРОШЁЛ.");
         else
            XConsole::PrintError("ТЕСТ ЗАВЕРШИЛСЯ НЕУДАЧЕЙ.");
   }

   XConsole::PrintIntensive(true,"*** ВСЁ ***");

   if (bWaitKeyForExit)
   {
      if (nExitCode)
         Console::Beep(880,200);
      XConsole::Print();
      XConsole::PressEnterOrEscapeToExitDialog();
   }

   return nExitCode;
}

//==================================================================================================

static void ProcessUnhandledException(Exception^ e)
{
   XConsole::PrintError( true,
   "===============================================================================" );
   XConsole::PrintError(
   "=== Необработанное исключение (обратный хронологический порядок цепи): ========" );

   for (Exception^ x=e; x; x=x->InnerException)
   {
      XConsole::Print();
      if (e->InnerException)
         XConsole::TypeError("* ");
      XConsole::PrintError(x->ToString());
   }

   XConsole::PrintError( true,
   "===============================================================================" );
}

//==================================================================================================
