﻿#include "Precompiled.h"
#include "SharedIncludes.h"

namespace XDeviceTest {

//==================================================================================================

// Секция "WRITE" объявлена линкеру как "только для записи".
#pragma data_seg("WRITE")
   // "Плохой" буфер ввода (строка в памяти "только для записи"):
   static $ACH g_aszBadInputBuffer[10]="";
#pragma data_seg()

// Секция "READ" объявлена линкеру как "только для чтения".
#pragma data_seg("READ")
   // "Плохой" буфер вывода (строка в памяти "только для чтения"):
   static $ACH g_aszBadOuputBuffer[10]="";
#pragma data_seg()

//==================================================================================================

void Program::Run()
{
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   static $CACH
      // Имя текущей функции (строка в памяти "только для чтения"):
      caszFunctionName[]=__FUNCTION__;

   Char
      szStringFromDriver [c_nDriverStringBufferCch];
   $ACH
      aszStringForDriver [c_nDriverStringBufferCch],
      aszStringFromDriver[c_nDriverStringBufferCch];
   DWORD
      dwBytesRead, dwBytesWritten, dwBytesReturned;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   XConsole::PrintOperation("Получение символьной ссылки устройства");

   String^
      strDeviceLink=DeviceAccess::ObtainLink();
   if (!strDeviceLink)
      throw gcnew AppFailureException("Устройство \"XDevice\" в системе не обнаружено.",0);
   XConsole::PrintExpressive(strDeviceLink);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   XConsole::PrintOperation(true,"Открытие 1-го дескриптора устройства (только чтение)");
   s_hDeviceR=DeviceAccess::OpenHandle(strDeviceLink,GENERIC_READ);
   s_bCleanupRequest=true;

   XConsole::PrintOperation("Открытие 2-го дескриптора устройства (только запись)");
   s_hDeviceW=DeviceAccess::OpenHandle(strDeviceLink,GENERIC_WRITE);

   XConsole::PrintOperation("Открытие 3-го дескриптора устройства (чтение и запись)");
   s_hDeviceRW=DeviceAccess::OpenHandle(strDeviceLink,GENERIC_READ|GENERIC_WRITE);

   XConsole::PrintOperation("Открытие 4-го дескриптора устройства (права не заданы)");
   s_hDevice=DeviceAccess::OpenHandle(strDeviceLink,GENERIC_READ|GENERIC_WRITE);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   XConsole::PrintOperation(true,"Тестирование чтения из устройства");

   if (!::ReadFile(s_hDeviceR,$ARRAY_N_SIZE_OF(szStringFromDriver),&dwBytesRead,NULL))
      throw gcnew Win32Exception();

   XConsole::PrintInformation(gcnew String(szStringFromDriver));
   PrintBytesRead(dwBytesRead);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   XConsole::PrintOperation(true,"Тестирование записи в устройство");

   if (!::GetModuleFileNameA( SysUtils::ProcessModuleHandle,
                              $ARRAY_N_COUNT_OF(aszStringForDriver) ))
      throw gcnew Win32Exception();

   if (!::WriteFile(s_hDeviceW,$ARRAY_N_SIZE_OF(aszStringForDriver),&dwBytesWritten,NULL))
      throw gcnew Win32Exception();

   PrintBytesWritten(dwBytesWritten);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   XConsole::PrintOperation( true,
      "Посылка в драйвер команды для трассировки приветствия (из режима ядра)" );

   if (!::DeviceIoControl( s_hDeviceW, IOCTL_XDEVICE_TRACE_HELLO,
                           NULL,0, NULL,0, &dwBytesReturned, NULL ))
      throw gcnew Win32Exception();

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   XConsole::PrintOperation(true,"Получение даты-времени компиляции драйвера");

   if (!::DeviceIoControl( s_hDeviceR, IOCTL_XDEVICE_RETRIEVE_DRIVER_BUILD,
                           NULL,0, $ARRAY_N_SIZE_OF(aszStringFromDriver),
                           &dwBytesRead, NULL ))
      throw gcnew Win32Exception();

   XConsole::PrintInformation(gcnew String(aszStringFromDriver));
   PrintBytesRead(dwBytesRead);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   XConsole::PrintOperation(true,"Обмен сообщениями для трассировки");

   if (!::DeviceIoControl( s_hDevice, IOCTL_XDEVICE_EXCHANGE_TRACE_MESSAGES,
                           (PVOID)caszFunctionName, sizeof(caszFunctionName),
                           $ARRAY_N_SIZE_OF(aszStringFromDriver),
                           &dwBytesReturned, NULL ))
      throw gcnew Win32Exception();

   XConsole::PrintInformation(gcnew String(aszStringFromDriver));

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   XConsole::PrintOperation( true,
      "Проверка на ошибку (посылка неподдерживаемого \"IOCTL\"-кода)");
   ::SetLastError(NO_ERROR);
   ::DeviceIoControl( s_hDeviceW,
      CTL_CODE( FILE_DEVICE_UNKNOWN, 0x8FF, METHOD_NEITHER, FILE_ANY_ACCESS ),
      NULL,0, NULL,0, &dwBytesReturned, NULL );
   if (::GetLastError()!=ERROR_INVALID_FUNCTION)
      throw gcnew Win32Exception(::GetLastError());
   XConsole::PrintError(
      SysUtils::ErrorCodeToMessage(::GetLastError()) );

   XConsole::PrintOperation("Проверка на ошибку (буфер ввода - память \"только для записи\")" );
   ::SetLastError(NO_ERROR);
   ::DeviceIoControl( s_hDevice, IOCTL_XDEVICE_EXCHANGE_TRACE_MESSAGES,
                      $ARRAY_N_SIZE_OF(g_aszBadInputBuffer), // <-- память "только для записи"
                      $ARRAY_N_SIZE_OF(aszStringFromDriver),
                      &dwBytesReturned, NULL );
   if (::GetLastError()!=ERROR_NOACCESS)
      throw gcnew Win32Exception(::GetLastError());
   XConsole::PrintError(
      SysUtils::ErrorCodeToMessage(::GetLastError()) );

   XConsole::PrintOperation("Проверка на ошибку (буфер вывода - память \"только для чтения\")");
   ::SetLastError(NO_ERROR);
   ::DeviceIoControl( s_hDevice, IOCTL_XDEVICE_EXCHANGE_TRACE_MESSAGES,
                      (PVOID)caszFunctionName, sizeof(caszFunctionName),
                      $ARRAY_N_SIZE_OF(g_aszBadOuputBuffer), // <-- память "только для чтения"
                      &dwBytesReturned, NULL );
   if (::GetLastError()!=ERROR_NOACCESS)
      throw gcnew Win32Exception(::GetLastError());
   XConsole::PrintError(
      SysUtils::ErrorCodeToMessage(::GetLastError()) );

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   Program::PerformCleanup();

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}

//==================================================================================================

void Program::PerformCleanup()
{
   if (!s_bCleanupRequest)
      return;
   s_bCleanupRequest=false;

   XConsole::PrintOperation(true,"Закрытие всех объектов");

   Handle::InvalidateAndClose(s_hDevice  );
   Handle::InvalidateAndClose(s_hDeviceRW);
   Handle::InvalidateAndClose(s_hDeviceW );
   Handle::InvalidateAndClose(s_hDeviceR );
}

//==================================================================================================

} // XDeviceTest
