:SetVar Procedure "paycoll_GetCompaniesPropertiesList"
exec sys_DropProcedure '$(Procedure)'
GO
create procedure [$(Procedure)] (@IsSelfManagedSubdiv bit)
AS
/*================================================================================================*/
/*
##   /    :
*/
/*================================================================================================*/
$(BEGIN)
----------------------------------------------------------------------------------------------------

if @IsSelfManagedSubdiv is null
   raiserror ('   "$(Procedure)".',11,1)

----------------------------------------------------------------------------------------------------

create table #COMPANIES
(
   ID                int               not null
                                       primary key,
   NAME              $(dt_Name)        not null,
   PROP_ID           uniqueidentifier  null,
   PAYCOLL_DIV_ID_1  uniqueidentifier  null,
   PAYCOLL_DIV_ID_2  uniqueidentifier  null
)

if @IsSelfManagedSubdiv=0
   insert #COMPANIES
      select
         mc.[ ],
         RTRIM(mc.[ ]),
         pp.ID,
         pp.PAYCOLL_DIV_ID_1,
         pp.PAYCOLL_DIV_ID_2
      from
         [ ] mc (nolock)
      left outer join
         PAYCOLL_PROPERTIES pp (nolock)
         on mc.[ ]=pp.MANAGEMENT_COMPANY_ID
      where
         exists( select * from TECH_SUBDIVISIONS (nolock)
                 where MC_ID=mc.[ ] )
else
   insert #COMPANIES
      select
         t.ID,
         RTRIM(t.NAME),
         pp.ID,
         pp.PAYCOLL_DIV_ID_1,
         pp.PAYCOLL_DIV_ID_2
      from
         TECH_SUBDIVISIONS t (nolock)
      left outer join
         PAYCOLL_PROPERTIES pp (nolock)
         on t.ID=pp.TECHSUBDIV_ID
/*
      where
         t.MC_ID is null
*/
----------------------------------------------------------------------------------------------------

select
   co.ID                'COMPANY_ID',
   co.NAME              'COMPANY_NAME',
   case
      when @IsSelfManagedSubdiv=1 and LEN(LTRIM(STR(co.ID)))<=3 then
         SUBSTRING('000',1,3-LEN(LTRIM(STR(co.ID))))+
         LTRIM(STR(co.ID))+': '
      else
         ''
   end +
   co.NAME              'COMPANY_NAME_EX',
   pd_1.BIK             'BIK_1',
   pd_2.BIK             'BIK_2',
   pd_1.NAME            'BANK_NAME_1',
   pd_2.NAME            'BANK_NAME_2',
   BANK_ACCOUNT_ID_1,
   BANK_ACCOUNT_ID_2,
   ADDRESS_1,
   ADDRESS_2,
   PHONE_1,
   PHONE_2,
   PHONE_3,
   PHONE_4,
   FINGROUP_CODE
from
   #COMPANIES           co    with (nolock,index=0)
left outer join
   PAYCOLL_PROPERTIES   pp    (nolock) on co.PROP_ID=pp.ID
left outer join
   PAYCOLL_DIVISIONS    pd_1  (nolock) on co.PAYCOLL_DIV_ID_1=pd_1.ID
left outer join
   PAYCOLL_DIVISIONS    pd_2  (nolock) on co.PAYCOLL_DIV_ID_2=pd_2.ID
order by
   case @IsSelfManagedSubdiv
      when 1 then co.ID
      else 0
   end,
   co.NAME

----------------------------------------------------------------------------------------------------
$(END)
/*================================================================================================*/
GO
/*
exec paycoll_GetCompaniesPropertiesList 0
exec paycoll_GetCompaniesPropertiesList 1
*/
grant exec on [$(Procedure)] to [$(ident_AuthorizedGroup)]
GO
