:SetVar Procedure "paycoll_SetCompanyProperties"
exec sys_DropProcedure '$(Procedure)'
GO
create procedure [$(Procedure)]
(
   @IsSelfManagedSubdiv bit,
   @CompanyID           int,
   @Bik_1               $(dt_Bik),
   @Bik_2               $(dt_Bik),
   @BankAccountID_1     $(dt_BankAccount),
   @BankAccountID_2     $(dt_BankAccount),
   @Address_1           $(dt_Address),
   @Address_2           $(dt_Address),
   @Phone_1             $(dt_Phone),
   @Phone_2             $(dt_Phone),
   @Phone_3             $(dt_Phone),
   @Phone_4             $(dt_Phone),
   @FingroupCode        int
)
AS
/*================================================================================================*/
/*
## /    ( -):
*/
/*================================================================================================*/
$(BEGIN)
----------------------------------------------------------------------------------------------------
exec [paycoll_VerifyStateAndSecurity]
----------------------------------------------------------------------------------------------------

declare @error_message $(dt_Message)

if @IsSelfManagedSubdiv is null or @CompanyID is null
   raiserror ('   "$(Procedure)".',11,1)

if @Bik_1     is not null and LEN(@Bik_1    )=0 or
   @Bik_2     is not null and LEN(@Bik_2    )=0 or
   @Address_1 is not null and LEN(@Address_1)=0 or
   @Address_2 is not null and LEN(@Address_2)=0 or
   @Phone_1   is not null and LEN(@Phone_1  )=0 or
   @Phone_2   is not null and LEN(@Phone_2  )=0 or
   @Phone_3   is not null and LEN(@Phone_3  )=0 or
   @Phone_4   is not null and LEN(@Phone_4  )=0
begin
   set @error_message=
      '   "$(Procedure)".'+$(NL)+
      '(       "null"-.)'
   raiserror (@error_message,11,1)
end

----------------------------------------------------------------------------------------------------

if @Bik_1     is null     and @BankAccountID_1 is not null or
   @Bik_1     is not null and @BankAccountID_1 is     null or
   @Bik_2     is null     and @BankAccountID_2 is not null or
   @Bik_2     is not null and @BankAccountID_2 is     null or
   @Bik_1     is null     and @Bik_2           is not null or
   @Address_1 is null     and @Address_2       is not null or
   @Phone_1   is null     and @Phone_2         is not null or
   @Phone_2   is null     and @Phone_3         is not null or
   @Phone_3   is null     and @Phone_4         is not null
   raiserror ('    .',11,1)

----------------------------------------------------------------------------------------------------

declare
   @PaycollDivID_1 uniqueidentifier,
   @PaycollDivID_2 uniqueidentifier

if @Bik_1 is not null
begin
   exec [paycoll_VerifyBankAccount] @Bik_1,@BankAccountID_1
   exec [paycoll_BikToIdentOfBank ] @Bik_1,@PaycollDivID_1 out
end

if @Bik_2 is not null
begin try
   exec [paycoll_VerifyBankAccount] @Bik_2,@BankAccountID_2
   exec [paycoll_BikToIdentOfBank ] @Bik_2,@PaycollDivID_2 out
end try
begin catch
   set @error_message=ERROR_MESSAGE()+$(NL)+'(2-  .)'
   raiserror (@error_message,11,1)
end catch

----------------------------------------------------------------------------------------------------

if @Address_1 is not null and LTRIM(@Address_1)<>@Address_1
   raiserror ('   .',11,1)
if @Address_2 is not null and LTRIM(@Address_2)<>@Address_2
   raiserror ('   2- .',11,1)

if @Phone_1 is not null and LTRIM(@Phone_1)<>@Phone_1
   raiserror ('   1-  .',11,1)
if @Phone_2 is not null and LTRIM(@Phone_2)<>@Phone_2
   raiserror ('   2-  .',11,1)
if @Phone_3 is not null and LTRIM(@Phone_3)<>@Phone_3
   raiserror ('   3-  .',11,1)
if @Phone_4 is not null and LTRIM(@Phone_4)<>@Phone_4
   raiserror ('   4-  .',11,1)

----------------------------------------------------------------------------------------------------
$(BEGIN_TRANSACTION)
----------------------------------------------------------------------------------------------------

if @IsSelfManagedSubdiv=0 and not exists( select * from PAYCOLL_PROPERTIES (nolock)
                                          where MANAGEMENT_COMPANY_ID=@CompanyID )
   insert PAYCOLL_PROPERTIES (MANAGEMENT_COMPANY_ID) values (@CompanyID)

if @IsSelfManagedSubdiv=1 and not exists( select * from PAYCOLL_PROPERTIES (nolock)
                                          where TECHSUBDIV_ID=@CompanyID )
   insert PAYCOLL_PROPERTIES (TECHSUBDIV_ID) values (@CompanyID)

----------------------------------------------------------------------------------------------------

if @IsSelfManagedSubdiv=0
   update
      PAYCOLL_PROPERTIES
   set
      PAYCOLL_DIV_ID_1 = @PaycollDivID_1,
      PAYCOLL_DIV_ID_2 = @PaycollDivID_2,
      BANK_ACCOUNT_ID_1= @BankAccountID_1,
      BANK_ACCOUNT_ID_2= @BankAccountID_2,
      ADDRESS_1        = @Address_1,
      ADDRESS_2        = @Address_2,
      PHONE_1          = @Phone_1,
      PHONE_2          = @Phone_2,
      PHONE_3          = @Phone_3,
      PHONE_4          = @Phone_4,
      FINGROUP_CODE    = @FingroupCode
   where
      MANAGEMENT_COMPANY_ID=@CompanyID
else
   update
      PAYCOLL_PROPERTIES
   set
      PAYCOLL_DIV_ID_1 = @PaycollDivID_1,
      PAYCOLL_DIV_ID_2 = @PaycollDivID_2,
      BANK_ACCOUNT_ID_1= @BankAccountID_1,
      BANK_ACCOUNT_ID_2= @BankAccountID_2,
      ADDRESS_1        = @Address_1,
      ADDRESS_2        = @Address_2,
      PHONE_1          = @Phone_1,
      PHONE_2          = @Phone_2,
      PHONE_3          = @Phone_3,
      PHONE_4          = @Phone_4,
      FINGROUP_CODE    = @FingroupCode
   where
      TECHSUBDIV_ID=@CompanyID

----------------------------------------------------------------------------------------------------
$(COMMIT_TRANSACTION)
----------------------------------------------------------------------------------------------------
$(END)
/*================================================================================================*/
GO
grant exec on [$(Procedure)] to [$(ident_AuthorizedGroup)]
GO
