:SetVar Procedure "QuickInfo:GetAccountInfo"
exec sys_DropProcedure '$(Procedure)'
GO
create procedure [$(Procedure)] (@AccountId int)
AS
/*================================================================================================*/
/*
##
##     :
##
*/
/*================================================================================================*/
$(BEGIN)
----------------------------------------------------------------------------------------------------

if @AccountId is null
   raiserror ('   "$(Procedure)".',11,1)

----------------------------------------------------------------------------------------------------

declare
   @DeliveryRegionId    int,
   @DeliveryRegionName  $(dt_Name),
   @DeliveryRegion      $(dt_String),
   @PrintFintermId      int,
   @PrintFinterm        $(dt_String)

----------------------------------------------------------------------------------------------------

select
   @DeliveryRegionId=dc.REGION_ID
from
   OCCUPATIONS o  (nolock)
join
   FLATS f (nolock)
   on f.ID=o.FLAT_ID
join
   DELIVERY_CORRESPONDENCE dc (nolock)
   on dc.BLDN_ID=f.BLDN_ID
where
   o.ID=@AccountId

if @DeliveryRegionId is not null
begin
   set @DeliveryRegion=LTRIM(STR(@DeliveryRegionId))

   select @DeliveryRegionName=LTRIM(RTRIM(IsNull(NAME,'')))
   from DELIVERY_REGIONS (nolock)
   where ID=@DeliveryRegionId

   if DataLength(@DeliveryRegionName)>0
      set @DeliveryRegion=@DeliveryRegion+' ('+@DeliveryRegionName+')'
end
else
   set @DeliveryRegion='< >'

select
   o.INITIALS                                                     'INITIALS',
   RTRIM(s.NAME)+', .'+RTRIM(b.BLDN_NO)+', .'+RTRIM(f.FLAT_NO) 'ADDRESS',
   RTRIM(pd.NAME)+' ('+LTRIM(STR(pd.ID))+')'                      'GEO_DIVISION',
   RTRIM(d.NAME) +' ('+LTRIM(STR(d.ID ))+')'                      'DIVISION',
   RTRIM(t.NAME) +' ('+LTRIM(STR(t.ID ))+')'                      'SUBDIVISION',
   @DeliveryRegion                                                'DELIVERY_REGION'
from
   OCCUPATIONS o (nolock)
join
   FLATS f (nolock)
   on f.ID=o.FLAT_ID
join
   BUILDINGS b (nolock)
   on b.ID=f.BLDN_ID
join
   STREETS s (nolock)
   on s.ID=b.STREET_ID
join
   TECH_SUBDIVISIONS t (nolock)
   on t.ID=b.TECHSUBDIV_ID
join
   DIVISIONS d (nolock)
   on d.ID=t.DIV_ID
join
   POLITICAL_DIVISIONS pd (nolock)
   on pd.ID=b.PD_ID
where
   o.ID=@AccountId

----------------------------------------------------------------------------------------------------

select @PrintFintermId=MAX(ID) from FIN_TERMS (nolock) where CLOSED=1

if @PrintFintermId is null
   return 0

select
   @PrintFinterm = RTRIM(mn.NAME)+
      STR(DATEPART(year,START_DATE),5)+
      '. (-'+LTRIM(STR(@PrintFintermId))+')'
from
   FIN_TERMS ft (nolock)
join
   MONTH_NAMES mn (nolock)
   on mn.NO=DATEPART(month,ft.START_DATE)
where
   ft.ID=@PrintFintermId

select
   @PrintFinterm  'PRINT_FINTERM',
   PACK_NO        'PRINT_PACK_NO',
   ORIGINATED     'PRINT_PACK_DATE',
   TOTAL_TO_PAY   'PRINT_TOTAL_TO_PAY'
from
   PRINT_VERIFICATION (nolock)
where
   FINTERM_ID=@PrintFintermId and OCC_ID=@AccountId

----------------------------------------------------------------------------------------------------
$(END)
/*================================================================================================*/
GO
grant exec on [$(Procedure)] to [$(ident_AuthorizedGroup)]
GO
/*
exec [QuickInfo:GetAccountInfo] 123456789
*/
