:SetVar ProcBaseName "ListProcesses"
if exists( select * from [sysobjects] (nolock) where [type]='P' and [name]='$(SysPrefix)$(ProcBaseName)' )
   drop procedure [$(SysPrefix)$(ProcBaseName)]
GO
create procedure [$(SysPrefix)$(ProcBaseName)] (@DatabaseName $(dt_SysName)='*')
AS
/*================================================================================================*/
/* System Helpers; Author: Sergei Kitaev
----------------------------------------------------------------------------------------------------
##
## === SERVER PROCESSES LISTING UTILITY: ===
##
## Usage: exec <sysprefix>ListProcesses                  -- all processes
##        exec <sysprefix>ListProcesses '<DatabaseName>' -- specified database
##        exec <sysprefix>ListProcesses '.'              -- current database
##        exec <sysprefix>ListProcesses '*'              -- all processes
##        exec <sysprefix>ListProcesses null             -- processes without DB context
*/
/*================================================================================================*/
$(BEGIN)
----------------------------------------------------------------------------------------------------
declare @error_message $(dt_Message), @DatabaseID int
----------------------------------------------------------------------------------------------------

if @DatabaseName is not null
begin
   set @DatabaseName=RTRIM(@DatabaseName)
   if DATALENGTH(@DatabaseName)=0 or LTRIM(@DatabaseName)<>@DatabaseName
      raiserror ('Incorrect call of "$(SysPrefix)$(ProcBaseName)" procedure.',11,1)
end

----------------------------------------------------------------------------------------------------

if @DatabaseName is not null and @DatabaseName<>'*'
   if @DatabaseName='.'
      select
         @DatabaseID=db.[dbid]
      from
         [master]..[sysprocesses] p (nolock)
      join
         [master]..[sysdatabases] db (nolock)
         on p.[dbid]=db.[dbid] and
            p.[spid]=@@SPID
   else
   begin
      select @DatabaseID=[dbid] from [master]..[sysdatabases] (nolock) where [name]=@DatabaseName
      if @@rowcount=0
      begin
         set @error_message='Database not found by name: "'+@DatabaseName+'".'
         raiserror (@error_message,11,1)
      end
   end

----------------------------------------------------------------------------------------------------

select
   CONVERT( bit,
            case when p.[spid]=@@spid then 1 else 0 end )
                        'IsCallingProcess',
   ISNULL(db.[name],'') 'DatabaseName',
   p.[spid]             'SPID',
   case p.[status]
      when 'runnable' then UPPER(p.[status])
      else p.[status]
   end                  'Status',
   p.[cmd]              'Command',
   p.[cpu]              'CPU',
   p.[loginame]         'LoginName',
   p.[program_name]     'AppName',
   p.[hostname]         'Host',
   p.[login_time]       'LoginTime',
   p.[last_batch]       'LastBatch'
from
   [master]..[sysprocesses] p (nolock)
left outer join
   [master]..[sysdatabases] db (nolock)
   on p.[dbid]=db.[dbid]
where
   @DatabaseName='*' or
   ( @DatabaseName is not null and db.[dbid]=@DatabaseID ) or
   ( @DatabaseName is     null and db.[dbid] is null     )
order by
   case
      when db.[dbid] is null then '1'
      else '0'
   end,
   IsNull(db.[name],''),
   p.[spid]

----------------------------------------------------------------------------------------------------
$(END)
/*================================================================================================*/
GO
/*
exec sys_ListProcesses
exec sys_ListProcesses ''
exec sys_ListProcesses null
exec sys_ListProcesses 'Emerald'
exec sys_ListProcesses '*'
*/
