﻿using System;
using System.Threading;
using System.Globalization;
using System.Runtime.InteropServices;

using XHelpers;

namespace ConsoleCalculator {

//**************************************************************************************************
static partial class Program {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

// AUXILIARY FUNCTIONS:

static void PrintLogoInformation()
{
   Console.Clear();
   XConsole.PrintTitleHeader( false, true,
      "Simple Console Calculator (XHelpers Library) {0} coded by Sergei Kitaev",
      XConsole.LongDash );
}

//--------------------------------------------------------------------------------------------------

static void PrintHotKeysInfo()
{
   XConsole.PrintDialogKeysLegend
   (
      "Keys for operand input:",
      new string[]{ "Accept input", "Clear input field" },
      new ConsoleKey[]{ ConsoleKey.Enter, ConsoleKey.Backspace },
      new ConsoleModifiers[]{ 0, ConsoleModifiers.Control }
   );
   XConsole.PrintDialogKeysLegend
   (
      true,
      "Operation keys (on numeric keyboard, or their alternatives):",
      _OperationCaptions_,
      _MainOperationKeysArray_,
      _MainOperationModifiersArray_
   );
   XConsole.PrintDialogKeysLegend
   (
      true,
      "Everywhere in calculation routine:",
      new string[]{ "Abandon (cancel currently active procedure)" },
      new ConsoleKey[]{ ConsoleKey.Escape }
   );
}

//==================================================================================================

static void PrintRedefineLastOperation()
{
   XConsole.PrintFaint(true,"<< Redefine the operation: >>");
}

//--------------------------------------------------------------------------------------------------

static void ReportErrorRedefineOperand(string message)
{
   XConsole.ReportError( message );
   XConsole.PrintFaint("<< Redefine the operand: >>");
}

static void ReportErrorRedefineOperand(Exception e)
{
   ReportErrorRedefineOperand( XException.GetErrorMessage(e) );
}

//--------------------------------------------------------------------------------------------------

static void ReportErrorRedefineOperation(string message)
{
   XConsole.ReportError( message );
   PrintRedefineLastOperation();
}

static void ReportErrorRedefineOperation(Exception e)
{
   ReportErrorRedefineOperation( XException.GetErrorMessage(e) );
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // Program
//**************************************************************************************************

} // ConsoleCalculator
