﻿using System;
using System.Reflection;

using XHelpers;

namespace ConsoleCalculator {

//**************************************************************************************************
static partial class Program {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

/// <summary>
/// This is, in fact, the main function in this application.
/// </summary>
public static void Run()
{
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Displaying startup information:

   PrintLogoInformation();

   if (XProgram.HasEntryPointArguments)
      throw new XAppFailedException(
         "No command line arguments were expected." );

   PrintHotKeysInfo();

   XSound.Play( Sounds.Startup );

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// This cycle invokes new calculation many times, until the user will deside to leave the app.:

   for (;;)
   {
      XConsole.PrintDoubleSplitter(true,true);

      // Calling new calculation sequence routine:
      PerformCalculation();

      XConsole.PrintDoubleSplitter(true,true);

      XConsole.PrintFeaturedInscription(
         XConsole.FaintColor,
         "Calculation has been reset." );

      // Proceeding new calculation dialog:
      if (XConsole.PerformDialog(
         true,
         "Confirm proceeding another calculation:",
         new string[] { "New calculation", "Exit from the application" },
         new ConsoleKey[]{ ConsoleKey.Enter, ConsoleKey.X },
         new ConsoleModifiers[]{ 0, ConsoleModifiers.Alt }
      )==1)
         break;

      Console.Clear();
      PrintLogoInformation();
      PrintHotKeysInfo();
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Saying "Goodbye":

   XConsole.Print();
   XConsole.OutputExclamation( "Bye! " );
   XConsole.PrintFaint( "Good luck!" );

   XConsole.PrintTitleFooter( true,
      "{0} {1} {2}",
      Assembly.GetExecutingAssembly().GetName().Name,
      XConsole.LongDash,
      "OUT" );

   XSound.SyncPlay( Sounds.Exit );

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // Program
//**************************************************************************************************

} // ConsoleCalculator
