﻿using System;
using System.Threading;
using System.Reflection;
using System.Globalization;
using System.Configuration;

using XHelpers;

namespace ConsoleCalculator {

//**************************************************************************************************
/// <summary>
/// Application bootup class (it contains application entry point).
/// </summary>
public static class Startup {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

/// <summary>
/// Application entry point.
/// </summary>
public static int Main(string[] args)
{
   // Initialization (optional, but useful):
   XHelpersLibrary.Initialize();

   // Specifying runtime information:
   XProgram.SpecifyMainThread();
   XProgram.SpecifyInteractsWithDesktop( true );
   XFatalError.Flags = XFatalErrorFlags.ProcessOwnerDefaults;

   // The program (its main thread and UI) will be insensitive to the current system culture.
   // (Those formatting rules as we have in the EN-culture will be used.)
   // (Default UI-strings are in english.)
   Thread.CurrentThread.CurrentCulture =
      Thread.CurrentThread.CurrentUICulture =
         CultureInfo.InvariantCulture;

   // Installing XHelpers cancellation routine:
   XConsole.HandlesCancelEvent = true;
   // Disabling interruption by Ctrl+C (but Ctrl+Break is always enabled):
   Console.TreatControlCAsInput = true;

   // Sound initialization (optional, but useful):
   XSound.Initialize();

   // Setting console title:
   if (XAssembly.IsEntryAssembly())
      Console.Title = string.Format( "{0} {1} {2}",
         XProgram.SimpleName, XUnicode.EmDash,
         "based on XHelpers Library (Freeware)" );

   // Applying "Mute" key from "appSettings" section of ".exe.config"
   XSound.IsMute =
      ConfigurationManager.AppSettings[ "Mute" ] == "1";

   // Running the application:
   try
   {
      // Calling "Program.Run"-function (which is, actually, main function in this application):
      Program.Run();
   }
   catch (XAppFailedException e)
   {
   // Processing exception of a special type - XAppExitException (derived from XEscapeException):

      if (e is XAppFailedException)
         XConsole.ReportError( e );
      else
         XConsole.PrintExpressive( e.Message );
   // Performing pause before exit:
      XConsole.PerformExitionDialog(true);

   // Setting program exit code (which was until equal to zero):
      XProgram.ExitCode = e.ExitCode;
   }

   // Termination message:
   XConsole.PrintOperation(true,"Exiting from the program");

   // Completing execution (performing all pending finalizers)
   // and returning exit code to the system:
   return XProgram.PrepareToExit();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // Startup
//**************************************************************************************************

} // ConsoleCalculator
