﻿using System;
using System.Reflection;
using System.Configuration;

using XHelpers;

namespace PaycorMaster {

//**************************************************************************************************
public class AppSettings: XApplicationSettingsBase {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

[UserScopedSetting]
public bool StartMaximized
{
   get { return GetValue<bool>(MethodBase.GetCurrentMethod()); }
   set { SetValue(MethodBase.GetCurrentMethod(),value); }
}

//--------------------------------------------------------------------------------------------------

[UserScopedSetting]
[DefaultSettingValue("true")]
public bool UseBeeperAccompaniment
{
   get { return GetValue<bool>(MethodBase.GetCurrentMethod()); }
   set { SetValue(MethodBase.GetCurrentMethod(),value); }
}

//--------------------------------------------------------------------------------------------------

[UserScopedSetting]
public bool UseFluctuationEffect
{
   get { return GetValue<bool>(MethodBase.GetCurrentMethod()); }
   set { SetValue(MethodBase.GetCurrentMethod(),value); }
}

//==================================================================================================

public static readonly AppSettings
   Default = new AppSettings(true);

//--------------------------------------------------------------------------------------------------

public AppSettings(): this(false)
{}

public AppSettings(bool useDefaultValues)
{
   Constructor( useDefaultValues );
}

//--------------------------------------------------------------------------------------------------

public AppSettings Duplicate()
{
   return (AppSettings)MemberwiseClone();
}

//==================================================================================================

public bool IsDefault
{
   get { return Equals( Default ); }
}

//--------------------------------------------------------------------------------------------------

public override bool Equals(object obj)
{
   if (this==obj)
      return true;
   var settings = obj as AppSettings;
   if (settings==null)
      return false;
   return
      settings.StartMaximized         == StartMaximized         &&
      settings.UseBeeperAccompaniment == UseBeeperAccompaniment &&
      settings.UseFluctuationEffect   == UseFluctuationEffect;
}

public override int GetHashCode()
{
   return base.GetHashCode();
}

//==================================================================================================

protected override Type ActualType
{
   get { return typeof(AppSettings); }
}

//--------------------------------------------------------------------------------------------------

protected override void ProbeValues()
{
   ProbeValue( StartMaximized         );
   ProbeValue( UseBeeperAccompaniment );
   ProbeValue( UseFluctuationEffect   );
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // AppSettings
//**************************************************************************************************

} // PaycorMaster
