﻿using System;
using System.Collections;
using System.ComponentModel;
using System.Collections.ObjectModel;

using System.Windows;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Threading;
using System.Windows.Media;

using XHelpers;

namespace PaycorMaster {

//**************************************************************************************************
public class CustomListBox: ListBox {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public ListBoxItem SelectedListBoxItem
{
   get { return _LastSelectedListBoxItem_; }
}

//==================================================================================================

public void ArrangeBehaviour(Window parentWindow)
{
   XArgCheck.VerifyNotNull( parentWindow );
   parentWindow.LostKeyboardFocus -= ParentWindow_LostKeyboardFocus;
   parentWindow.LostKeyboardFocus += ParentWindow_LostKeyboardFocus;
}

//--------------------------------------------------------------------------------------------------

public void SetItemsSource(IEnumerable value)
{
   ItemsSource = value; SetSelectedIndex(0);
}

//--------------------------------------------------------------------------------------------------

public void RefreshView()
{
   ((ListCollectionView)CollectionViewSource.GetDefaultView( ItemsSource )).Refresh();
}

//--------------------------------------------------------------------------------------------------

public bool SetSelectedIndex(int value)
{
   _LastSelectedListBoxItem_ = null;
   SelectedIndex = Math.Min(value,Items.Count-1);
   UpdateLayout(); FocusLastSelectedItem();
   return ScrollSelectedItemIntoView();
}

public void SetSelectedItem(object value)
{
   SelectedItem = value; ScrollSelectedItemIntoView();
}

public bool ScrollSelectedItemIntoView()
{
   if (SelectedItem!=null)
   {
      ScrollIntoView( SelectedItem );
      return true;
   }
   return false;
}

//--------------------------------------------------------------------------------------------------

public void AddAndSelectItem(object value)
{
   ((IList)ItemsSource).Add( value );
   SetSelectedItem( value );
}

public bool RemoveSelectedItem()
{
   if (SelectedItem==null)
      return false;

   int selectedIndex = SelectedIndex;

   _IgnoreFocusOnSelect_ = true;
   ((IList)ItemsSource).Remove( SelectedItem );
   SetSelectedIndex( selectedIndex );
   _IgnoreFocusOnSelect_ = false;

   return true;
}

//--------------------------------------------------------------------------------------------------

public bool IsChildListBoxItem(ListBoxItem lbi)
{
   for ( DependencyObject element=lbi; element!=null;
         element=VisualTreeHelper.GetParent(element) )
      if (element==this)
         return true;
   return false;
}

public bool IsItemMouseClick(MouseButtonEventArgs e)
{
   var element = e.OriginalSource as FrameworkElement;

   return element!=null && (
      element.TemplatedParent is ListBoxItem ||
      element.TemplatedParent is ContentPresenter );
}

//==================================================================================================

protected override void OnInitialized(EventArgs e)
{
   base.OnInitialized(e);

   if (!DesignerProperties.GetIsInDesignMode(this))
   {
      var parentWindow = Window.GetWindow(this);
      if (parentWindow!=null)
         ArrangeBehaviour( parentWindow );
      AddHandler(
         ListBoxItem.SelectedEvent,
         new RoutedEventHandler( ListBoxItem_Selected ),
         true );
   }
}

protected override void OnPreviewMouseDown(MouseButtonEventArgs e)
{
   base.OnPreviewMouseDown(e);
   if (e.Handled) return;
   if (IsItemMouseClick(e) && !IsKeyboardFocusWithin)
      _IsInItemMouseDownFocusing_ = true;
}

protected override void OnGotKeyboardFocus(KeyboardFocusChangedEventArgs e)
{
   base.OnGotKeyboardFocus(e);
   if (e.Handled) return;

   if (XOps.DefaultExchange(ref _IsInItemMouseDownFocusing_))
      return;

   if (e.NewFocus is ListBoxItem)
      SelectedItem = ItemContainerGenerator.ItemFromContainer( (ListBoxItem)e.NewFocus );
}

//==================================================================================================

void FocusLastSelectedItem()
{
   if (_LastSelectedListBoxItem_!=null && IsKeyboardFocusWithin)
      _LastSelectedListBoxItem_.Focus();
}

void ListBoxItem_Selected(object sender,RoutedEventArgs e)
{
   _LastSelectedListBoxItem_ = (ListBoxItem)e.OriginalSource;
   if (!_IgnoreFocusOnSelect_)
      FocusLastSelectedItem();
}

void ParentWindow_LostKeyboardFocus(object sender,KeyboardFocusChangedEventArgs e)
{
   if ( !_IsInItemMouseDownFocusing_ &&
        ( e.NewFocus==this || e.OldFocus is ListBox ||
          !IsChildListBoxItem(e.OldFocus as ListBoxItem) ) )
      FocusLastSelectedItem();
}

//==================================================================================================

ListBoxItem
   _LastSelectedListBoxItem_;
bool
   _IgnoreFocusOnSelect_,
   _IsInItemMouseDownFocusing_;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // CustomListBox
//**************************************************************************************************

} // PaycorMaster
