﻿using System;
using System.ComponentModel;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;

using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Threading;

using XHelpers;

namespace PaycorMaster {

//**************************************************************************************************
partial class ConfigurationPage {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void CreateCause()
{
   _SelectedListBox_.ScrollSelectedItemIntoView();
   CorrectionCauseDocTypeWindow.PerformCreateCauseDialog
   (
      listboxExcludedCauses,
      delegate (CorrectionCause cause)
      {
         XDispatcher.Synchronize(delegate{
            listboxExcludedCauses.AddAndSelectItem( cause );
            listboxExcludedCauses.Focus();
         });
         if (LoadDocumentTypes())
            XSound.PlaySuccess();
      }
   );
}

//--------------------------------------------------------------------------------------------------

void EditCause()
{
   _SelectedListBox_.ScrollSelectedItemIntoView();
   CorrectionCauseDocTypeWindow.PerformEditCauseDialog
   (
      _SelectedListBox_,
      SelectedCause( _SelectedListBox_ ),
      delegate (CorrectionCause cause)
      {
         XDispatcher.Synchronize(delegate{
            _SelectedListBox_.RefreshView();
            _SelectedListBox_.SetSelectedItem( cause );
         });
         XSound.PlaySuccess();
      }
   );
}

//--------------------------------------------------------------------------------------------------

void RemoveCause()
{
   _SelectedListBox_.ScrollSelectedItemIntoView();
   if (XMessageBox.ShowExclamation(
         "Подтверждаете удаление причины корректировки?",
         MessageBoxButton.OKCancel ) == MessageBoxResult.OK )
   {
      var causeId =
         SelectedCause( _SelectedListBox_ ).Id;
      XBackgroundOperation.Start
      (
         ConfigurationCommands.RemoveCause.Text,
         delegate
         {
            int
               errorCode;
            string
               errorMessage = null;
            try
            {
               errorCode = App.Database.PaycorMaster_RemoveCause(
                  causeId, ref errorMessage );
            }
            catch (Exception x)
            {
               App.CapitulateDbConnection(x);
               return;
            }
            if (errorCode!=0)
            {
               XAppWaiting.ReportError(errorMessage);
               return;
            }
            XDispatcher.Synchronize(delegate{
               _SelectedListBox_.RemoveSelectedItem(); });
            if (LoadDocumentTypes())
               XSound.PlaySuccess();
         }
      );
   }
}

//==================================================================================================

void CreateDocumentType()
{
   _SelectedListBox_.ScrollSelectedItemIntoView();
   CorrectionCauseDocTypeWindow.PerformCreateDocumentTypeDialog
   (
      listboxUnboundDocumentTypes,
      delegate (CorrectionDocumentType documentType)
      {
         XDispatcher.Synchronize(delegate{
            listboxUnboundDocumentTypes.AddAndSelectItem( documentType );
            listboxUnboundDocumentTypes.Focus();
         });
         XSound.PlaySuccess();
      }
   );
}

//--------------------------------------------------------------------------------------------------

void EditDocumentType()
{
   _SelectedListBox_.ScrollSelectedItemIntoView();
   CorrectionCauseDocTypeWindow.PerformEditDocumentTypeDialog
   (
      _SelectedListBox_,
      SelectedDocumentType( _SelectedListBox_ ),
      delegate (CorrectionDocumentType documentType)
      {
         XDispatcher.Synchronize(delegate{
            _SelectedListBox_.RefreshView();
            _SelectedListBox_.SetSelectedItem( documentType );
         });
         XSound.PlaySuccess();
      }
   );
}

//--------------------------------------------------------------------------------------------------

void RemoveDocumentType()
{
   _SelectedListBox_.ScrollSelectedItemIntoView();
   if (XMessageBox.ShowExclamation(
         "Подтверждаете удаление типа документа корректировки?",
         MessageBoxButton.OKCancel ) == MessageBoxResult.OK )
   {
      var documentTypeId =
         SelectedDocumentType( _SelectedListBox_ ).Id;
      XBackgroundOperation.Start
      (
         ConfigurationCommands.RemoveDocumentType.Text,
         delegate
         {
            int
               errorCode;
            string
               errorMessage = null;
            try
            {
               errorCode = App.Database.PaycorMaster_RemoveDocumentType(
                  documentTypeId, ref errorMessage );
            }
            catch (Exception x)
            {
               App.CapitulateDbConnection(x);
               return;
            }
            if (errorCode!=0)
            {
               XAppWaiting.ReportError(errorMessage);
               return;
            }
            XDispatcher.Synchronize(delegate{
               _SelectedListBox_.RemoveSelectedItem(); });
            XSound.PlaySuccess();
         }
      );
   }
}

//==================================================================================================

void IncludeCause()
{
   var
      cause = SelectedCause( listboxExcludedCauses );
   bool?
      percentalParameter = checkboxPercental.IsChecked,
      affectsUnitsParameter = checkboxAffectsUnits.IsChecked;
   XBackgroundOperation.Start
   (
      ConfigurationCommands.IncludeCause.Text,
      delegate
      {
         int
            errorCode;
         string
            errorMessage = null;
         try
         {
            errorCode = App.Database.PaycorMaster_IncludeCause
            (
               cause.Id,
               percentalParameter,
               affectsUnitsParameter,
               ref errorMessage
            );
         }
         catch (Exception x)
         {
            App.CapitulateDbConnection(x);
            return;
         }
         if (errorCode!=0)
         {
            XAppWaiting.ReportError(errorMessage);
            return;
         }
         XDispatcher.Synchronize(delegate{
            listboxExcludedCauses.RemoveSelectedItem();
            CausesListBoxItemsSource( listboxIncludedCauses ).Add( cause );
            listboxIncludedCauses.SetSelectedItem( cause );
         });
         XSound.PlaySuccess();
      }
   );
}

//--------------------------------------------------------------------------------------------------

void ExcludeCause()
{
   var
      cause = SelectedCause( listboxIncludedCauses );
   bool?
      percentalParameter = checkboxPercental.IsChecked,
      affectsUnitsParameter = checkboxAffectsUnits.IsChecked;
   XBackgroundOperation.Start
   (
      ConfigurationCommands.ExcludeCause.Text,
      delegate
      {
         int
            errorCode;
         string
            errorMessage = null;
         try
         {
            errorCode = App.Database.PaycorMaster_ExcludeCause
            (
               cause.Id,
               percentalParameter,
               affectsUnitsParameter,
               ref errorMessage
            );
         }
         catch (Exception x)
         {
            App.CapitulateDbConnection(x);
         }
         if (errorMessage!=null)
         {
            XAppWaiting.ReportError(errorMessage);
            return;
         }
         XDispatcher.Synchronize(delegate{
            listboxIncludedCauses.RemoveSelectedItem();
            CausesListBoxItemsSource( listboxExcludedCauses ).Add( cause );
            listboxExcludedCauses.SetSelectedItem( cause );
         });
         XSound.PlaySuccess();
      }
   );
}

//==================================================================================================

void BindDocumentType()
{
   var documentType =
      SelectedDocumentType( listboxUnboundDocumentTypes );
   XBackgroundOperation.Start
   (
      ConfigurationCommands.BindDocumentType.Text,
      delegate
      {
         int
            errorCode;
         string
            errorMessage = null;
         try
         {
            errorCode = App.Database.PaycorMaster_BindDocumentType
            (
               _ActiveCause_.Id, documentType.Id, ref errorMessage
            );
         }
         catch (Exception x)
         {
            App.CapitulateDbConnection(x);
            return;
         }
         if (errorCode!=0)
         {
            XAppWaiting.ReportError(errorMessage);
            return;
         }
         XDispatcher.Synchronize(delegate{
            listboxUnboundDocumentTypes.RemoveSelectedItem();
            DocumentsListBoxItemsSource( listboxBoundDocumentTypes ).Add( documentType );
            listboxBoundDocumentTypes.SetSelectedItem( documentType );
         });
         XSound.PlaySuccess();
      }
   );
}

//--------------------------------------------------------------------------------------------------

void UnbindDocumentType()
{
   var documentType =
      SelectedDocumentType( listboxBoundDocumentTypes );
   XBackgroundOperation.Start
   (
      ConfigurationCommands.UnbindDocumentType.Text,
      delegate
      {
         int
            errorCode;
         string
            errorMessage = null;
         try
         {
            errorCode = App.Database.PaycorMaster_UnbindDocumentType
            (
               _ActiveCause_.Id, documentType.Id, ref errorMessage
            );
         }
         catch (Exception x)
         {
            App.CapitulateDbConnection(x);
            return;
         }
         if (errorCode!=0)
         {
            XAppWaiting.ReportError(errorMessage);
            return;
         }
         XDispatcher.Synchronize(delegate{
            listboxBoundDocumentTypes.RemoveSelectedItem();
            DocumentsListBoxItemsSource( listboxUnboundDocumentTypes ).Add( documentType );
            listboxUnboundDocumentTypes.SetSelectedItem( documentType );
         });
         XSound.PlaySuccess();
      }
   );
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // ConfigurationPage
//**************************************************************************************************

} // PaycorMaster
