﻿using System;
using System.ComponentModel;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;

using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Threading;

using XHelpers;

namespace PaycorMaster {

//**************************************************************************************************
partial class ConfigurationPage {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

bool LoadCauses()
{
//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   var includedCauses = new ObservableCollection<CorrectionCause>();
   var excludedCauses = new ObservableCollection<CorrectionCause>();

   bool?
      percentalParameter = null,
      affectsUnitsParameter = null;

   XDispatcher.Synchronize(delegate{
      if (!_IsInitialized_)
         ActivateListBox(listboxIncludedCauses,false);
      percentalParameter = checkboxPercental.IsChecked;
      affectsUnitsParameter = checkboxAffectsUnits.IsChecked;
   });

//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   XAppWaiting.DisplayStatus( _LoadingCausesCaption_ );

   try
   {
//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      var includedCausesResult = App.Database.PaycorMaster_GetActiveCauses(
         percentalParameter, affectsUnitsParameter );
      XDispatcher.Synchronize(delegate{
         foreach (var resultRow in includedCausesResult)
               includedCauses.Add(new CorrectionCause(
                  resultRow.Id, resultRow.Name, resultRow.CountersRelated ));
      });
//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      var excludedCausesResult = App.Database.PaycorMaster_GetCandidateCauses(
         percentalParameter, affectsUnitsParameter );
      XDispatcher.Synchronize(delegate{
         foreach (var resultRow in excludedCausesResult)
               excludedCauses.Add(new CorrectionCause(
                  resultRow.Id, resultRow.Name, resultRow.CountersRelated ));
      });
//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   }
   catch (Exception e)
   {
      return ShowErrorOrCapitulate(e);
   }

   XDispatcher.Synchronize(delegate{
      SetListBoxItemsSource( listboxIncludedCauses, includedCauses );
      SetListBoxItemsSource( listboxExcludedCauses, excludedCauses );
   });

//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   XAppWaiting.DisplayStatus( _LoadingDocumentTypesCaption_ );

   if (!LoadDocumentTypes())
      return false;

//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   XAppWaiting.RevertStatus();

   return true;

//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}

//==================================================================================================

bool PreloadDocumentTypes()
{
   if (_CausesActiveListBox_.SelectedItem==_ActiveCause_)
      return true;
   if (_CausesActiveListBox_.SelectedItem==null)
   {
      listboxBoundDocumentTypes.ItemsSource =
      listboxUnboundDocumentTypes.ItemsSource = null;
      _ActiveCause_ = null;
      return true;
   }
   if ( (CorrectionCause)_CausesActiveListBox_.SelectedItem == _ActiveCause_ )
      return true;
   return false;
}

//==================================================================================================

bool LoadDocumentTypes()
{
//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   if (XDispatcher.Synchronize(
      (XResultDelegate<bool>)delegate
      {
         if (PreloadDocumentTypes())
            return true;
         _ActiveCause_ = (CorrectionCause)_CausesActiveListBox_.SelectedItem;
         return false;
      }
   ))
      return true;
//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   using (new XAppWaiting(_LoadingDocumentTypesCaption_))
   try
   {
//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      var boundDocumentTypes =
         new ObservableCollection<CorrectionDocumentType>();
      var boundDocumentTypesResult =
         App.Database.PaycorMaster_GetBoundDocumentTypes( _ActiveCause_.Id );
      XDispatcher.Synchronize(delegate{
         foreach (var resultRow in boundDocumentTypesResult)
            boundDocumentTypes.Add(new CorrectionDocumentType(
               resultRow.Id, resultRow.Name ));
         SetListBoxItemsSource( listboxBoundDocumentTypes, boundDocumentTypes );
      });
//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      var unboundDocumentTypes =
         new ObservableCollection<CorrectionDocumentType>();
      var unboundDocumentTypesResult =
         App.Database.PaycorMaster_GetUnboundDocumentTypes( _ActiveCause_.Id );
      XDispatcher.Synchronize(delegate{
         foreach (var resultRow in unboundDocumentTypesResult)
            unboundDocumentTypes.Add(new CorrectionDocumentType(
               resultRow.Id, resultRow.Name ));
         SetListBoxItemsSource( listboxUnboundDocumentTypes, unboundDocumentTypes );
      });
//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   }
   catch (Exception x)
   {
      return ShowErrorOrCapitulate(x);
   }
//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   return true;
//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // ConfigurationPage
//**************************************************************************************************

} // PaycorMaster
