﻿using System;
using System.ComponentModel;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;

using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Threading;

using XHelpers;

namespace PaycorMaster {

//**************************************************************************************************
partial class ConfigurationPage {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static readonly DependencyProperty
   IsActiveProperty,
   CreateObjectCommandProperty,
   EditObjectCommandProperty,
   RemoveObjectCommandProperty,
   IncludeItemCommandProperty,
   ExcludeItemCommandProperty,
   ListBoxItemMenuProperty;

//==================================================================================================

public ConfigurationPage()
{
   InitializeComponent();
}

//==================================================================================================

public static void OpenNew()
{
   XBackgroundOperation.Start
   (
      _LoadingDataCaption_,
      delegate
      {
         var page = XDispatcher.Synchronize(
            (XResultDelegate<ConfigurationPage>)delegate {
               return new ConfigurationPage(); } );
         if (page.LoadCauses())
            XDispatcher.Synchronize(delegate{
               App.Navigate( page ); });
      }
   );
}

//==================================================================================================

static ConfigurationPage()
{
   IsActiveProperty = DependencyProperty.Register(
      "IsActive", typeof(bool), typeof(ConfigurationPage) );

   CreateObjectCommandProperty = DependencyProperty.Register(
      "CreateObjectCommand", typeof(RoutedUICommand), typeof(ConfigurationPage) );
   EditObjectCommandProperty = DependencyProperty.Register(
      "EditObjectCommand", typeof(RoutedUICommand), typeof(ConfigurationPage) );
   RemoveObjectCommandProperty = DependencyProperty.Register(
      "RemoveObjectCommand", typeof(RoutedUICommand), typeof(ConfigurationPage) );

   IncludeItemCommandProperty = DependencyProperty.Register(
      "IncludeItemCommand", typeof(RoutedUICommand), typeof(ConfigurationPage) );
   ExcludeItemCommandProperty = DependencyProperty.Register(
      "ExcludeItemCommand", typeof(RoutedUICommand), typeof(ConfigurationPage) );

   ListBoxItemMenuProperty = DependencyProperty.Register(
      "ListBoxItemMenu", typeof(ContextMenu), typeof(ConfigurationPage) );
}

//--------------------------------------------------------------------------------------------------

ObservableCollection<CorrectionCause> CausesListBoxItemsSource(ListBox listBox)
{
   return (ObservableCollection<CorrectionCause>)listBox.ItemsSource;
}

ObservableCollection<CorrectionDocumentType> DocumentsListBoxItemsSource(ListBox listBox)
{
   return (ObservableCollection<CorrectionDocumentType>)listBox.ItemsSource;
}

//--------------------------------------------------------------------------------------------------

CorrectionCause SelectedCause(ListBox listBox)
{
   return (CorrectionCause)listBox.SelectedItem;
}

CorrectionDocumentType SelectedDocumentType(ListBox listBox)
{
   return (CorrectionDocumentType)listBox.SelectedItem;
}

//--------------------------------------------------------------------------------------------------

static void SetListBoxItemsSource(CustomListBox listBox,IEnumerable collection)
{
   var view = (ListCollectionView)CollectionViewSource.GetDefaultView( collection );
   view.SortDescriptions.Add(
      new SortDescription("Name",ListSortDirection.Ascending) );
   listBox.SetItemsSource( collection );
}

//--------------------------------------------------------------------------------------------------

bool IsCauseListBox(ListBox listBox)
{
   return XOps.IsOneOf(listBox,listboxIncludedCauses,listboxExcludedCauses);
}

bool IsDocumentTypeListBox(ListBox listBox)
{
   return XOps.IsOneOf(listBox,listboxBoundDocumentTypes,listboxUnboundDocumentTypes);
}

//--------------------------------------------------------------------------------------------------

bool IsListBoxSelectedWithItem(CustomListBox listBox)
{
   return listBox==_SelectedListBox_ && listBox.SelectedItem!=null;
}

void SelectListBox(CustomListBox listBox)
{
   ActivateListBox(listBox,true);
}

void ActivateListBox(CustomListBox listBox,bool select)
{
   CustomListBox
      previousCausesActiveListBox = null;
   if (IsCauseListBox(listBox))
      previousCausesActiveListBox = XOps.Exchange(ref _CausesActiveListBox_,listBox);

   var previousSelectedListFox = _SelectedListBox_;
   _SelectedListBox_ = select ? listBox : null;

   if ( previousSelectedListFox != null &&
        previousSelectedListFox != listBox )
      if ( !IsCauseListBox( listBox ) &&
           previousSelectedListFox == _CausesActiveListBox_ )
         SetListBoxActive( previousSelectedListFox );
      else
         SetListBoxInactive( previousSelectedListFox );

   if ( IsCauseListBox(listBox) &&
        previousCausesActiveListBox != null &&
        previousCausesActiveListBox != listBox )
      SetListBoxInactive( previousCausesActiveListBox );

   if (select)
      SetListBoxSelected( listBox );
   else
      SetListBoxActive( listBox );
}

//--------------------------------------------------------------------------------------------------

void SetListBoxSelected(ListBox listBox)
{
   listBox.SetValue(Selector.IsSelectedProperty,true);
   SetListBoxBorderStyle(listBox,"SelectedListBoxBorderStyle");
}

void SetListBoxActive(ListBox listBox)
{
   listBox.SetValue(IsActiveProperty,true);
   listBox.ClearValue(Selector.IsSelectedProperty);
   SetListBoxBorderStyle(listBox,"ActiveListBoxBorderStyle");
}

void SetListBoxInactive(ListBox listBox)
{
   listBox.ClearValue(IsActiveProperty);
   listBox.ClearValue(Selector.IsSelectedProperty);
   SetListBoxBorderStyle(listBox,"InactiveListBoxBorderStyle");
}

//--------------------------------------------------------------------------------------------------

void SetListBoxBorderStyle(ListBox listBox,string styleName)
{
   var style = (Style)Resources[styleName];
   var border = ListBoxBorder(listBox);
   if ( border.Style != style )
      border.Style = style;
}

//--------------------------------------------------------------------------------------------------

Border ListBoxBorder(ListBox listBox)
{
   if (listBox==listboxIncludedCauses)
      return borderActiveCauses;
   if (listBox==listboxExcludedCauses)
      return borderCandidateCauses;
   if (listBox==listboxBoundDocumentTypes)
      return borderBoundDocumentTypes;
   if (listBox==listboxUnboundDocumentTypes)
      return borderUnboundDocumentTypes;
   throw XArgCheck.ArgumentException_IncorrectArguments;
}

//--------------------------------------------------------------------------------------------------

RoutedUICommand GetListBoxCommand(ListBox listBox,DependencyProperty property)
{
   return (RoutedUICommand)listBox.GetValue( property );
}

ContextMenu GetListBoxItemMenu(ListBox listBox)
{
   return (ContextMenu)listBox.GetValue( ListBoxItemMenuProperty );
}

//--------------------------------------------------------------------------------------------------

void BuildListBoxMenus(ListBox listBox)
{
   RoutedCommand createObjectCommand = null;
   if (XOps.IsOneOf(listBox,listboxExcludedCauses,listboxUnboundDocumentTypes))
      createObjectCommand = GetListBoxCommand( listBox, CreateObjectCommandProperty );

   var menu = new ContextMenu();
   if (createObjectCommand!=null)
      XMenu.AppendCommand(menu,createObjectCommand);
   listBox.ContextMenu = menu.Items.Count>0 ? menu : null;

   menu = new ContextMenu();
   if (createObjectCommand!=null)
      XMenu.AppendCommand(menu,createObjectCommand);
   XMenu.AppendCommand(menu,GetListBoxCommand( listBox, EditObjectCommandProperty   ),true);
   XMenu.AppendCommand(menu,GetListBoxCommand( listBox, RemoveObjectCommandProperty )     );
   XMenu.AppendCommand(menu,GetListBoxCommand( listBox, ExcludeItemCommandProperty  ),true);
   listBox.SetValue(ListBoxItemMenuProperty,menu);
}

//--------------------------------------------------------------------------------------------------

void UpdateMainMenuContextItem()
{
   var tabItem = (TabItem)tabcontrol.SelectedItem;
   MenuItem menuItem = null;
   if (tabItem == tabitemCausesDocTypes)
      menuItem = (MenuItem)Resources["CausesDocTypesMainMenuContextItem"];
   MainWindow.SetMainMenuContextItem( menuItem );
}

//==================================================================================================

bool ShowErrorOrCapitulate(Exception e)
{
   return _IsInitialized_ ?
      App.CapitulateDbConnection(e) :
      XAppWaiting.ReportError(e);
}

//==================================================================================================

void ForceAttachListBoxMouseEventHandlers(ListBox listBox)
{
   listBox.AddHandler(
      UIElement.MouseDownEvent,
      new MouseButtonEventHandler( listboxCausesDocTypes_MouseDown ),
      true );
   listBox.AddHandler(
      UIElement.MouseUpEvent,
      new MouseButtonEventHandler( listboxCausesDocTypes_MouseUp ),
      true );
}

//==================================================================================================

void listboxCausesDocTypes_MouseDown(object sender,MouseButtonEventArgs e)
{
   _ClickHoldListBoxItem_ = null;

   var listBox = (CustomListBox)sender;
   var lbi = listBox.SelectedListBoxItem;

   if (!listBox.IsKeyboardFocusWithin)
      listBox.Focus();

   if (!(
      listBox==_SelectedListBox_ &&
      listBox.SelectedItem!=null &&
      lbi!=null && listBox.IsItemMouseClick(e)
   ))
      return;

   if (XInput.IsSoleLeftMouseButtonSingleClick(e))
   {
      _ClickHoldListBoxItem_ = lbi;
      return;
   }

   if (XInput.IsSoleLeftMouseButtonDoubleClick(e))
   {
      XCommand.TryExecute(GetListBoxCommand(
         listBox, EditObjectCommandProperty ));
      return;
   }
}

//--------------------------------------------------------------------------------------------------

void listboxCausesDocTypes_MouseUp(object sender,MouseButtonEventArgs e)
{
   var listBox = (CustomListBox)sender;
   var lbi = listBox.SelectedListBoxItem;

   if (!listBox.IsKeyboardFocusWithin)
      listBox.Focus();

   if (!(
      listBox==_SelectedListBox_ &&
      listBox.SelectedItem!=null &&
      lbi!=null && listBox.IsItemMouseClick(e)
   ))
      return;

   if (XInput.IsSoleRightMouseButtonUp(e))
   {
      ContextMenuService.SetPlacement(lbi,PlacementMode.MousePoint);
      return;
   }
}

//==================================================================================================

const string
   _LoadingDataCaption_          = "Загрузка конфигурационных данных",
   _LoadingCausesCaption_        = "Загрузка причин корректировки",
   _LoadingDocumentTypesCaption_ = "Загрузка типов документов корректировки";

//--------------------------------------------------------------------------------------------------

bool
   _IsInitialized_;
CustomListBox
   _SelectedListBox_,
   _CausesActiveListBox_;
CorrectionCause
   _ActiveCause_;
ListBoxItem
   _ClickHoldListBoxItem_;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // ConfigurationPage
//**************************************************************************************************

} // PaycorMaster
