﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;

using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using XHelpers;

namespace PaycorMaster {

//**************************************************************************************************
public partial class CorrectionCauseDocTypeWindow: CustomWindow {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

CorrectionCauseDocTypeDialogData _DialogData_;

//--------------------------------------------------------------------------------------------------

public CorrectionCauseDocTypeWindow()
{
   XCursor.SetWait();
   InitializeComponent();
}

//==================================================================================================

public static void PerformCreateCauseDialog( UIElement centerByElement,
                                             CorrectionCauseCallback posteriourCallback )
{
   PerformDialog( centerByElement,
                  CorrectionCauseDocTypeDialogKind.Cause,
                  null, false,
                  posteriourCallback );
}

public static void PerformEditCauseDialog( UIElement centerByElement,
                                           CorrectionCause cause,
                                           CorrectionCauseCallback posteriourCallback )
{
   PerformDialog( centerByElement,
                  CorrectionCauseDocTypeDialogKind.Cause,
                  cause, cause.IsCountersRelated,
                  posteriourCallback );
}

//--------------------------------------------------------------------------------------------------

public static void PerformCreateDocumentTypeDialog( UIElement centerByElement,
                                                    CorrectionDocTypeCallback posteriourCallback )
{
   PerformDialog( centerByElement,
                  CorrectionCauseDocTypeDialogKind.DocumentType,
                  null, false, posteriourCallback );
}

public static void PerformEditDocumentTypeDialog( UIElement centerByElement,
                                                  CorrectionDocumentType documentType,
                                                  CorrectionDocTypeCallback posteriourCallback )
{
   PerformDialog( centerByElement,
                  CorrectionCauseDocTypeDialogKind.DocumentType,
                  documentType, false,
                  posteriourCallback );
}

//==================================================================================================

static void PerformDialog( UIElement centerByElement,
                           CorrectionCauseDocTypeDialogKind dialogKind,
                           object theObject, bool isCauseCountersRelated,
                           object posteriourCallback )
{
   var dd = new CorrectionCauseDocTypeDialogData();

   dd.CenterByElement = centerByElement;

   dd.TheObject = theObject;

   dd.DialogKind = dialogKind;
   dd.IsCreationDialog = theObject==null;

   if (!dd.IsCreationDialog)
   {
      if (dialogKind==CorrectionCauseDocTypeDialogKind.Cause)
      {
         dd.ObjectId   = ((CorrectionCause)theObject).Id;
         dd.ObjectName = ((CorrectionCause)theObject).Name;
      }
      if (dialogKind==CorrectionCauseDocTypeDialogKind.DocumentType)
      {
         dd.ObjectId   = ((CorrectionDocumentType)theObject).Id;
         dd.ObjectName = ((CorrectionDocumentType)theObject).Name;
      }
   }
   if (dialogKind==CorrectionCauseDocTypeDialogKind.Cause)
      dd.IsCauseCountersRelated = isCauseCountersRelated;

   dd.PosteriourCallback = posteriourCallback;

   var window = new CorrectionCauseDocTypeWindow();
   window._DialogData_ = dd;
   window.Owner = App.MainWindow;

   window.ShowModal();
}

//--------------------------------------------------------------------------------------------------

void EnableControls()
{
   buttonAccept.IsEnabled =
      !XString.IsBlank(textboxObjectName.Text) &&
      !Validation.GetHasError(textboxObjectName);
}

//--------------------------------------------------------------------------------------------------

void BackgroundOperation()
{
//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   var
      dd = _DialogData_;
   int
      errorCode;
   string
      errorMessage = null;
//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   try
   {
      switch (dd.DialogKind)
      {
      case CorrectionCauseDocTypeDialogKind.Cause:
         errorCode = dd.IsCreationDialog ?
            App.Database.PaycorMaster_AddCause(
               ref dd.ObjectId,
               dd.ObjectName,
               dd.IsCauseCountersRelated,
               ref errorMessage ) :
            App.Database.PaycorMaster_ModifyCause(
               dd.ObjectId,
               dd.ObjectName,
               dd.IsCauseCountersRelated,
               ref errorMessage );
         break;
      case CorrectionCauseDocTypeDialogKind.DocumentType:
         errorCode = dd.IsCreationDialog ?
            App.Database.PaycorMaster_AddDocumentType(
               ref dd.ObjectId,
               dd.ObjectName,
               ref errorMessage ) :
            App.Database.PaycorMaster_ModifyDocumentType(
               dd.ObjectId,
               dd.ObjectName,
               ref errorMessage );
         break;
      default:
         throw new XDeadCodeBranchException();
      }
   }
   catch (Exception x)
   {
      DialogResult = false;
      App.CapitulateDbConnection(x);
      return;
   }
//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   if (errorCode!=0)
   {
      XAppWaiting.ReportError(errorMessage);
      return;
   }
//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   DialogResult = true;
//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   switch (dd.DialogKind)
   {
   case CorrectionCauseDocTypeDialogKind.Cause:
      var causeCallback =
         (CorrectionCauseCallback)dd.PosteriourCallback;
      if (dd.IsCreationDialog)
         causeCallback(
            new CorrectionCause(
               dd.ObjectId.Value, dd.ObjectName, dd.IsCauseCountersRelated.Value ));
      else
      {
         var cause = (CorrectionCause)dd.TheObject;
         cause.Name = dd.ObjectName;
         cause.IsCountersRelated = dd.IsCauseCountersRelated.Value;
         causeCallback( cause );
      }
      break;
   case CorrectionCauseDocTypeDialogKind.DocumentType:
      var doctypeCallback =
         (CorrectionDocTypeCallback)dd.PosteriourCallback;
      if (dd.IsCreationDialog)
         doctypeCallback( new CorrectionDocumentType( dd.ObjectId.Value, dd.ObjectName ) );
      else
      {
         var documentType = (CorrectionDocumentType)dd.TheObject;
         documentType.Name = dd.ObjectName;
         doctypeCallback( documentType );
      }
      break;
   }
//-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}

//==================================================================================================

private void Window_Loaded(object sender,RoutedEventArgs e)
{
   var dd = _DialogData_;

   switch (dd.DialogKind)
   {
   case CorrectionCauseDocTypeDialogKind.Cause:
      Title = dd.IsCreationDialog ?
         "Заведение новой причины корректировки (как неподключенной)" :
         ConfigurationCommands.EditCause.Text;
      buttonAccept.Content = dd.IsCreationDialog ?
         "Созд_ать новую причину" :
         "Измен_ить причину";
      checkboxIsCountersRelatedCause.Visibility =
         Visibility.Visible;
      break;
   case CorrectionCauseDocTypeDialogKind.DocumentType:
      Title = dd.IsCreationDialog ?
         "Заведение нового типа документа корректировки (как непривязанного)" :
         ConfigurationCommands.EditDocumentType.Text;
      buttonAccept.Content = dd.IsCreationDialog ?
         "Созд_ать новый тип документа" :
         "Измен_ить тип документа";
      break;
   }

   if (dd.ObjectName!=null)
      textboxObjectName.Text = dd.ObjectName;
   checkboxIsCountersRelatedCause.IsChecked =
      dd.IsCauseCountersRelated;

   Center( dd.CenterByElement );

   buttonAccept.IsEnabled = false;

   textboxObjectName.Focus();

   XCursor.Reset();
}

//--------------------------------------------------------------------------------------------------

private void textboxObjectName_TextChanged(object sender,TextChangedEventArgs e)
{
   EnableControls();
}

private void checkboxIsCountersRelatedCause_Click(object sender,RoutedEventArgs e)
{
   EnableControls();
}

//--------------------------------------------------------------------------------------------------

private void buttonAccept_Click(object sender,RoutedEventArgs e)
{
   var dd = _DialogData_;

   dd.ObjectName = textboxObjectName.Text.TrimEnd();
   dd.IsCauseCountersRelated = checkboxIsCountersRelatedCause.IsChecked;

   XBackgroundOperation.Start
   (
      dd.DialogKind == CorrectionCauseDocTypeDialogKind.Cause ?
         dd.IsCreationDialog ?
            "Добавление причины корректировки" :
            "Обновление причины корректировки" :
         dd.IsCreationDialog ?
            "Добавление типа документа корректировки" :
            "Обновление типа документа корректировки",
      BackgroundOperation
   );
}

//--------------------------------------------------------------------------------------------------

private void buttonCancel_Click(object sender,RoutedEventArgs e)
{
   DialogResult = false;
}

//--------------------------------------------------------------------------------------------------

private void Window_KeyDown(object sender,KeyEventArgs e)
{
   if (XInput.IsSoleEnterKey(e) && !buttonAccept.IsEnabled)
      XSound.PlayDisabled();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // CorrectionCauseDocumentTypeWindow
//**************************************************************************************************

} // PaycorMaster
