﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;

using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using XHelpers;

namespace PaycorMaster {

//**************************************************************************************************
public partial class LoginWindow: CustomWindow {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static DependencyProperty PasswordValidationErrorMessageProperty;

//--------------------------------------------------------------------------------------------------

public LoginWindow()
{
   XCursor.SetWait();
   InitializeComponent();
}

//--------------------------------------------------------------------------------------------------

public string PasswordValidationErrorMessage
{
   get
   {
      return (string)passwordboxUser.GetValue(PasswordValidationErrorMessageProperty);
   }
   set
   {
      passwordboxUser.SetValue(PasswordValidationErrorMessageProperty,value);
   }
}

//--------------------------------------------------------------------------------------------------

public string PasswordValidationErrorMessageSource { get; set; }

//==================================================================================================

public new static bool ShowDialog()
{
   return XWindow.ShowModal(typeof(LoginWindow),App.MainWindow,true).Value;
}

//==================================================================================================

static LoginWindow()
{
   PasswordValidationErrorMessageProperty = DependencyProperty.Register
   (
      "PasswordValidationErrorMessage", typeof(string), typeof(LoginWindow)
   );
}

//--------------------------------------------------------------------------------------------------

void EnableControls()
{
   textboxUser.IsEnabled = passwordboxUser.IsEnabled =
      !checkboxIntegratedSecurity.IsChecked.Value;

   buttonConnect.IsEnabled =
      checkboxIntegratedSecurity.IsChecked.Value ||
      textboxUser.Text!="" &&
      !Validation.GetHasError(textboxUser) &&
      !Validation.GetHasError(passwordboxUser);
}

//==================================================================================================

private void Window_Loaded(object sender,RoutedEventArgs e)
{
   Loaded -= Window_Loaded;

   Title = string.Format(
      "Подключение к [{0}].[{1}]",
      App.Configuration.ServerName,
      App.Configuration.DatabaseName );

   checkboxIntegratedSecurity.Content =
      string.Format( "_Системная аутентификация ({0})",
                     XEnvironment.DomainUserName );

   if (App.DbConnector.IntegratedSecutity)
      checkboxIntegratedSecurity.IsChecked = true;
   else
   {
      textboxUser.Text        = App.DbConnector.UserLogName  ?? "";
      passwordboxUser.Password= App.DbConnector.UserPassword ?? "";
   }

   EnableControls();

   Keyboard.Focus( checkboxIntegratedSecurity.IsChecked.Value ?
                   (IInputElement)checkboxIntegratedSecurity : textboxUser );

   XCursor.Reset();
}

private void Window_KeyDown(object sender,KeyEventArgs e)
{
   if (e.Key==Key.Escape)
      DialogResult = false;
}

//--------------------------------------------------------------------------------------------------

private void textboxUser_TextChanged(object sender,TextChangedEventArgs e)
{
   passwordboxUser.Clear(); EnableControls();
}

private void textboxUser_KeyDown(object sender,KeyEventArgs e)
{
   if (e.Key==Key.Return)
   {
      if (textboxUser.Text!="")
         passwordboxUser.Focus();
      else
         XSound.PlayDisabled();
      e.Handled=true;
   }
}

//--------------------------------------------------------------------------------------------------

private void passwordboxUser_PasswordChanged(object sender,RoutedEventArgs e)
{
   PasswordValidationErrorMessage =
      passwordboxUser.Password.Contains(';') ?
      "Недопустимый(е) символ(ы) в пароле.":
      null;
    EnableControls();
}

private void passwordboxUser_KeyDown(object sender,KeyEventArgs e)
{
   if (e.Key==Key.Return)
      if (textboxUser.Text=="")
      {
         textboxUser.Focus();
         e.Handled=true;
      }
      else
         if (!buttonConnect.IsEnabled)
            XSound.PlayDisabled();
}

//--------------------------------------------------------------------------------------------------

private void checkboxIntegratedSecurity_Click(object sender,RoutedEventArgs e)
{
   if (checkboxIntegratedSecurity.IsChecked.Value)
      textboxUser.Text = passwordboxUser.Password = "";

   EnableControls();

   Keyboard.Focus( checkboxIntegratedSecurity.IsChecked.Value ?
                   (IInputElement)buttonConnect : textboxUser );
}

private void checkboxIntegratedSecurity_KeyDown(object sender,KeyEventArgs e)
{
   if (!buttonConnect.IsEnabled)
      XSound.PlayDisabled();
}

//--------------------------------------------------------------------------------------------------

private void buttonConnect_Click(object sender,RoutedEventArgs e)
{
   var connector = App.DbConnector.Duplicate();

   if (!( connector.IntegratedSecutity =
          checkboxIntegratedSecurity.IsChecked.Value ))
   {
      connector.UserLogName = textboxUser.Text;
      connector.UserPassword= passwordboxUser.Password;
   }

   IsBusy = true;
   XThread.StartWithInheritedCultures
   (
      delegate
      {
         bool ok = App.EstablishDbConnection(connector,false);
         XDispatcher.Synchronize(delegate{
            IsBusy = false;
            if (ok)
               DialogResult = true;
            else
               if (!checkboxIntegratedSecurity.IsChecked.Value && buttonConnect.IsFocused)
                  textboxUser.Focus();
         });
      }
   );
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // LoginWindow
//**************************************************************************************************


//**************************************************************************************************
public class LoginUserNameValidationRule: ValidationRule
{
   public override ValidationResult Validate(object value,CultureInfo cultureInfo)
   {
      string userName=(string)value;

      if ( userName.Contains(' ') ||
           userName.Contains(';') )
         return new ValidationResult( false,
            "Имя пользователя не должно содержать пробелов и знака \";\"." );

      return new ValidationResult(true,null);
   }
}
//**************************************************************************************************


//**************************************************************************************************
public class LoginPasswordValidationRule: ValidationRule
{
   public override ValidationResult Validate(object value,CultureInfo cultureInfo)
   {
      return new ValidationResult(value==null,value);
   }
}
//**************************************************************************************************

} // PaycorMaster
