﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using XHelpers;

namespace PaycorMaster {

//**************************************************************************************************
public partial class MainWindow: CustomWindow {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static NavigationService Navigation
{
   get { return App.MainWindow.framePageHost.NavigationService; }
}

//==================================================================================================

public MainWindow()
{
   InitializeComponent();
   Title = App.Title;
   SetConnectionInformation();
   if (App.Settings.StartMaximized)
      WindowState = WindowState.Maximized;
}

//==================================================================================================

public void SetStatusCaption(string value)
{
   textblockStatusCaption.Text = IsBusy ?
      value!=null ? string.Format("{0} ...",value) : "" :
      "Готово к работе";
   textblockStatusCaption.FontWeight =
      IsBusy ? FontWeights.Bold : FontWeights.Normal;
}

//--------------------------------------------------------------------------------------------------

public void SetConnectionInformation()
{
   if (App.IsConnectedToDatabase)
   {
      Title = string.Format(
         "{0} \u2014 {1}",
         App.Title, App.DbConnector.UserInformationString );
      textblockConnection.Text =
         string.Format(
            "{0} / {1} \u2014 {2}",
            App.DbConnector.ServerName,
            App.DbConnector.DatabaseName,
            App.DbConnector.UserLogName );
   }
   else
   {
      Title = App.Title;
      textblockConnection.Text = "не подключено";
   }
   textblockConnection.FontWeight =
      App.IsConnectedToDatabase ? FontWeights.Bold : FontWeights.Normal;
}

//--------------------------------------------------------------------------------------------------

public void SetPageTitle(string title)
{
   title = title ?? "";
   textblockPageTitle.Text = title;
   gridPageHeader.Visibility = title!="" ? Visibility.Visible : Visibility.Collapsed;
}

//--------------------------------------------------------------------------------------------------

public static void SetMainMenuContextItem(MenuItem menuItem)
{
   var window = App.MainWindow;
   var menu = App.MainWindow.menu;
   var contextMenuItem = window.menuitemContext;
   var contextItemIndex = App.MainWindow.menu.Items.IndexOf( contextMenuItem );

   if (menuItem!=null)
   {
      menuItem.FontWeight = FontWeights.Bold;
      XLogicalTreeHelper.SetChildrenValue( menuItem,
         Control.FontWeightProperty, FontWeights.Normal );
      XInput.ProvideMenuItemsWithToolTips( menuItem );
   }
   else
   {
      menuItem = new MenuItem();
      menuItem.Visibility = Visibility.Collapsed;
   }
   menuItem.ToolTip = contextMenuItem.ToolTip;

   menu.Items.Remove( contextMenuItem );
   menu.Items.Insert( contextItemIndex, menuItem );

   window.menuitemContext = menuItem;
}

public static void ResetMainMenuContextItem()
{
   SetMainMenuContextItem(null);
}

//==================================================================================================

private void Window_Loaded(object sender,RoutedEventArgs e)
{
   Loaded -= Window_Loaded;
   App.Navigate(typeof(StartPage));
   XInput.ProvideToolTipsWithGestures(this);
   XInput.ProvideMenuItemsWithToolTips( menu );
   XLogicalTreeHelper.ClearChildrenValue( menuitemPage, FrameworkElement.ToolTipProperty );
}

//--------------------------------------------------------------------------------------------------

private void Window_Closing(object sender,CancelEventArgs e)
{
   e.Cancel = true;

   if ( App.IsConnectedToDatabase &&
        XMessageBox.ShowExclamation(
            "Отключиться и выйти?",
            MessageBoxButton.OKCancel ) != MessageBoxResult.OK )
      return;

   Closing -= Window_Closing;

   XBackgroundOperation.Start
   (
      AppCommands.Close.Text,
      delegate {
         App.DisconnectDatabase();
         XDispatcher.Synchronize( Close ); }
   );
}

//--------------------------------------------------------------------------------------------------

private void Window_KeyDown(object sender,KeyEventArgs e)
{
   if ( XInput.IsSoleEscapeKey(e) &&
        !App.IsConnectedToDatabase &&
        XMessageBox.ShowExclamation(
            "Подтвердите закрытие программы!",
            MessageBoxButton.OKCancel ) == MessageBoxResult.OK )
   {
      Close(); return;
   }
   App.SelectedPage.DoKeyDown(sender,e);
}

//==================================================================================================

private void CloseCommand_Executed(object sender,ExecutedRoutedEventArgs e)
{
   Close();
}

//--------------------------------------------------------------------------------------------------

private void ClosePageCommand_Executed(object sender,ExecutedRoutedEventArgs e)
{
   App.Navigate(typeof(MainPage));
}

private void ClosePageCommand_CanExecute(object sender,CanExecuteRoutedEventArgs e)
{
   e.CanExecute = gridPageHeader.Visibility == Visibility.Visible;
}

//--------------------------------------------------------------------------------------------------

private void ConnectDatabaseCommand_Executed(object sender,ExecutedRoutedEventArgs e)
{
   XBackgroundOperation.Start
   (
      AppCommands.ConnectDatabase.Text,
      delegate
      {
         if (App.ConnectDatabase())
            App.Navigate(typeof(MainPage));
      }
   );
}

private void ConnectDatabaseCommand_CanExecute(object sender,CanExecuteRoutedEventArgs e)
{
   e.CanExecute = !App.IsConnectedToDatabase;
}

//--------------------------------------------------------------------------------------------------

private void DisconnectDatabaseCommand_Executed(object sender,ExecutedRoutedEventArgs e)
{
   ResetMainMenuContextItem();
   XBackgroundOperation.Start
   (
      AppCommands.DisconnectDatabase.Text,
      delegate
      {
         App.DisconnectDatabase();
         App.Navigate(typeof(StartPage));
      }
   );
}

private void DisconnectDatabaseCommand_CanExecute(object sender, CanExecuteRoutedEventArgs e)
{
   e.CanExecute = App.IsConnectedToDatabase;
}

//--------------------------------------------------------------------------------------------------

private void OpenConfigurationPageCommand_Executed(object sender,ExecutedRoutedEventArgs e)
{
   ConfigurationPage.OpenNew();
}

private void OpenConfigurationPageCommand_CanExecute(object sender,CanExecuteRoutedEventArgs e)
{
   e.CanExecute = App.IsConnectedToDatabase && App.IsInAdminMode &&
                  !(App.SelectedPage is ConfigurationPage);
}

//--------------------------------------------------------------------------------------------------

private void CommandBinding_Executed(object sender,ExecutedRoutedEventArgs e)
{
   OptionsWindow.ShowDialog();
}

private void OptionsCommand_CanExecute(object sender,CanExecuteRoutedEventArgs e)
{
   e.CanExecute = App.Settings.InitializationException==null;
}

//--------------------------------------------------------------------------------------------------

private void AboutCommand_Executed(object sender,ExecutedRoutedEventArgs e)
{
   AboutWindow.ShowDialog();
}

//--------------------------------------------------------------------------------------------------

private void PageSpecificCommand_Executed(object sender,ExecutedRoutedEventArgs e)
{
   App.SelectedPage.DoCommandExecuted(sender,e);
}

private void PageSpecificCommand_CanExecute(object sender,CanExecuteRoutedEventArgs e)
{
   App.SelectedPage.DoCommandCanExecute(sender,e);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // MainWindow
//**************************************************************************************************

} // PaycorMaster
