﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using XHelpers;

namespace PaycorMaster {

//**************************************************************************************************
public partial class OptionsWindow: CustomWindow {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public OptionsWindow()
{
   XCursor.SetWait();
   InitializeComponent();
}

//==================================================================================================

public new static void ShowDialog()
{
   new OptionsWindow().ShowModal(App.MainWindow);
}

//==================================================================================================

AppSettings GetSettingsFromElements()
{
   var settings = new AppSettings();

   settings.StartMaximized         = checkboxStartMaximized        .IsChecked.Value;
   settings.UseBeeperAccompaniment = checkboxUseBeeperAccompaniment.IsChecked.Value;
   settings.UseFluctuationEffect   = checkboxUseFluctuationEffect  .IsChecked.Value;

   return settings;
}

void SetElementsBySettings(AppSettings settings)
{
   checkboxStartMaximized        .IsChecked = settings.StartMaximized;
   checkboxUseBeeperAccompaniment.IsChecked = settings.UseBeeperAccompaniment;
   checkboxUseFluctuationEffect  .IsChecked = settings.UseFluctuationEffect;
}

//--------------------------------------------------------------------------------------------------

void ProcessElementsChanged()
{
   buttonApplyAndSave.IsEnabled =
   buttonApplyOnly   .IsEnabled =
   buttonDefaults    .IsEnabled = true;
}

//--------------------------------------------------------------------------------------------------

void Window_CheckedUnchecked(object sender,RoutedEventArgs e)
{
   if (e.Source is CheckBox)
      ProcessElementsChanged();
}

//==================================================================================================

private void Window_Loaded(object sender,RoutedEventArgs e)
{
   Loaded -= Window_Loaded;

   var settings = App.Settings;

   SetElementsBySettings( settings );

   buttonApplyAndSave.IsEnabled = App.HasNonsavedSettings;
   buttonDefaults    .IsEnabled = !settings.IsDefault;

   AddHandler( ToggleButton.CheckedEvent  , new RoutedEventHandler(Window_CheckedUnchecked) );
   AddHandler( ToggleButton.UncheckedEvent, new RoutedEventHandler(Window_CheckedUnchecked) );

   XFocus.MoveToFirstElementOf(this);

   XCursor.Reset();
}

//--------------------------------------------------------------------------------------------------

private void buttonApplyAndSave_Click(object sender,RoutedEventArgs e)
{
   var settings = GetSettingsFromElements();
   XBackgroundOperation.Start
   (
      "Сохранение настроек приложения",
      delegate
      {
         if (App.ApplySettings(settings,true))
            XDispatcher.Synchronize(delegate{
               DialogResult = true; });
      }
   );
}

//--------------------------------------------------------------------------------------------------

private void buttonApplyOnly_Click(object sender,RoutedEventArgs e)
{
   using (new XWaitCursor())
   {
      App.ApplySettings(GetSettingsFromElements());
      DialogResult = true;
   }
}

//--------------------------------------------------------------------------------------------------

private void buttonDefaults_Click(object sender,RoutedEventArgs e)
{
   SetElementsBySettings(AppSettings.Default);
   buttonDefaults.IsEnabled = false;
   buttonApplyAndSave.Focus();
}

//--------------------------------------------------------------------------------------------------

private void buttonCancel_Click(object sender,RoutedEventArgs e)
{
   DialogResult = false;
}

//--------------------------------------------------------------------------------------------------

private void Window_KeyDown(object sender,KeyEventArgs e)
{
   if (XInput.IsSoleEnterKey(e) && !buttonApplyAndSave.IsEnabled)
      XSound.PlayDisabled();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // OptionsWindow
//**************************************************************************************************

} // PaycorMaster
