﻿using System;
using System.Threading;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Runtime.InteropServices;
using System.Globalization;

using XHelpers;

namespace ConsoleApp {

//**************************************************************************************************
static partial class Program {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

static int Main(string[] args)
{
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   XProgram.SpecifyMainThread();
   XProgram.SpecifyInteractsWithDesktop( true );

   XHelpersLibrary.Initialize( true );

   XFatalError.Flags = XFatalErrorFlags.ProcessOwnerDefaults;

   Thread.CurrentThread.CurrentUICulture = Thread.CurrentThread.CurrentCulture;
/*
   Thread.CurrentThread.CurrentUICulture =
      Thread.CurrentThread.CurrentCulture =
         new CultureInfo("it-IT");
*/
   //XConsole.ExactDisplaying = true;

   XConsole.HandlesCancelEvent = true;
   //Console.TreatControlCAsInput = true;

   XSound.Initialize();
   //XSound.IsMute = true;
   //XBeeper.IsMute = true;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   Console.Title = string.Format(
      "ConsoleApp {0} Test Program (based on XHelpers Library)",
      XUnicode.EmDash );

   XConsole.PrintTitleHeader(
      "\"ConsoleApp\" {0} Test Program (based on XHelpers Library):",
      XConsole.LongDash );

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
/*
   try
   {
      throw new ApplicationException("Error-1.");
   }
   catch (Exception e)
   {
      try
      {
         throw new ApplicationException("Error-2.",e);
      }
      catch (Exception x)
      {
         throw new ApplicationException("Error-3.",x);
      }
   }
*/
/*
   XCriticalOperation.Execute(delegate{
      throw new ApplicationException("Artificial interruption.");
   });
*/
/*
   XFatalError.Raise(
      "Internal consistency error.",      // <-- category
      "Can't restore previous state." );  // <-- message
*/
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   if (XProgram.HasEntryPointArguments)
      XConsole.PrintNumberedList
      (
         true,
         "Entry Point Arguments:",
         XProgram.GetEntryPointArguments()
      );

   XConsole.PrintAlignedBulletedList
   (
      true,
      XProgram.HasEntryPointArguments ?
         "SQL Connection Properties:" : null,
      new string[]{ "Server Name", "Database Name" },
      new string[]{ "Luna", "Emerald" }
   );

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   if (XConsole.IsInVisualMode)
   {
      var table = new[,]
      {
         {   1,   2,   3,   4,   5 },
         {  10,  20,  30,  40,  50 },
         { 100, 200, 300, 400, 500 },
      };
      int[] vector = null;

      switch (XConsole.PerformNumberedDialog(
         true, "Array2D Extraction Test:",
         new string[]{ "Horizontal extraction", "Vertical extraction" },
         "Ignore this test" ))
      {
      case 0:
         vector = XArray2D.BackwardExtract( table, 0, 2, null, 3 );
         break;
      case 1:
         vector = XArray2D.Extract( table, 1, 3, 0 );
         break;
      }

      if (vector!=null)
      {
         XConsole.Print();
         XConsole.OutputExclamation( "Vector: " );
         XConsole.PrintExpressive( XString.SeparatedString(vector) );
      }
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   if ( XConsole.IsInVisualMode &&
        XConsole.PerformConfirmationDialog(true,"Input Test:") )
   {
      XConsole.Print();
      string inputString = XConsole.InputString("Input string");
      if (!string.IsNullOrEmpty(inputString))
         XConsole.PrintFeaturedInscription(true,inputString);
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   if ( !XConsole.IsInVisualMode ||
        XConsole.PerformConfirmationDialog(true,"Perform Main Test (RUN):") )
   {
      XConsole.PrintDoubleSplitter(true);
      Run();
   }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   XConsole.PrintTitleFooter(true,"End of \"ConsoleApp\".");

   if (XConsole.IsInVisualMode)
   {
      XConsole.PerformExitionDialog(true);
      XConsole.PrintOperation(true,"Exiting from the program (finalization)");
   }
   return XProgram.PrepareToExit(_HasErrors_ ? 1 : 0);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}

//==================================================================================================

static void Run()
{
   XConsole.PrintOperation(true,"Accessing Win32: \"GetCurrentDirectory\"");
   try
   {
      XConsole.PrintExpressive("Currect directory is: \"{0}\".",GetCurrentDirectory());
   }
   catch (Win32Exception e)
   {
      XConsole.PrintError(e.Message);
      _HasErrors_ = true;
   }

   XConsole.PrintOperation(true,"Accessing Win32: \"GetTempPath\"");
   try
   {
      XConsole.PrintExpressive("Temp path is: \"{0}\".",GetTempPath());
   }
   catch (Win32Exception e)
   {
      XConsole.PrintError(e.Message);
      _HasErrors_ = true;
   }
/*
   int a=57,b=21;
   XConsole.PrintInformation(
      "{0}+{1} = {2}",
      a, b, XEnvironment.Add(a,b) );
*/
/*
   try
      { throw new ApplicationException("Error-1."); }
   catch (Exception e)
      { throw new ApplicationException("Error-2.",e); }
*/
/*
   XCriticalOperation.Execute(delegate{
      try
         { throw new ApplicationException("Error-1."); }
      catch (Exception e)
         { throw new ApplicationException("Error-2.",e); }
   });
*/
/*
   XFatalError.Raise("Artificial failure.");
*/
/*
   XFatalError.Raise("Explicit breakdown.","Unrecoverable state.");
*/
}

//==================================================================================================

static bool
   _HasErrors_;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // Program
//**************************************************************************************************

} // ConsoleApp
