﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Threading;

using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Browser;

using XHelpers;

namespace SilverlightApp {

//**************************************************************************************************
public class BlockingControl: UserControl
{
   public new UIElement Content
   {
      get { return base.Content; }
      set { base.Content = value; }
   }
}
//**************************************************************************************************


//**************************************************************************************************
public partial class App: Application {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

static Popup
   _DisablingPopup_;
static Control
   _PreDisablingFocus_;

//==================================================================================================

public App()
{
   XProgram.SpecifyMainModuleName("SilverlightApp.xap");
   XProgram.SpecifyMainThread();

   XDebug.PrintOperation("Starting the application");

   InitializeComponent();
}

//==================================================================================================

public new static App Current
{
   get { return (App)Current; }
}

public static FrontalLayout RootControl
{
   get { return (FrontalLayout)XApplication.RootControl;  }
}

//==================================================================================================

static void NavigationFrameSizeChanged(object sender,SizeChangedEventArgs e)
{
   if (_DisablingPopup_!=null)
   {
      var blockingControl = (BlockingControl)_DisablingPopup_.Child;
      blockingControl.Width = e.NewSize.Width;
      blockingControl.Height = e.NewSize.Height;
   }
}

//--------------------------------------------------------------------------------------------------

static void XApplication_IsBusyChanged()
{
   if (XApplication.IsBusy)
   {
      var rectangle = new Rectangle()
      {
         HorizontalAlignment = HorizontalAlignment.Stretch,
         VerticalAlignment = VerticalAlignment.Stretch,
         Opacity = 0.75, Fill=new SolidColorBrush( Colors.LightGray ),
      };

      var contentControl = new ContentControl();

      var grid = new Grid();
      grid.Children.Add( rectangle );
      grid.Children.Add( contentControl );

      var navigationFrame = RootControl.navigationFrame;

      var blockingControl = new BlockingControl()
      {
         Content = grid,
         Width = RootControl.navigationFrame.ActualWidth,
         Height = RootControl.navigationFrame.ActualHeight
      };

      Point navigationFrameOffset =
         XUIElement.GetRelativeOffset(
            navigationFrame, RootControl );

      _DisablingPopup_ = new Popup()
      {
         Child = blockingControl,
         HorizontalOffset = navigationFrameOffset.X,
         VerticalOffset = navigationFrameOffset.Y
      };

      navigationFrame.SizeChanged -= NavigationFrameSizeChanged;
      navigationFrame.SizeChanged += NavigationFrameSizeChanged;

      _PreDisablingFocus_ = FocusManager.GetFocusedElement() as Control;

      XApplication.LayoutRoot.Children.Add( _DisablingPopup_ );
      _DisablingPopup_.IsOpen = true;
      XFocus.StrictFocus( blockingControl );

      XApplication.RootControl.IsEnabled = false;
      XApplication.LayoutRoot.Background =
         new SolidColorBrush(Colors.Black);
   }
   else
   {
      XApplication.LayoutRoot.Background =
         new SolidColorBrush(Colors.White);
      XApplication.RootControl.IsEnabled = true;

      _DisablingPopup_.IsOpen = false;
      XApplication.LayoutRoot.Children.Remove( _DisablingPopup_ );

      _DisablingPopup_ = null;
      if (_PreDisablingFocus_!=null)
         XFocus.StrictFocus( _PreDisablingFocus_ );
   }
}

//==================================================================================================

private void Application_Startup(object sender,StartupEventArgs e)
{
   RootVisual = new FrontalLayout();

   XApplication.IsBusyChanged += XApplication_IsBusyChanged;
}

private void Application_Exit(object sender,EventArgs e)
{
   XDebug.PrintOperation("Exiting from the application");
}

private void Application_UnhandledException
(
   object sender, ApplicationUnhandledExceptionEventArgs e
)
{
   XFatalError.ProcessUnhandledException( e.ExceptionObject );
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // App
//**************************************************************************************************

} // SilverlightApp
