﻿using System;
using System.Threading;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Windows;
using System.Windows.Data;

using XHelpers;

namespace WpfApplication
{
   /// <summary>
   /// Interaction logic for App.xaml
   /// </summary>
   public partial class App: Application
   {
      public new static App Current
         { get { return (App)Application.Current; } }

      public new static MainWindow MainWindow
         { get { return (MainWindow)Application.Current.MainWindow; } }

      void MergeDictionary(string relativePath)
      {
          Resources.MergedDictionaries.Add(
             (ResourceDictionary)LoadComponent(
                new Uri(relativePath, UriKind.Relative)));
      }

      void PrepareResources()
      {
          Resources.MergedDictionaries.Clear();

          MergeDictionary("MainCode/Styles/BusyStyles.xaml");

          Resources.Add
          (
             "BusyToIsEnabledBinding",
             new Binding()
             {
                 Path = new PropertyPath("IsBusy"),
                 RelativeSource = new RelativeSource(RelativeSourceMode.FindAncestor, typeof(Window), 1),
                 Converter = new XBusyToIsEnabledConverter()
             }
          );
      }

      static App()
      {
         Thread.CurrentThread.CurrentUICulture =
            Thread.CurrentThread.CurrentCulture;

         XProgram.SpecifyMainThread();
         XProgram.SpecifyInteractsWithDesktop( true );

         XFatalError.Flags = XFatalErrorFlags.ProcessOwnerDefaults;

         XDebug.PrintOperation("Starting application");
         AppDomain.CurrentDomain.ProcessExit += AppDomain_ProcessExit;
      }

      static void AppDomain_ProcessExit(object sender,EventArgs e)
      {
         XDebug.PrintOperation("Process completion");
      }

      private void Application_Startup(object sender,StartupEventArgs e)
      {
         XApplication.SpecifyDispatcher();
         XApplication.AnchorWindow = App.MainWindow;

         XMessageBox.CaptionFormatString = "Test: {0}";
         XMessageBox.BeforeShowHandler += delegate
         {
            if ( MainWindow!=null &&
                 MainWindow.WindowState==WindowState.Minimized )
               MainWindow.Restore();
         };

         var mainWindow = new MainWindow();
         XApplication.AnchorWindow = mainWindow;
         mainWindow.Show();
      }
   }
}
