﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
partial class XConsole {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static void OutputArrowAlignedCaption( [Optional] string caption,
                                              [Optional] int maxCaptionLength )
{
   OutputArrowAlignedCaption(caption,null,maxCaptionLength);
}

public static void OutputArrowAlignedCaption( [Optional] string caption,
                                              ConsoleColor? captionColor,
                                              int maxCaptionLength )
{
   OutputArrowAlignedCaption(caption,captionColor,maxCaptionLength,null,null,null);
}

//--------------------------------------------------------------------------------------------------

public static void OutputArrowAlignedCaption( [Optional] string caption,
                                              [Optional] int maxCaptionLength,
                                              char? dashChar )
{
   OutputArrowAlignedCaption(caption,maxCaptionLength,dashChar,null);
}

public static void OutputArrowAlignedCaption( [Optional] string caption,
                                              [Optional] int maxCaptionLength,
                                              char? dashChar, char? pointerChar )
{
   OutputArrowAlignedCaption(caption,null,maxCaptionLength,dashChar,pointerChar,null);
}

//--------------------------------------------------------------------------------------------------

public static void OutputArrowAlignedCaption( [Optional] string caption,
                                              ConsoleColor? captionColor,
                                              int maxCaptionLength,
                                              char? dashChar )
{
   OutputArrowAlignedCaption(caption,captionColor,maxCaptionLength,dashChar,null);
}

public static void OutputArrowAlignedCaption( [Optional] string caption,
                                              ConsoleColor? captionColor,
                                              int maxCaptionLength,
                                              char? dashChar, char? pointerChar )
{
   OutputArrowAlignedCaption(caption,captionColor,maxCaptionLength,dashChar,pointerChar,null);
}

//==================================================================================================

public static void OutputArrowAlignedCaption
(
   [Optional] string caption, ConsoleColor? captionColor, [Optional] int maxCaptionLength,
   char? dashChar, char? pointerChar, ConsoleColor? arrowColor
)
{
   if (maxCaptionLength<0)
      throw new ArgumentOutOfRangeException();

   caption = caption ?? "";
   arrowColor = arrowColor ?? ArrowedCaptionArrowColor;
   dashChar = dashChar ?? ArrowedCaptionDashChar;

   lock (typeof(XConsole))
      if (caption=="" && maxCaptionLength==0)
         Output( arrowColor, "{0}{1} ",
            new string( dashChar.Value, ArrowedCaptionDashMinLength ),
            ArrowedCaptionPointerChar );
      else
      {
         Output(captionColor ?? ArrowedCaptionColor,caption);
         Output( arrowColor,
            caption!="" ? " {0}{1} " : "{0}{1} ",
            new string( dashChar.Value,
               ArrowedCaptionDashMinLength +
               XArith.ConfinePositive( maxCaptionLength-caption.Length ) ),
            pointerChar ?? ArrowedCaptionPointerChar );
      }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XConsole
//**************************************************************************************************

} // XHelpers
