﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
partial class XConsole {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static int PerformDialog( bool insertBlankLine, string caption,
                                 string[] actions, [Optional] bool[] enablings,
                                 ConsoleKey[] keysArray )
{
   return PerformDialog( insertBlankLine, caption,
      actions, false, enablings, keysArray );
}

//--------------------------------------------------------------------------------------------------

public static int PerformDialog( bool insertBlankLine, string caption,
                                 string[] actions,
                                 [Optional] bool[] enablings,
                                 ConsoleKey[] keysArray,
                                 [Optional] string cancelAction )
{
   return PerformDialog( insertBlankLine, caption,
      actions, enablings, keysArray, null,
      cancelAction );
}

public static int PerformDialog( bool insertBlankLine,
                                 string caption,
                                 string[] actions,
                                 [Optional] bool[] enablings,
                                 ConsoleKey[] keysArray,
                                 [Optional] string cancelAction,
                                 [Optional] ConsoleKey cancelKey )
{
   return PerformDialog( insertBlankLine, caption,
      actions, enablings, keysArray,
      cancelAction, cancelKey, 0 );
}

public static int PerformDialog( bool insertBlankLine,
                                 string caption,
                                 string[] actions,
                                 [Optional] bool[] enablings,
                                 ConsoleKey[] keysArray,
                                 [Optional] string cancelAction,
                                 [Optional] ConsoleKey cancelKey,
                                 [Optional] ConsoleModifiers cancelModifiers )
{
   return PerformDialog( insertBlankLine,
      caption, actions, enablings, keysArray, null,
      cancelAction, cancelKey, cancelModifiers );
}

//--------------------------------------------------------------------------------------------------

public static int PerformDialog( bool insertBlankLine, string caption,
                                 string[] actions, bool isLastActionCancel,
                                 [Optional] bool[] enablings, ConsoleKey[] keysArray )
{
   return PerformDialog( insertBlankLine, caption,
      actions, isLastActionCancel, enablings, keysArray, null );
}

//--------------------------------------------------------------------------------------------------

public static int PerformDialog( bool insertBlankLine,
                                 string caption,
                                 string[] actions,
                                 [Optional] bool[] enablings,
                                 ConsoleKey[] keysArray,
                                 [Optional] ConsoleModifiers[] modifiersArray )
{
   return PerformDialog( insertBlankLine, caption,
      actions, false, enablings, keysArray, modifiersArray );
}

//==================================================================================================

public static int PerformDialog( bool insertBlankLine,
                                 string caption,
                                 string[] actions,
                                 [Optional] bool[] enablings,
                                 ConsoleKey[] keysArray,
                                 [Optional] ConsoleModifiers[] modifiersArray,
                                 [Optional] string cancelAction )
{
   XArgCheck.VerifyNotNull( cancelAction );
   return PerformDialog( insertBlankLine,
      caption, actions, enablings, keysArray, modifiersArray,
      cancelAction, cancelAction!=null ? ConsoleKey.Escape : 0 );
}

public static int PerformDialog( bool insertBlankLine, string caption,
                                 string[] actions,
                                 [Optional] bool[] enablings,
                                 ConsoleKey[] keysArray,
                                 [Optional] ConsoleModifiers[] modifiersArray,
                                 [Optional] string cancelAction,
                                 [Optional] ConsoleKey cancelKey )
{
   return PerformDialog( insertBlankLine,
      caption, actions, enablings, keysArray, modifiersArray,
      cancelAction, cancelKey, 0 );
}

public static int PerformDialog( bool insertBlankLine,
                                 string caption,
                                 string[] actions,
                                 [Optional] bool[] enablings,
                                 ConsoleKey[] keysArray,
                                 [Optional] ConsoleModifiers[] modifiersArray,
                                 [Optional] string cancelAction,
                                 [Optional] ConsoleKey cancelKey,
                                 [Optional] ConsoleModifiers cancelModifiers )
{
   XArgCheck.VerifyNotNulls( caption, actions, keysArray );

   if ( !( cancelAction!=null && cancelKey!=0 ||
           cancelAction==null && cancelKey==0 && cancelModifiers==0 ) ||
        enablings!=null && actions.Length!=enablings.Length )
      throw XArgCheck.ArgumentException_IncorrectArguments;

   if (cancelAction!=null)
   {
      XArray.Append(ref keysArray,cancelKey);
      if (modifiersArray!=null)
         XArray.Append(ref modifiersArray,cancelModifiers);
      XArray.Append(ref actions,cancelAction);
   }

   return PerformDialog( insertBlankLine, caption,
      actions, cancelAction!=null, enablings, keysArray, modifiersArray );
}

//--------------------------------------------------------------------------------------------------

public static int PerformDialog( bool insertBlankLine,
                                 string caption,
                                 string[] actions,
                                 bool isLastActionCancel,
                                 [Optional] bool[] enablings,
                                 ConsoleKey[] keysArray,
                                 [Optional] ConsoleModifiers[] modifiersArray )
{
   XArgCheck.VerifyNotNulls( caption, keysArray, actions );

   if
   (
      keysArray.Length==0 ||
      modifiersArray!=null && modifiersArray.Length!=keysArray.Length ||
      actions.Length!=keysArray.Length ||
      enablings!=null &&
      !(
         !isLastActionCancel && enablings.Length==actions.Length ||
         isLastActionCancel  && enablings.Length==actions.Length-1
      )
   )
      throw XArgCheck.ArgumentException_IncorrectArguments;

   VerifyIsInVisualMode();

   PrintUnderbarCaption( insertBlankLine, DialogCaptionChar, DialogCaptionColor, caption );

   int itemsCount = actions.Length;

   var keyStrings = new string[ itemsCount ];
   for (int i=0; i<keyStrings.Length; i++)
      keyStrings[i] = KeyToString( XArray.GetAtZeroBasedIndex(keysArray,i),
         modifiersArray!=null ? XArray.GetAtZeroBasedIndex(modifiersArray,i) : 0 );

   int maxCaptionLength = XString.MaxLength( keyStrings );

   Predicate<int> isItemEnabledDelegate = (i) =>
      enablings==null || isLastActionCancel && i==itemsCount-1 ||
      XArray.GetAtZeroBasedIndex(enablings,i);

   for (int i=0; i<itemsCount; i++)
   {
      OutputDashAlignedListItemCaption
      (
         null, null, keyStrings[i],
         isItemEnabledDelegate(i) ? DialogKeyColor : DialogKeyDisablingColor,
         maxCaptionLength, null, DialogDashColor
      );
      Print
      (
         i==keyStrings.Length-1 && isLastActionCancel ?
         DialogCancelActionColor : DialogActionColor,
         XArray.GetAtZeroBasedIndex(actions,i)
      );
   }

   OutputDialogPrompt();

   int idx;

   for (;;)
   {
      try
         { idx = AcceptInputKey( keysArray, modifiersArray ); }
      catch (XAbortConsoleInputException)
         { Print(); throw; }
      idx = XArray.ZeroBasedIndex( keysArray, idx );

      if (isItemEnabledDelegate(idx))
      {
         Print();
         break;
      }
      PlayDisabledSound();
   }

   return isLastActionCancel && idx==itemsCount-1 ?
      -1 : idx;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XConsole
//**************************************************************************************************

} // XHelpers
