﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
partial class XConsole {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static int PerformNumberedDialog( bool insertBlankLine, string caption,
                                         string[] actions, [Optional] bool[] enablings )
{
   return PerformNumberedDialog(insertBlankLine,caption,actions,enablings,null);
}

//--------------------------------------------------------------------------------------------------

public static int PerformNumberedDialog( bool insertBlankLine,
                                         string caption,
                                         string[] actions,
                                         [Optional] bool[] enablings,
                                         [Optional] string cancelAction )
{
   return PerformNumberedDialog(
      insertBlankLine, caption,
      null, actions, enablings,
      cancelAction, cancelAction!=null ? ConsoleKey.Escape : 0 );
}

public static int PerformNumberedDialog( bool insertBlankLine,
                                         string caption,
                                         string[] actions,
                                         [Optional] bool[] enablings,
                                         [Optional] string cancelAction,
                                         [Optional] ConsoleKey cancelKey )
{
   return PerformNumberedDialog(
      insertBlankLine, caption,
      null, actions, enablings,
      cancelAction, cancelKey );
}

public static int PerformNumberedDialog( bool insertBlankLine,
                                         string caption,
                                         string[] actions,
                                         [Optional] bool[] enablings,
                                         [Optional] string cancelAction,
                                         [Optional] ConsoleKey cancelKey,
                                         [Optional] ConsoleModifiers cancelModifiers )
{
   return PerformNumberedDialog(
      insertBlankLine, caption,
      null, actions, enablings,
      cancelAction, cancelKey, cancelModifiers );
}

//==================================================================================================

public static int PerformNumberedDialog( bool insertBlankLine,
                                         string caption,
                                         [Optional] string zeroAction,
                                         string[] actions,
                                         [Optional] bool[] enablings )
{
   return PerformNumberedDialog(
      insertBlankLine, caption,
      zeroAction, false, actions, enablings,
      null, 0 );
}

//--------------------------------------------------------------------------------------------------

public static int PerformNumberedDialog( bool insertBlankLine,
                                         string caption,
                                         [Optional] string zeroAction,
                                         string[] actions,
                                         [Optional] bool[] enablings,
                                         [Optional] string cancelAction )
{
   return PerformNumberedDialog(
      insertBlankLine, caption,
      zeroAction, false, actions, enablings,
      cancelAction, cancelAction!=null ? ConsoleKey.Escape : 0 );
}

public static int PerformNumberedDialog( bool insertBlankLine,
                                         string caption,
                                         [Optional] string zeroAction,
                                         string[] actions,
                                         [Optional] bool[] enablings,
                                         [Optional] string cancelAction,
                                         [Optional] ConsoleKey cancelKey )
{
   return PerformNumberedDialog(
      insertBlankLine, caption,
      zeroAction, false, actions, enablings,
      cancelAction, cancelKey );
}

public static int PerformNumberedDialog( bool insertBlankLine,
                                         string caption,
                                         [Optional] string zeroAction,
                                         string[] actions,
                                         [Optional] bool[] enablings,
                                         [Optional] string cancelAction,
                                         [Optional] ConsoleKey cancelKey,
                                         [Optional] ConsoleModifiers cancelModifiers )
{
   return PerformNumberedDialog(
      insertBlankLine, caption,
      zeroAction, false, actions, enablings,
      cancelAction, cancelKey, cancelModifiers );
}

//==================================================================================================

public static int PerformNumberedDialog( bool insertBlankLine,
                                         string caption,
                                         [Optional] string zeroAction,
                                         bool placeZeroActionDown,
                                         string[] actions,
                                         [Optional] bool[] enablings )
{
   return PerformNumberedDialog(
      insertBlankLine, caption,
      zeroAction, placeZeroActionDown, actions, enablings,
      null, 0 );
}

//--------------------------------------------------------------------------------------------------

public static int PerformNumberedDialog( bool insertBlankLine,
                                         string caption,
                                         [Optional] string zeroAction,
                                         bool placeZeroActionDown,
                                         string[] actions,
                                         [Optional] bool[] enablings,
                                         [Optional] string cancelAction )
{
   return PerformNumberedDialog(
      insertBlankLine, caption,
      zeroAction, placeZeroActionDown, actions, enablings,
      cancelAction, cancelAction!=null ? ConsoleKey.Escape : 0 );
}

public static int PerformNumberedDialog( bool insertBlankLine,
                                         string caption,
                                         [Optional] string zeroAction,
                                         bool placeZeroActionDown,
                                         string[] actions,
                                         [Optional] bool[] enablings,
                                         [Optional] string cancelAction,
                                         [Optional] ConsoleKey cancelKey )
{
   return PerformNumberedDialog(
      insertBlankLine, caption,
      zeroAction, placeZeroActionDown, actions, enablings,
      cancelAction, cancelKey, 0 );
}

public static int PerformNumberedDialog( bool insertBlankLine,
                                         string caption,
                                         [Optional] string zeroAction,
                                         bool placeZeroActionDown,
                                         string[] actions,
                                         [Optional] bool[] enablings,
                                         [Optional] string cancelAction,
                                         [Optional] ConsoleKey cancelKey,
                                         [Optional] ConsoleModifiers cancelModifiers )
{
   XArgCheck.VerifyNotNull( caption );
   XArgCheck.VerifyNotNulls( actions );

   if ( zeroAction==null && placeZeroActionDown ||
        !( cancelAction!=null && cancelKey!=0 ||
           cancelAction==null && cancelKey==0 && cancelModifiers==0 ) ||
        enablings!=null && enablings.Length!=actions.Length )
      throw XArgCheck.ArgumentException_IncorrectArguments;

   int actionsCount = actions.Length;

   if (!XOps.IsBetween(actionsCount,1,9))
      throw new ArgumentOutOfRangeException( null,
         "Numbered dialog actions count must be within 1..9." );

   int nonZeroActionsCount = actionsCount;

   if (zeroAction!=null)
      actionsCount++;

   bool startWithZeroAction =
      zeroAction!=null && !placeZeroActionDown;

   var keysArray = new ConsoleKey[ actionsCount ];

   if (zeroAction!=null)
   {
      actions = startWithZeroAction ?
         XArray.Prefix( actions, zeroAction ) :
         XArray.Append( actions, zeroAction );
      int zeroActionIdx =
         startWithZeroAction ? 0 : actionsCount-1;
      keysArray[ zeroActionIdx ] = ConsoleKey.D0;
      actions  [ zeroActionIdx ] = zeroAction;
   }

   for ( int i=0, j=0;
         i < nonZeroActionsCount;
         i++, j++ )
      keysArray[ i + (startWithZeroAction ? 1 : 0) ] =
         (ConsoleKey)( (int)ConsoleKey.D1 + j );

   int dialogResult = PerformDialog(
      insertBlankLine, caption,
      actions, enablings, keysArray,
      cancelAction, cancelKey, cancelModifiers );

   if (dialogResult<0)
      return dialogResult;
   if (placeZeroActionDown)
      return dialogResult < actionsCount-1 ? dialogResult+1 : 0;
   return dialogResult;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XConsole
//**************************************************************************************************

} // XHelpers
