﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Threading;
using System.ComponentModel;

namespace XHelpers {

//**************************************************************************************************
partial class XConsole {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

/// <exception cref="ArgumentNullException" />
public static string CancellationFormatString
{
   get
   {
      lock(_CancellationSyncObject_)
         return _CancellationFormatString_;
   }
   set
   {
      XArgCheck.VerifyNotNull( value );
      lock(_CancellationSyncObject_)
         _CancellationFormatString_ = value;
   }
}

//--------------------------------------------------------------------------------------------------

public static ConsoleColor? CancellationColor
{
   get { lock (_CancellationSyncObject_) return _CancellationColor_; }
   set { lock (_CancellationSyncObject_) _CancellationColor_ = value; }
}

//--------------------------------------------------------------------------------------------------

/// <exception cref="ArgumentException" />
public static int CancellationBeepFrequency
{
   get
   {
      lock (_CancellationSyncObject_)
         return _CancellationBeepFrequency_;
   }
   set
   {
      XBeeper.VerifyFrequency( value );
      lock (_CancellationSyncObject_)
         _CancellationBeepFrequency_ = value;
   }
}

/// <exception cref="ArgumentException" />
public static int CancellationBeepDuration
{
   get
   {
      lock (_CancellationSyncObject_)
         return _CancellationBeepDuration_;
   }
   set
   {
      XBeeper.VerifyDuration( value );
      lock (_CancellationSyncObject_)
         _CancellationBeepDuration_ = value;
   }
}

//==================================================================================================

static string
   _CancellationFormatString_ = "* {0} ...";
static ConsoleColor?
   _CancellationColor_ = ConsoleColor.Red;
static int
   _CancellationBeepFrequency_ = 880,
   _CancellationBeepDuration_  = 500;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XConsole
//**************************************************************************************************

} // XHelpers
