﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
partial class XConsole {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

enum HeaderFooterType { Normal, Shallow, Title }

//**************************************************************************************************

static void PrintHeaderFooter( HeaderFooterType headerFooterType,
                               bool footer,
                               bool insertPrecedingBlankLine,
                               bool insertPosteriorBlankLine,
                               string caption )
{
   var color = HeaderFooterColor;
   var fillChar = HeaderFooterChar;

   switch (headerFooterType)
   {
   case HeaderFooterType.Shallow:
      color = ShallowHeaderFooterColor;
      fillChar = ShallowHeaderFooterChar;
      break;
   case HeaderFooterType.Title:
      color = TitleHeaderFooterColor;
      fillChar = TitleHeaderFooterChar;
      break;
   }

   int maxCaptionLength =
      Width - (HeaderFooterPaddingWidth+1)*2;
   if (caption.Length>maxCaptionLength)
      caption = caption.Substring( 0,
         XArith.ConfinePositive( maxCaptionLength ) );

   lock (typeof(XConsole))
   {
      if (insertPrecedingBlankLine)
         Print();
      if (!footer)
         Print(color,new string(fillChar,Width));
      Print
      (
         color, "{1} {0} {2}", caption,
         new string(fillChar,HeaderFooterPaddingWidth),
         new string( fillChar,
            Math.Max( Width-caption.Length-HeaderFooterPaddingWidth-2,
                      HeaderFooterPaddingWidth ) )
      );
      if (footer || headerFooterType==HeaderFooterType.Title)
         Print(color,new string(fillChar,Width));
      if (insertPosteriorBlankLine)
         Print();
   }
}

//--------------------------------------------------------------------------------------------------

static void PrintHeaderFooter( HeaderFooterType headerFooterType,
                               bool footer,
                               bool insertPrecedingBlankLine,
                               bool insertPosteriorBlankLine,
                               string captionFmt,
                               object[] parameters )
{
   PrintHeaderFooter( headerFooterType, footer,
      insertPrecedingBlankLine, insertPosteriorBlankLine,
      string.Format(captionFmt,parameters) );
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XConsole
//**************************************************************************************************

} // XHelpers
