﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
partial class XConsole {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static void PrintAlignedList( bool insertBlankLine, [Optional] string caption,
                                     string[] captions, string[] values )
{
   PrintListCaption( insertBlankLine, caption );
   PrintAlignedList( false, captions, values );
}

//==================================================================================================

public static void PrintAlignedList(bool insertBlankLine,string[] captions,string[] values)
{
   PrintAlignedList(
      insertBlankLine,
      captions, null,
      values, null,
      null, null,
      null, null );
}

public static void PrintAlignedList( bool insertBlankLine, string[] captions,
                                     string[] values, [Optional] string pairLigament )
{
   PrintAlignedList(
      insertBlankLine,
      captions, null,
      values, null,
      null, null,
      pairLigament, null );
}

//--------------------------------------------------------------------------------------------------

public static void PrintAlignedList( bool insertBlankLine,
                                     string[] captions,
                                     ConsoleColor? captionColor,
                                     string[] values )
{
   PrintAlignedList(
      insertBlankLine,
      captions, captionColor,
      values, null,
      null, null,
      null, null );
}

public static void PrintAlignedList( bool insertBlankLine,
                                     string[] captions, ConsoleColor? captionColor,
                                     string[] values, [Optional] string pairLigament )
{
   PrintAlignedList(
      insertBlankLine,
      captions, captionColor,
      values, null,
      null, null,
      pairLigament, null );
}

//--------------------------------------------------------------------------------------------------

public static void PrintAlignedList( bool insertBlankLine,
                                     string[] captions,
                                     string[] values,
                                     ConsoleColor? valueColor )
{
   PrintAlignedList(
      insertBlankLine,
      captions, null,
      values, valueColor,
      null, null,
      null, null );
}

public static void PrintAlignedList( bool insertBlankLine, string[] captions,
                                     string[] values, ConsoleColor? valueColor,
                                     [Optional] string pairLigament )
{
   PrintAlignedList(
      insertBlankLine,
      captions, null,
      values, valueColor,
      null, null,
      pairLigament, null );
}

//--------------------------------------------------------------------------------------------------

public static void PrintAlignedList( bool insertBlankLine,
                                     string[] captions, ConsoleColor? captionColor,
                                     string[] values, ConsoleColor? valueColor )
{
   PrintAlignedList(
      insertBlankLine,
      captions, captionColor,
      values, valueColor,
      null, null,
      null, null );
}

public static void PrintAlignedList( bool insertBlankLine,
                                     string[] captions, ConsoleColor? captionColor,
                                     string[] values, ConsoleColor? valueColor,
                                     [Optional] string pairLigament )
{
   PrintAlignedList(
      insertBlankLine,
      captions, captionColor,
      values, valueColor,
      null, null,
      pairLigament, null );
}

//--------------------------------------------------------------------------------------------------

public static void PrintAlignedList( bool insertBlankLine,
                                     string[] captions, ConsoleColor? captionColor,
                                     string[] values, ConsoleColor? valueColor,
                                     char? bulletChar, ConsoleColor? bulletColor,
                                     [Optional] string pairLigament,
                                     ConsoleColor? pairLigamentColor )
{
   XArgCheck.VerifyNotNulls(captions,values);
   XArgCheck.VerifyNotNulls(captions);
   XArgCheck.VerifyNotNulls(values);

   if (captions.Length!=values.Length)
      throw XArgCheck.ArgumentException_IncorrectArguments;

   if (insertBlankLine)
      Print();

   int maxCaptionLength = XString.MaxLength( captions );

   lock (typeof(XConsole))
      for (int i=0; i<captions.Length; i++)
      {
         OutputAlignedListItemCaption
         (
            bulletChar, bulletColor,
            XArray.GetAtZeroBasedIndex(captions,i), captionColor, maxCaptionLength,
            pairLigament, pairLigamentColor
         );
         Print( valueColor ?? ListItemValueColor,
                XArray.GetAtZeroBasedIndex(values,i) );
      }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XConsole
//**************************************************************************************************

} // XHelpers
