﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
partial class XConsole {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static void PrintDashAlignedList( bool insertBlankLine, [Optional] string caption,
                                         string[] captions, string[] values )
{
   PrintListCaption( insertBlankLine, caption );
   PrintDashAlignedList( false, captions, values );
}

//==================================================================================================

public static void PrintDashAlignedList(bool insertBlankLine,string[] captions,string[] values)
{
   PrintDashAlignedList(
      insertBlankLine,
      captions, null,
      values, null,
      null, null,
      null, null );
}

public static void PrintDashAlignedList( bool insertBlankLine,
                                         string[] captions,
                                         ConsoleColor? captionColor,
                                         string[] values )
{
   PrintDashAlignedList(
      insertBlankLine,
      captions, captionColor,
      values, null,
      null, null,
      null, null );
}

//--------------------------------------------------------------------------------------------------

public static void PrintDashAlignedList( bool insertBlankLine,
                                         string[] captions,
                                         string[] values,
                                         ConsoleColor? valueColor )
{
   PrintDashAlignedList(
      insertBlankLine,
      captions, null,
      values, valueColor,
      null, null,
      null, null );
}

public static void PrintDashAlignedList( bool insertBlankLine,
                                         string[] captions, ConsoleColor? captionColor,
                                         string[] values, ConsoleColor? valueColor )
{
   PrintDashAlignedList(
      insertBlankLine,
      captions, captionColor,
      values, valueColor,
      null, null,
      null, null );
}

//--------------------------------------------------------------------------------------------------

public static void PrintDashAlignedList( bool insertBlankLine,
                                         string[] captions, ConsoleColor? captionColor,
                                         string[] values, ConsoleColor? valueColor,
                                         ConsoleColor? dashColor )
{
   PrintDashAlignedList(
      insertBlankLine,
      captions, captionColor,
      values, valueColor,
      null, null,
      null, dashColor );
}

//--------------------------------------------------------------------------------------------------

public static void PrintDashAlignedList( bool insertBlankLine,
                                         string[] captions, ConsoleColor? captionColor,
                                         string[] values, ConsoleColor? valueColor,
                                         char? bulletChar, ConsoleColor? bulletColor,
                                         char? dashChar, ConsoleColor? dashColor )
{
   XArgCheck.VerifyNotNulls(captions,values);
   XArgCheck.VerifyNotNulls(captions);
   XArgCheck.VerifyNotNulls(values);

   if (captions.Length!=values.Length)
      throw XArgCheck.ArgumentException_IncorrectArguments;

   if (insertBlankLine)
      Print();

   int maxCaptionLength = XString.MaxLength( captions );

   lock (typeof(XConsole))
      for (int i=0; i<captions.Length; i++)
      {
         OutputDashAlignedListItemCaption
         (
            bulletChar, bulletColor,
            XArray.GetAtZeroBasedIndex(captions,i), captionColor, maxCaptionLength,
            dashChar, dashColor
         );
         Print( valueColor ?? ListItemValueColor,
                XArray.GetAtZeroBasedIndex(values,i) );
      }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XConsole
//**************************************************************************************************

} // XHelpers
