﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
partial class XConsole {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static void PrintNumberedList( bool insertBlankLine,
                                      [Optional] string caption,
                                      string[] values )
{
   PrintListCaption( insertBlankLine, caption );
   PrintNumberedList( false, values );
}

//==================================================================================================

public static void PrintNumberedList(bool insertBlankLine,string[] values)
{
   PrintNumberedList(insertBlankLine,values,null);
}

public static void PrintNumberedList( bool insertBlankLine, string[] values,
                                      [Optional] ConsoleColor? valueColor )
{
   PrintNumberedList(insertBlankLine,values,valueColor,null);
}

//--------------------------------------------------------------------------------------------------

public static void PrintNumberedList( bool insertBlankLine, string[] values,
                                      [Optional] ConsoleColor? valueColor,
                                      [Optional] ConsoleColor? numberColor )
{
   PrintNumberedList(insertBlankLine,values,valueColor,numberColor,null);
}

public static void PrintNumberedList( bool insertBlankLine, string[] values,
                                      [Optional] ConsoleColor? valueColor,
                                      [Optional] ConsoleColor? numberColor,
                                      [Optional] string numberValueSeparator )
{
   PrintNumberedList(insertBlankLine,values,valueColor,numberColor,numberValueSeparator,null);
}

//--------------------------------------------------------------------------------------------------

public static void PrintNumberedList( bool insertBlankLine, string[] values,
                                      [Optional] ConsoleColor? valueColor,
                                      [Optional] ConsoleColor? numberColor,
                                      [Optional] string numberValueSeparator,
                                      [Optional] ConsoleColor? numberValueSeparatorColor )
{
   XArgCheck.VerifyNotNulls( values );
   if (values.Length==0)
      throw XArgCheck.ArgumentException_IncorrectArguments;

   int itemCount = values.Length;
   var numCaptions = new string[ itemCount ];
   int maxNumberWidth = XInt32.ToNormalString( itemCount ).Length;

   for (int i=0; i<itemCount; i++)
      numCaptions[i] = XInt32.ToNormalString( i+1 ).PadLeft( maxNumberWidth );

   numberColor = numberColor ?? ListItemNumberColor;

   PrintAlignedList( insertBlankLine,
      numCaptions, numberColor,
      values, valueColor ?? ListItemNumberedValueColor,
      null, null,
      numberValueSeparator ?? ListItemNumberValueSeparator,
      numberValueSeparatorColor ?? ListItemNumberValueSeparatorColor ?? numberColor );
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XConsole
//**************************************************************************************************

} // XHelpers
