﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
partial class XConsole {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static void Print(bool insertBlankLine,object obj)
{
   Print(insertBlankLine,(ConsoleColor?)null,obj);
}

public static void Print(bool insertBlankLine,string str)
{
   Print(insertBlankLine,null,str);
}

public static void Print(bool insertBlankLine,string format,params object[] parameters)
{
   Print(insertBlankLine,null,format,parameters);
}

//--------------------------------------------------------------------------------------------------

public static void Print(bool insertBlankLine,ConsoleColor? color,object obj)
{
   Print(insertBlankLine,color,obj.ToString());
}

public static void Print(bool insertBlankLine,ConsoleColor? color,string str)
{
   lock (typeof(XConsole))
      Output( color, "{0}{1}\n",
         insertBlankLine ? "\n" : "",
         str );
}

public static void Print( bool insertBlankLine, ConsoleColor? color,
                          string format, params object[] parameters )
{
   Print(insertBlankLine,color,string.Format(format,parameters));
}

//--------------------------------------------------------------------------------------------------

public static void PrintDefault(bool insertBlankLine,object obj)
{
   Print(insertBlankLine,Console.ForegroundColor,obj);
}

public static void PrintDefault(bool insertBlankLine,string str)
{
   Print(insertBlankLine,Console.ForegroundColor,str);
}

public static void PrintDefault(bool insertBlankLine,string format,params object[] parameters)
{
   Print(insertBlankLine,Console.ForegroundColor,string.Format(format,parameters));
}

//==================================================================================================

public static void PrintIntensive(bool insertBlankLine,object obj)
   { Print(insertBlankLine,IntensiveColor,obj); }

public static void PrintIntensive(bool insertBlankLine,string str)
   { Print(insertBlankLine,IntensiveColor,str); }

public static void PrintIntensive(bool insertBlankLine,string format,params object[] parameters)
   { Print(insertBlankLine,IntensiveColor,format,parameters); }

//--------------------------------------------------------------------------------------------------

public static void PrintFaint(bool insertBlankLine,object obj)
   { Print(insertBlankLine,FaintColor,obj); }

public static void PrintFaint(bool insertBlankLine,string str)
   { Print(insertBlankLine,FaintColor,str); }

public static void PrintFaint(bool insertBlankLine,string format,params object[] parameters)
   { Print(insertBlankLine,FaintColor,format,parameters); }

//--------------------------------------------------------------------------------------------------

public static void PrintExpressive(bool insertBlankLine,object obj)
   { Print(insertBlankLine,ExpressiveColor,obj); }

public static void PrintExpressive(bool insertBlankLine,string str)
   { Print(insertBlankLine,ExpressiveColor,str); }

public static void PrintExpressive(bool insertBlankLine,string format,params object[] parameters)
   { Print(insertBlankLine,ExpressiveColor,format,parameters); }

//--------------------------------------------------------------------------------------------------

public static void PrintImportant(bool insertBlankLine,object obj)
   { Print(insertBlankLine,ImportantColor,obj); }

public static void PrintImportant(bool insertBlankLine,string str)
   { Print(insertBlankLine,ImportantColor,str); }

public static void PrintImportant(bool insertBlankLine,string format,params object[] parameters)
   { Print(insertBlankLine,ImportantColor,format,parameters); }

//--------------------------------------------------------------------------------------------------

public static void PrintExclamation(bool insertBlankLine,object obj)
   { Print(insertBlankLine,ExclamationColor,obj); }

public static void PrintExclamation(bool insertBlankLine,string str)
   { Print(insertBlankLine,ExclamationColor,str); }

public static void PrintExclamation( bool insertBlankLine,
                                     string format,
                                     params object[] parameters )
   { Print(insertBlankLine,ExclamationColor,format,parameters); }

//--------------------------------------------------------------------------------------------------

public static void PrintInformation(bool insertBlankLine,object obj)
   { Print(insertBlankLine,InformationColor,obj); }

public static void PrintInformation(bool insertBlankLine,string str)
   { Print(insertBlankLine,InformationColor,str); }

public static void PrintInformation( bool insertBlankLine,
                                     string format,
                                     params object[] parameters )
   { Print(insertBlankLine,InformationColor,format,parameters); }

//--------------------------------------------------------------------------------------------------

public static void PrintWarning(bool insertBlankLine,object obj)
   { Print(insertBlankLine,WarningColor,obj); }

public static void PrintWarning(bool insertBlankLine,string str)
   { Print(insertBlankLine,WarningColor,str); }

public static void PrintWarning( bool insertBlankLine,
                                 string format,
                                 params object[] parameters )
   { Print(insertBlankLine,WarningColor,format,parameters); }

//==================================================================================================

public static void PrintError(bool insertBlankLine,string str)
   { Print(insertBlankLine,ErrorColor,str); }

public static void PrintError(bool insertBlankLine,string format,params object[] parameters)
   { Print(insertBlankLine,ErrorColor,format,parameters); }

public static void PrintError(bool insertBlankLine,Exception e)
   { Print(insertBlankLine,ErrorColor,XException.GetErrorMessage(e)); }

//--------------------------------------------------------------------------------------------------

public static void PrintSuccess(bool insertBlankLine,string str)
   { Print(insertBlankLine,SuccessColor,str); }

public static void PrintSuccess(bool insertBlankLine,string format,params object[] parameters)
   { Print(insertBlankLine,SuccessColor,format,parameters); }

//==================================================================================================

public static void PrintOperation(bool insertBlankLine,string operation)
{
   Print( insertBlankLine, OperationColor,
          string.Format(OperationFormatString,operation) );
}

public static void PrintOperation( bool insertBlankLine,
                                   string operationFmt,
                                   params object[] parameters )
{
   PrintOperation(insertBlankLine,string.Format(operationFmt,parameters));
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XConsole
//**************************************************************************************************

} // XHelpers
