﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
[XSyncOnlyDisposable]
public partial class XConsoleColorRetention: XDisposable {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public XConsoleColorRetention(ConsoleColor? newForegroundColor)
   :this(newForegroundColor,null)
{}

public XConsoleColorRetention( ConsoleColor? newForegroundColor,
                               ConsoleColor? newBackgroungColor )
{
   if (newForegroundColor.HasValue)
   {
      _PreviousForegroundColor_ = Console.ForegroundColor;
      Console.ForegroundColor = (_ForegroundColor_=newForegroundColor).Value;
   }
   if (newBackgroungColor.HasValue)
   {
      _PreviousBackgroundColor_ = Console.BackgroundColor;
      Console.BackgroundColor = (_BackgroundColor_=newBackgroungColor).Value;
   }
   ValidateDisposable();
}

//--------------------------------------------------------------------------------------------------

public ConsoleColor? ForegroundColor
   { get { return _ForegroundColor_; } }
public ConsoleColor? BackgroundColor
   { get { return _BackgroundColor_; } }

public ConsoleColor? PreviousForegroundColor
   { get { return _PreviousForegroundColor_; } }
public ConsoleColor? PreviousBackgroundColor
   { get { return _PreviousBackgroundColor_; } }

//==================================================================================================

protected override void OnDispose(bool synchronous)
{
   if (_PreviousForegroundColor_.HasValue)
      Console.ForegroundColor = _PreviousForegroundColor_.Value;
   if (_PreviousBackgroundColor_.HasValue)
      Console.BackgroundColor = _PreviousBackgroundColor_.Value;
}

//==================================================================================================

static XConsoleColorRetention()
{
   Library.Access();
}
//==================================================================================================

ConsoleColor?
   _ForegroundColor_, _BackgroundColor_,
   _PreviousForegroundColor_, _PreviousBackgroundColor_;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XConsoleColorRetention
//**************************************************************************************************

} // XHelpers
