﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
[XSyncOnlyDisposable]
public partial class XConsoleInputIdlePollingContext: XDisposable {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public XConsoleInputIdlePollingContext(XConsoleInputIdlePollingCallback callback)
   :this( callback, XConsole.InputIdlePollingDefaultInterval )
{}

public XConsoleInputIdlePollingContext( XConsoleInputIdlePollingCallback callback,
                                        int interval )
{
   XArgCheck.VerifyNotNull( callback );
   if (interval<0)
      throw new ArgumentOutOfRangeException();
   lock (typeof(XConsole))
   {
      if (XConsole.InputIdlePollingContext!=null)
         throw new InvalidOperationException(
            "Input idle polling context is already active." );
      _Callback_ = callback;
      _Interval_ = interval;
      ValidateDisposable();
      XConsole.SetInputIdlePollingContext( this );
   }
}

//--------------------------------------------------------------------------------------------------

public XConsoleInputIdlePollingCallback Callback
{
   get { return _Callback_; }
}

public int Interval
{
   get { return _Interval_; }
}

//==================================================================================================

protected override void OnDispose(bool synchronous)
{
   XConsole.SetInputIdlePollingContext( null );
}

//==================================================================================================

static XConsoleInputIdlePollingContext()
{
   Library.Access();
}

//==================================================================================================

XConsoleInputIdlePollingCallback
   _Callback_;
int
   _Interval_;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XConsoleInputIdlePollingContext
//**************************************************************************************************

} // XHelpers
