﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
partial class XConsole {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool InputStringModify([Optional] ref string inputString)
{
   return InputStringModify(null,ref inputString);
}

public static bool InputStringModify( [Optional] ref string inputString,
                                      XConsoleInputOptions options )
{
   return InputStringModify(null,ref inputString,options);
}

public static bool InputStringModify( [Optional] ref string inputString,
                                      XValidateConsoleInputCallback validationCallback )
{
   return InputStringModify(null,ref inputString,validationCallback);
}

public static bool InputStringModify( [Optional] ref string inputString,
                                      XConsoleInputOptions options,
                                      XValidateConsoleInputCallback validationCallback )
{
   return InputStringModify(null,ref inputString,options,validationCallback);
}

//==================================================================================================

public static bool InputStringModify( [Optional] string caption,
                                      [Optional] ref string inputString )
{
   return InputStringModify(caption,0,ref inputString);
}

public static bool InputStringModify( [Optional] string caption,
                                      [Optional] ref string inputString,
                                      XConsoleInputOptions options )
{
   return InputStringModify(caption,0,ref inputString,options);
}

public static bool InputStringModify
(
   [Optional] string caption,
   [Optional] ref string inputString,
   [Optional] XValidateConsoleInputCallback validationCallback
)
{
   return InputStringModify(caption,0,ref inputString,validationCallback);
}

public static bool InputStringModify
(
   [Optional] string caption,
   [Optional] ref string inputString,
   XConsoleInputOptions options,
   [Optional] XValidateConsoleInputCallback validationCallback
)
{
   return InputStringModify(caption,0,ref inputString,options,validationCallback);
}

//--------------------------------------------------------------------------------------------------

public static bool InputStringModify( [Optional] string caption,
                                      ConsoleColor? captionColor,
                                      [Optional] ref string inputString )
{
   return InputStringModify(caption,captionColor,0,ref inputString);
}

public static bool InputStringModify( [Optional] string caption,
                                      ConsoleColor? captionColor,
                                      [Optional] ref string inputString,
                                      XConsoleInputOptions options )
{
   return InputStringModify(caption,captionColor,0,ref inputString,options);
}

public static bool InputStringModify
(
   [Optional] string caption,
   ConsoleColor? captionColor,
   [Optional] ref string inputString,
   [Optional] XValidateConsoleInputCallback validationCallback
)
{
   return InputStringModify(caption,captionColor,0,ref inputString,validationCallback);
}

public static bool InputStringModify
(
   [Optional] string caption,
   ConsoleColor? captionColor,
   [Optional] ref string inputString,
   XConsoleInputOptions options,
   [Optional] XValidateConsoleInputCallback validationCallback
)
{
   return InputStringModify(caption,captionColor,0,ref inputString,options,validationCallback);
}

//==================================================================================================

public static bool InputStringModify( string caption,
                                      [Optional] int maxCaptionLength,
                                      [Optional] ref string inputString )
{
   return InputStringModify(
      caption, null, maxCaptionLength,
      ref inputString, null, null, 0, null );
}

public static bool InputStringModify( string caption,
                                      [Optional] int maxCaptionLength,
                                      [Optional] ref string inputString,
                                      XConsoleInputOptions options )
{
   return InputStringModify(
      caption, null, maxCaptionLength,
      ref inputString, null, null, options, null );
}

public static bool InputStringModify
(
   string caption,
   [Optional] int maxCaptionLength,
   [Optional] ref string inputString,
   [Optional] XValidateConsoleInputCallback validationCallback
)
{
   return InputStringModify(
      caption, null, maxCaptionLength,
      ref inputString, null, null, 0, validationCallback );
}

public static bool InputStringModify
(
   string caption,
   [Optional] int maxCaptionLength,
   [Optional] ref string inputString,
   XConsoleInputOptions options,
   [Optional] XValidateConsoleInputCallback validationCallback
)
{
   return InputStringModify(
      caption, null, maxCaptionLength,
      ref inputString, null, null, options, validationCallback );
}

//--------------------------------------------------------------------------------------------------

public static bool InputStringModify( string caption,
                                      ConsoleColor? captionColor,
                                      [Optional] int maxCaptionLength,
                                      [Optional] ref string inputString )
{
   return InputStringModify(
      caption, captionColor, maxCaptionLength,
      ref inputString, null, null, 0, null );
}

public static bool InputStringModify( string caption,
                                      ConsoleColor? captionColor,
                                      [Optional] int maxCaptionLength,
                                      [Optional] ref string inputString,
                                      XConsoleInputOptions options )
{
   return InputStringModify(
      caption, captionColor, maxCaptionLength,
      ref inputString, null, null, options, null );
}

public static bool InputStringModify
(
   string caption,
   ConsoleColor? captionColor,
   [Optional] int maxCaptionLength,
   [Optional] ref string inputString,
   [Optional] XValidateConsoleInputCallback validationCallback
)
{
   return InputStringModify(
      caption, captionColor, maxCaptionLength,
      ref inputString, null, null, 0, validationCallback );
}

public static bool InputStringModify
(
   string caption,
   ConsoleColor? captionColor,
   [Optional] int maxCaptionLength,
   [Optional] ref string inputString,
   XConsoleInputOptions options,
   [Optional] XValidateConsoleInputCallback validationCallback
)
{
   return InputStringModify(
      caption, captionColor, maxCaptionLength,
      ref inputString, null, null, options, validationCallback );
}

//==================================================================================================

public static bool InputStringModify
(
   [Optional] string caption,
   ConsoleColor? captionColor,
   [Optional] int maxCaptionLength,
   [Optional] ref string inputString,
   ConsoleColor? foregroundColor,
   ConsoleColor? backgroundColor,
   XConsoleInputOptions options,
   [Optional] XValidateConsoleInputCallback validationCallback
)
{
   if (caption!=null)
      OutputArrowAlignedInputPrompt( caption,
         captionColor ?? InputCaptionColor,
         maxCaptionLength );

   lock (typeof(XConsole))
   {
      foregroundColor = foregroundColor ?? InputForegroundColor;
      backgroundColor = backgroundColor ?? InputBackgroundColor;

      bool result = false;

      try
      {
         using (var colorRetention=new XConsoleColorRetention(foregroundColor,backgroundColor))
         {
            string inputRoutineResult =
               InputRoutine(colorRetention,inputString,options,validationCallback);
            if (inputRoutineResult!=null)
            {
               inputString = inputRoutineResult;
               result = true;
            }
         }
      }
      catch (XAbortConsoleInputException)
      {
         Print(); throw;
      }

      Print();

      return result;
   }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XConsole
//**************************************************************************************************

} // XHelpers
