﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Threading;
using System.ComponentModel;

namespace XHelpers {

//**************************************************************************************************
partial class XConsole {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

[DefaultValue(false)]
public static bool HandlesCancelEvent
{
   get
   {
      lock (typeof(XConsole))
         return _HandlesCancelEvent_;
   }
   set
   {
      lock (typeof(XConsole))
      {
         if (value && !_HandlesCancelEvent_)
            Console.CancelKeyPress += CancelEventHandler;
         else
            if (!value && _HandlesCancelEvent_)
               Console.CancelKeyPress -= CancelEventHandler;
         _HandlesCancelEvent_ = value;
      }
   }
}

//==================================================================================================

public static void ProcessCancelEvent()
{
   lock (_CancellationSyncObject_)
   {
      Monitor.TryEnter(typeof(XConsole),100);

      Console.ResetColor();
      Console.WriteLine();

      if ( CancellationColor.HasValue )
         Console.ForegroundColor = CancellationColor.Value;
      Console.WriteLine(
         CancellationFormatString,
         CancellationMessage );

      Console.ResetColor();

      Console.Beep(
         CancellationBeepFrequency,
         CancellationBeepDuration );
   }
}

//==================================================================================================

static void CancelEventHandler(object sender,ConsoleCancelEventArgs e)
{
   ProcessCancelEvent();
}

//==================================================================================================

static bool
   _HandlesCancelEvent_;

static readonly object
   _CancellationSyncObject_ = new object();

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XConsole
//**************************************************************************************************

} // XHelpers
