﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Text;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
partial class XConsole {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool IsCharPrintable(char ch)
{
   return IsCharPrintable(ch,true);
}

public static bool IsCharPrintable(char ch,bool allowSpecial)
{
   if (ch=='\0')
      return false;

   byte[] oemChars = new byte[1];
   char[] unicodeChars = new char[1];

   Win32.CharToOemBuff( new char[] { ch }, oemChars, 1 );
   Win32.OemToCharBuff( oemChars, unicodeChars, 1 );

   if ( unicodeChars[0]!=ch ||
        XOps.IsOneOf( (int)oemChars[0],
            XAscii.BEL, XAscii.BS, XAscii.HT, XAscii.LF, XAscii.CR ) )
      return false;

   if (allowSpecial)
      return true;

   int oemCode = (int)oemChars[0];

   return
      !XAscii.IsControl(oemCode,true) &&
      !XOps.IsBetween(oemCode,0xB0,0xDF) &&
      oemCode!=0xFF;
}

//--------------------------------------------------------------------------------------------------

public static char CharToPrintable(char ch)
{
   return
      !IsCharPrintable(ch) ||
      ExactDisplaying && (!IsCharPrintable(ch) || ch==XUnicode.Bullet) ?
         NonDisplayableSubstChar :
         ch==XUnicode.Bullet ? BulletChar : ch;
}

public static string StringToPrintable(string str)
{
   var sb=new StringBuilder();
   foreach (char ch in str)
      sb.Append( CharToPrintable(ch) );
   return sb.ToString();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XConsole
//**************************************************************************************************

} // XHelpers
