﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Text;

namespace XHelpers {

//**************************************************************************************************
partial class XConsole {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static void Output(object obj)
{
   Output(obj.ToString());
}

public static void Output(string str)
{
   Output(null,str);
}

public static void Output(string format,params object[] parameters)
{
   Output(null,format,parameters);
}

//--------------------------------------------------------------------------------------------------

public static void Output(ConsoleColor? color,object obj)
{
   Output(color,obj.ToString());
}

public static void Output(ConsoleColor? color,string str)
{
   lock (typeof(XConsole))
      using (new XConsoleColorRetention(
         color.HasValue ? color.Value : NormalColor.Value ))
      {
         var sb=new StringBuilder();
         char? prevChar=null;
         foreach (char ch in str)
            if (ch=='\n')
            {
               if (prevChar=='\r')
                  sb.Length--;
               PerformOutput(sb,true);
               sb.Length=0;
               prevChar=null;
            }
            else
               sb.Append(prevChar=ch);
         PerformOutput(sb,false);
      }
}

public static void Output(ConsoleColor? color,string format,params object[] parameters)
{
   Output(color,string.Format(format,parameters));
}

//==================================================================================================

public static void OutputDefault(object obj)
{
   Output(Console.ForegroundColor,obj);
}

public static void OutputDefault(string str)
{
   Output(Console.ForegroundColor,str);
}

public static void OutputDefault(string format,params object[] parameters)
{
   Output(Console.ForegroundColor,string.Format(format,parameters));
}

//==================================================================================================

public static void OutputIntensive(object obj)
   { Output(IntensiveColor,obj); }

public static void OutputIntensive(string str)
   { Output(IntensiveColor,str); }

public static void OutputIntensive(string format,params object[] parameters)
   { Output(IntensiveColor,format,parameters); }

//--------------------------------------------------------------------------------------------------

public static void OutputFaint(object obj)
   { Output(FaintColor,obj); }

public static void OutputFaint(string str)
   { Output(FaintColor,str); }

public static void OutputFaint(string format,params object[] parameters)
   { Output(FaintColor,format,parameters); }

//--------------------------------------------------------------------------------------------------

public static void OutputExpressive(object obj)
   { Output(ExpressiveColor,obj); }

public static void OutputExpressive(string str)
   { Output(ExpressiveColor,str); }

public static void OutputExpressive(string format,params object[] parameters)
   { Output(ExpressiveColor,format,parameters); }

//--------------------------------------------------------------------------------------------------

public static void OutputImportant(object obj)
   { Output(ImportantColor,obj); }

public static void OutputImportant(string str)
   { Output(ImportantColor,str); }

public static void OutputImportant(string format,params object[] parameters)
   { Output(ImportantColor,format,parameters); }

//--------------------------------------------------------------------------------------------------

public static void OutputExclamation(object obj)
   { Output(ExclamationColor,obj); }

public static void OutputExclamation(string str)
   { Output(ExclamationColor,str); }

public static void OutputExclamation( 
                                     string format,
                                     params object[] parameters )
   { Output(ExclamationColor,format,parameters); }

//--------------------------------------------------------------------------------------------------

public static void OutputInformation(object obj)
   { Output(InformationColor,obj); }

public static void OutputInformation(string str)
   { Output(InformationColor,str); }

public static void OutputInformation( 
                                     string format,
                                     params object[] parameters )
   { Output(InformationColor,format,parameters); }

//--------------------------------------------------------------------------------------------------

public static void OutputWarning(object obj)
   { Output(WarningColor,obj); }

public static void OutputWarning(string str)
   { Output(WarningColor,str); }

public static void OutputWarning( 
                                 string format,
                                 params object[] parameters )
   { Output(WarningColor,format,parameters); }

//==================================================================================================

static void PerformOutput(StringBuilder sb,bool feedLine)
{
   for (int i=0; i<sb.Length; i++)
      sb[i] = CharToPrintable( sb[i] );
   if ( IsInVisualMode && sb.Length>0 &&
        (Console.CursorLeft+sb.Length) % Console.BufferWidth == 0 )
      feedLine = false;
   if (feedLine)
      Console.WriteLine( sb.ToString() );
   else
      Console.Write( sb.ToString() );
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XConsole
//**************************************************************************************************

} // XHelpers
