﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Text;
using System.Security.Permissions;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
[PermissionSetAttribute(SecurityAction.Demand,Unrestricted=true)]
public static partial class XConsole {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool IsInVisualMode
{
   get
   {
      lock (typeof(XConsole))
         return _IsInVisualMode_;
   }
}

/// <exception cref="InvalidOperationException" />
/// <exception cref="XOperationFailedException" />
public static IntPtr WindowHandle
{
   get
   {
      VerifyIsInVisualMode();
      IntPtr handle = Win32.GetConsoleWindow();
      if (handle==IntPtr.Zero)
         throw new XOperationFailedException();
      return handle;
   }
}

//--------------------------------------------------------------------------------------------------

public static int Width
{
   get
   {
      return IsInVisualMode ? Console.BufferWidth : NormalWidth;
   }
}

public static int Height
{
   get
   {
      return IsInVisualMode ? Console.BufferHeight : NormalHeight;
   }
}

//==================================================================================================

/// <exception cref="InvalidOperationException" />
public static void VerifyIsInVisualMode()
{
   if (!IsInVisualMode)
      throw new InvalidOperationException(
         "Requested console operation requires visual mode." );
}

//==================================================================================================

static XConsole()
{
   Library.Access();

   IntPtr hStdOutput =
      Win32.GetStdHandle( Win32.STD_OUTPUT_HANDLE );
   if (hStdOutput==Win32.INVALID_HANDLE_VALUE)
      throw XWin32Error.CreateExceptionFromLastError();
   Win32.CONSOLE_SCREEN_BUFFER_INFO
      screenBufferInfo;
   if (Win32.GetConsoleScreenBufferInfo(hStdOutput,out screenBufferInfo))
      _IsInVisualMode_ = true;
   else
   {
      if (XWin32Error.LastErrorCode!=Win32.ERROR_INVALID_HANDLE)
         throw XWin32Error.CreateExceptionFromLastError();
      CustomizeForAsciiAppearance();
   }
}

//==================================================================================================

static bool
   _IsInVisualMode_;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XConsole
//**************************************************************************************************

} // XHelpers
