﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XOps {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static Action Nop { get { return _Nop_; } }

//==================================================================================================

public static bool IsDefault<T>(T value) where T: struct
{
   return value.Equals( default(T) );
}

//==================================================================================================

public static void Swap<T>(ref T value_1,ref T value_2)
{
   T temp=value_1; value_1=value_2; value_2=temp;
}

public static T Exchange<T>(ref T target,T value)
{
   T prev=target; target=value; return prev;
}

public static T NullExchange<T>(ref T target) where T: class
{
   return Exchange(ref target,null);
}

public static T DefaultExchange<T>(ref T target)
{
   return Exchange(ref target,default(T));
}

//==================================================================================================

public static T NullSubstitution<T>(T value,T subst) where T: class
{
   return value!=null ? value : subst;
}

public static void NullSubstitution<T>(ref T operand,T subst) where T: class
{
   if (operand==null)
      operand = subst;
}

//--------------------------------------------------------------------------------------------------

public static T DefaultSubstitution<T>(T value,T subst) where T: struct
{
   return !IsDefault(value) ? value : subst;
}

public static void DefaultSubstitution<T>(ref T operand,T subst) where T: struct
{
   if (IsDefault(operand))
      operand = subst;
}

//==================================================================================================

public static T Min<T>(T value_1,T value_2) where T: IComparable
{
   return value_1.CompareTo(value_2)<=0 ? value_1 : value_2;
}

public static T Max<T>(T value_1,T value_2) where T: IComparable
{
   return value_1.CompareTo(value_2)>=0 ? value_1 : value_2;
}

//==================================================================================================

public static bool IsBetween<T>(T value,T lowerBound,T upperBound) where T: IComparable<T>
{
   return value.CompareTo(lowerBound)>=0 && value.CompareTo(upperBound)<=0;
}

public static bool IsExBetween<T>(T value,T lowerBound,T upperBound) where T: IComparable<T>
{
   return value.CompareTo(lowerBound)>0 && value.CompareTo(upperBound)<0;
}

//==================================================================================================

public static bool IsOneOf(object value,params object[] values)
{
   XArgCheck.VerifyArrayNotEmpty( values );
   return XObjectCollection.Contains(values,value);
}

//--------------------------------------------------------------------------------------------------

public static int IndexOf(object value,params object[] values)
{
   XArgCheck.VerifyArrayNotEmpty( values );
   return XObjectCollection.IndexOf(values,value);
}

public static int StrictIndexOf(object value,params object[] values)
{
   XArgCheck.VerifyArrayNotEmpty( values );
   return XObjectCollection.StrictIndexOf(values,value);
}

//==================================================================================================

public static bool CheckNotNulls(params object[] values)
{
   XArgCheck.VerifyArrayNotEmpty( values );
   foreach (object value in values)
      if (value==null)
         return false;
   return true;
}

//==================================================================================================

static XOps()
{
   Library.Access();
}
//==================================================================================================

static readonly Action
   _Nop_ = delegate {};

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XOps
//**************************************************************************************************

} // XHelpers
