﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Collections.Generic;

namespace XHelpers {

//**************************************************************************************************
partial class XString {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static int MinLength(IEnumerable<string> collection)
{
   int index; return GetMinLength(collection,out index);
}

public static int MaxLength(IEnumerable<string> collection)
{
   int index; return GetMaxLength(collection,out index);
}

public static int MinLength(params string[] array)
{
   return MinLength((IEnumerable<string>)array);
}

public static int MaxLength(params string[] array)
{
   return MaxLength((IEnumerable<string>)array);
}

//--------------------------------------------------------------------------------------------------

public static int GetMinLength(IEnumerable<string> collection,out int index)
{
   return GetMinOrMaxLength(collection,XMinOrMax.Min,out index);
}

public static int GetMaxLength(IEnumerable<string> collection,out int index)
{
   return GetMinOrMaxLength(collection,XMinOrMax.Max,out index);
}

//--------------------------------------------------------------------------------------------------

public static int MinLengthIndex(IEnumerable<string> collection)
{
   int index; GetMinLength(collection,out index); return index;
}

public static int MaxLengthIndex(IEnumerable<string> collection)
{
   int index; GetMaxLength(collection,out index); return index;
}

public static int MinLengthIndex(params string[] array)
{
   return MinLengthIndex((IEnumerable<string>)array);
}

public static int MaxLengthIndex(params string[] array)
{
   return MaxLengthIndex((IEnumerable<string>)array);
}

//==================================================================================================

public static int GetMinOrMaxLength( IEnumerable<string> collection,
                                     XMinOrMax minmax,
                                     out int index )
{
   XArgCheck.VerifyNotNull(collection);

   int result=0; index=-1;

   using (IEnumerator<string> enumerator=collection.GetEnumerator())
   {
      if (!enumerator.MoveNext())
         return result;
      int
         idx=0;
      do
      {
         string element=enumerator.Current;

         if (element!=null)
         {
            int len=element.Length;

            if ( index<0 ||
                 minmax==XMinOrMax.Min && len<result ||
                 minmax==XMinOrMax.Max && len>result )
            {
               result=len; index=idx;
            }
         }

         idx++;
      }
      while (enumerator.MoveNext());
   }

   return result;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XString
//**************************************************************************************************

} // XHelpers
