﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Text;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
[Flags]
public enum XStringSplitOptions
{
   Auto = 0,

   TrimElementStart          = 1 << 0,
   TrimElementEnd            = 1 << 1,
   RemoveEmptyEntries        = 1 << 2,
   AlwaysReturnNonEmptyArray = 1 << 3,

   TrimElement =
      TrimElementStart | TrimElementEnd,
   WhiteSpaceSeparatorDefaults =
      RemoveEmptyEntries,
   NonWhiteSpaceSeparatorDefaults =
      TrimElement
}
//**************************************************************************************************


//**************************************************************************************************
partial class XString {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static string[] Split(string str)
{
   return Split(str,(char[])null);
}

public static string[] Split(string str,XStringSplitOptions options)
{
   return Split(str,(char[])null,options);
}

//--------------------------------------------------------------------------------------------------

public static string[] Split(string str,char separator)
{
   return Split(str,new char[]{separator});
}

public static string[] Split(string str,char separator,XStringSplitOptions options)
{
   return Split(str,new char[]{separator},options);
}

//--------------------------------------------------------------------------------------------------

public static string[] Split(string str,[Optional] string separators)
{
   return Split(str,ToCharArray(separators));
}

public static string[] Split(string str,[Optional] string separators,XStringSplitOptions options)
{
   return Split(str,ToCharArray(separators),options);
}

//--------------------------------------------------------------------------------------------------

public static string[] Split(string str,[Optional] char[] separators)
{
   return Split(str,separators,XStringSplitOptions.Auto);
}

public static string[] Split(string str,[Optional] char[] separators,XStringSplitOptions options)
{
   XArgCheck.VerifyNotNull( str );

   if (options==XStringSplitOptions.Auto)
      options |=
         separators==null ||
         XArray.Exists( separators, ch => char.IsWhiteSpace( ch ) ) ?
            XStringSplitOptions.WhiteSpaceSeparatorDefaults :
            XStringSplitOptions.NonWhiteSpaceSeparatorDefaults;
   var
      list = new List<string>();
   int
      len = str.Length,
      i = 0;

   while (i<len)
   {
      var sb = new StringBuilder();
      bool hasSeparator = false;

      while (i<len)
      {
         char ch = str[i];
         if ( separators!=null && XArray.Contains(separators,ch) ||
              separators==null && char.IsWhiteSpace(ch) )
         {
            hasSeparator = true;
            break;
         }
         sb.Append( ch );
         i++;
      }

      string element = sb.ToString();

      if ( (options & XStringSplitOptions.TrimElementStart)!=0 &&
           (options & XStringSplitOptions.TrimElementEnd  )!=0 )
         element = element.Trim();
      else
         if ((options & XStringSplitOptions.TrimElementStart)!=0)
            element = element.TrimStart();
         else
            if ((options & XStringSplitOptions.TrimElementEnd)!=0)
               element = element.TrimEnd();

      if (!(
         element=="" &&
         (options & XStringSplitOptions.RemoveEmptyEntries)!=0
      ))
         list.Add( element );

      if ( i==len-1 && hasSeparator &&
           (options & XStringSplitOptions.RemoveEmptyEntries)==0 )
         list.Add("");

      i++;
   }

   if ( list.Count==0 &&
        (options & XStringSplitOptions.AlwaysReturnNonEmptyArray)!=0 )
      list.Add( "" );

   return list.ToArray();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XString
//**************************************************************************************************

} // XHelpers
