﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Text;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XString {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool Equal(string str_1,string str_2)
{
   return Equal(str_1,str_2,false);
}

public static bool Equal(string str_1,string str_2,bool ignoreCase)
{
   return CompareOrdinal(str_1,str_2,ignoreCase)==0;
}

//--------------------------------------------------------------------------------------------------

public static int CompareOrdinal(string str_1,string str_2)
{
   return CompareOrdinal(str_1,str_2,false);
}

public static int CompareOrdinal(string str_1,string str_2,bool ignoreCase)
{
   return
      string.Compare( str_1, str_2,
         ignoreCase ?
            StringComparison.OrdinalIgnoreCase :
            StringComparison.Ordinal );
}

//--------------------------------------------------------------------------------------------------

public static int CompareInvariant(string str_1,string str_2)
{
   return CompareInvariant(str_1,str_2,false);
}

public static int CompareInvariant(string str_1,string str_2,bool ignoreCase)
{
   return
      string.Compare( str_1, str_2,
         ignoreCase ?
            StringComparison.InvariantCultureIgnoreCase :
            StringComparison.InvariantCulture );
}

//--------------------------------------------------------------------------------------------------

public static bool IsOneOf(string str,params string[] strings)
{
   return IsOneOf(str,strings,false);
}

public static bool IsOneOf(string str,string[] strings,bool ignoreCase)
{
   int idx=GetIndexOf(str,strings,ignoreCase);
   return idx>=strings.GetLowerBound(0);
}

//--------------------------------------------------------------------------------------------------

public static int GetIndexOf(string str,params string[] strings)
{
   return GetIndexOf(str,strings,false);
}

public static int GetIndexOf(string str,string[] strings,bool ignoreCase)
{
   for (int i=strings.GetLowerBound(0); i<=strings.GetUpperBound(0); i++)
      if (Equal(str,strings[i],ignoreCase))
         return i;
   return strings.GetLowerBound(0)-1;
}

//--------------------------------------------------------------------------------------------------

public static int StrictIndexOf(string str,params string[] strings)
{
   return StrictIndexOf(str,strings,false);
}

public static int StrictIndexOf(string str,string[] strings,bool ignoreCase)
{
   int idx=GetIndexOf(str,strings,ignoreCase);
   if (idx<strings.GetLowerBound(0))
      throw new XElementNotFoundException();
   return idx;
}

//==================================================================================================

public static bool IsBlank(string str)
{
   return str.Trim()=="";
}

public static bool IsSpaceNormalizedName(string str)
{
   return !( IsBlank(str) || StartsWithSpaces(str) ||
             EndsWithSpaces(str) || ContainsDoubleSpaces(str) );
}

//--------------------------------------------------------------------------------------------------

public static bool StartsWithSpaces(string str)
{
   return str.TrimStart()!=str;
}

public static bool EndsWithSpaces(string str)
{
   return str.TrimEnd()!=str;
}

public static bool ContainsSpaces(string str)
{
   return str.Contains(" ");
}

public static bool ContainsDoubleSpaces(string str)
{
   return str.Contains("  ");
}

//==================================================================================================

public static char[] ToCharArray([Optional] string str)
{
   return ToCharArray(str,false);
}

public static char[] ToCharArray([Optional] string str,bool nullToEmpty)
{
   return str!=null ?
      str.ToCharArray() :
      (nullToEmpty ? new char[0] : null);
}

//--------------------------------------------------------------------------------------------------

public static string FromCharArray([Optional] char[] chars)
{
   return FromCharArray(chars,false);
}

public static string FromCharArray([Optional] char[] chars,bool nullToEmpty)
{
   return chars!=null ?
      new string(chars) :
      (nullToEmpty ? "" : null);
}

//==================================================================================================

public static string SeparatedString<T>(T[] parameters)
{
   return SeparatedString(parameters,null);
}

public static string SeparatedString<T>(T[] parameters,char separator)
{
   return SeparatedString(parameters,separator.ToString());
}

public static string SeparatedString<T>(T[] parameters,[Optional] string separator)
{
   XArgCheck.VerifyNotNulls( parameters );
   separator = separator ?? ", ";
   var sb = new StringBuilder();
   bool wasAppended = false;
   foreach (T value in parameters)
   {
      if (wasAppended)
         sb.Append( separator );
      sb.Append( value.ToString() );
      wasAppended = true;
   }
   return sb.ToString();
}

//==================================================================================================

static XString()
{
   Library.Access();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XString
//**************************************************************************************************

} // XHelpers
