﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
public static partial class XArgCheck {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static void VerifyIsTypeOf(Type type,Type comparandType)
{
   VerifyNotNulls( type, comparandType );
   if (!XType.IsTypeOf( type, comparandType ))
      throw new XArgumentTypeException(string.Format(
         "Type is not \"{0}\" type.",
         comparandType ));
}

public static void VerifyIsValueOf(object value,Type comparandType)
{
   VerifyNotNull( value );
   if (!XType.IsValueOf( value, comparandType ))
      throw new XArgumentTypeException(string.Format(
         "Value is not of \"{0}\" type.",
         comparandType ));
}

//==================================================================================================

public static void VerifyTypeIsByRef(Type type)
{
   VerifyNotNull( type );
   if (!type.IsByRef)
      throw new XArgumentTypeException(string.Format(
         "Type \"{0}\" is not reference type.",
         type ));
}

public static void VerifyTypeIsValueType(Type type)
{
   VerifyNotNull( type );
   if (!type.IsValueType)
      throw new XArgumentTypeException(string.Format(
         "Type \"{0}\" is not value type.",
         type ));
}

public static void VerifyTypeIsEnum(Type type)
{
   VerifyNotNull( type );
   if (!type.IsEnum)
      throw new XArgumentTypeException(string.Format(
         "Type \"{0}\" is not enumeration.",
         type ));
}

//--------------------------------------------------------------------------------------------------

public static void VerifyTypeIsStdValue(Type type)
{
   VerifyNotNull( type );
   if (!XType.IsStdValue( type ))
      throw new XArgumentTypeException(string.Format(
         "Type \"{0}\" is not standard value type.",
         type ));
}

//--------------------------------------------------------------------------------------------------

public static void VerifyTypeIsIntegralNumber(Type type)
{
   VerifyNotNull( type );
   if (!XType.IsIntegralNumber( type ))
      throw new XArgumentTypeException(string.Format(
         "Type \"{0}\" is not integral type.",
         type ));
}

public static void VerifyTypeIsIntegralSignedlNumber(Type type)
{
   VerifyNotNull( type );
   if (!XType.IsIntegralSignedNumber( type ))
      throw new XArgumentTypeException(string.Format(
         "Type \"{0}\" is not integral signed number type.",
         type ));
}

public static void VerifyTypeIsIntegralUnsignedNumber(Type type)
{
   VerifyNotNull( type );
   if (!XType.IsIntegralUnsignedNumber( type ))
      throw new XArgumentTypeException(string.Format(
         "Type \"{0}\" is not integral unsigned number type.",
         type ));
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XArgCheck
//**************************************************************************************************

} // XHelpers
