﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
partial class XArgCheck {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static void VerifyNotNull(object value)
{
   if (value==null)
      throw new ArgumentNullException();
}

public static void VerifyNotNulls(params object[] values)
{
   if (values==null || !XOps.CheckNotNulls(values))
      throw new ArgumentNullException();
}

//==================================================================================================

public static void VerifyNotDefalutValue<T>(T value) where T: struct,IComparable<T>
{
   if (value.CompareTo(default(T))==0)
      throw ArgumentException_DefaultValue;
}

//--------------------------------------------------------------------------------------------------

public static void VerifyNotZeroPointer(IntPtr pointer)
{
   if (pointer==IntPtr.Zero)
      throw ArgumentException_ZeroPointer;
}

public static void VerifyNotInvalidPointer(IntPtr pointer)
{
   if (pointer==XIntPtr.MinusOneValue)
      throw ArgumentException_InvalidPointer;
}

//==================================================================================================

public static void VerifyStringNotEmpty(string str)
{
   VerifyNotNull( str );
   if (str=="")
      throw ArgumentException_EmptyString;
}

public static void VerifyStringsNotEmpty(params string[] strings)
{
   VerifyNotNull( strings );
   foreach (string str in strings)
      VerifyStringNotEmpty( str );
}

//==================================================================================================

public static void VerifyArrayNotEmpty(Array array)
{
   VerifyNotNull( array );
   if (array.Length==0)
      throw ArgumentException_EmptyArray;
}

//--------------------------------------------------------------------------------------------------

public static void VerifyArrayIndex(Array array,int dimension,int index)
{
   VerifyNotNull( array );
   if (!XArray.IsIndexInValidBounds(array,dimension,index))
      throw ArgumentOutOfRangeException_IncorrectArrayIndex;
}

public static void VerifyArrayIndex<T>(T[] array,int index)
{
   VerifyNotNull( array );
   if (!XArray.IsIndexInValidBounds(array,index))
      throw ArgumentOutOfRangeException_IncorrectArrayIndex;
}

public static void VerifyArrayIndex<T>(T[,] array,int dimension,int index)
{
   VerifyNotNull( array );
   if (!XArray2D.IsIndexInValidBounds(array,dimension,index))
      throw ArgumentOutOfRangeException_IncorrectArrayIndex;
}

//--------------------------------------------------------------------------------------------------

public static void VerifyArrayDimension(Array array,int dimension)
{
   VerifyNotNull( array );
   if (!XArray.IsValidDimension(array,dimension))
      throw ArgumentOutOfRangeException_IncorrectArrayDimension;
}

public static void VerifyArrayDimension<T>(T[,] array,int dimension)
{
   VerifyNotNull( array );
   if (!XArray2D.IsValidDimension(array,dimension))
      throw ArgumentOutOfRangeException_IncorrectArrayDimension;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XArgCheck
//**************************************************************************************************

} // XHelpers
