﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XGenericTypeCheck {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static void VerifyIsTypeOf(Type type,Type comparandType)
{
   XArgCheck.VerifyNotNulls( type, comparandType );
   if (!XType.IsTypeOf( type, comparandType ))
      throw new XGenericTypeException(string.Format(
         "Generic type is not \"{0}\" type.",
         comparandType ));
}

public static void VerifyIsTypeOf<T>(Type comparandType)
{
   VerifyIsTypeOf( typeof(T), comparandType );
}

//==================================================================================================

public static void VerifyIsByRef(Type type)
{
   XArgCheck.VerifyNotNull( type );
   if (!type.IsByRef)
      throw new XGenericTypeException(string.Format(
         "Generic type is not reference type.",
         type ));
}

public static void VerifyIsValueType(Type type)
{
   XArgCheck.VerifyNotNull( type );
   if (!type.IsValueType)
      throw new XGenericTypeException(string.Format(
         "Generic type is not value type.",
         type ));
}

public static void VerifyIsEnum(Type type)
{
   XArgCheck.VerifyNotNull( type );
   if (!type.IsEnum)
      throw new XGenericTypeException(string.Format(
         "Generic type is not enumeration.",
         type ));
}

//--------------------------------------------------------------------------------------------------

public static void VerifyIsStdValueType(Type type)
{
   XArgCheck.VerifyNotNull( type );
   if (!XType.IsStdValue( type ))
      throw new XGenericTypeException(string.Format(
         "Generic type is not standerd value type.",
         type ));
}

//--------------------------------------------------------------------------------------------------

public static void VerifyIsIntegralNumber(Type type)
{
   XArgCheck.VerifyNotNull( type );
   if (!XType.IsIntegralNumber( type ))
      throw new XGenericTypeException(string.Format(
         "Generic type is not integral number type.",
         type ));
}

public static void VerifyIsIntegralSignedlNumber(Type type)
{
   XArgCheck.VerifyNotNull( type );
   if (!XType.IsIntegralSignedNumber( type ))
      throw new XGenericTypeException(string.Format(
         "Generic type is not integral signed number type.",
         type ));
}

public static void VerifyIsIntegralUnsignedNumber(Type type)
{
   XArgCheck.VerifyNotNull( type );
   if (!XType.IsIntegralUnsignedNumber( type ))
      throw new XGenericTypeException(string.Format(
         "Generic type is not integral unsigned number type.",
         type ));
}

//==================================================================================================

static XGenericTypeCheck()
{
   Library.Access();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XGenericTypeCheck
//**************************************************************************************************

} // XHelpers
