﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Collections.Generic;

namespace XHelpers {

//**************************************************************************************************
partial class XCollection {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static T Min<T>(IEnumerable<T> collection) where T: IComparable
{
   int index;
   T value = GetMin(collection,out index);
   return index>=0 ? value : default(T);
}

public static T Max<T>(IEnumerable<T> collection) where T: IComparable
{
   int index;
   T value = GetMax(collection,out index);
   return index>=0 ? value : default(T);
}

//--------------------------------------------------------------------------------------------------

/// <exception cref="XElementNotFoundException" />
public static T StrictMin<T>(IEnumerable<T> collection) where T: IComparable
{
   int index;
   T value = GetMin(collection,out index);
   if (index<0)
      throw new XEmptySetException();
   return value;
}

/// <exception cref="XElementNotFoundException" />
public static T StrictMax<T>(IEnumerable<T> collection) where T: IComparable
{
   int index;
   T value = GetMax(collection,out index);
   if (index<0)
      throw new XEmptySetException();
   return value;
}

//==================================================================================================

public static int IndexOfMin<T>(IEnumerable<T> collection) where T: IComparable
{
   int index; GetMin(collection,out index); return index;
}

public static int IndexOfMax<T>(IEnumerable<T> collection) where T: IComparable
{
   int index; GetMax(collection,out index); return index;
}

//--------------------------------------------------------------------------------------------------

/// <exception cref="XElementNotFoundException" />
public static int StrictIndexOfMin<T>(IEnumerable<T> collection) where T: IComparable
{
   int index = IndexOfMin( collection );
   if (index<0)
      throw new XEmptySetException();
   return index;
}

/// <exception cref="XElementNotFoundException" />
public static int StrictIndexOfMax<T>(IEnumerable<T> collection) where T: IComparable
{
   int index = IndexOfMax( collection );
   if (index<0)
      throw new XEmptySetException();
   return index;
}

//==================================================================================================

public static T GetMin<T>(IEnumerable<T> collection,out int index) where T: IComparable
{
   return GetMinOrMax(collection,XMinOrMax.Min,out index);
}

public static T GetMax<T>(IEnumerable<T> collection,out int index) where T: IComparable
{
   return GetMinOrMax(collection,XMinOrMax.Max,out index);
}

//==================================================================================================

static T GetMinOrMax<T>(IEnumerable<T> collection,XMinOrMax minmax,out int index)
   where T: IComparable
{
   XArgCheck.VerifyNotNull( collection );

   T result=default(T); index=-1;

   using (IEnumerator<T> enumerator=collection.GetEnumerator())
   {
      if (!enumerator.MoveNext())
         return result;
      int
         idx=0;
      do
      {
         T element=enumerator.Current;

         if ( element!=null && ( index<0 ||
              minmax==XMinOrMax.Min && element.CompareTo(result)<0 ||
              minmax==XMinOrMax.Max && element.CompareTo(result)>0 ) )
         {
            result=element; index=idx;
         }

         idx++;
      }
      while (enumerator.MoveNext());
   }

   return result;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XCollection
//**************************************************************************************************

} // XHelpers
