﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XCollection {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool IsEmpty<T>(IEnumerable<T> collection)
{
   XArgCheck.VerifyNotNull( collection );
   var
      enumerator = collection.GetEnumerator();
   try
   {
      return !enumerator.MoveNext();
   }
   finally
   {
      if (enumerator is IDisposable)
         ((IDisposable)enumerator).Dispose();
   }
}

//--------------------------------------------------------------------------------------------------

public static int Count<T>(IEnumerable<T> collection)
{
   XArgCheck.VerifyNotNull( collection );
   var
      enumerator = collection.GetEnumerator();
   try
   {
      int count=0;
      while (enumerator.MoveNext())
         count++;
      return count;
   }
   finally
   {
      if (enumerator is IDisposable)
         ((IDisposable)enumerator).Dispose();
   }
}

public static bool Contains<T>(IEnumerable<T> collection,object obj)
{
   return IndexOf(collection,obj)>=0;
}

//==================================================================================================

public static int IndexOf<T>(IEnumerable<T> collection,object obj)
{
   int idx;
   GetIndexOf(collection,obj,out idx);
   return idx;
}

public static int StrictIndexOf<T>(IEnumerable<T> collection,object obj)
{
   int idx;
   if (GetIndexOf(collection,obj,out idx))
      return idx;
   throw new XElementNotFoundException();
}

public static bool GetIndexOf<T>(IEnumerable<T> collection,object obj,out int index)
{
   int idx=-1;
   foreach (var o in collection)
   {
      idx++;
      if (object.Equals(o,obj))
      {
         index = idx;
         return true;
      }
   }
   index = -1;
   return false;
}

//==================================================================================================

public static bool ConventEquals<T>( [Optional] IEnumerable<T> collection_1,
                                     [Optional] IEnumerable<T> collection_2 )
{
   if (collection_1==collection_2)
      return true;
   if ( collection_1==null && collection_2!=null ||
        collection_1!=null && collection_2==null )
      return false;
   var
      enumerator_1 = collection_1.GetEnumerator();
   try
   {
      var
         enumerator_2 = collection_2.GetEnumerator();
      try
      {
         for (;;)
         {
            bool
               end_1 =! enumerator_1.MoveNext(),
               end_2 =! enumerator_2.MoveNext();
            if (end_1 && end_2)
               return true;
            if (end_1!=end_2)
               return false;
            T
               element_1 = enumerator_1.Current,
               element_2 = enumerator_2.Current;
            if (!object.Equals(element_1,element_2))
               return false;
         }
      }
      finally
      {
         if (enumerator_2 is IDisposable)
            ((IDisposable)enumerator_2).Dispose();
      }
   }
   finally
   {
      if (enumerator_1 is IDisposable)
         ((IDisposable)enumerator_1).Dispose();
   }
}

//==================================================================================================

static XCollection()
{
   Library.Access();
}


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XCollection
//**************************************************************************************************

} // XHelpers
