﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
partial class XArray {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static T[] Extract<T>(T[] array,int? startIdx)
{
   return Extract(array,startIdx,null,false);
}

public static T[] Extract<T>(T[] array,int? startIdx,int? count)
{
   return Extract(array,startIdx,count,false);
}

//--------------------------------------------------------------------------------------------------

public static T[] BackwardExtract<T>(T[] array)
{
   return Extract(array,null,null,true);
}

public static T[] BackwardExtract<T>(T[] array,int? startIdx)
{
   return Extract(array,startIdx,null,true);
}

public static T[] BackwardExtract<T>(T[] array,int? startIdx,int? count)
{
   return Extract(array,startIdx,count,true);
}

//==================================================================================================

static T[] Extract<T>(T[] array,int? startIdx,int? count,bool backwardScan)
{
   XArgCheck.VerifyNotNull( array );

   int
      lowerBound = array.GetLowerBound(0),
      upperBound = array.GetUpperBound(0),
      fromIdx    = startIdx ?? (backwardScan ? upperBound : lowerBound),
      length     = count ?? (backwardScan ? fromIdx-lowerBound+1 : upperBound-fromIdx+1);

   var arrayCapture = new XCapture<T[]>();
   var indexCapture = new XCapture<int>();

   Scan
   (
      array,
      (XContinueEnumerationWithIndexCallback<T>)delegate (T element,int index)
      {
         if (arrayCapture.Value==null)
            arrayCapture.Value = new T[ length ];
         arrayCapture.Value[ indexCapture.Value++ ] =
            array[ index ];
         return true;
      },
      startIdx, count, backwardScan
   );

   return arrayCapture.Value ?? new T[0];
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XArray
//**************************************************************************************************

} // XHelpers
