﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Collections.Generic;

namespace XHelpers {

//**************************************************************************************************
partial class XArray {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool Find<T>(T[] array,Predicate<T> match,out T element)
{
   return Find(array,match,null,null,false,out element);
}

public static bool Find<T>(T[] array,Predicate<T> match,int? startIdx,out T element)
{
   return Find(array,match,startIdx,null,false,out element);
}

public static bool Find<T>(T[] array,Predicate<T> match,int? startIdx,int? count,out T element)
{
   return Find(array,match,startIdx,count,false,out element);
}

//--------------------------------------------------------------------------------------------------

public static bool FindLast<T>(T[] array, Predicate<T> match,out T element)
{
   return Find(array,match,null,null,true,out element);
}

public static bool FindLast<T>(T[] array,Predicate<T> match,int? startIdx,out T element)
{
   return Find(array,match,startIdx,null,true,out element);
}

public static bool FindLast<T>(T[] array,Predicate<T> match,int? startIdx,int? count,out T element)
{
   return Find(array,match,startIdx,count,true,out element);
}

//==================================================================================================

public static int? FindIndex<T>(T[] array,Predicate<T> match)
{
   return FindIndex(array,match,null,null,false);
}

public static int? FindIndex<T>(T[] array,Predicate<T> match,int? startIdx)
{
   return FindIndex(array,match,startIdx,null,false);
}

public static int? FindIndex<T>(T[] array,Predicate<T> match,int? startIdx,int? count)
{
   return FindIndex(array,match,startIdx,count,false);
}

//--------------------------------------------------------------------------------------------------

public static int? FindLastIndex<T>(T[] array,Predicate<T> match)
{
   return FindIndex(array,match,null,null,true);
}

public static int? FindLastIndex<T>(T[] array,Predicate<T> match,int? startIdx)
{
   return FindIndex(array,match,startIdx,null,true);
}

public static int? FindLastIndex<T>(T[] array,Predicate<T> match,int? startIdx,int? count)
{
   return FindIndex(array,match,startIdx,count,true);
}

//==================================================================================================

public static T[] FindAll<T>(T[] array,Predicate<T> match)
{
   return FindAll(array,match,null,null,false);
}

public static T[] FindAll<T>(T[] array,Predicate<T> match,int? startIdx)
{
   return FindAll(array,match,startIdx,null,false);
}

public static T[] FindAll<T>(T[] array,Predicate<T> match,int? startIdx,int? count)
{
   return FindAll(array,match,startIdx,count,false);
}

//--------------------------------------------------------------------------------------------------

public static T[] BackwardFindAll<T>(T[] array,Predicate<T> match)
{
   return FindAll(array,match,null,null,true);
}

public static T[] BackwardFindAll<T>(T[] array,Predicate<T> match,int? startIdx)
{
   return FindAll(array,match,startIdx,null,true);
}

public static T[] BackwardFindAll<T>(T[] array,Predicate<T> match,int? startIdx,int? count)
{
   return FindAll(array,match,startIdx,count,true);
}

//==================================================================================================

static bool Find<T>( T[] array, Predicate<T> match,
                     int? startIdx, int? count, bool backwardScan,
                     out T element )
{
   int? idx=FindIndex(array,match,startIdx,count,backwardScan);
   if (idx!=null)
   {
      element=array[ idx.Value ];
      return true;
   }
   element=default(T);
   return false;
}

//--------------------------------------------------------------------------------------------------

static int? FindIndex<T>( T[] array, Predicate<T> match,
                          int? startIdx, int? count, bool backwardScan )
{
   var indexCapture =
      new XCapture<int?>();
   Scan
   (
      array,
      (XContinueEnumerationWithIndexCallback<T>)delegate (T element,int index)
      {
         if (match(element))
         {
            indexCapture.Value = index;
            return false;
         }
         return true;
      },
      startIdx, count, backwardScan
   );
   return indexCapture;
}

//--------------------------------------------------------------------------------------------------

static T[] FindAll<T>( T[] array, Predicate<T> match,
                       int? startIdx, int? count, bool backwardScan )
{
   XArgCheck.VerifyNotNull( match );

   var list = new List<T>();

   ForEach( array,
      delegate (T e){ list.Add(e); },
      startIdx, count, backwardScan );

   return list.ToArray();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XArray
//**************************************************************************************************

} // XHelpers
