﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
partial class XArray {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static T Min<T>(T[] array)
   where T: IComparable
{
   return MinOrMax(array,XMinOrMax.Min,null,null);
}

public static T Min<T>(T[] array,int? startIdx)
   where T: IComparable
{
   return MinOrMax(array,XMinOrMax.Min,startIdx,null);
}

public static T Min<T>(T[] array,int? startIdx,int? count)
   where T: IComparable
{
   return MinOrMax(array,XMinOrMax.Min,startIdx,count);
}

//--------------------------------------------------------------------------------------------------

public static T Max<T>(T[] array)
   where T: IComparable
{
   return MinOrMax(array,XMinOrMax.Max,null,null);
}

public static T Max<T>(T[] array,int? startIdx)
   where T: IComparable
{
   return MinOrMax(array,XMinOrMax.Max,startIdx,null);
}

public static T Max<T>(T[] array,int? startIdx,int? count)
   where T: IComparable
{
   return MinOrMax(array,XMinOrMax.Max,startIdx,count);
}

//==================================================================================================

public static int? IndexOfMin<T>(T[] array)
   where T: IComparable
{
   return IndexOfMinOrMax(array,XMinOrMax.Min,null,null);
}

public static int? IndexOfMin<T>(T[] array,int? startIdx)
   where T: IComparable
{
   return IndexOfMinOrMax(array,XMinOrMax.Min,startIdx,null);
}

public static int? IndexOfMin<T>(T[] array,int? startIdx,int? count)
   where T: IComparable
{
   return IndexOfMinOrMax(array,XMinOrMax.Min,startIdx,count);
}

//--------------------------------------------------------------------------------------------------

public static int? IndexOfMax<T>(T[] array)
   where T: IComparable
{
   return IndexOfMinOrMax(array,XMinOrMax.Max,null,null);
}

public static int? IndexOfMax<T>(T[] array,int? startIdx)
   where T: IComparable
{
   return IndexOfMinOrMax(array,XMinOrMax.Max,startIdx,null);
}

public static int? IndexOfMax<T>(T[] array,int? startIdx,int? count)
   where T: IComparable
{
   return IndexOfMinOrMax(array,XMinOrMax.Max,startIdx,count);
}

//==================================================================================================

static T MinOrMax<T>(T[] array,XMinOrMax minmax,int? startIdx,int? count)
   where T: IComparable
{
   int? index;
   return GetMinOrMax(array,minmax,startIdx,count,out index);
}

//--------------------------------------------------------------------------------------------------

static int? IndexOfMinOrMax<T>(T[] array,XMinOrMax minmax,int? startIdx,int? count)
   where T: IComparable
{
   int? index;
   GetMinOrMax(array,minmax,startIdx,count,out index);
   return index;
}

//--------------------------------------------------------------------------------------------------

static T GetMinOrMax<T>(T[] array,XMinOrMax minmax,int? startIdx,int? count,out int? index)
   where T: IComparable
{
   var indexCapture =
      new XCapture<int?>();
   var minmaxCapture =
      new XCapture<T>();
   Scan
   (
      array,
      (XContinueEnumerationWithIndexCallback<T>)delegate (T element,int idx)
      {
         if ( indexCapture.Value == null ||
              minmax==XMinOrMax.Min && element.CompareTo( minmaxCapture.Value )<0 ||
              minmax==XMinOrMax.Max && element.CompareTo( minmaxCapture.Value )>0 )
         {
            indexCapture.Value = idx;
            minmaxCapture.Value = element;
         }
         return true;
      },
      startIdx, count, false
   );
   index = indexCapture.Value;
   return minmaxCapture;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XArray
//**************************************************************************************************

} // XHelpers
