﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
public static partial class XArray {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static T[] Concatenate<T>(T[] array_1,T[] array_2)
{
   XArgCheck.VerifyNotNulls( array_1, array_2 );
   var result = new T[ array_1.Length + array_2.Length ];
   array_1.CopyTo
   (
      result, array_1.GetLowerBound(0)
   );
   Array.Copy
   (
      array_2, array_2.GetLowerBound(0),
      result, array_1.Length,
      array_2.Length
   );
   return result;
}

//==================================================================================================

public static T[] Append<T>(T[] array,params T[] values)
{
   return Concatenate( array, values );
}

public static void Append<T>(ref T[] array,params T[] values)
{
   XArgCheck.VerifyNotNull( array );
   XArgCheck.VerifyNotNull( values );

   if (values.Length>0)
      array = Append( array, values );
}

//==================================================================================================

public static T[] Insert<T>(T[] array,int index,params T[] values)
{
   XArgCheck.VerifyNotNull( array );
   if (index!=array.GetUpperBound(0))
      XArgCheck.VerifyArrayIndex( array, index );
   XArgCheck.VerifyNotNull( values );

   int
      len          = array.Length,
      insLen       = values.Length,
      sumLen       = len + insLen,
      lb           = array.GetLowerBound(0),
      headElemCount= index - lb;
   var
      result = new T[ sumLen ];

   Array.Copy( array, result, headElemCount );
   values.CopyTo( result, lb+headElemCount );

   int remainedElemCount =
      sumLen-headElemCount-insLen;

   if (remainedElemCount>0)
      Array.Copy(
         array, lb+headElemCount,
         result, headElemCount+insLen,
         remainedElemCount );

   return result;
}

public static void Insert<T>(ref T[] array,int index,params T[] values)
{
   array = Insert( array, index, values );
}

//==================================================================================================

public static T[] Prefix<T>(T[] array,T value)
{
   return Concatenate( new T[]{ value }, array );
}

public static void Prefix<T>(ref T[] array,T value)
{
   array = Prefix( array, value );
}

//==================================================================================================

public static T[] Delete<T>(T[] array,int index)
{
   return Delete( array, index, 1 );
}

public static T[] Delete<T>(T[] array,int fromIdx,int count)
{
   XArgCheck.VerifyNotNull( array );
   XArgCheck.VerifyArrayIndex( array, fromIdx );
   if (count<0)
      throw new ArgumentOutOfRangeException();
   if (count==0)
      return array;
   XArgCheck.VerifyArrayIndex( array, fromIdx+count-1 );

   int
      lb           = array.GetLowerBound(0),
      headElemCount= fromIdx - lb,
      len          = array.Length;
   var
      result = new T[ len-1 ];

   Array.Copy( array, result,
               headElemCount );

   int remainedElemCount=
      len-headElemCount-1;

   if (remainedElemCount>0)
      Array.Copy(
         array, lb+headElemCount+1,
         result, headElemCount+1,
         remainedElemCount );

   return result;
}

//--------------------------------------------------------------------------------------------------

public static void Delete<T>(ref T[] array,int index)
{
   array = Delete( array, index );
}

public static void Delete<T>(ref T[] array,int fromIdx,int count)
{
   array = Delete( array, fromIdx, count );
}

//==================================================================================================

public static T[] Truncate<T>(T[] array,int fromIdx)
{
   XArgCheck.VerifyNotNull( array );
   XArgCheck.VerifyArrayIndex( array, fromIdx );
   return Delete( array, fromIdx,
      array.GetUpperBound(0)-fromIdx+1 );
}

public static void Truncate<T>(ref T[] array,int fromIdx)
{
   array = Truncate( array, fromIdx );
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XArray
//**************************************************************************************************

} // XHelpers
