﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
partial class XArray {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool Scan<T>( T[] array,
                            XContinueEnumerationWithIndexCallback<T> enumerationCallback )
{
   return Scan(array,enumerationCallback,null,null,false);
}

public static bool Scan<T>( T[] array,
                            XContinueEnumerationWithIndexCallback<T> enumerationCallback,
                            int? startIdx )
{
   return Scan(array,enumerationCallback,startIdx,null,false);
}

public static bool Scan<T>( T[] array,
                            XContinueEnumerationWithIndexCallback<T> enumerationCallback,
                            int? startIdx, int? count )
{
   return Scan(array,enumerationCallback,startIdx,count,false);
}

//--------------------------------------------------------------------------------------------------

public static bool BackwardScan<T>( T[] array,
                                    XContinueEnumerationWithIndexCallback<T> enumerationCallback )
{
   return Scan(array,enumerationCallback,null,null,true);
}

public static bool BackwardScan<T>( T[] array,
                                    XContinueEnumerationWithIndexCallback<T> enumerationCallback,
                                    int? startIdx )
{
   return Scan(array,enumerationCallback,startIdx,null,true);
}

public static bool BackwardScan<T>( T[] array,
                                    XContinueEnumerationWithIndexCallback<T> enumerationCallback,
                                    int? startIdx, int? count )
{
   return Scan(array,enumerationCallback,startIdx,count,true);
}

//==================================================================================================

static bool Scan<T>( T[] array,
                     XContinueEnumerationWithIndexCallback<T> enumerationCallback,
                     int? startIdx, int? count, bool backwardScan )
{
   XArgCheck.VerifyNotNull( array );

   if (startIdx!=null)
   {
      XArgCheck.VerifyArrayIndex( array, startIdx.Value );
      if (count!=null)
      {
         if (count<0)
            throw new ArgumentOutOfRangeException();
         XArgCheck.VerifyArrayIndex( array, backwardScan ?
            startIdx.Value-count.Value+1 : startIdx.Value+count.Value-1 );
      }
   }

   int
      lowerBound = array.GetLowerBound(0),
      upperBound = array.GetUpperBound(0),
      fromIdx = startIdx ?? (backwardScan ? upperBound : lowerBound),
      i;

   if (!backwardScan)
      for ( i=fromIdx;
            count==null && i<=upperBound ||
            count!=null && i-fromIdx<count;
            i++ )
         if (!enumerationCallback(array[i],i))
            return false;

   if (backwardScan)
      for ( i=fromIdx;
            count==null && i>=lowerBound ||
            count!=null && fromIdx-i<count;
            i-- )
         if (!enumerationCallback(array[i],i))
            return false;

   return true;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XArray
//**************************************************************************************************

} // XHelpers
