﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XArray {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool IsNullOrEmpty(Array array)
{
   return array==null || array.Length==0;
}

public static bool IsNullOrEmpty<T>(T[] array)
{
   return array==null || array.Length==0;
}

//--------------------------------------------------------------------------------------------------

public static int RelativeIndex<T>(T[] array,int zeroBazedIdx)
{
   return array.GetLowerBound(0)+zeroBazedIdx;
}

public static int RelativeIndex<T>(T[] array,int zeroBazedIdx,bool accountNotFoundIndexValue)
{
   return accountNotFoundIndexValue && zeroBazedIdx==-1 ?
      NotFoundElementIndex(array) : RelativeIndex(array,zeroBazedIdx);
}

//--------------------------------------------------------------------------------------------------

public static int ZeroBasedIndex<T>(T[] array,int relativeIdx)
{
   return relativeIdx-array.GetLowerBound(0);
}

public static int ZeroBasedIndex<T>(T[] array,int relativeIdx,bool accountNotFoundIndexValue)
{
   return accountNotFoundIndexValue && relativeIdx==NotFoundElementIndex(array) ?
      -1 : ZeroBasedIndex(array,relativeIdx);
}

//--------------------------------------------------------------------------------------------------

public static bool IsIndexInValidBounds<T>(T[] array,int index)
{
   XArgCheck.VerifyNotNull( array );
   return XOps.IsBetween( index,
      array.GetLowerBound(0),
      array.GetUpperBound(0) );
}

public static bool IsIndexInValidBounds(Array array,int dimension,int index)
{
   XArgCheck.VerifyNotNull( array );
   XArgCheck.VerifyArrayDimension( array, dimension );
   return XOps.IsBetween( index,
      array.GetLowerBound( dimension ),
      array.GetUpperBound( dimension ) );
}

public static bool IsValidDimension(Array array,int dimension)
{
   XArgCheck.VerifyNotNull( array );
   return XOps.IsBetween(dimension,0,array.Rank-1);
}

//--------------------------------------------------------------------------------------------------

public static int NotFoundElementIndex<T>(int baseIndex)
{
   return baseIndex>int.MinValue ? baseIndex-1 : int.MaxValue;
}

public static int NotFoundElementIndex<T>(T[] array)
{
   XArgCheck.VerifyNotNull(array);
   return NotFoundElementIndex<T>(array.GetLowerBound(0));
}

//--------------------------------------------------------------------------------------------------

/// <exception cref="System.ArgumentNullException" />
/// <exception cref="XElementNotFoundException" />
public static void VerifyElementFound<T>(T[] array,int index)
{
   if (!IsIndexInValidBounds(array,index))
      throw new XElementNotFoundException();
}

//--------------------------------------------------------------------------------------------------

public static T GetAtZeroBasedIndex<T>(T[] array,int zeroBazedIdx)
{
   return array[ RelativeIndex(array,zeroBazedIdx) ];
}

public static void SetAtZeroBasedIndex<T>(T[] array,int zeroBazedIdx,T value)
{
   array[ RelativeIndex(array,zeroBazedIdx) ] = value;
}

//--------------------------------------------------------------------------------------------------

public static bool Contains<T>(T[] array,T value)
{
   return XCollection.Contains(array,value);
}

//--------------------------------------------------------------------------------------------------

public static T[] Duplicate<T>(T[] array)
{
   return (T[])array.Clone();
}

//==================================================================================================

static XArray()
{
   Library.Access();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XArray
//**************************************************************************************************

} // XHelpers
