﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
partial class XArray2D {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static T[] Extract<T>(T[,] array,int dimension,int subarrayIdx)
{
   return Extract(array,dimension,subarrayIdx,null,null,false);
}

public static T[] Extract<T>(T[,] array,int dimension,int subarrayIdx,int? startIdx)
{
   return Extract(array,dimension,subarrayIdx,startIdx,null,false);
}

public static T[] Extract<T>(T[,] array,int dimension,int subarrayIdx,int? startIdx,int? count)
{
   return Extract(array,dimension,subarrayIdx,startIdx,count,false);
}

//--------------------------------------------------------------------------------------------------

public static T[] BackwardExtract<T>(T[,] array,int dimension,int subarrayIdx)
{
   return Extract(array,dimension,subarrayIdx,null,null,true);
}

public static T[] BackwardExtract<T>(T[,] array,int dimension,int subarrayIdx,int? startIdx)
{
   return Extract(array,dimension,subarrayIdx,startIdx,null,true);
}

public static T[] BackwardExtract<T>( T[,] array, int dimension, int subarrayIdx,
                                      int? startIdx, int? count )
{
   return Extract(array,dimension,subarrayIdx,startIdx,count,true);
}

//==================================================================================================

static T[] Extract<T>( T[,] array, int dimension, int subarrayIdx,
                       int? startIdx, int? count, bool backwardScan )
{
   XArgCheck.VerifyArrayIndex( array, dimension, subarrayIdx );

   int
      lowerBound = array.GetLowerBound( 1-dimension ),
      upperBound = array.GetUpperBound( 1-dimension ),
      fromIdx    = startIdx ?? (backwardScan ? upperBound : lowerBound),
      length     = count ?? (backwardScan ? fromIdx-lowerBound+1 : upperBound-fromIdx+1);

   var arrayCapture = new XCapture<T[]>();
   var indexCapture = new XCapture<int>();

   Scan
   (
      array, dimension, subarrayIdx,
      (XContinueEnumerationWithIndexCallback<T>)delegate (T element,int index)
      {
         if (arrayCapture.Value==null)
            arrayCapture.Value = new T[ length ];
         arrayCapture.Value[ indexCapture.Value++ ] =
            GetValue(array,dimension,subarrayIdx,index);
         return true;
      },
      startIdx, count, backwardScan
   );

   return arrayCapture.Value ?? new T[0];
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XArray2D
//**************************************************************************************************

} // XHelpers
