﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Collections.Generic;

namespace XHelpers {

//**************************************************************************************************
partial class XArray2D {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool Find<T>( T[,] array, int dimension, int subarrayIdx,
                            Predicate<T> match, out T element )
{
   return Find(array,dimension,subarrayIdx,match,null,null,false,out element);
}

public static bool Find<T>( T[,] array, int dimension, int subarrayIdx,
                            Predicate<T> match, int? startIdx, out T element )
{
   return Find(array,dimension,subarrayIdx,match,startIdx,null,false,out element);
}

public static bool Find<T>( T[,] array, int dimension, int subarrayIdx,
                            Predicate<T> match, int? startIdx, int? count,
                            out T element )
{
   return Find(array,dimension,subarrayIdx,match,startIdx,count,false,out element);
}

//--------------------------------------------------------------------------------------------------

public static bool FindLast<T>( T[,] array, int dimension, int subarrayIdx,
                                Predicate<T> match, out T element )
{
   return Find(array,dimension,subarrayIdx,match,null,null,true,out element);
}

public static bool FindLast<T>( T[,] array, int dimension, int subarrayIdx,
                                Predicate<T> match, int? startIdx, out T element )
{
   return Find(array,dimension,subarrayIdx,match,startIdx,null,true,out element);
}

public static bool FindLast<T>( T[,] array, int dimension, int subarrayIdx,
                                Predicate<T> match, int? startIdx, int? count,
                                out T element )
{
   return Find(array,dimension,subarrayIdx,match,startIdx,count,true,out element);
}

//==================================================================================================

public static int? FindIndex<T>( T[,] array, int dimension, int subarrayIdx,
                                 Predicate<T> match )
{
   return FindIndex(array,dimension,subarrayIdx,match,null,null,false);
}

public static int? FindIndex<T>( T[,] array, int dimension, int subarrayIdx,
                                 Predicate<T> match, int? startIdx )
{
   return FindIndex(array,dimension,subarrayIdx,match,startIdx,null,false);
}

public static int? FindIndex<T>( T[,] array, int dimension, int subarrayIdx,
                                 Predicate<T> match, int? startIdx, int? count )
{
   return FindIndex(array,dimension,subarrayIdx,match,startIdx,count,false);
}

//--------------------------------------------------------------------------------------------------

public static int? FindLastIndex<T>( T[,] array, int dimension, int subarrayIdx,
                                     Predicate<T> match  )
{
   return FindIndex(array,dimension,subarrayIdx,match,null,null,true);
}

public static int? FindLastIndex<T>( T[,] array, int dimension, int subarrayIdx,
                                     Predicate<T> match, int? startIdx )
{
   return FindIndex(array,dimension,subarrayIdx,match,startIdx,null,true);
}

public static int? FindLastIndex<T>( T[,] array, int dimension, int subarrayIdx,
                                     Predicate<T> match, int? startIdx, int? count )
{
   return FindIndex(array,dimension,subarrayIdx,match,startIdx,count,true);
}

//==================================================================================================

public static T[] FindAll<T>( T[,] array, int dimension, int subarrayIdx,
                              Predicate<T> match )
{
   return FindAll(array,dimension,subarrayIdx,match,null,null,false);
}

public static T[] FindAll<T>( T[,] array, int dimension, int subarrayIdx,
                              Predicate<T> match, int? startIdx )
{
   return FindAll(array,dimension,subarrayIdx,match,startIdx,null,false);
}

public static T[] FindAll<T>( T[,] array, int dimension, int subarrayIdx,
                              Predicate<T> match, int? startIdx, int? count )
{
   return FindAll(array,dimension,subarrayIdx,match,startIdx,count,false);
}

//--------------------------------------------------------------------------------------------------

public static T[] BackwardFindAll<T>( T[,] array, int dimension, int subarrayIdx,
                                      Predicate<T> match )
{
   return FindAll(array,dimension,subarrayIdx,match,null,null,true);
}

public static T[] BackwardFindAll<T>( T[,] array, int dimension, int subarrayIdx,
                                      Predicate<T> match, int? startIdx )
{
   return FindAll(array,dimension,subarrayIdx,match,startIdx,null,true);
}

public static T[] BackwardFindAll<T>( T[,] array, int dimension, int subarrayIdx,
                                      Predicate<T> match, int? startIdx, int? count )
{
   return FindAll(array,dimension,subarrayIdx,match,startIdx,count,true);
}

//==================================================================================================

static bool Find<T>( T[,] array, int dimension, int subarrayIdx,
                     Predicate<T> match, int? startIdx, int? count, bool backwardScan,
                     out T element )
{
   int? idx=FindIndex(array,dimension,subarrayIdx,match,startIdx,count,backwardScan);
   if (idx!=null)
   {
      element=GetValue(array,dimension,subarrayIdx,idx.Value);
      return true;
   }
   element=default(T);
   return false;
}

//--------------------------------------------------------------------------------------------------

static int? FindIndex<T>( T[,] array, int dimension, int subarrayIdx,
                          Predicate<T> match, int? startIdx, int? count, bool backwardScan )
{
   var indexCapture =
      new XCapture<int?>();
   Scan
   (
      array, dimension, subarrayIdx,
      (XContinueEnumerationWithIndexCallback<T>)delegate (T element,int index)
      {
         if (match(element))
         {
            indexCapture.Value = index;
            return false;
         }
         return true;
      },
      startIdx, count, backwardScan
   );
   return indexCapture;
}

//--------------------------------------------------------------------------------------------------

static T[] FindAll<T>( T[,] array, int dimension, int subarrayIdx,
                       Predicate<T> match, int? startIdx, int? count, bool backwardScan )
{
   XArgCheck.VerifyNotNull(match);

   var list = new List<T>();

   ForEach( array, dimension, subarrayIdx,
      delegate (T e){ list.Add(e); },
      startIdx, count, backwardScan );

   return list.ToArray();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XArray2D
//**************************************************************************************************

} // XHelpers
