﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
partial class XArray2D {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static void ForEach<T>( T[,] array, int dimension, int subarrayIdx,
                               Action<T> action )
{
   ForEach(array,dimension,subarrayIdx,action,null,null,false);
}

public static void ForEach<T>( T[,] array, int dimension, int subarrayIdx,
                               Action<T> action, int? startIdx )
{
   ForEach(array,dimension,subarrayIdx,action,startIdx,null,false);
}

public static void ForEach<T>( T[,] array, int dimension, int subarrayIdx,
                               Action<T> action, int? startIdx, int? count )
{
   ForEach(array,dimension,subarrayIdx,action,startIdx,count,false);
}

//--------------------------------------------------------------------------------------------------

public static void BackwardForEach<T>( T[,] array, int dimension, int subarrayIdx,
                                       Action<T> action )
{
   ForEach(array,dimension,subarrayIdx,action,null,null,true);
}

public static void BackwardForEach<T>( T[,] array, int dimension, int subarrayIdx,
                                       Action<T> action, int? startIdx )
{
   ForEach(array,dimension,subarrayIdx,action,startIdx,null,true);
}

public static void BackwardForEach<T>( T[,] array, int dimension, int subarrayIdx,
                                       Action<T> action, int? startIdx, int? count )
{
   ForEach(array,dimension,subarrayIdx,action,startIdx,count,true);
}

//==================================================================================================

static void ForEach<T>( T[,] array, int dimension, int subarrayIdx,
                        Action<T> action, int? startIdx, int? count, bool backwardScan )
{
   Scan
   (
      array, dimension, subarrayIdx,
      (XContinueEnumerationWithIndexCallback<T>)delegate (T element,int index)
      {
         action( GetValue(array,dimension,subarrayIdx,index) );
         return true;
      },
      startIdx, count, backwardScan
   );
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XArray2D
//**************************************************************************************************

} // XHelpers
