﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
partial class XArray2D {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static T StrictMin<T>(T[,] array,int dimension,int subarrayIdx)
   where T: IComparable
{
   return StrictMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Min,null,null);
}

public static T StrictMin<T>(T[,] array,int dimension,int subarrayIdx,int? startIdx)
   where T: IComparable
{
   return StrictMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Min,startIdx,null);
}

public static T StrictMin<T>(T[,] array,int dimension,int subarrayIdx,int? startIdx,int? count)
   where T: IComparable
{
   return StrictMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Min,startIdx,count);
}

//--------------------------------------------------------------------------------------------------

public static T StrictMax<T>(T[,] array,int dimension,int subarrayIdx)
   where T: IComparable
{
   return StrictMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Max,null,null);
}

public static T StrictMax<T>(T[,] array,int dimension,int subarrayIdx,int? startIdx)
   where T: IComparable
{
   return StrictMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Max,startIdx,null);
}

public static T StrictMax<T>(T[,] array,int dimension,int subarrayIdx,int? startIdx,int? count)
   where T: IComparable
{
   return StrictMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Max,startIdx,count);
}

//==================================================================================================

public static int? StrictIndexOfMin<T>(T[,] array,int dimension,int subarrayIdx)
   where T: IComparable
{
   return StrictIndexOfMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Min,null,null);
}

public static int? StrictIndexOfMin<T>(T[,] array,int dimension,int subarrayIdx,int? startIdx)
   where T: IComparable
{
   return StrictIndexOfMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Min,startIdx,null);
}

public static int? StrictIndexOfMin<T>( T[,] array, int dimension, int subarrayIdx,
                                        int? startIdx, int? count )
   where T: IComparable
{
   return StrictIndexOfMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Min,startIdx,count);
}

//--------------------------------------------------------------------------------------------------

public static int? StrictIndexOfMax<T>(T[,] array,int dimension,int subarrayIdx)
   where T: IComparable
{
   return StrictIndexOfMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Max,null,null);
}

public static int? StrictIndexOfMax<T>(T[,] array,int dimension,int subarrayIdx,int? startIdx)
   where T: IComparable
{
   return StrictIndexOfMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Max,startIdx,null);
}

public static int? StrictIndexOfMax<T>( T[,] array, int dimension, int subarrayIdx,
                                        int? startIdx, int? count )
   where T: IComparable
{
   return StrictIndexOfMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Max,startIdx,count);
}

//==================================================================================================

static T StrictMinOrMax<T>( T[,] array, int dimension, int subarrayIdx,
                            XMinOrMax minmax, int? startIdx, int? count )
   where T: IComparable
{
   int? index;
   T value = GetMinOrMax(array,dimension,subarrayIdx,minmax,startIdx,count,out index);
   if (index==null)
      throw new XEmptySetException();
   return value;
}

//--------------------------------------------------------------------------------------------------

static int? StrictIndexOfMinOrMax<T>( T[,] array, int dimension, int subarrayIdx,
                                      XMinOrMax minmax, int? startIdx, int? count )
   where T: IComparable
{
   int? index;
   GetMinOrMax(array,dimension,subarrayIdx,minmax,startIdx,count,out index);
   if (index==null)
      throw new XEmptySetException();
   return index;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XArray2D
//**************************************************************************************************

} // XHelpers
