﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
partial class XArray2D {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static T Min<T>(T[,] array,int dimension,int subarrayIdx)
   where T: IComparable
{
   return MinOrMax(array,dimension,subarrayIdx,XMinOrMax.Min,null,null);
}

public static T Min<T>(T[,] array,int dimension,int subarrayIdx,int? startIdx)
   where T: IComparable
{
   return MinOrMax(array,dimension,subarrayIdx,XMinOrMax.Min,startIdx,null);
}

public static T Min<T>(T[,] array,int dimension,int subarrayIdx,int? startIdx,int? count)
   where T: IComparable
{
   return MinOrMax(array,dimension,subarrayIdx,XMinOrMax.Min,startIdx,count);
}

//--------------------------------------------------------------------------------------------------

public static T Max<T>(T[,] array,int dimension,int subarrayIdx)
   where T: IComparable
{
   return MinOrMax(array,dimension,subarrayIdx,XMinOrMax.Max,null,null);
}

public static T Max<T>(T[,] array,int dimension,int subarrayIdx,int? startIdx)
   where T: IComparable
{
   return MinOrMax(array,dimension,subarrayIdx,XMinOrMax.Max,startIdx,null);
}

public static T Max<T>(T[,] array,int dimension,int subarrayIdx,int? startIdx,int? count)
   where T: IComparable
{
   return MinOrMax(array,dimension,subarrayIdx,XMinOrMax.Max,startIdx,count);
}

//==================================================================================================

public static int? IndexOfMin<T>(T[,] array,int dimension,int subarrayIdx)
   where T: IComparable
{
   return IndexOfMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Min,null,null);
}

public static int? IndexOfMin<T>(T[,] array,int dimension,int subarrayIdx,int? startIdx)
   where T: IComparable
{
   return IndexOfMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Min,startIdx,null);
}

public static int? IndexOfMin<T>(T[,] array,int dimension,int subarrayIdx,int? startIdx,int? count)
   where T: IComparable
{
   return IndexOfMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Min,startIdx,count);
}

//--------------------------------------------------------------------------------------------------

public static int? IndexOfMax<T>(T[,] array,int dimension,int subarrayIdx)
   where T: IComparable
{
   return IndexOfMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Max,null,null);
}

public static int? IndexOfMax<T>(T[,] array,int dimension,int subarrayIdx,int? startIdx)
   where T: IComparable
{
   return IndexOfMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Max,startIdx,null);
}

public static int? IndexOfMax<T>(T[,] array,int dimension,int subarrayIdx,int? startIdx,int? count)
   where T: IComparable
{
   return IndexOfMinOrMax(array,dimension,subarrayIdx,XMinOrMax.Max,startIdx,count);
}

//==================================================================================================

static T MinOrMax<T>( T[,] array, int dimension, int subarrayIdx,
                      XMinOrMax minmax, int? startIdx, int? count )
   where T: IComparable
{
   int? index;
   return GetMinOrMax(array,dimension,subarrayIdx,minmax,startIdx,count,out index);
}

//--------------------------------------------------------------------------------------------------

static int? IndexOfMinOrMax<T>( T[,] array, int dimension, int subarrayIdx,
                                XMinOrMax minmax, int? startIdx, int? count )
   where T: IComparable
{
   int? index;
   GetMinOrMax(array,dimension,subarrayIdx,minmax,startIdx,count,out index);
   return index;
}

//--------------------------------------------------------------------------------------------------

static T GetMinOrMax<T>( T[,] array, int dimension, int subarrayIdx,
                         XMinOrMax minmax, int? startIdx, int? count,
                         out int? index )
   where T: IComparable
{
   var indexCapture =
      new XCapture<int?>();
   var minmaxCapture =
      new XCapture<T>();
   Scan
   (
      array, dimension, subarrayIdx,
      (XContinueEnumerationWithIndexCallback<T>)delegate (T element,int idx)
      {
         if ( indexCapture.Value == null ||
              minmax==XMinOrMax.Min && element.CompareTo( minmaxCapture.Value )<0 ||
              minmax==XMinOrMax.Max && element.CompareTo( minmaxCapture.Value )>0 )
         {
            indexCapture.Value = idx;
            minmaxCapture.Value = element;
         }
         return true;
      },
      startIdx, count, false
   );
   index = indexCapture.Value;
   return minmaxCapture;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XArray2D
//**************************************************************************************************

} // XHelpers
