﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
partial class XArray2D {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool Scan<T>( T[,] array, int dimension, int subarrayIdx,
                            XContinueEnumerationWithIndexCallback<T> enumerationCallback )
{
   return Scan(array,dimension,subarrayIdx,enumerationCallback,null,null,false);
}

public static bool Scan<T>( T[,] array, int dimension, int subarrayIdx,
                            XContinueEnumerationWithIndexCallback<T> enumerationCallback,
                            int? startIdx )
{
   return Scan(array,dimension,subarrayIdx,enumerationCallback,startIdx,null,false);
}

public static bool Scan<T>( T[,] array, int dimension, int subarrayIdx,
                            XContinueEnumerationWithIndexCallback<T> enumerationCallback,
                            int? startIdx, int? count )
{
   return Scan(array,dimension,subarrayIdx,enumerationCallback,startIdx,count,false);
}

//--------------------------------------------------------------------------------------------------

public static bool BackwardScan<T>( T[,] array, int dimension, int subarrayIdx,
                                    XContinueEnumerationWithIndexCallback<T> enumerationCallback )
{
   return Scan(array,dimension,subarrayIdx,enumerationCallback,null,null,true);
}

public static bool BackwardScan<T>( T[,] array, int dimension, int subarrayIdx,
                                    XContinueEnumerationWithIndexCallback<T> enumerationCallback,
                                    int? startIdx )
{
   return Scan(array,dimension,subarrayIdx,enumerationCallback,startIdx,null,true);
}

public static bool BackwardScan<T>( T[,] array, int dimension, int subarrayIdx,
                                    XContinueEnumerationWithIndexCallback<T> enumerationCallback,
                                    int? startIdx, int? count )
{
   return Scan(array,dimension,subarrayIdx,enumerationCallback,startIdx,count,true);
}

//==================================================================================================

static bool Scan<T>( T[,] array, int dimension, int subarrayIdx,
                     XContinueEnumerationWithIndexCallback<T> enumerationCallback,
                     int? startIdx, int? count, bool backwardScan )
{
   XArgCheck.VerifyArrayIndex( array, dimension, subarrayIdx );

   int
      lowerBound = array.GetLowerBound( 1-dimension ),
      upperBound = array.GetUpperBound( 1-dimension );

   if (startIdx!=null)
   {
      if (!XOps.IsBetween(startIdx.Value,lowerBound,upperBound))
         throw XArgCheck.ArgumentOutOfRangeException_IncorrectArrayIndex;
      if (count!=null)
      {
         if (count<0)
            throw new ArgumentOutOfRangeException();
         if (!XOps.IsBetween(
               backwardScan ?
                  startIdx.Value-count.Value+1 :
                  startIdx.Value+count.Value-1,
               lowerBound, upperBound ))
            throw XArgCheck.ArgumentOutOfRangeException_IncorrectArrayIndex;
      }
   }

   int
      fromIdx = startIdx ?? (backwardScan ? upperBound : lowerBound),
      i;

   if (!backwardScan)
      for ( i=fromIdx;
            count==null && i<=upperBound ||
            count!=null && i-fromIdx<count;
            i++ )
         if (!enumerationCallback( GetValue(array,dimension,subarrayIdx,i),
                                   i ))
            return false;

   if (backwardScan)
      for ( i=fromIdx;
            count==null && i>=lowerBound ||
            count!=null && fromIdx-i<count;
            i-- )
         if (!enumerationCallback( GetValue(array,dimension,subarrayIdx,i),
                                   i ))
            return false;

   return true;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XArray2D
//**************************************************************************************************

} // XHelpers
