﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XArray2D {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool IsNullOrEmpty<T>(T[,] array)
{
   return array==null || array.Length==0;
}

//--------------------------------------------------------------------------------------------------

public static bool IsIndexInValidBounds<T>(T[,] array,int dimension,int index)
{
   XArgCheck.VerifyNotNull( array );
   XArgCheck.VerifyArrayDimension( array, dimension );
   return XOps.IsBetween( index,
      array.GetLowerBound( dimension ),
      array.GetUpperBound( dimension ) );
}

public static bool IsValidDimension<T>(T[,] array,int dimension)
{
   XArgCheck.VerifyNotNull( array );
   return XOps.IsOneOf(dimension,0,1);
}

//--------------------------------------------------------------------------------------------------

public static T[,] Duplicate<T>(T[,] array)
{
   return (T[,])array.Clone();
}

//==================================================================================================

static XArray2D()
{
   Library.Access();
}

//--------------------------------------------------------------------------------------------------

static T GetValue<T>(T[,] array,int dimension,int subarrayIdx,int elementIdx)
{
   return dimension==0 ?
      array[ subarrayIdx, elementIdx ] :
      array[ elementIdx, subarrayIdx ];
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XArray2D
//**************************************************************************************************

} // XHelpers
