﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#if SILVERLIGHT
   #define DEBUG
#else
   #define TRACE
#endif

using System;
using System.IO;
using System.Threading;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Runtime.CompilerServices;

namespace XHelpers {

//**************************************************************************************************
partial class XFatalError {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

static string FatalErrorCaption
{
   get
   {
      return string.Format("{0}: Fatal Error",_AppSimpleName_);
   }
}

//--------------------------------------------------------------------------------------------------

static bool IsInRaiseThread
{
   get { return Thread.CurrentThread == _RaiseThread_; }
}

//--------------------------------------------------------------------------------------------------

static DateTime ErrorTime
{
   get
   {
      lock (_ErrorTimeAccessSyncObject_)
         return _ErrorTime_;
   }
}

//==================================================================================================

static void ObtainApplicationName()
{
   try
   {
      _AppSimpleName_ = XProgram.SimpleName;
      _MainModuleName_ = XProgram.MainModuleName;
   }
   catch
   {
      _AppSimpleName_ = _MainModuleName_ = "("+AppDomain.CurrentDomain.FriendlyName+")";
   }
}

//==================================================================================================

static void OutputTraceMessage(string message)
{
#if SILVERLIGHT
   Debug.WriteLine(Environment.NewLine+message);
#else
   System.Diagnostics.Trace.IndentLevel=0;
   System.Diagnostics.Trace.WriteLine(Environment.NewLine+message);
   System.Diagnostics.Trace.Flush();
#endif
}

//--------------------------------------------------------------------------------------------------

static void Trace(string message)
{
   OutputTraceMessage(string.Format(
      "=== Breakdown of fatal error performing in \"{0}\": ===\r\n"+
      "{1}\r\n"+
      "=== End of message (breakdown). ===",
      _MainModuleName_, message ));
}

static void Trace(Exception e)
{
   Trace(e.ToString());
}

//==================================================================================================

#if !SILVERLIGHT

//--------------------------------------------------------------------------------------------------

[MethodImpl(MethodImplOptions.NoInlining)]
static void FailFast(string message)
{
   Environment.FailFast(Environment.NewLine+message);
}

//--------------------------------------------------------------------------------------------------

[MethodImpl(MethodImplOptions.NoInlining)]
static void SetWaitCursor()
{
   IntPtr cursorHandle =
      Win32.LoadCursor( IntPtr.Zero, Win32.IDC_WAIT );
   if (cursorHandle!=IntPtr.Zero)
      Win32.SetCursor( cursorHandle );
}

//--------------------------------------------------------------------------------------------------

[MethodImpl(MethodImplOptions.NoInlining)]
static void Beep()
{
   Console.Beep(880,300);
}

//--------------------------------------------------------------------------------------------------

static bool ShowErrorBox(string errorText,string caption)
{
   return ShowErrorBox(errorText,caption,false);
}

//--------------------------------------------------------------------------------------------------

[MethodImpl(MethodImplOptions.NoInlining)]
static bool ShowErrorBox(string errorText,string caption,bool yesNoMode)
{
   return Win32.MessageBox( IntPtr.Zero,
      errorText, caption,
      Win32.MB_ICONERROR | Win32.MB_SYSTEMMODAL |
      (yesNoMode ? Win32.MB_YESNO : Win32.MB_OK) ) == Win32.IDYES;
}

//--------------------------------------------------------------------------------------------------

[MethodImpl(MethodImplOptions.NoInlining)]
static void TryReportToConsole(string cause,string message)
{
   try
   {
      if (Win32.GetStdHandle( Win32.STD_OUTPUT_HANDLE )==Win32.INVALID_HANDLE_VALUE)
         return;
      Console.ResetColor();
      Console.ForegroundColor = ConsoleColor.Red;
      Console.WriteLine
      (
         "\r\n"+
         "## === Fatal error in <{0}>: ===\r\n"+
         "{1}"+
         "## {2}\r\n"+
         "## === The process is terminated! ===",
         _MainModuleName_,
         cause!=null ?
            string.Format("## << {0} >>\r\n",cause) :
            "",
         message.Replace(XUnicode.Bullet,'*').Replace("\r\n","\r\n## ")
      );
      Console.ResetColor();
   }
   catch
   {}
}

//--------------------------------------------------------------------------------------------------

[MethodImpl(MethodImplOptions.NoInlining)]
static void DetachUnhandledExceptionHandler()
{
   AppDomain.CurrentDomain.UnhandledException -= AppDomain_UnhandledExceptionHandler;
}

//--------------------------------------------------------------------------------------------------

#endif // !SILVERLIGHT

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XFatalError
//**************************************************************************************************

} // XHelpers
