﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Threading;
using System.Reflection;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Runtime.CompilerServices;

#if SILVERLIGHT
using System.Windows.Threading;
#endif

namespace XHelpers {

//**************************************************************************************************
partial class XFatalError {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static XFatalErrorFlags Flags
{
   get
   {
      lock (typeof(XFatalError))
         return _Flags_;
   }
   set
   {
      lock (typeof(XFatalError))
      {
#if !SILVERLIGHT
         bool
            raiseOnUnhandledException_Old =
               (_Flags_ & XFatalErrorFlags.RaiseOnUnhandledException) != 0,
            raiseOnUnhandledException_New =
               (value & XFatalErrorFlags.RaiseOnUnhandledException) != 0;
         if (raiseOnUnhandledException_Old!=raiseOnUnhandledException_New)
            if (raiseOnUnhandledException_New)
               AppDomain.CurrentDomain.UnhandledException +=
                  AppDomain_UnhandledExceptionHandler;
            else
               AppDomain.CurrentDomain.UnhandledException -=
                  AppDomain_UnhandledExceptionHandler;
#endif // !SILVERLIGHT
         _Flags_ = value;
      }
   }
}

//--------------------------------------------------------------------------------------------------

#if !SILVERLIGHT
public static bool RaiseOnUnhandledException
{
   get
   {
      lock (typeof(XFatalError))
         return _RaiseOnUnhandledException_;
   }
   set
   {
      lock (typeof(XFatalError))
         if (_RaiseOnUnhandledException_!=value)
         {
            if (value)
               AppDomain.CurrentDomain.UnhandledException +=
                  AppDomain_UnhandledExceptionHandler;
            else
               AppDomain.CurrentDomain.UnhandledException -=
                  AppDomain_UnhandledExceptionHandler;
            _RaiseOnUnhandledException_ = value;
         }
   }
}
#endif // !SILVERLIGHT

//--------------------------------------------------------------------------------------------------

public static event XFatalErrorEmergencyHandler EmergencyHandler
{
   add    { lock (typeof(XFatalError)) _EmergencyHandler_ += value; }
   remove { lock (typeof(XFatalError)) _EmergencyHandler_ -= value; }
}

public static event XFatalErrorEmergencyHandler EmergencyFinalHandler
{
   add    { lock (typeof(XFatalError)) _EmergencyFinalHandler_ += value; }
   remove { lock (typeof(XFatalError)) _EmergencyFinalHandler_ -= value; }
}

//--------------------------------------------------------------------------------------------------

public static bool IsInTerminationThread
{
   get { return _IsInTerminationThread_; }
}

//==================================================================================================

[MethodImpl(MethodImplOptions.NoInlining)]
public static void Raise(string message)
{
#if SILVERLIGHT
   RaiseEx(Assembly.GetCallingAssembly(),null,message,null,new StackTrace());
#else
   RaiseEx(Assembly.GetCallingAssembly(),null,message,null,new StackTrace(1,true));
#endif
}

[MethodImpl(MethodImplOptions.NoInlining)]
public static void Raise([Optional] string cause,string message)
{
#if SILVERLIGHT
   RaiseEx(Assembly.GetCallingAssembly(),cause,message,null,new StackTrace());
#else
   RaiseEx(Assembly.GetCallingAssembly(),cause,message,null,new StackTrace(1,true));
#endif
}

[MethodImpl(MethodImplOptions.NoInlining)]
public static void Raise([Optional] string cause,string messageFmt,params object[] parameters)
{
#if SILVERLIGHT
   RaiseEx( Assembly.GetCallingAssembly(),
      cause, string.Format(messageFmt,parameters), null,
      new StackTrace() );
#else
   RaiseEx( Assembly.GetCallingAssembly(),
      cause, string.Format(messageFmt,parameters), null,
      new StackTrace(1,true) );
#endif
}

//--------------------------------------------------------------------------------------------------

#if !SILVERLIGHT

[MethodImpl(MethodImplOptions.NoInlining)]
public static void Raise(int skipFrames,string message)
{
   RaiseEx(Assembly.GetCallingAssembly(),null,message,null,new StackTrace(skipFrames+1,true));
}

[MethodImpl(MethodImplOptions.NoInlining)]
public static void Raise(int skipFrames,[Optional] string cause,string message)
{
   RaiseEx(Assembly.GetCallingAssembly(),cause,message,null,new StackTrace(skipFrames+1,true));
}

[MethodImpl(MethodImplOptions.NoInlining)]
public static void Raise( int skipFrames, [Optional] string cause,
                          string messageFmt, params object[] parameters )
{
   RaiseEx( Assembly.GetCallingAssembly(),
      cause, string.Format(messageFmt,parameters), null,
      new StackTrace(skipFrames+1,true) );
}

#endif // !SILVERLIGHT

//--------------------------------------------------------------------------------------------------

[MethodImpl(MethodImplOptions.NoInlining)]
public static void Raise(string cause,Exception e)
{
   RaiseEx(Assembly.GetCallingAssembly(),cause,null,e,null);
}

//==================================================================================================

#if !SILVERLIGHT

[MethodImpl(MethodImplOptions.NoInlining)]
public static void Raise(Assembly assembly,string message)
{
   RaiseEx(assembly,null,message,null,new StackTrace(1,true));
}

[MethodImpl(MethodImplOptions.NoInlining)]
public static void Raise(Assembly assembly,[Optional] string cause,string message)
{
   RaiseEx(assembly,cause,message,null,new StackTrace(1,true));
}

[MethodImpl(MethodImplOptions.NoInlining)]
public static void Raise( Assembly assembly,
                          [Optional] string cause,
                          string messageFmt,
                          params object[] parameters )
{
   RaiseEx(assembly,cause,string.Format(messageFmt,parameters),null,new StackTrace(1,true));
}

#endif // !SILVERLIGHT

//--------------------------------------------------------------------------------------------------

#if !SILVERLIGHT

[MethodImpl(MethodImplOptions.NoInlining)]
public static void Raise(Assembly assembly,int skipFrames,string message)
{
   RaiseEx(assembly,null,message,null,new StackTrace(skipFrames+1,true));
}

[MethodImpl(MethodImplOptions.NoInlining)]
public static void Raise(Assembly assembly,int skipFrames,[Optional] string cause,string message)
{
   RaiseEx(assembly,cause,message,null,new StackTrace(skipFrames+1,true));
}

[MethodImpl(MethodImplOptions.NoInlining)]
public static void Raise( Assembly assembly,int skipFrames, 
                          [Optional] string cause, string messageFmt,
                          params object[] parameters )
{
   RaiseEx( assembly,
      cause, string.Format(messageFmt,parameters), null,
      new StackTrace(skipFrames+1,true) );
}

#endif // !SILVERLIGHT

//--------------------------------------------------------------------------------------------------

#if !SILVERLIGHT
[MethodImpl(MethodImplOptions.NoInlining)]
public static void Raise(Assembly assembly,string cause,Exception e)
{
   RaiseEx(assembly,cause,null,e,null);
}
#endif

//==================================================================================================

public static bool IsExceptionFatal(Exception e)
{
   return e.Data[_FatalErrorExceptionKey_] != null;
}

public static bool IsFatalErrorExceptionChain(Exception e)
{
   for (;e!=null;e=e.InnerException)
      if (IsExceptionFatal(e))
         return true;
   return false;
}

//--------------------------------------------------------------------------------------------------

public static void ProcessUnhandledException(Exception e)
{
   Thread.MemoryBarrier();
   if (!IsFatalErrorExceptionChain(e))
   {
      Thread mainThread =
         XProgram.GetMainThreadSpecification();
      Raise
      (
         mainThread!=null ?
            Thread.CurrentThread==mainThread ?
               "Unhandled exception in main program thread." :
               "Unhandled exception in secondary program thread." :
            "Unhandled exception in the program.",
         e
      );
   }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XFatalError
//**************************************************************************************************

} // XHelpers
