﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define TRACE
#define DEBUG

using System;
using System.Diagnostics;

namespace XHelpers {

//**************************************************************************************************
class Tracer {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public Tracer(bool debug)
{
   _Debug_ = debug;
}

//==================================================================================================

public void Print(string message)
{
   Print(XTraceMessageCategory.Neutral,message);
}

public void Print(string messageFmt,params object[] parameters)
{
   Print(XTraceMessageCategory.Neutral,string.Format(messageFmt,parameters));
}

public void Print(object obj)
{
   Print(XTraceMessageCategory.Neutral,obj.ToString());
}

//--------------------------------------------------------------------------------------------------

public void Print(XTraceMessageCategory messageCategory,string message)
{
   lock (typeof(Tracer))
   {
#if !SILVERLIGHT
      int IndentLevel=Trace.IndentSize;
      if (_Debug_)
         Debug.IndentLevel=0;
      else
         Trace.IndentLevel=0;
#endif
      try
      {
         string outputString = string.Format(
            "{0}{1} {2} >> {3}",
            XOps.Exchange(ref _WasPrinted_,true) ? "" : Environment.NewLine,
            XProgram.MainModuleName,
            GetMessageCategotyChar(messageCategory),
            message );
#if SILVERLIGHT
         Debug.WriteLine(outputString);
#else
         if (_Debug_)
         {
            Debug.WriteLine(outputString);
            Debug.Flush();
         }
         else
         {
            Trace.WriteLine(outputString);
            Trace.Flush();
         }
#endif
      }
      finally
      {
#if !SILVERLIGHT
         if (_Debug_)
            Debug.IndentLevel=IndentLevel;
         else
            Trace.IndentLevel=IndentLevel;
#endif
      }
   }
}

public void Print( XTraceMessageCategory messageCategory,
                   string messageFmt, params object[] parameters )
{
   Print(messageCategory,string.Format(messageFmt,parameters));
}

//--------------------------------------------------------------------------------------------------

public void PrintInformation(string message)
{
   Print(XTraceMessageCategory.Information,message);
}

public void PrintInformation(string messageFmt,params object[] parameters)
{
   Print(XTraceMessageCategory.Information,string.Format(messageFmt,parameters));
}

//--------------------------------------------------------------------------------------------------

public void PrintWarning(string message)
{
   Print(XTraceMessageCategory.Warning,message);
}

public void PrintWarning(string messageFmt,params object[] parameters)
{
   Print(XTraceMessageCategory.Warning,string.Format(messageFmt,parameters));
}

//--------------------------------------------------------------------------------------------------

public void PrintError(string message)
{
   Print(XTraceMessageCategory.Error,message);
}

public void PrintError(string messageFmt,params object[] parameters)
{
   Print(XTraceMessageCategory.Error,string.Format(messageFmt,parameters));
}

public void PrintError(Exception e)
{
   PrintError(e.Message);
}

//==================================================================================================

public void PrintOperation(string message)
{
   PrintOperation(false,message);
}

public void PrintOperation(string messageFmt,params object[] parameters)
{
   PrintOperation(false,messageFmt,parameters);
}

//--------------------------------------------------------------------------------------------------

public void PrintOperation(bool informationCategory,string message)
{
   Print( informationCategory ?
      XTraceMessageCategory.Information :
      XTraceMessageCategory.Neutral,
      "* {0} ...", message );
}

public void PrintOperation( bool informationCategory,
                            string messageFmt,
                            params object[] parameters )
{
   PrintOperation(informationCategory,string.Format(messageFmt,parameters));
}

//==================================================================================================

public void DetailedPrint(string caption,string details)
{
   DetailedPrint(XTraceMessageCategory.Neutral,caption,details);
}

public void DetailedPrint(string caption,string detailsFmt,params object[] parameters)
{
   DetailedPrint(XTraceMessageCategory.Neutral,caption,detailsFmt,parameters);
}

//--------------------------------------------------------------------------------------------------

public void DetailedPrint(XTraceMessageCategory messageCategory,string caption,string details)
{
   Print(messageCategory,"{0}: \"{1}\".",caption,details);
}

public void DetailedPrint( XTraceMessageCategory messageCategory, string caption,
                           string detailsFmt, params object[] parameters )
{
   DetailedPrint(messageCategory,caption,string.Format(detailsFmt,parameters));
}

//--------------------------------------------------------------------------------------------------

public void DetailedPrintInformation(string caption,string details)
{
   DetailedPrint(XTraceMessageCategory.Information,caption,details);
}

public void DetailedPrintInformation(string caption,string detailsFmt,params object[] parameters)
{
   DetailedPrint(XTraceMessageCategory.Information,caption,detailsFmt,parameters);
}

//--------------------------------------------------------------------------------------------------

public void DetailedPrintWarning(string caption,string details)
{
   DetailedPrint(XTraceMessageCategory.Warning,caption,details);
}

public void DetailedPrintWarning(string caption,string detailsFmt,params object[] parameters)
{
   DetailedPrint(XTraceMessageCategory.Warning,caption,detailsFmt,parameters);
}

//--------------------------------------------------------------------------------------------------

public void DetailedPrintError(string caption,string details)
{
   DetailedPrint(XTraceMessageCategory.Error,caption,details);
}

public void DetailedPrintError(string caption,string detailsFmt,params object[] parameters)
{
   DetailedPrint(XTraceMessageCategory.Error,caption,detailsFmt,parameters);
}

public void DetailedPrintError(XTraceMessageCategory messageCategory,string caption,Exception e)
{
   DetailedPrintError(caption,e.Message);
}

//==================================================================================================

static char GetMessageCategotyChar(XTraceMessageCategory messageCategory)
{
   switch (messageCategory)
   {
   case XTraceMessageCategory.Neutral    : return '-';
   case XTraceMessageCategory.Information: return 'I';
   case XTraceMessageCategory.Warning    : return 'W';
   case XTraceMessageCategory.Error      : return 'E';
   }
   throw XArgCheck.ArgumentException_IncorrectArguments;
}

//==================================================================================================

bool
   _Debug_;

static bool
   _WasPrinted_;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // Tracer
//**************************************************************************************************

} // XHelpers
