﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define TRACE

using System;
using System.Reflection;
using System.Diagnostics;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
[AttributeUsage(AttributeTargets.Assembly)]
public sealed class XTraceConfigurationAttribute: Attribute {}
//**************************************************************************************************


//**************************************************************************************************
/// <summary>
/// </summary>
public enum XTraceMessageCategory { Neutral, Information, Warning, Error }
//**************************************************************************************************

#if !SILVERLIGHT

//**************************************************************************************************
/// <summary>
/// </summary>
[XSyncOnlyDisposable]
public partial class XTrace: XDisposable {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public XTrace(string blockName)
{
   if (XAssembly.IsConfiguredForTrace( Assembly.GetCallingAssembly() ))
      _BlockTracer_ = new BlockTracer(false,blockName);
   ValidateDisposable();
}

public XTrace(string blockNameFmt,params object[] parameters)
{
   if (XAssembly.IsConfiguredForTrace( Assembly.GetCallingAssembly() ))
      _BlockTracer_ = new BlockTracer(false,blockNameFmt,parameters);
   ValidateDisposable();
}

public XTrace(MethodBase method)
{
   if (XAssembly.IsConfiguredForTrace( Assembly.GetCallingAssembly() ))
      _BlockTracer_ = new BlockTracer(false,method);
   ValidateDisposable();
}

//==================================================================================================

[Conditional("TRACE")]
public static void Print(string message)
{
   Tracer.Print(message);
}

[Conditional("TRACE")]
public static void Print(string messageFmt,params object[] parameters)
{
   Tracer.Print(messageFmt,parameters);
}

[Conditional("TRACE")]
public static void Print(object obj)
{
   Tracer.Print(obj);
}

//--------------------------------------------------------------------------------------------------

[Conditional("TRACE")]
public static void Print(XTraceMessageCategory messageCategory,string message)
{
   Tracer.Print(messageCategory,message);
}

[Conditional("TRACE")]
public static void Print( XTraceMessageCategory messageCategory,
                          string messageFmt, params object[] parameters )
{
   Tracer.Print(messageCategory,messageFmt,parameters);
}

//--------------------------------------------------------------------------------------------------

[Conditional("TRACE")]
public static void PrintInformation(string message)
{
   Tracer.PrintInformation(message);
}

[Conditional("TRACE")]
public static void PrintInformation(string messageFmt,params object[] parameters)
{
   Tracer.PrintInformation(messageFmt,parameters);
}

//--------------------------------------------------------------------------------------------------

[Conditional("TRACE")]
public static void PrintWarning(string message)
{
   Tracer.PrintWarning(message);
}

[Conditional("TRACE")]
public static void PrintWarning(string messageFmt,params object[] parameters)
{
   Tracer.PrintWarning(messageFmt,parameters);
}

//--------------------------------------------------------------------------------------------------

[Conditional("TRACE")]
public static void PrintError(string message)
{
   Tracer.PrintError(message);
}

[Conditional("TRACE")]
public static void PrintError(string messageFmt,params object[] parameters)
{
   Tracer.PrintError(messageFmt,parameters);
}

[Conditional("TRACE")]
public static void PrintError(Exception e)
{
   Tracer.PrintError(e);
}

//==================================================================================================

[Conditional("TRACE")]
public static void PrintOperation(string message)
{
   Tracer.PrintOperation(message);
}

[Conditional("TRACE")]
public static void PrintOperation(string messageFmt,params object[] parameters)
{
   Tracer.PrintOperation(messageFmt,parameters);
}

//--------------------------------------------------------------------------------------------------

[Conditional("TRACE")]
public static void PrintOperation(bool informationCategory,string message)
{
   Tracer.PrintOperation(informationCategory,message);
}

[Conditional("TRACE")]
public static void PrintOperation( bool informationCategory,
                                   string messageFmt,
                                   params object[] parameters )
{
   Tracer.PrintOperation(informationCategory,messageFmt,parameters);
}

//==================================================================================================

[Conditional("TRACE")]
public static void DetailedPrint(string caption,string details)
{
   Tracer.DetailedPrint(caption,details);
}

[Conditional("TRACE")]
public static void DetailedPrint(string caption,string detailsFmt,params object[] parameters)
{
   Tracer.DetailedPrint(caption,detailsFmt,parameters);
}

//--------------------------------------------------------------------------------------------------

[Conditional("TRACE")]
public static void DetailedPrint( XTraceMessageCategory messageCategory,
                                  string caption, string details )
{
   Tracer.DetailedPrint(messageCategory,caption,details);
}

[Conditional("TRACE")]
public static void DetailedPrint( XTraceMessageCategory messageCategory, string caption,
                                  string detailsFmt, params object[] parameters )
{
   Tracer.DetailedPrint(messageCategory,caption,detailsFmt,parameters);
}

//--------------------------------------------------------------------------------------------------

[Conditional("TRACE")]
public static void DetailedPrintInformation(string caption,string details)
{
   Tracer.DetailedPrintInformation(caption,details);
}

[Conditional("TRACE")]
public static void DetailedPrintInformation( string caption,
                                             string detailsFmt,
                                             params object[] parameters )
{
   Tracer.DetailedPrintInformation(caption,detailsFmt,parameters);
}

//--------------------------------------------------------------------------------------------------

[Conditional("TRACE")]
public static void DetailedPrintWarning(string caption,string details)
{
   Tracer.DetailedPrintWarning(caption,details);
}

[Conditional("TRACE")]
public static void DetailedPrintWarning( string caption,
                                         string detailsFmt,
                                         params object[] parameters )
{
   Tracer.DetailedPrintWarning(caption,detailsFmt,parameters);
}

//--------------------------------------------------------------------------------------------------

[Conditional("TRACE")]
public static void DetailedPrintError(string caption,string details)
{
   Tracer.DetailedPrintError(caption,details);
}

[Conditional("TRACE")]
public static void DetailedPrintError( string caption,
                                       string detailsFmt,
                                       params object[] parameters )
{
   Tracer.DetailedPrintError(caption,detailsFmt,parameters);
}

[Conditional("TRACE")]
public static void DetailedPrintError( XTraceMessageCategory messageCategory,
                                       string caption, Exception e )
{
   Tracer.DetailedPrintError(messageCategory,caption,e);
}

//==================================================================================================

protected override void OnDispose(bool synchronous)
{
   if (!synchronous)
      throw XInvOpExceptionCreation.ObjectMustBeDisposedSynchronously( this );
   BlockTracer.Dispose( _BlockTracer_ );
}

//==================================================================================================

static XTrace()
{
   Library.Access();
}

//==================================================================================================

internal static Tracer
   Tracer = new Tracer(false);

//==================================================================================================

BlockTracer
   _BlockTracer_;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XTrace
//**************************************************************************************************

#endif // !SILVERLIGHT

} // XHelpers
