﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.IO;
using System.ComponentModel;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XDirectory {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool CheckExists(string path,out Exception e)
{
   return XFile.CheckExists(path,true,out e);
}

public static void VerifyExists(string path)
{
   XFile.VerifyExists(path,true);
}

//==================================================================================================

public static bool IsReadonly(string path)
{
   return XFile.IsReadonly(path,true);
}

//--------------------------------------------------------------------------------------------------

public static void SetReadonlyAttribute(string path)
{
   XFile.SetReadonlyAttribute(path,true);
}

public static void ResetReadonlyAttribute(string path)
{
   XFile.ResetReadonlyAttribute(path,true);
}

//==================================================================================================

public static FileAttributes GetAttributes(string path)
{
   return XFile.GetAttributes(path,true);
}

//--------------------------------------------------------------------------------------------------

/// <exception cref="XOperationFailedException" />
public static void SetAttributes(string path,FileAttributes attributes)
{
   XFile.SetAttributes(path,true,attributes);
}

/// <exception cref="XOperationFailedException" />
public static FileAttributes AddAttributes(string path,FileAttributes attributes)
{
   return XFile.AddAttributes(path,true,attributes);
}

/// <exception cref="XOperationFailedException" />
public static FileAttributes RemoveAttributes(string path,FileAttributes attributes)
{
   return XFile.RemoveAttributes(path,true,attributes);
}

/// <exception cref="XOperationFailedException" />
public static FileAttributes ModifyAttributes( string path,
                                               FileAttributes attributesToAdd,
                                               FileAttributes attributesToRemove )
{
   return XFile.ModifyAttributes(path,true,attributesToAdd,attributesToRemove);
}

//==================================================================================================

public static void Delete(string path)
{
   Delete(path,false);
}

public static void Delete(string path,bool ignoreNotFoundError)
{
   Delete( path,
      ignoreNotFoundError ?
      XFileDirDeletionOptions.IgnoreNotFoundError : 0 );
}

public static void Delete(string path,XFileDirDeletionOptions options)
{
   XFile.VerifyPath( path );

   if ((options & XFileDirDeletionOptions.IgnoreNotFoundError)!=0)
   {
      Exception e;
      if (CheckExists(path,out e))
         return;
      throw e;
   }

   string[]
      directories = Directory.GetDirectories( path, "*", SearchOption.AllDirectories ),
      files       = Directory.GetFiles      ( path, "*", SearchOption.AllDirectories );

   if ((options & XFileDirDeletionOptions.RemoveReadonly)!=0)
   {
      ResetReadonlyAttribute( path );
      foreach (string directory in directories)
         ResetReadonlyAttribute( directory );
      foreach (string file in files)
         XFile.ResetReadonlyAttribute( file );
   }
   else
   {
      if (IsReadonly( path ))
         throw new XOperationFailedException(string.Format(
            "Folder is readonly: \"{0}\".",
            path ));
      foreach (string directory in directories)
         if (IsReadonly( directory ))
            throw new XOperationFailedException(string.Format(
               "Folder \"{0}\" contains readonly subfolders.",
               path ));
      foreach (string file in files)
         if (XFile.IsReadonly( file ))
            throw new XOperationFailedException(string.Format(
               "Folder \"{0}\" contains readonly files.",
               path ));
   }

   Directory.Delete( path, true );
}

//--------------------------------------------------------------------------------------------------

public static void DeleteReadonly(string path)
{
   DeleteReadonly(path,false);
}

public static void DeleteReadonly(string path,bool ignoreNotFoundError)
{
   Delete( path,
      XFileDirDeletionOptions.RemoveReadonly |
      ( ignoreNotFoundError ?
        XFileDirDeletionOptions.IgnoreNotFoundError : 0 ) );
}

//==================================================================================================

static XDirectory()
{
   Library.Access();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XDirectory
//**************************************************************************************************

} // XHelpers
