﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.IO;
using System.ComponentModel;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
[Flags]
public enum XFileDirDeletionOptions
{
   None = 0,

   RemoveReadonly      = 1 << 0,
   IgnoreNotFoundError = 1 << 1
}
//**************************************************************************************************


//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XFile {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool CheckExists(string path,out Exception e)
{
   return CheckExists(path,false,out e);
}

public static void VerifyExists(string path)
{
   VerifyExists(path,false);
}

//==================================================================================================

public static bool IsReadonly(string path)
{
   return IsReadonly(path,false);
}

//--------------------------------------------------------------------------------------------------

public static void SetReadonlyAttribute(string path)
{
   SetReadonlyAttribute(path,false);
}

public static void ResetReadonlyAttribute(string path)
{
   ResetReadonlyAttribute(path,false);
}

//==================================================================================================

public static FileAttributes GetAttributes(string path)
{
   return GetAttributes(path,false);
}

//--------------------------------------------------------------------------------------------------

/// <exception cref="XOperationFailedException" />
public static void SetAttributes(string path,FileAttributes attributes)
{
   SetAttributes(path,false,attributes);
}

/// <exception cref="XOperationFailedException" />
public static FileAttributes AddAttributes(string path,FileAttributes attributes)
{
   return AddAttributes(path,false,attributes);
}

/// <exception cref="XOperationFailedException" />
public static FileAttributes RemoveAttributes(string path,FileAttributes attributes)
{
   return RemoveAttributes(path,false,attributes);
}

/// <exception cref="XOperationFailedException" />
public static FileAttributes ModifyAttributes( string path,
                                               FileAttributes attributesToAdd,
                                               FileAttributes attributesToRemove )
{
   return ModifyAttributes(path,false,attributesToAdd,attributesToRemove);
}

//==================================================================================================

public static void Delete(string path)
{
   Delete(path,false);
}

public static void Delete(string path,bool ignoreNotFoundError)
{
   Delete( path,
      ( ignoreNotFoundError ?
      XFileDirDeletionOptions.IgnoreNotFoundError : 0 ) );
}

public static void Delete(string path,XFileDirDeletionOptions options)
{
   VerifyPath( path );

   if ((options & XFileDirDeletionOptions.IgnoreNotFoundError)==0)
      VerifyExists( path );

   if ((options & XFileDirDeletionOptions.RemoveReadonly)!=0)
      RemoveAttributes(path,FileAttributes.ReadOnly);

   File.Delete(path);
}

//--------------------------------------------------------------------------------------------------

public static void DeleteReadonly(string path)
{
   DeleteReadonly(path,false);
}

public static void DeleteReadonly(string path,bool ignoreNotFoundError)
{
   Delete( path,
      XFileDirDeletionOptions.RemoveReadonly |
      ( ignoreNotFoundError ?
        XFileDirDeletionOptions.IgnoreNotFoundError : 0 ) );
}

//==================================================================================================

static XFile()
{
   Library.Access();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XFile
//**************************************************************************************************

} // XHelpers
