﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.IO;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XPath {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static char[] WildcardChars
{
   get { return new char[]{'*','?'}; }
}

//==================================================================================================

public static string DirectorySeparator
{
   get { return Path.DirectorySeparatorChar.ToString(); }
}

//==================================================================================================

public static string GetDirectorySeparator(bool useAltSeparatorChar)
{
   return(
      useAltSeparatorChar ?
         Path.AltDirectorySeparatorChar :
         Path.DirectorySeparatorChar
   ).ToString();
}

//--------------------------------------------------------------------------------------------------

public static bool ContainsVolumeSeparatorChar(string path)
{
   return path.IndexOf( Path.VolumeSeparatorChar )>=0;
}

public static bool ContainsDirectorySeparatorChar(string path)
{
   return
      path.IndexOf( Path.DirectorySeparatorChar    )>=0 ||
      path.IndexOf( Path.AltDirectorySeparatorChar )>=0;
}

public static bool ContainsPathSeparatorChar(string path)
{
   return path.IndexOf( Path.PathSeparator )>=0;
}

public static bool ContainsWildcardChar(string path)
{
   return path.IndexOfAny( WildcardChars )>=0;
}

public static bool ContainsInvalidChar(string path)
{
   return
      ContainsInvalidFileNameChar( path ) ||
      ContainsInvalidPathChar    ( path );
}

public static bool ContainsInvalidFileNameChar(string path)
{
   return path.IndexOfAny( Path.GetInvalidFileNameChars() )>=0;
}

public static bool ContainsInvalidPathChar(string path)
{
   return path.IndexOfAny( Path.GetInvalidPathChars() )>=0;
}

//--------------------------------------------------------------------------------------------------

public static bool HasTrailingDirSeparator(string path)
{
   if (path=="")
      return false;
   char
      tailChar = path[ path.Length-1 ];
   return
      tailChar ==
         Path.DirectorySeparatorChar ||
         tailChar==Path.AltDirectorySeparatorChar;
}

public static string AppendTrailingDirSeparator(string path)
{
   return AppendTrailingDirSeparator(path,false);
}

public static string AppendTrailingDirSeparator(string path,bool useAltSeparatorChar)
{
   var dirSeparator =
      GetDirectorySeparator( useAltSeparatorChar );
   if (path=="")
      return "."+dirSeparator;
   return HasTrailingDirSeparator( path ) ?
      path : path+dirSeparator;
}

//--------------------------------------------------------------------------------------------------

public static string ProvideWithUNCPrefix(string path)
{
   if (path.Length>=2)
   {
      var prefix=path.Substring(0,2);
      if ( prefix=="//" || prefix==@"\\" ||
           XAscii.IsLetter(prefix[0]) &&
           prefix[1]==Path.VolumeSeparatorChar )
         return path;
   }
   if (path.Length>0 && path[0]=='.')
      return path;
   return @"\\?\"+path;
}

//==================================================================================================

public static void VerifyNotContainsWildcardChars(string path)
{
   XArgCheck.VerifyNotNull(path);
   if (ContainsWildcardChar(path))
      throw new ArgumentException("Path to single file must not contain wildcard characters.");
}

//==================================================================================================

static XPath()
{
   Library.Access();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XPath
//**************************************************************************************************

} // XHelpers
