﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XArith {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool IsOdd(int n)
{
   return n%2!=0;
}

public static bool IsEven(int n)
{
   return n%2==0;
}

//==================================================================================================

public static int Sign(int n)
{
   if (n>0) return  1;
   if (n<0) return -1;
   return 0;
}

public static double Sign(double x)
{
   if (x>0) return  1d;
   if (x<0) return -1d;
   return 0;
}

//--------------------------------------------------------------------------------------------------

public static bool AreSameSignValues(int m,int n)
{
   return Sign(m)*Sign(n)>=0;
}

public static bool AreSameSignValues(double x,double y)
{
   return Sign(x)*Sign(y)>=0;
}

//--------------------------------------------------------------------------------------------------

public static int ConfinePositive(int n)
{
   return Math.Max(n,0);
}

public static int ConfineNegative(int n)
{
   return Math.Min(n,0);
}

//--------------------------------------------------------------------------------------------------

public static double ConfinePositive(double x)
{
   return Math.Max(x,0);
}

public static double ConfineNegative(double x)
{
   return Math.Min(x,0);
}

//==================================================================================================

public static int RoundDiv(int m,int n)
{
   int div = m/n;
   return
      Math.Abs(m % n * 2) >= Math.Abs(n) ?
      (Math.Abs(div)+1)*Sign(m)*Sign(n) :
      div;
}

//==================================================================================================

public static int IntPower(int m,int n)
{
   if (n<0)
      throw new ArgumentOutOfRangeException();
   int result=1;
   for (int i=0; i<n; i++)
      result*=m;
   return result;
}

//==================================================================================================

public static int Min(params int[] values)
{
   XArgCheck.VerifyArrayNotEmpty( values );
   int? min = null;
   foreach (int value in values)
      if (min==null || value<min)
         min = value;
   return min.Value;
}

public static int Max(params int[] values)
{
   XArgCheck.VerifyArrayNotEmpty( values );
   int? max = null;
   foreach (int value in values)
      if (max==null || value>max)
         max = value;
   return max.Value;
}

//--------------------------------------------------------------------------------------------------

public static double Min(params double[] values)
{
   XArgCheck.VerifyArrayNotEmpty( values );
   double? min = null;
   foreach (double value in values)
      if (min==null || value<min)
         min = value;
   return min.Value;
}

public static double Max(params double[] values)
{
   XArgCheck.VerifyArrayNotEmpty( values );
   double? max = null;
   foreach (double value in values)
      if (max==null || value>max)
         max = value;
   return max.Value;
}

//==================================================================================================

static XArith()
{
   Library.Access();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XArith
//**************************************************************************************************

} // XHelpers
