﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Reflection;

namespace XHelpers {

//**************************************************************************************************
partial class XAssembly {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool IsEntryAssembly()
{
   return IsEntryAssembly( Assembly.GetCallingAssembly() );
}

public static bool IsEntryAssembly(Assembly assembly)
{
   XArgCheck.VerifyNotNull( assembly );
   var entryAssembly =
      Assembly.GetEntryAssembly();
   return
      entryAssembly!=null &&
      assembly.FullName==entryAssembly.FullName;
}

//--------------------------------------------------------------------------------------------------

/// <exception cref="XElementNotFoundException" />
public static Assembly AssemblyByName(string fullOrPartialName)
{
   return GetByName(fullOrPartialName,true);
}

/// <exception cref="XElementNotFoundException" />
public static Assembly GetByName(string fullOrPartialName)
{
   return GetByName(fullOrPartialName,false);
}

/// <exception cref="XElementNotFoundException" />
public static Assembly GetByName(string fullOrPartialName,bool throwOnFailure)
{
   XArgCheck.VerifyStringNotEmpty(fullOrPartialName);

   foreach (Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
   {
      var assemblyName=assembly.GetName();
      if ( XString.Equal(assemblyName.FullName,fullOrPartialName,true) ||
           XString.Equal(assemblyName.Name,fullOrPartialName,true) ) 
         return assembly;
   }

   if (throwOnFailure)
      throw new XElementNotFoundException(string.Format(
         "Assembly \"{0}\" has not been loaded into domain.",
         fullOrPartialName ));

   return null;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XAssembly
//**************************************************************************************************

} // XHelpers
