﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Reflection;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XAssembly {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static string GetModuleName()
{
   return GetModuleName( Assembly.GetCallingAssembly() );
}

public static string GetModuleName(Assembly assembly)
{
   XArgCheck.VerifyNotNull( assembly );
   return assembly.GetModules()[0].Name;
}

//==================================================================================================

public static string GetConfigurationName()
{
   return GetConfigurationName( Assembly.GetCallingAssembly() );
}

public static string GetConfigurationName(Assembly assembly)
{
   var configAttribute =
      GetAttribute<AssemblyConfigurationAttribute>( assembly );
   return configAttribute!=null ?
      configAttribute.Configuration : null;
}

//==================================================================================================

public static bool IsConfiguredForDebug()
{
   return IsConfiguredForDebug( Assembly.GetCallingAssembly() );
}

public static bool IsConfiguredForDebug(Assembly assembly)
{
   return XString.Equal(
      GetConfigurationName(assembly),
      "Debug", true );
}

//--------------------------------------------------------------------------------------------------

public static bool IsConfiguredForTrace()
{
   return IsConfiguredForTrace( Assembly.GetCallingAssembly() );
}

public static bool IsConfiguredForTrace(Assembly assembly)
{
   return IsMarkedWithAttribute( assembly,
      typeof(XTraceConfigurationAttribute) );
}

//==================================================================================================

public static Attribute GetAttribute(Type attributeType)
{
   return GetAttribute(
      Assembly.GetCallingAssembly(),
      attributeType );
}

public static Attribute GetAttribute(Assembly assembly,Type attributeType)
{
   XArgCheck.VerifyNotNulls( assembly, attributeType );
   XArgCheck.VerifyIsTypeOf( attributeType, typeof(Attribute) );

   var attributes =
      (Attribute[])assembly.GetCustomAttributes( attributeType, false );

   switch (attributes.Length)
   {
   case 0: return null;
   case 1: return attributes[0];
   default:
      throw XOpFailedExceptionCreation.Create(
         "Multiple attribute of type \"{0}\" on assembly \"{1}\".",
         attributeType, assembly.GetName().Name );
   }
}

//--------------------------------------------------------------------------------------------------

public static TAttribute GetAttribute<TAttribute>()
   where TAttribute: Attribute
{
   return GetAttribute<TAttribute>( Assembly.GetCallingAssembly() );
}

public static TAttribute GetAttribute<TAttribute>(Assembly assembly)
   where TAttribute: Attribute
{
   return (TAttribute)GetAttribute( assembly, typeof(TAttribute) );
}

//--------------------------------------------------------------------------------------------------

public static bool IsMarkedWithAttribute(Type attributeType)
{
   return IsMarkedWithAttribute(
      Assembly.GetCallingAssembly(),
      attributeType );
}

public static bool IsMarkedWithAttribute(Assembly assembly,Type attributeType)
{
   return GetAttribute(assembly,attributeType) != null;
}

//==================================================================================================

static XAssembly()
{
   Library.Access();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XAssembly
//**************************************************************************************************

} // XHelpers
