﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Reflection;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XPropertyInfo {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static string ExtractNameByGetMethod(MethodBase getMethod)
{
   if (getMethod==null)
      throw new ArgumentNullException();
   string
      methodName = getMethod.Name;
   if ((!methodName.StartsWith("get_") && methodName.Length>4))
      throw new ArgumentException(
         "Method is not a \"get\"-method of the property." );
   return methodName.Substring(4);
}

public static string ExtractNameBySetMethod(MethodBase setMethod)
{
   if (setMethod==null)
      throw new ArgumentNullException();
   string
      methodName = setMethod.Name;
   if ((!methodName.StartsWith("set_") && methodName.Length>4))
      throw new ArgumentException(
         "Method is not a \"set\"-method of the property." );
   return methodName.Substring(4);
}

//==================================================================================================

public static object GetStaticValue(Type type,string propertyName)
{
   return GetStaticValueUsingVisibilityFlags(type,propertyName,BindingFlags.Public);
}

public static void SetStaticValue(Type type,string propertyName,object value)
{
   SetStaticValueUsingVisibilityFlags(type,propertyName,BindingFlags.Public,value);
}

//--------------------------------------------------------------------------------------------------

public static object GetInstanceValue(object obj,string propertyName)
{
   return GetInstanceValueUsingVisibilityFlags(obj,propertyName,BindingFlags.Public);
}

public static void SetInstanceValue(object obj,string propertyName,object value)
{
   SetInstanceValueUsingVisibilityFlags(obj,propertyName,BindingFlags.Public,value);
}

//==================================================================================================

public static object GetStaticValueUsingVisibilityFlags( Type type,
                                                         string propertyName,
                                                         BindingFlags visibilityFlags )
{
   return type.InvokeMember( propertyName,
      visibilityFlags | BindingFlags.Static | BindingFlags.GetProperty,
      null, null, new object[0] );
}

public static void SetStaticValueUsingVisibilityFlags( Type type,
                                                       string propertyName,
                                                       BindingFlags visibilityFlags,
                                                       object value )
{
   type.InvokeMember( propertyName,
      visibilityFlags | BindingFlags.Static | BindingFlags.SetProperty,
      null, null, new object[]{value} );
}

//--------------------------------------------------------------------------------------------------

public static object GetInstanceValueUsingVisibilityFlags( object obj,
                                                           string propertyName,
                                                           BindingFlags visibilityFlags )
{
   return obj.GetType().InvokeMember( propertyName,
      visibilityFlags | BindingFlags.Instance | BindingFlags.GetProperty,
      null, null, new object[0] );
}

public static void SetInstanceValueUsingVisibilityFlags( object obj,
                                                         string propertyName,
                                                         BindingFlags visibilityFlags,
                                                         object value )
{
   obj.GetType().InvokeMember( propertyName,
      visibilityFlags | BindingFlags.Instance | BindingFlags.SetProperty,
      null, null, new object[]{value} );
}

//==================================================================================================

static XPropertyInfo()
{
   Library.Access();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XPropertyInfo
//**************************************************************************************************

} // XHelpers
