﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Reflection;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
public static partial class XType {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool IsStdValue(Type type)
{
   XArgCheck.VerifyNotNull(type);

   return Type.GetTypeCode(type)!=TypeCode.Object;
}

//--------------------------------------------------------------------------------------------------

public static bool IsIntegral(Type type)
{
   XArgCheck.VerifyNotNull(type);

   return XOps.IsOneOf( type,
      typeof(SByte), typeof(Int16), typeof(Int32), typeof(Int64),
      typeof(Byte), typeof(UInt16), typeof(UInt32), typeof(UInt64),
      typeof(Char) );
}

public static bool IsIntegralNumber(Type type)
{
   XArgCheck.VerifyNotNull(type);

   return IsIntegral(type) && type!=typeof(Char);
}

public static bool IsIntegralSignedNumber(Type type)
{
   XArgCheck.VerifyNotNull(type);

   return XOps.IsOneOf( type,
      typeof(SByte), typeof(Int16), typeof(Int32), typeof(Int64) );
}

public static bool IsIntegralUnsignedNumber(Type type)
{
   XArgCheck.VerifyNotNull(type);

   return XOps.IsOneOf( type,
      typeof(Byte), typeof(UInt16), typeof(UInt32), typeof(UInt64) );
}

//--------------------------------------------------------------------------------------------------

public static bool IsStdNumber(Type type)
{
   XArgCheck.VerifyNotNull(type);

   return IsStdSignedNumber(type) || IsStdUnsignedNumber(type);
}

public static bool IsStdSignedNumber(Type type)
{
   XArgCheck.VerifyNotNull(type);

   return
      IsIntegralSignedNumber(type) ||
      XOps.IsOneOf( type,
         typeof(Single), typeof(Double), typeof(Decimal) );
}

public static bool IsStdUnsignedNumber(Type type)
{
   return IsIntegralUnsignedNumber(type);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XType
//**************************************************************************************************

} // XHelpers
