﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Reflection;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XType {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool IsTypeOf(Type type,Type comparandType)
{
   XArgCheck.VerifyNotNulls( type, comparandType );
   return type==comparandType || type.IsSubclassOf(comparandType);
}

public static bool IsTypeOf<T>(Type comparandType)
{
   return IsTypeOf( typeof(T), comparandType );
}

//--------------------------------------------------------------------------------------------------

public static bool IsValueOf(object value,Type comparandType)
{
   XArgCheck.VerifyNotNull( comparandType );
   return value!=null ?
      IsTypeOf(value.GetType(),comparandType) :
      false;
}

//==================================================================================================

public static bool IsOneOf(Type type,params Type[] comparandTypes)
{
   XArgCheck.VerifyNotNull( type );
   XArgCheck.VerifyNotNulls( comparandTypes );
   foreach (Type t in comparandTypes)
      if (XType.IsTypeOf(type,t))
         return true;
   return false;
}

public static bool IsOneOf<T>(params Type[] comparandTypes)
{
   return IsOneOf( typeof(T), comparandTypes );
}

//--------------------------------------------------------------------------------------------------

public static bool IsValueOneOf(object value,params Type[] comparandTypes)
{
   XArgCheck.VerifyNotNull( value );
   XArgCheck.VerifyNotNulls( comparandTypes );
   foreach (Type t in comparandTypes)
      if (XType.IsValueOf(value,t))
         return true;
   return false;
}

//==================================================================================================

public static object CreateInstance(Type type)
{
   return CreateInstance(type,null);
}

public static object CreateInstance(Type type,[Optional] params object[] parameters)
{
   return CreateInstanceUsingVisibilityFlags(type,BindingFlags.Public,parameters);
}

//--------------------------------------------------------------------------------------------------

public static T CreateInstance<T>()
{
   return (T)CreateInstance(typeof(T));
}

public static T CreateInstance<T>([Optional] params object[] parameters)
{
   return (T)CreateInstance(typeof(T),parameters);
}

//==================================================================================================

public static object CreateInstanceUsingVisibilityFlags(Type type,BindingFlags visibilityFlags)
{
   return CreateInstanceUsingVisibilityFlags(type,visibilityFlags,null);
}

public static object CreateInstanceUsingVisibilityFlags( Type type,
                                                         BindingFlags visibilityFlags,
                                                         [Optional] params object[] parameters )
{
   return type.InvokeMember( null,
      visibilityFlags | BindingFlags.Instance | BindingFlags.CreateInstance,
      null, null, parameters ?? new object[0] );
}

//--------------------------------------------------------------------------------------------------

public static T CreateInstanceUsingVisibilityFlags<T>(BindingFlags visibilityFlags)
{
   return (T)CreateInstanceUsingVisibilityFlags(typeof(T),visibilityFlags);
}

public static T CreateInstanceUsingVisibilityFlags<T>( BindingFlags visibilityFlags,
                                                       [Optional] params object[] parameters )
{
   return (T)CreateInstanceUsingVisibilityFlags(typeof(T),visibilityFlags,parameters);
}

//==================================================================================================

public static Attribute GetAttribute(Type type,Type attributeType)
{
   XArgCheck.VerifyNotNulls( type, attributeType );
   XArgCheck.VerifyIsTypeOf( attributeType, typeof(Attribute) );

   var attributes =
      (Attribute[])type.GetCustomAttributes( attributeType, false );

   switch (attributes.Length)
   {
   case 0: return null;
   case 1: return attributes[0];
   default:
      throw XInvOpExceptionCreation.Create(
         "Multiple attribute of type \"{0}\" on type \"{1}\".",
         attributeType, type );
   }
}

public static TAttribute GetAttribute<TAttribute>(Type type)
   where TAttribute: Attribute
{
   return (TAttribute)GetAttribute( type, typeof(TAttribute) );
}

//--------------------------------------------------------------------------------------------------

public static bool IsMarkedWithAttribute(Type type,Type attributeType)
{
   return GetAttribute(type,attributeType) != null;
}

//==================================================================================================

static XType()
{
   Library.Access();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XType
//**************************************************************************************************

} // XHelpers
