﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.ComponentModel;

namespace XHelpers {

//**************************************************************************************************
public delegate bool XOverwriteExitCodeCallback(int oldValue,int newValue);
//**************************************************************************************************


//**************************************************************************************************
partial class XProgram {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public const int
   SuccessfulExitCodeDefault = 0,
   FailureExitCodeDefault    = 1;

//==================================================================================================

[DefaultValue(SuccessfulExitCodeDefault)]
public static int SuccessfulExitCode
{
   get { return _SuccessfulExitCode_; }
   set { lock (typeof(XProgram)) _SuccessfulExitCode_ = value; }
}

[DefaultValue(FailureExitCodeDefault)]
public static int FailureExitCode
{
   get { return _FailureExitCode_; }
   set { lock (typeof(XProgram)) _FailureExitCode_ = value; }
}

//--------------------------------------------------------------------------------------------------

public static XOverwriteExitCodeCallback OverwriteExitCodeCallback
{
   get { return _OverwriteExitCodeCallback_; }
   set { lock (typeof(XProgram)) _OverwriteExitCodeCallback_ = value; }
}

//--------------------------------------------------------------------------------------------------

public static int ExitCode
{
   get
   {
      return _ExitCode_;
   }
   set
   {
      lock (_ExitSyncObject_)
         if (value!=_ExitCode_)
         {
            _ExitCode_ = value;
#if !SILVERLIGHT
            if (!XAppDomain.IsDefault())
               return;
#endif // !SILVERLIGHT
            Environment.ExitCode = _ExitCode_;
         }
   }
}

//==================================================================================================

public static bool OverwriteExitCode(int value)
{
   lock (_ExitSyncObject_)
   {
      var overwriteCallback =
         _OverwriteExitCodeCallback_;
      if ( overwriteCallback!=null && overwriteCallback(_ExitCode_,value) ||
           overwriteCallback==null && value!=_ExitCode_ )
      {
         ExitCode = value;
         return true;
      }
      return false;
   }
}

//--------------------------------------------------------------------------------------------------

public static int PrepareToExit()
{
   return PrepareToExit(_ExitCode_);
}

public static int PrepareToExit(int exitCode)
{
   GC.Collect();
   GC.WaitForPendingFinalizers();
   return ExitCode = exitCode;
}

//==================================================================================================

static volatile int
   _SuccessfulExitCode_ = SuccessfulExitCodeDefault,
   _FailureExitCode_    = FailureExitCodeDefault;

static volatile int
   _ExitCode_ = SuccessfulExitCodeDefault;

static volatile XOverwriteExitCodeCallback
   _OverwriteExitCodeCallback_;

static readonly object
   _ExitSyncObject_ = new object();

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XProgram
//**************************************************************************************************

} // XHelpers
