﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.IO;
using System.Reflection;
using System.Diagnostics;
using System.Runtime.CompilerServices;

namespace XHelpers {

//**************************************************************************************************
partial class XProgram {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool IsInteractsWithDesktopSpecified
{
   get { return _InteractsWithDesktop_>=0; }
}

public static bool InteractsWithDesktop
{
   get
   {
      int interactsWithDesktop =
         _InteractsWithDesktop_;
      if (interactsWithDesktop==-1)
         throw new InvalidOperationException(
            "Desktop interaction flag was not specified." );
      return interactsWithDesktop==1;
   }
}

//--------------------------------------------------------------------------------------------------

public static string EntryModuleFullName
{
   [MethodImpl(MethodImplOptions.NoInlining)]
   get
   {
      var entryAssembly =
         Assembly.GetEntryAssembly();
      return entryAssembly!=null ?
         entryAssembly.ManifestModule.FullyQualifiedName :
         Process.GetCurrentProcess().MainModule.FileName;
   }
}

public static string EntryModuleDirectoryName
{
   get
   {
      return Path.GetDirectoryName( EntryModuleFullName );
   }
}

//--------------------------------------------------------------------------------------------------

static string ProcessModuleName
{
   [MethodImpl(MethodImplOptions.NoInlining)]
   get
   {
      return Process.GetCurrentProcess().MainModule.ModuleName;
   }
}

//--------------------------------------------------------------------------------------------------

public static Version Version
{
   get { return Assembly.GetEntryAssembly().GetName().Version; }
}

//==================================================================================================

public static void SpecifyInteractsWithDesktop(bool value)
{
   lock (typeof(XProgram))
      _InteractsWithDesktop_ = value ? 1 : 0;
}

public static bool? GetInteractsWithDesktopSpecification()
{
   switch (_InteractsWithDesktop_)
   {
   case 0: return false;
   case 1: return true;
   }
   return null;
}

//==================================================================================================

[MethodImpl(MethodImplOptions.NoInlining)]
static int GetEntryPointArgsCountFromCommandLine()
{
   return Environment.GetCommandLineArgs().Length - 1;
}

[MethodImpl(MethodImplOptions.NoInlining)]
static string[] GetEntryPointArgsFromCommandLine()
{
   string[] arguments =
      Environment.GetCommandLineArgs();
   return arguments.Length > 1 ?
      XArray.Extract(arguments,1) :
      new string[0];
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XProgram
//**************************************************************************************************

} // XHelpers
