﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.IO;
using System.Reflection;
using System.Diagnostics;
using System.Threading;
using System.ComponentModel;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XProgram {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static string SimpleName
{
   get { return Path.GetFileNameWithoutExtension( MainModuleName ); }
}

//--------------------------------------------------------------------------------------------------

public static bool IsMainModuleNameSpecified
{
   get { return _MainModuleName_!=null; }
}

public static string MainModuleName
{
   get
   {
      string mainModuleName =
         _MainModuleName_;
      if (mainModuleName!=null)
         return mainModuleName;
#if !SILVERLIGHT
      var entryAssembly =
         Assembly.GetEntryAssembly();
      return entryAssembly!=null ?
         entryAssembly.ManifestModule.Name :
         ProcessModuleName;
#endif
      throw new InvalidOperationException(
         "Main program module name was not specified." );
   }
}

//--------------------------------------------------------------------------------------------------

public static bool IsMainThreadSpecified
{
   get { return _MainThread_!=null; }
}

public static Thread MainThread
{
   get
   {
      var result = _MainThread_;
      if (result!=null) return result;
      throw new InvalidOperationException(
         "Main program thread was not specified." );
   }
}

//--------------------------------------------------------------------------------------------------

public static bool AreEntryPointArgumentsSpecified
{
   get { return _EntryPointArguments_ != null; }
}

public static bool HasEntryPointArguments
{
   get
   {
      return EntryPointArgumentsCount > 0;
   }
}

public static int EntryPointArgumentsCount
{
   get
   {
#if SILVERLIGHT
      var arguments = _EntryPointArguments_;
      if (arguments!=null) return arguments.Length;
      throw new InvalidOperationException(_EntryPointArgsNotSpecified_);
#else
      return GetEntryPointArgsCountFromCommandLine();
#endif
   }
}

//==================================================================================================

public static void SpecifyMainModuleName(string value)
{
   _MainModuleName_ = value;
}

public static string GetMainModuleNameSpecification()
{
   return _MainModuleName_;
}

//--------------------------------------------------------------------------------------------------

public static bool IsInMainThread()
{
   return MainThread == Thread.CurrentThread;
}

public static void SpecifyMainThread()
{
   SpecifyMainThread( Thread.CurrentThread );
}

public static void SpecifyMainThread(Thread value)
{
   _MainThread_ = value;
}

public static Thread GetMainThreadSpecification()
{
   return _MainThread_;
}

//--------------------------------------------------------------------------------------------------

public static void VerifyIsInMainThread()
{
   if (!IsInMainThread())
      throw new InvalidOperationException(
         "Current thread is not main program thread." );
}

public static bool IsMainThread(Thread thread)
{
   return thread == MainThread;
}

//--------------------------------------------------------------------------------------------------

public static string[] GetEntryPointArguments()
{
#if SILVERLIGHT
   var arguments = _EntryPointArguments_;
   if (arguments!=null) return arguments;
   throw new InvalidOperationException(_EntryPointArgsNotSpecified_);
#else
   return _EntryPointArguments_ ?? GetEntryPointArgsFromCommandLine();
#endif
}

public static string[] GetEntryPointArgumentsSpecification()
{
   return _EntryPointArguments_;
}

public static void SpecifyEntryPointArguments(string[] arguments)
{
   XArgCheck.VerifyNotNulls( arguments );
   _EntryPointArguments_ = arguments;
}

//==================================================================================================

internal static void Access()
{}

//==================================================================================================

static XProgram()
{
   Library.Access();
}

//==================================================================================================

#if SILVERLIGHT
const string _EntryPointArgsNotSpecified_ =
   "Entry point arguments were not specified.";
#endif // SILVERLIGHT

static volatile string
   _MainModuleName_;
static volatile Thread
   _MainThread_;

#if !SILVERLIGHT
static volatile int
   _InteractsWithDesktop_;
#endif // !SILVERLIGHT

static volatile string[]
   _EntryPointArguments_;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XProgram
//**************************************************************************************************

} // XHelpers
