﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XOperatingSystem {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static readonly OperatingSystem
   Windows2K    = new OperatingSystem( PlatformID.Win32NT, new Version(5,0) ),
   WindowsXP    = new OperatingSystem( PlatformID.Win32NT, new Version(5,1) ),
   WindowsVista = new OperatingSystem( PlatformID.Win32NT, new Version(6,0) ),
   Windows7     = new OperatingSystem( PlatformID.Win32NT, new Version(6,1) );

//==================================================================================================

public static bool IsWindows2KOrLater()
{
   return IsWindows2KOrLater( Environment.OSVersion );
}

public static bool IsWindowsXPOrLater()
{
   return IsWindowsXPOrLater( Environment.OSVersion );
}

public static bool IsWindowsVistaOrLater()
{
   return IsWindowsVistaOrLater( Environment.OSVersion );
}

public static bool IsWindows7OrLater()
{
   return IsWindows7OrLater( Environment.OSVersion );
}

//--------------------------------------------------------------------------------------------------

public static bool IsWindows2KOrLater(OperatingSystem system)
{
   return CheckOperatingSystem( Windows2K );
}

public static bool IsWindowsXPOrLater(OperatingSystem system)
{
   return CheckOperatingSystem( WindowsXP );
}

public static bool IsWindowsVistaOrLater(OperatingSystem system)
{
   return CheckOperatingSystem( WindowsVista );
}

public static bool IsWindows7OrLater(OperatingSystem system)
{
   return CheckOperatingSystem( Windows7 );
}

//==================================================================================================

public static bool CheckOperatingSystem(OperatingSystem requiredMinimalOS)
{
   XArgCheck.VerifyNotNull( requiredMinimalOS );

   OperatingSystem
      system = Environment.OSVersion;
   return
      system.Platform == requiredMinimalOS.Platform &&
      system.Version  >= requiredMinimalOS.Version;
}

//==================================================================================================

public static string GetNameOrVersionString(OperatingSystem system)
{
   XArgCheck.VerifyNotNull( system );

   if (system.Platform == PlatformID.Win32NT)
   {
      if (system == Windows2K   ) return "Windows 2000" ;
      if (system == WindowsXP   ) return "Windows XP"   ;
      if (system == WindowsVista) return "Windows Vista";
      if (system == Windows7    ) return "Windows 7"    ;
   }

#if SILVERLIGHT
   return system.Version.ToString();
#else
   return system.VersionString;
#endif
}

//==================================================================================================

static XOperatingSystem()
{
   Library.Access();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XOperatingSystem
//**************************************************************************************************

} // XHelpers
