﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.ComponentModel;
using System.Security.Permissions;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
[PermissionSetAttribute(SecurityAction.Demand,Unrestricted=true)]
public static partial class XWin32Error {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

[CLSCompliant(false)]
public static uint LastErrorCode
{
   get
   {
      try
         { throw new Win32Exception(); }
      catch (Win32Exception e)
         { return (uint)e.NativeErrorCode; }
   }
}

public static string LastErrorMessage
{
   get { return ErrorCodeToMessage(LastErrorCode); }
}

//==================================================================================================

public static string NormalizeErrorMessage(string message)
{
   XArgCheck.VerifyStringNotEmpty(message);

   message = message.TrimEnd();
   if (message!="" && !char.IsPunctuation(message[message.Length-1]))
      message+='.';

   return message;
}

//--------------------------------------------------------------------------------------------------

[CLSCompliant(false)]
public static string ErrorCodeToMessage(uint errorCode)
{
   return ErrorCodeToMessage((int)errorCode);
}

public static string ErrorCodeToMessage(int errorCode)
{
   try
   {
      throw new Win32Exception(errorCode);
   }
   catch (Win32Exception e)
   {
      return NormalizeErrorMessage(e.Message);
   }
}

//--------------------------------------------------------------------------------------------------

[CLSCompliant(false)]
public static Win32Exception CreateException(uint errorCode)
{
   return CreateException((int)errorCode);
}

public static Win32Exception CreateException(int errorCode)
{
   return new Win32Exception(errorCode,ErrorCodeToMessage(errorCode));
}

//--------------------------------------------------------------------------------------------------

[CLSCompliant(false)]
public static Win32Exception CreateException(uint errorCode,string message)
{
   return CreateException((int)errorCode,message);
}

public static Win32Exception CreateException(int errorCode,string message)
{
   return new Win32Exception(errorCode,NormalizeErrorMessage(message));
}

//--------------------------------------------------------------------------------------------------

public static Win32Exception CreateException(Win32Exception e)
{
   return CreateException(e.NativeErrorCode,e.Message);
}

//--------------------------------------------------------------------------------------------------

public static Win32Exception CreateExceptionFromLastError()
{
   return CreateException(LastErrorCode);
}

//==================================================================================================

static XWin32Error()
{
   Library.Access();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XWin32Error
//**************************************************************************************************

} // XHelpers
