﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Reflection;
using System.Globalization;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
partial class XValueParser {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool TryParse<T>( string src, [Optional] object[] additionalParams,
                                out T dst, out XParseValueException e ) where T: struct
   { return TryParseEx(src,additionalParams,null,out dst,out e); }

public static bool TryParse<T>( string src, [Optional] object[] additionalParams, T def,
                                out T dst, out XParseValueException e ) where T: struct
   { return TryParseEx(src,additionalParams,(T?)def,out dst,out e); }

//--------------------------------------------------------------------------------------------------

public static bool TryParse<T>(string src,out T dst,out XParseValueException e) where T: struct
   { return TryParse(src,(object[])null,out dst,out e); }
public static bool TryParse<T>( string src, T def,
                                out T dst, out XParseValueException e ) where T: struct
   { return TryParse(src,(object[])null,def,out dst,out e); }

public static bool TryParse<T>( string src, IFormatProvider provider,
                                out T dst, out XParseValueException e ) where T: struct
   { return TryParse(src,new object[]{provider},out dst,out e); }
public static bool TryParse<T>( string src, IFormatProvider provider, T def,
                                out T dst, out XParseValueException e ) where T: struct
   { return TryParse(src,new object[]{provider},def,out dst,out e); }

//--------------------------------------------------------------------------------------------------

public static bool TryParseInvariant<T>( string src,
                                         out T dst, out XParseValueException e ) where T: struct
   { return TryParse(src,new object[]{(IFormatProvider)null},out dst,out e); }
public static bool TryParseInvariant<T>( string src, T def,
                                         out T dst, out XParseValueException e ) where T: struct
   { return TryParse(src,new object[]{(IFormatProvider)null},def,out dst,out e); }

//==================================================================================================

internal static bool TryParseEx<T>( string src, [Optional] object[] additionalParams,
                                    T? def, out T dst, out XParseValueException e )
   where T: struct
{
   return TryParseEx
   (
      src, "Parse",
      XArray.Concatenate(
         new object[]{ src },
         additionalParams ?? new object[0] ),
      def, out dst, out e
   );
}

//--------------------------------------------------------------------------------------------------

internal static bool TryParseEx<T>( string src, string parseMethodName, object[] parameters,
                                    T? def, out T dst, out XParseValueException e )
   where T: struct
{
   Type type = typeof(T);

   XGenericTypeCheck.VerifyIsStdValueType( type );
   XArgCheck.VerifyNotNull( src );

   src = src.Trim();

   if (src=="")
      if (def!=null)
      {
         dst=def.Value; e=null; return true;
      }
      else
      {
         dst = default(T);
         e = new XParseValueException( "Value string is empty." );
         return false;
      }

   try
   {
      dst = (T)XMethodInfo.InvokeStaticMethod( type, parseMethodName, parameters );
   }
   catch (TargetInvocationException x)
   {
      var ie=x.InnerException;
      if (ie==null)
         throw;
      if (!(ie is FormatException || ie is OverflowException))
         throw ie;
      dst = default(T);
      e = new XParseValueException( ie.Message );
      return false;
   }
   e = null;
   return true;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XValueParser
//**************************************************************************************************

} // XHelpers
