﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Reflection;
using System.Globalization;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
partial class XValueParser {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool TryParse<T>( string src, [Optional] object[] additionalParams,
                                out T dst ) where T: struct
   { return TryParseEx(src,additionalParams,null,out dst); }

public static bool TryParse<T>( string src, [Optional] object[] additionalParams, T def,
                                out T dst ) where T: struct
   { return TryParseEx(src,additionalParams,(T?)def,out dst); }

//--------------------------------------------------------------------------------------------------

public static bool TryParse<T>(string src,out T dst) where T: struct
   { return TryParse(src,(object[])null,out dst); }
public static bool TryParse<T>(string src,T def,out T dst) where T: struct
   { return TryParse(src,(object[])null,def,out dst); }

public static bool TryParse<T>(string src,IFormatProvider provider,out T dst) where T: struct
   { return TryParse(src,new object[]{provider},out dst); }
public static bool TryParse<T>(string src,IFormatProvider provider,T def,out T dst) where T: struct
   { return TryParse(src,new object[]{provider},def,out dst); }

//--------------------------------------------------------------------------------------------------

public static bool TryParseInvariant<T>(string src,out T dst) where T: struct
   { return TryParse(src,new object[]{(IFormatProvider)null},out dst); }
public static bool TryParseInvariant<T>(string src,T def,out T dst) where T: struct
   { return TryParse(src,new object[]{(IFormatProvider)null},def,out dst); }

//==================================================================================================

internal static bool TryParseEx<T>(string src,[Optional] object[] additionalParams,T? def,out T dst)
   where T: struct
{
   XParseValueException e;
   return TryParseEx(src,additionalParams,def,out dst,out e);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XValueParser
//**************************************************************************************************

} // XHelpers
