﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

namespace XHelpers {

using System;
using System.Threading;

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XBeeper {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

static XBeeper()
{
   Library.Access();
}

//--------------------------------------------------------------------------------------------------

static void PlayBeeps(int[] beeps)
{
   for (int i=0; i<beeps.Length && !_StopFlag_; i+=2)
   {
      if (Monitor.TryEnter(_StopSignalSyncObject_))
         Monitor.Exit(_StopSignalSyncObject_);
      else
      {
         if (Monitor.TryEnter(_StopControlSyncObject_))
            Monitor.Exit(_StopControlSyncObject_);
         else
            _StopOwnerThread_ = _StopInitiatorThread_;
         break;
      }
      int
         frequency= beeps[i],
         duration = beeps[i+1];
      if (frequency>0)
         Console.Beep( frequency, duration );
      else
         Thread.Sleep( duration );
   }
}

//--------------------------------------------------------------------------------------------------

static int[] PrepareBeeps(double[] beeps)
{
   if (!XArith.IsEven(beeps.Length))
      throw new ArgumentException(
         "Beep sequence must be of pari length." );

   var result = new int[ beeps.Length ];

   lock (typeof(XBeeper))
      for (int i=0; i<beeps.Length; i+=2)
      {
         double
            frequencyScale= beeps[i  ],
            durationScale = beeps[i+1];
         int
            frequency, duration;
         try
         {
            frequency = (int)( BaseFrequency*frequencyScale );
            duration  = (int)( BaseDuration *durationScale  );
         }
         catch (Exception e)
         {
            if (e is OverflowException)
               throw new ArgumentException("Incorrect beep sequence.");
            throw;
         }
         if ( frequencyScale!=0 && !IsValidFrequency(frequency) )
            if (frequency!=0)
               throw new ArgumentException( string.Format(
                  "Incorrect beep frequency: {0} (Hz).\r\n"+
                  "Valid range: {1}..{2} (Hz) or 0 (for silence).",
                  frequency, MinFrequency, MaxFrequency ));
            else
               throw new ArgumentException( string.Format(
                  "Incorrect beep frequency.\r\n"+
                  "Valid range: {0}..{1} (Hz) or 0 (for silence).",
                  MinFrequency, MaxFrequency ));
         VerifyDuration( duration );

         result[i  ] = frequency;
         result[i+1] = duration;
      }

   return result;
}

//--------------------------------------------------------------------------------------------------

static void PlaybackThread()
{
   lock (_ThreadActivitySyncObject_)
   {
      Monitor.Pulse(_ThreadActivitySyncObject_);
      for (;;)
      {
         Monitor.Wait(_ThreadActivitySyncObject_);
         lock (_PlaybackSyncObject_)
         {
            _StopFlag_ = false;
            _StartedEvent_.Set();
            PlayBeeps( _PlaybackBeeps_ );
            _PlaybackThread_.Priority = _DefaultPriority_;
         }
      }
   }
}

//==================================================================================================

const ThreadPriority
   _DefaultPriority_ = ThreadPriority.AboveNormal;

static volatile int
   _BaseFrequency_ = BaseFrequencyDefault,
   _BaseDuration_  = BaseDurationDefault;
static volatile bool
   _IsMute_,
   _IsInitialized_,
   _StopFlag_;
static volatile int[]
   _PlaybackBeeps_;
static volatile AutoResetEvent
   _StartedEvent_;
static volatile Thread
   _PlaybackThread_,
   _StopInitiatorThread_,
   _StopOwnerThread_;

static readonly object
   _InitializationSyncObject_ = new object(),
   _StartSyncObject_          = new object(),
   _ThreadActivitySyncObject_ = new object(),
   _PlaybackSyncObject_       = new object(),
   _StopSignalSyncObject_     = new object(),
   _StopControlSyncObject_    = new object();

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XBeeper
//**************************************************************************************************

} // XHelpers
