﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.IO;
using System.Media;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public delegate bool XPlaySoundCallback(XSound sound,bool synchronously);
//**************************************************************************************************


//**************************************************************************************************
/// <summary>
/// </summary>
public delegate bool XAbortSoundCallback();
//**************************************************************************************************


//**************************************************************************************************
/// <summary>
/// </summary>
public sealed partial class XSound {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool IsMute
{
   get { return _IsMute_; }
   set { _IsMute_ = value; }
}

//--------------------------------------------------------------------------------------------------

public static XPlaySoundCallback PlaySoundCallback
{
   get { return _PlaySoundCallback_; }
   set { _PlaySoundCallback_ = value; }
}

public static XAbortSoundCallback AbortSoundCallback
{
   get { return _AbortSoundCallback_; }
   set { _AbortSoundCallback_ = value; }
}

//--------------------------------------------------------------------------------------------------

public static bool IsInitialized
{
   get { return _IsInitialized_; }
}

//==================================================================================================

public static void Initialize()
{
   lock (typeof(XSound))
      if (!_IsInitialized_)
      {
         XBeeper.Initialize();
         _IsInitialized_ = true;
      }
}

//==================================================================================================

public static void Play(XSound sound)
{
   Play(sound,false);
}

public static void Play(XSound sound,bool ignoreMute)
{
   Play(sound,ignoreMute,false);
}

public static void Play(XSound sound,bool ignoreMute,bool synchronously)
{
   if (sound!=XSounds.None && (ignoreMute || !IsMute))
   {
      var callback = PlaySoundCallback;

      if (callback==null || !callback( sound, synchronously ))
         foreach (var layout in XSounds.DefaultBeepsLayouts)
            if (layout.Sound==sound)
            {
               XBeeper.Play( synchronously, layout.Beeps );
               return;
            }
   }
}

//--------------------------------------------------------------------------------------------------

public static void SyncPlay(XSound sound)
{
   SyncPlay(sound,false);
}

public static void SyncPlay(XSound sound,bool ignoreMute)
{
   Play(sound,ignoreMute,true);
}

//--------------------------------------------------------------------------------------------------

public static void Abort()
{
   var callback = AbortSoundCallback;
   if (callback==null || !callback())
      XBeeper.StopActivePlayback( false );
}

//==================================================================================================

public static void PlaySuccess    () { PlaySuccess    ( false ); }
public static void PlayNotSuccess () { PlayNotSuccess ( false ); }
public static void PlayInformation() { PlayInformation( false ); }
public static void PlayError      () { PlayError      ( false ); }
public static void PlayWarning    () { PlayWarning    ( false ); }
public static void PlayExclamation() { PlayExclamation( false ); }
public static void PlayQuestion   () { PlayQuestion   ( false ); }
public static void PlayDisabled   () { PlayDisabled   ( false ); }

//--------------------------------------------------------------------------------------------------

public static void PlaySuccess    ( bool ignoreMute ) { Play( XSounds.Success    , ignoreMute ); }
public static void PlayNotSuccess ( bool ignoreMute ) { Play( XSounds.NotSuccess , ignoreMute ); }
public static void PlayInformation( bool ignoreMute ) { Play( XSounds.Information, ignoreMute ); }
public static void PlayError      ( bool ignoreMute ) { Play( XSounds.Error      , ignoreMute ); }
public static void PlayWarning    ( bool ignoreMute ) { Play( XSounds.Warning    , ignoreMute ); }
public static void PlayExclamation( bool ignoreMute ) { Play( XSounds.Exclamation, ignoreMute ); }
public static void PlayQuestion   ( bool ignoreMute ) { Play( XSounds.Question   , ignoreMute ); }
public static void PlayDisabled   ( bool ignoreMute ) { Play( XSounds.Disabled   , ignoreMute ); }

//==================================================================================================

static XSound()
{
   Library.Access();
}

//==================================================================================================

static volatile bool
   _IsMute_,
   _IsInitialized_;

static volatile XPlaySoundCallback
   _PlaySoundCallback_;
static volatile XAbortSoundCallback
   _AbortSoundCallback_;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XSound
//**************************************************************************************************

} // XHelpers
