﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.IO;
using System.Media;
using System.Runtime.InteropServices;
using System.Runtime.CompilerServices;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XSoundPlayer {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static void PlayStream([Optional] Stream stream)
{
   PlayStream(stream,false);
}

public static void PlayStream([Optional] Stream stream,bool ignoreMute)
{
   PlayStream(stream,ignoreMute,false);
}

[MethodImpl(MethodImplOptions.NoInlining)]
public static void PlayStream([Optional] Stream stream,bool ignoreMute,bool synchronously)
{
   if (stream!=null && (ignoreMute || !XSound.IsMute))
   {
      var soundPlayer = new SoundPlayer();
      soundPlayer.Stream = stream;
      if (synchronously)
         soundPlayer.PlaySync();
      else
         soundPlayer.Play();
   }
}

//==================================================================================================

public static bool PlayFile([Optional] string entryModuleRelativeLocation)
{
   return PlayFile(entryModuleRelativeLocation,false);
}

public static bool PlayFile([Optional] string entryModuleRelativeLocation,bool ignoreMute)
{
   return PlayFile(entryModuleRelativeLocation,ignoreMute,false);
}

[MethodImpl(MethodImplOptions.NoInlining)]
public static bool PlayFile( [Optional] string entryModuleRelativeLocation,
                             bool ignoreMute, bool synchronously )
{
   if (entryModuleRelativeLocation!=null && (ignoreMute || !XSound.IsMute))
   {
      string
         filePath = Path.Combine( XProgram.EntryModuleDirectoryName,
                                  entryModuleRelativeLocation );
      if (!File.Exists( filePath ))
         return false;
      var soundPlayer =
         new SoundPlayer( filePath );
      if (synchronously)
         soundPlayer.PlaySync();
      else
         soundPlayer.Play();
   }
   return true;
}

//==================================================================================================

public static void SyncPlayStream([Optional] Stream stream)
{
   SyncPlayStream(stream,false);
}

public static void SyncPlayStream([Optional] Stream stream,bool ignoreMute)
{
   PlayStream(stream,ignoreMute,true);
}

//--------------------------------------------------------------------------------------------------

public static bool SyncPlayFile([Optional] string entryModuleRelativeLocation)
{
   return SyncPlayFile(entryModuleRelativeLocation,false);
}

public static bool SyncPlayFile([Optional] string entryModuleRelativeLocation,bool ignoreMute)
{
   return PlayFile(entryModuleRelativeLocation,ignoreMute,true);
}

//==================================================================================================

static XSoundPlayer()
{
   Library.Access();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XSoundPlayer
//**************************************************************************************************

} // XHelpers
