﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public partial class XDisposableHandle: XDisposable {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public XDisposableHandle(IntPtr handle)
   :this(handle,false)
{}

public XDisposableHandle( IntPtr handle,
                          bool isOfInvalidHandleValueCategory )
   :this(handle,isOfInvalidHandleValueCategory,null)
{}

public XDisposableHandle( IntPtr handle,
                          [Optional] XCloseHandleCallback closeHandleCallback )
   :this(handle,false,closeHandleCallback)
{}

public XDisposableHandle( IntPtr handle,
                          bool isOfInvalidHandleValueCategory,
                          [Optional] XCloseHandleCallback closeHandleCallback )
{
   XHandle.SimpleVerification(
      handle, isOfInvalidHandleValueCategory );

   _Handle_                         = handle;
   _IsOfInvalidHandleValueCategory_ = isOfInvalidHandleValueCategory;
   _CloseHandleCallback_            = closeHandleCallback;

   ValidateDisposable();
}

//--------------------------------------------------------------------------------------------------

public static implicit operator IntPtr(XDisposableHandle dh)
{
   XArgCheck.VerifyNotNull( dh );
   return dh.IsValid ? dh._Handle_ :
      XHandle.GetNullOrInvalidValue( dh._IsOfInvalidHandleValueCategory_ );
}

//--------------------------------------------------------------------------------------------------

public bool IsValid
{
   get
   {
      return IsValidDisposable;
   }
}

public IntPtr Value
{
   get
   {
      VerifyIsValidDisposable();
      return _Handle_;
   }
}

public bool IsOfInvalidHandleValueCategory
{
   get
   {
      VerifyIsValidDisposable();
      return _IsOfInvalidHandleValueCategory_;
   }
}

public XCloseHandleCallback CloseHandleCallback
{
   get
   {
      VerifyIsValidDisposable();
      return _CloseHandleCallback_;
   }
}

//==================================================================================================

protected override void OnDispose(bool synchronous)
{
   XHandle.ResetAndClose( ref _Handle_,
      _IsOfInvalidHandleValueCategory_,
      _CloseHandleCallback_ );
}

//==================================================================================================

static XDisposableHandle()
{
   Library.Access();
}

//==================================================================================================

IntPtr
   _Handle_;
bool
   _IsOfInvalidHandleValueCategory_;
XCloseHandleCallback
   _CloseHandleCallback_;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XDisposableHandle
//**************************************************************************************************

} // XHelpers
