﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public partial class XDisposableSZ: XDisposable {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public XDisposableSZ(): this(1)
{}

public XDisposableSZ(string str)
{
   _Buffer_ = XSZ.Duplicate( str );
   ValidateDisposable();
}

public XDisposableSZ(IntPtr psz)
{
   _Buffer_ = XSZ.Duplicate( psz );
   ValidateDisposable();
}

public XDisposableSZ(int capacity)
{
   _Buffer_ = XSZ.Allocate( capacity );
   ValidateDisposable();
}

//==================================================================================================

public unsafe char this[int charIdx]
{
   get
   {
      VerifyIsValidDisposable();
      VerifyCharIndex( charIdx );
      return ((char*)_Buffer_)[ charIdx ];
   }
   set
   {
      VerifyIsValidDisposable();
      VerifyCharIndex( charIdx );
      ((char*)_Buffer_)[ charIdx ] = value;
   }
}

//--------------------------------------------------------------------------------------------------

public static implicit operator IntPtr(XDisposableSZ dsz)
{
   return dsz!=null ? dsz.Buffer : IntPtr.Zero;
}

public static explicit operator string(XDisposableSZ dsz)
{
   return dsz!=null ? dsz.ToString() : null;
}

public static unsafe explicit operator char*(XDisposableSZ dsz)
{
   return (char*)(IntPtr)dsz;
}

//--------------------------------------------------------------------------------------------------

public bool IsValid
{
   get
   {
      return IsValidDisposable;
   }
}

public IntPtr Buffer
{
   get
   {
      VerifyIsValidDisposable();
      return _Buffer_;
   }
}

public int Capacity
{
   get
   {
      VerifyIsValidDisposable();
      return XSZ.GetAllocatedCapacity(_Buffer_);
   }
}

public int Length
{
   get
   {
      VerifyIsValidDisposable();
      return XSZ.Length( _Buffer_ );
   }
}

public int MaxLength
{
   get
   {
      return Capacity-1;
   }
}

public bool IsTwiceEnlargeCapacityPossible
{
   get
   {
      return unchecked(Capacity*2/2)==Capacity;
   }
}

//==================================================================================================

public override unsafe string ToString()
{
   return XSZ.ToString( Buffer );
}

//--------------------------------------------------------------------------------------------------

public void ReAllocate(int newCapacity)
{
   VerifyIsValidDisposable();
   XSZ.ReAllocate(ref _Buffer_,newCapacity);
}

public void ReAllocateByLength()
{
   VerifyIsValidDisposable();
   XSZ.ReAllocateByLength(ref _Buffer_);
}

public void TwiceEnlargeCapacity()
{
   if (!IsTwiceEnlargeCapacityPossible)
      throw new XOperationFailedException(
         "Memory allocation block is too large (for signed integer value)." );
   ReAllocate( Capacity*2 );
}

//==================================================================================================

protected override void OnDispose(bool synchronous)
{
   XSZ.Free(ref _Buffer_);
}

//==================================================================================================

void VerifyCharIndex(int idx)
{
   if (!XOps.IsBetween(idx,0,MaxLength-1))
      throw new ArgumentOutOfRangeException(
         null, "Incorrect character index." );
}

//==================================================================================================

static XDisposableSZ()
{
   Library.Access();
}

//==================================================================================================

IntPtr _Buffer_;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XDisposableSZ
//**************************************************************************************************

} // XHelpers
