﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Threading;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public delegate void XCloseHandleCallback(IntPtr handle);
//**************************************************************************************************


//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XHandle {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static readonly IntPtr
   NullValue = IntPtr.Zero;

public static readonly IntPtr
   InvalidValue = (IntPtr)(-1);

//==================================================================================================

public static bool SimpleValidationCheck( [Optional] IntPtr handle,
                                          bool isOfInvalidHandleValueCategory )
{
   return isOfInvalidHandleValueCategory ?
      handle!=InvalidValue : handle!=NullValue;
}

public static void SimpleVerification( [Optional] IntPtr handle,
                                       bool isOfInvalidHandleValueCategory )
{
   if (!SimpleValidationCheck(handle,isOfInvalidHandleValueCategory))
      throw new ArgumentException(
         isOfInvalidHandleValueCategory ?
            "Handle value is invalid." :
            "Handle value is null." );
}

//==================================================================================================

public static IntPtr GetNullOrInvalidValue(bool isOfInvalidHandleValueCategory)
{
   return isOfInvalidHandleValueCategory ? InvalidValue : NullValue;
}

//==================================================================================================

public static void Close([Optional] IntPtr handle)
{
   Close(handle,false);
}

public static void Close( [Optional] IntPtr handle,
                          bool isOfInvalidHandleValueCategory )
{
   Close(handle,isOfInvalidHandleValueCategory,null);
}

//--------------------------------------------------------------------------------------------------

public static void Close( [Optional] IntPtr handle,
                          [Optional] XCloseHandleCallback closeHandleCallback )
{
   Close(handle,false,closeHandleCallback);
}

public static void Close( [Optional] IntPtr handle,
                          bool isOfInvalidHandleValueCategory,
                          [Optional] XCloseHandleCallback closeHandleCallback )
{
   if (SimpleValidationCheck(handle,isOfInvalidHandleValueCategory))
      if (closeHandleCallback!=null)
         closeHandleCallback( handle );
      else
         if (!Win32.CloseHandle( handle ))
            throw XWin32Error.CreateExceptionFromLastError();
}

//==================================================================================================

public static void ZeroAndClose([Optional] ref IntPtr handle)
{
   ZeroAndClose(ref handle,null);
}

public static void ZeroAndClose( [Optional] ref IntPtr handle,
                                 [Optional] XCloseHandleCallback closeHandleCallback )
{
   ResetAndClose(ref handle,false,closeHandleCallback);
}

//--------------------------------------------------------------------------------------------------

public static void InvalidateAndClose([Optional] ref IntPtr handle)
{
   InvalidateAndClose(ref handle,null);
}

public static void InvalidateAndClose( [Optional] ref IntPtr handle,
                                       [Optional] XCloseHandleCallback closeHandleCallback )
{
   ResetAndClose(ref handle,true,closeHandleCallback);
}

//--------------------------------------------------------------------------------------------------

public static void ResetAndClose( [Optional] ref IntPtr handle,
                                  bool isOfInvalidHandleValueCategory )
{
   ResetAndClose(ref handle,isOfInvalidHandleValueCategory,null);
}

public static void ResetAndClose( [Optional] ref IntPtr handle,
                                  bool isOfInvalidHandleValueCategory,
                                  [Optional] XCloseHandleCallback closeHandleCallback )
{
   Close(
      Interlocked.Exchange( ref handle,
         GetNullOrInvalidValue( isOfInvalidHandleValueCategory ) ),
      isOfInvalidHandleValueCategory,
      closeHandleCallback );
}

//==================================================================================================

static XHandle()
{
   Library.Access();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XHandle
//**************************************************************************************************

} // XHelpers
