﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.ComponentModel;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
partial class XMemory {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static int ElementSize<T>() where T: struct
{
   return XMarshal.SizeOf( typeof(T) );
}

public static int CountToSize<T>(int count) where T: struct
{
   if (count<0)
      throw new ArgumentOutOfRangeException(
         null, "Incorrect memory elements count." );
   return ElementSize<T>()*count;
}

//==================================================================================================

public static IntPtr Clear<T>([Out] IntPtr destination,int count) where T: struct
{
   return Clear(destination,CountToSize<T>(count));
}

public static IntPtr Copy<T>([In] IntPtr source,[Out] IntPtr destination,int count) where T: struct
{
   return Copy(source,destination,CountToSize<T>(count));
}

//==================================================================================================

/// <exception cref="XOperationFailedException" />
public static IntPtr Allocate<T>(int capacity) where T: struct
{
   return Allocate( CountToSize<T>(capacity) );
}

/// <exception cref="XOperationFailedException" />
public static IntPtr ReAllocate<T>(IntPtr allocatedBuffer,int newCapacity) where T: struct
{
   return ReAllocate(allocatedBuffer,CountToSize<T>(newCapacity));
}

/// <exception cref="XOperationFailedException" />
public static IntPtr ReAllocate<T>(ref IntPtr allocatedBuffer,int newCapacity) where T: struct
{
   return ReAllocate(ref allocatedBuffer,CountToSize<T>(newCapacity));
}

/// <exception cref="XOperationFailedException" />
public static IntPtr Duplicate<T>([In] IntPtr allocatedBuffer) where T: struct
{
   VerifyAllocatedBufferSize<T>( allocatedBuffer );
   return Duplicate( allocatedBuffer );
}

//--------------------------------------------------------------------------------------------------

/// <exception cref="XOperationFailedException" />
public static int GetAllocatedCapacity<T>([In] IntPtr allocatedBuffer) where T: struct
{
   int size = GetAllocatedSize( allocatedBuffer );
   VerifyAllocatedBufferSize<T>( size );
   return size/ElementSize<T>();
}

//--------------------------------------------------------------------------------------------------

/// <exception cref="Win32Exception" />
public static void Free<T>([Optional] IntPtr allocatedBuffer) where T: struct
{
   if (allocatedBuffer!=IntPtr.Zero)
   {
      VerifyAllocatedBufferSize<T>( allocatedBuffer );
      Free( allocatedBuffer );
   }
}

/// <exception cref="Win32Exception" />
public static void Free<T>([Optional] ref IntPtr allocatedBuffer) where T: struct
{
   Free( XIntPtr.InterlockedZeroExchange(ref allocatedBuffer) );
}


//==================================================================================================

static void VerifyAllocatedBufferSize<T>(int sizeInBytes) where T: struct
{
   if ( sizeInBytes < ElementSize<T>() ||
        sizeInBytes % ElementSize<T>() != 0 )
      throw new ArgumentException(
         "Incorrect size of previously allocated structured memory buffer." );
}

static void VerifyAllocatedBufferSize<T>(IntPtr allocatedBuffer) where T: struct
{
   VerifyAllocatedBufferSize<T>(
      GetAllocatedSize( allocatedBuffer ) );
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XMemory
//**************************************************************************************************

} // XHelpers
