﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Runtime.InteropServices;

using System.Windows.Input;

namespace XHelpers {

//**************************************************************************************************
partial class XAppWaiting {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public XAppWaiting(): this(XAppWaitingStatus.Neutral)
{}

public XAppWaiting([Optional] string caption)
   :this(new XAppWaitingStatus(caption))
{}

public XAppWaiting(string captionFmt,params object[] parameters)
   :this(string.Format(captionFmt,parameters))
{}

public XAppWaiting(XAppWaitingStatus status)
{
   lock (typeof(XAppWaiting))
   {
      VerifyIsActive();
      _PreviousStatus_ = _CurrentStatus_;
      _PreviousInstance_ = XOps.Exchange(ref _CurrentInstance_,this);
      SetStatus( status!=XAppWaitingStatus.Neutral ? status : CurrentStatus );
      _InitialStatus_ = _CurrentStatus_;
      ShowStatus();
   }
   ValidateDisposable();
}

//==================================================================================================

public XAppWaitingStatus InitialStatus
{
   get
   {
      lock (this) {
         VerifyIsActive();
         return _InitialStatus_; }
   }
}

public string InitialCaption
{
   get
   {
      lock (this) {
         VerifyIsActive();
         return _InitialStatus_.Caption; }
   }
}

public XAppWaitingStatus PreviousStatus
{
   get
   {
      lock (this) {
         VerifyIsActive();
         return _PreviousStatus_; }
   }
}

//==================================================================================================

public void RevertStatusToInitial()
{
   lock (this)
      DisplayStatus( _InitialStatus_ );
}

//==================================================================================================

protected override void OnDispose(bool synchronous)
{
   lock (this)
   lock (typeof(XAppWaiting))
      if (!_SuppressBackOff_)
      {
         VerifyIsActive();
         _IsInDispose_ = true;
         try
            { DisplayStatus(_PreviousStatus_); }
         finally
            { _IsInDispose_ = false; }
         _CurrentInstance_ = _PreviousInstance_;
      }
}

//==================================================================================================

XAppWaitingStatus
   _InitialStatus_,
   _PreviousStatus_;
XAppWaiting
   _PreviousInstance_;
bool
   _SuppressBackOff_;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XAppWaiting
//**************************************************************************************************

} // PaycorMaster
