﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
partial class XAppWaiting {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool ReportError(string errorMessage)
{
   return ReportErrorWithData(errorMessage,null);
}

public static bool ReportError(Exception e)
{
   return ReportErrorWithData(e,null);
}

//--------------------------------------------------------------------------------------------------

public static bool ReportError(string errorMessage,bool deactivateAppWaiting)
{
   return ReportErrorWithData(errorMessage,null,deactivateAppWaiting);
}

public static bool ReportError(Exception e,bool deactivateAppWaiting)
{
   return ReportErrorWithData(e,null,deactivateAppWaiting);
}

//==================================================================================================

public static bool ReportErrorWithData(string errorMessage,[Optional] object data)
{
   return ReportErrorWithData(errorMessage,data,true);
}

public static bool ReportErrorWithData(Exception e,[Optional] object data)
{
   return ReportErrorWithData(e,data,true);
}

//--------------------------------------------------------------------------------------------------

public static bool ReportErrorWithData( string errorMessage,
                                        [Optional] object data,
                                        bool deactivateAppWaiting )
{
   XArgCheck.VerifyNotNull( errorMessage );
   XDispatcher.Synchronize(delegate
   {
      VerifyIsActive();
      var reportErrorCallback = ReportErrorCallback;
#if !SILVERLIGHT
      if (reportErrorCallback==null)
         reportErrorCallback =
            delegate (string em,string at,object d)
               { XMessageBox.ShowError(em,at); };
#endif // !SILVERLIGHT
      reportErrorCallback( errorMessage, Title );
      if (deactivateAppWaiting)
         Deactivate();
   });
   return false;
}

public static bool ReportErrorWithData( Exception e,
                                        [Optional] object data,
                                        bool deactivateAppWaiting )
{
   XArgCheck.VerifyNotNull( e );
   return ReportErrorWithData(
      XException.GetErrorMessage(e), data, deactivateAppWaiting );
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XAppWaiting
//**************************************************************************************************

} // PaycorMaster
