﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.ComponentModel;
using System.Runtime.InteropServices;

using System.Windows.Input;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
[XSyncOnlyDisposable]
public partial class XAppWaiting: XDisposable {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static event XDisplayAppWaitingStatusHandler
   DisplayAppWaitingStatusHandler;

//--------------------------------------------------------------------------------------------------

[DefaultValue(true)]
public static bool KeepApplicationBusyWhenActive
{
   get { return _KeepApplicationBusyWhenActive_; }
   set { _KeepApplicationBusyWhenActive_ = value; }
}

public static XReportAppWaitingErrorCallback ReportErrorCallback
{
   get { return _ReportErrorCallback_; }
   set { _ReportErrorCallback_ = value; }
}

//--------------------------------------------------------------------------------------------------

#if !SILVERLIGHT

[DefaultValue(true)]
public static bool ReleaseCaptureOnActivation
{
   get { return _ReleaseCaptureOnActivation_; }
   set { _ReleaseCaptureOnActivation_ = value; }
}

[DefaultValue(false)]
public static bool AffectsCursor
{
   get { return _AffectsCursor_; }
   set { _AffectsCursor_ = value; }
}

#endif // !SILVERLIGHT

//==================================================================================================

public static bool IsActive
{
   get
   {
      lock (typeof(XAppWaiting))
         return _CurrentStatus_ != null;
   }
}

public static string Title
{
   get
   {
      lock (typeof(XAppWaiting))
      {
         VerifyIsActive();
         return _Title_;
      }
   }
}

public static string Caption
{
   get
   {
      lock (typeof(XAppWaiting))
      {
         VerifyIsActive();
         return _CurrentStatus_.Caption;
      }
   }
}

public static object Data
{
   get
   {
      lock (typeof(XAppWaiting))
      {
         VerifyIsActive();
         return _CurrentStatus_.Data;
      }
   }
}

//--------------------------------------------------------------------------------------------------

public static XAppWaitingStatus CurrentStatus
{
   get
   {
      lock (typeof(XAppWaiting))
      {
         VerifyIsActive();
         return _CurrentStatus_;
      }
   }
}

public static XAppWaitingStatus RootStatus
{
   get
   {
      lock (typeof(XAppWaiting))
      {
         VerifyIsActive();
         return _RootStatus_;
      }
   }
}

//--------------------------------------------------------------------------------------------------

public static XAppWaiting CurrentInstance
{
   get
   {
      lock (typeof(XAppWaiting))
      {
         VerifyIsActive();
         if (_CurrentInstance_==null)
            throw new InvalidOperationException(
               "No instance of application waiting was created." );
         if (_IsInDispose_)
            throw new InvalidOperationException(
               "Current instance of application waiting can not be accessed during disposing." );
         return _CurrentInstance_;
      }
   }
}

//--------------------------------------------------------------------------------------------------

public static string LastTitle
{
   get
   {
      lock (typeof(XAppWaiting))
      {
         VerifyIsNotActive();
         return _LastTitle_;
      }
   }
}

public static XAppWaitingStatus LastRootStatus
{
   get
   {
      lock (typeof(XAppWaiting))
      {
         VerifyIsNotActive();
         return _LastRootStatus_;
      }
   }
}

//==================================================================================================

public static XAppWaiting CreateInstance([Optional] string caption,[Optional] object data)
{
   return new XAppWaiting(new XAppWaitingStatus(caption,data));
}

//==================================================================================================

public static void VerifyIsActive()
{
   if (!IsActive)
      throw new InvalidOperationException(
         "Application waiting is not active." );
}

public static void VerifyIsNotActive()
{
   if (IsActive)
      throw new InvalidOperationException(
         "Application waiting must not be active." );
}

//==================================================================================================

public static XAppWaitingStatus Activate([Optional] string title)
{
   return Activate(title,title);
}

public static XAppWaitingStatus Activate([Optional] string title,[Optional] string caption)
{
   return ActivateWithData(title,caption,null);
}

public static XAppWaitingStatus Activate( [Optional] string title,
                                          string captionFmt,
                                          params object[] parameters )
{
   return Activate(title,string.Format(captionFmt,parameters));
}

public static XAppWaitingStatus Activate([Optional] string title,XAppWaitingStatus status)
{
   lock (typeof(XAppWaiting))
   {
      if (IsActive)
         SuppressBackOff();
      _Title_ = title;
      var result = SetStatus( status );
      _RootStatus_ = _CurrentStatus_;
      _LastTitle_ = null;
      _LastRootStatus_ = null;
      SwitchToActiveAppearance();
      return result;
   }
}

//--------------------------------------------------------------------------------------------------

public static XAppWaitingStatus ActivateWithData([Optional] string title,[Optional] object data)
{
   return ActivateWithData(title,title,data);
}

public static XAppWaitingStatus ActivateWithData( [Optional] string title,
                                                  [Optional] string caption,
                                                  [Optional] object data )
{
   return Activate(title,new XAppWaitingStatus(caption,data));
}

//--------------------------------------------------------------------------------------------------

public static XAppWaitingStatus Deactivate()
{
   lock (typeof(XAppWaiting))
      if (IsActive)
      {
         SuppressBackOff();
         var result = SetStatus( null );
         _LastTitle_ = XOps.Exchange(ref _Title_,null);
         _LastRootStatus_ = XOps.Exchange(ref _RootStatus_,null);
         SwitchToInactiveAppearance();
         return result;
      }
      else
         return null;
}

//==================================================================================================

public static XAppWaitingStatus DisplayStatus()
{
   return DisplayStatus((string)null);
}

public static XAppWaitingStatus DisplayStatus([Optional] string caption)
{
   return DisplayStatusWithData(caption,null);
}

public static XAppWaitingStatus DisplayStatus(string captionFmt,params object[] parameters)
{
   return DisplayStatus(string.Format(captionFmt,parameters));
}

public static XAppWaitingStatus DisplayStatus(XAppWaitingStatus status)
{
   XArgCheck.VerifyNotNull(status);
   lock (typeof(XAppWaiting))
   {
      VerifyIsActive();
      var result = SetStatus(status);
      ShowStatus();
      return result;
   }
}

public static XAppWaitingStatus DisplayStatusWithData( [Optional] string caption,
                                                       [Optional] object data )
{
   return DisplayStatus(new XAppWaitingStatus(caption,data));
}

//--------------------------------------------------------------------------------------------------

public static void RefreshStatus()
{
   lock (typeof(XAppWaiting))
      ShowStatus();
}

public static void RevertStatus()
{
   lock (typeof(XAppWaiting))
   {
      VerifyIsActive();
      if (_CurrentInstance_!=null)
         DisplayStatus(_CurrentInstance_._PreviousStatus_);
      else
         DisplayStatus(_RootStatus_);
   }
}

//==================================================================================================

public static void SuppressBackOff()
{
   lock (typeof(XAppWaiting))
      if (IsActive)
      {
         for ( var instance=_CurrentInstance_;
               instance!=null;
               instance=instance._PreviousInstance_ )
            instance._SuppressBackOff_ = true;
         _CurrentInstance_ = null;
      }
}

//==================================================================================================

static XAppWaiting()
{
   Library.Access();
}

//--------------------------------------------------------------------------------------------------

static void SwitchToActiveAppearance()
{
   XDispatcher.Synchronize
   (
      delegate
      {
#if !SILVERLIGHT
         if (ReleaseCaptureOnActivation)
            Mouse.Capture( null );
         if (AffectsCursor)
            XCursor.SetWait();
#endif // !SILVERLIGHT
         if (KeepApplicationBusyWhenActive)
            XApplication.IsBusy = true;
      }
   );
   ShowStatus();
}

static void SwitchToInactiveAppearance()
{
   if (KeepApplicationBusyWhenActive)
      XDispatcher.Synchronize(delegate{
         XApplication.IsBusy = false;
      });
   ShowStatus();
#if !SILVERLIGHT
   if (AffectsCursor)
      XCursor.Reset();
#endif // !SILVERLIGHT
}

//--------------------------------------------------------------------------------------------------

static XAppWaitingStatus SetStatus([Optional] XAppWaitingStatus status)
{
   return XOps.Exchange(
      ref _CurrentStatus_,
      status!=null ? status.Duplicate() : null );
}

public static void ShowStatus()
{
   var displayStatusHandler =
      DisplayAppWaitingStatusHandler;
   if (displayStatusHandler!=null)
   {
      bool
         isActive = IsActive;
      string
         caption = isActive ? Caption : null;
      object
         data = isActive ? Data : null;
      XDispatcher.Synchronize(delegate{
         displayStatusHandler(isActive,caption,data); });
   }
}

//==================================================================================================

static volatile bool
   _KeepApplicationBusyWhenActive_ = true;
static volatile XReportAppWaitingErrorCallback
   _ReportErrorCallback_;

//--------------------------------------------------------------------------------------------------

#if !SILVERLIGHT
static volatile bool
   _ReleaseCaptureOnActivation_ = true,
   _AffectsCursor_;
#endif // !SILVERLIGHT

//--------------------------------------------------------------------------------------------------

static XAppWaitingStatus
   _RootStatus_,
   _CurrentStatus_,
   _LastRootStatus_;
static XAppWaiting
   _CurrentInstance_;
static string
   _Title_,
   _LastTitle_;
static bool
   _IsInDispose_;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XAppWaiting
//**************************************************************************************************

} // PaycorMaster
