﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

using System.Windows;
using System.Windows.Threading;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XApplication {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static event Action IsBusyChanged;

//==================================================================================================

public static bool IsDispatcherSpecified
{
   get
   {
      lock (typeof(XApplication))
         return _Dispatcher_!=null;
   }
}

public static Dispatcher Dispatcher
{
   get
   {
      lock (typeof(XApplication))
         if (IsDispatcherSpecified)
            return _Dispatcher_;
      throw new InvalidOperationException(
         "Application dispatcher was not specified." );
   }
}

//--------------------------------------------------------------------------------------------------

public static bool IsBusy
{
   get
   {
      lock (typeof(XApplication))
         return _IsBusy_;
   }
   set
   {
      XProgram.VerifyIsInMainThread();
      lock (typeof(XApplication))
      {
         bool changed = value != _IsBusy_;
         _IsBusy_=value;
#if !SILVERLIGHT
         UpdateWindowsBusy();
#endif // !SILVERLIGHT
         if (changed && IsBusyChanged!=null)
            IsBusyChanged();
      }
   }
}

//==================================================================================================

#if !SILVERLIGHT
public static void SpecifyDispatcher()
{
   XProgram.VerifyIsInMainThread();
   SpecifyDispatcher( Application.Current.Dispatcher );
}
#endif // !SILVERLIGHT

public static void SpecifyDispatcher(Dispatcher value)
{
   lock (typeof(XApplication))
      _Dispatcher_ = value;
}

public static Dispatcher GetDispatcherSpecification()
{
   lock (typeof(XApplication))
      return _Dispatcher_;
}

//==================================================================================================

static XApplication()
{
   Library.Access();
}

//==================================================================================================

static Dispatcher
   _Dispatcher_;
static bool
   _IsBusy_;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XApplication
//**************************************************************************************************

} // XHelpers
