﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Threading;
using System.Runtime.InteropServices;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XBackgroundOperation {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static bool IsActive
{
   get { return _IsActive_; }
}

public static bool IsInBackgroundThread
{
   get { return Thread.CurrentThread==_Thread_; }
}

public static Thread Thread
{
   get { return _Thread_; }
}

//==================================================================================================

public static void VerifyIsInBackgroundThread()
{
   if (!IsInBackgroundThread)
      throw new InvalidOperationException(
         "Execution is not not in background operation thread." );
}

//--------------------------------------------------------------------------------------------------

public static void DetachAppWaitingSuppressBackOff()
{
   XDispatcher.Synchronize(delegate{
      _UseAppWaiting_ = false;
      XAppWaiting.SuppressBackOff();
   });
}

//--------------------------------------------------------------------------------------------------

public static void WaitOperationDeactivateAppWaiting()
{
   XProgram.VerifyIsInMainThread();
   lock (_ActivitySyncObject_)
   {}
   XAppWaiting.Deactivate();
}

//==================================================================================================

static XBackgroundOperation()
{
   Library.Access();
}


//--------------------------------------------------------------------------------------------------

static void Initialize()
{
   if (!_IsInitialized_)
   {
      _Thread_ = new Thread( OperationThreadRoutine );
      _Thread_.IsBackground = true;
      lock (_ActivitySyncObject_)
      {
         _Thread_.Start();
         Monitor.Wait(_ActivitySyncObject_);
      }
      _IsInitialized_ = true;
   }
}

//--------------------------------------------------------------------------------------------------

static void StartEx( bool useAppWaiting,
                     [Optional] string appWaitingTitle,
                     [Optional] XAppWaitingStatus appWaitingStatus,
                     object priority, Action operation )
{
   XArgCheck.VerifyNotNull( operation );
   XProgram.VerifyIsInMainThread();
   if (_IsActive_)
      throw new InvalidOperationException(
         "Previous background operation was not completed." );
   Thread.MemoryBarrier();
   Initialize();
   lock (_ActivitySyncObject_)
   {
      if (useAppWaiting)
         XAppWaiting.Activate( appWaitingTitle, appWaitingStatus );
      _Thread_.CurrentCulture = Thread.CurrentThread.CurrentCulture;
      _Thread_.CurrentUICulture = Thread.CurrentThread.CurrentUICulture;
      _UseAppWaiting_ = useAppWaiting;
      _Operation_ = operation;
      _IsActive_ = true;
#if !SILVERLIGHT
      if (priority!=null)
         _Thread_.Priority = (ThreadPriority)priority;
#endif // !SILVERLIGHT
      Monitor.Pulse(_ActivitySyncObject_);
   }
}

//--------------------------------------------------------------------------------------------------

static void OperationThreadRoutine()
{
   lock (_ActivitySyncObject_)
   {
      Monitor.Pulse(_ActivitySyncObject_);
      for (;;)
      {
         Monitor.Wait(_ActivitySyncObject_);
         try
         {
            XDispatcher.Synchronize( XOps.Nop );
            _Operation_();
         }
         finally
         {
            Thread.MemoryBarrier();
         }
         XDispatcher.Synchronize(delegate{
            if (_UseAppWaiting_)
               XAppWaiting.Deactivate();
            _IsActive_ = false;
         });
#if !SILVERLIGHT
         _Thread_.Priority = ThreadPriority.Normal;
#endif // !SILVERLIGHT
      }
   }
}

//==================================================================================================

static bool
   _IsInitialized_,
   _UseAppWaiting_;
static Action
   _Operation_;

static volatile bool
   _IsActive_;
static volatile Thread
   _Thread_;

static readonly object
   _ActivitySyncObject_ = new object();

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XBackgroundOperation
//**************************************************************************************************

} // XHelpers
