﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.Threading;

using System.Windows;
using System.Windows.Threading;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XDispatcher {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

static XDispatcher()
{
   Library.Access();
}

//--------------------------------------------------------------------------------------------------

static void BeginInvokeEx
(
#if !SILVERLIGHT
   DispatcherPriority? priority,
#endif // !SILVERLIGHT
   Action operation
)
{
   Thread.MemoryBarrier();
   XApplication.Dispatcher.BeginInvoke
   (
      (Action)delegate
      {
         try
            { operation(); }
         finally
            { Thread.MemoryBarrier(); }
      }
#if !SILVERLIGHT
      ,priority ?? DispatcherPriority.Normal
#endif // !SILVERLIGHT
   );
}

//--------------------------------------------------------------------------------------------------

static void SynchronizeEx
(
#if !SILVERLIGHT
   DispatcherPriority? priority,
#endif // !SILVERLIGHT
   Action operation
)
{
   if (XProgram.IsInMainThread())
      operation();
   else
   {
#if SILVERLIGHT
      var operationFinishedEvent =
         new ManualResetEvent(false);
#endif // !SILVERLIGHT
      Thread.MemoryBarrier();
#if SILVERLIGHT
      XApplication.Dispatcher.BeginInvoke
#else
      XApplication.Dispatcher.Invoke
#endif
      (
         (Action)delegate
         {
            try
            {
               operation();
            }
            finally
            {
               Thread.MemoryBarrier();
#if SILVERLIGHT
               operationFinishedEvent.Set();
#endif // SILVERLIGHT
            }
         }
#if !SILVERLIGHT
         ,priority ?? DispatcherPriority.Normal
#endif // !SILVERLIGHT
      );
#if SILVERLIGHT
      operationFinishedEvent.WaitOne();
#endif // SILVERLIGHT
   }
}

//--------------------------------------------------------------------------------------------------

static T SynchronizeEx<T>
(
#if !SILVERLIGHT
   DispatcherPriority? priority,
#endif // !SILVERLIGHT
   XResultDelegate<T> operation
)
{
   var resultCapture = new XSyncCapture<T>();
   SynchronizeEx
   (
#if !SILVERLIGHT
      priority ?? DispatcherPriority.Normal,
#endif // !SILVERLIGHT
      delegate { resultCapture.Value = operation(); }
   );
   return resultCapture;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XDispatcher
//**************************************************************************************************

} // XHelpers
