﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;

using System.Windows;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public delegate void XProcessElementCallback(FrameworkElement element);
//**************************************************************************************************


//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XLogicalTreeHelper {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static void ProcessNodes(FrameworkElement root,XProcessElementCallback callback)
{
   ProcessNodes(root,true,callback);
}

public static void ProcessNodes( FrameworkElement root, bool processRoot,
                                 XProcessElementCallback callback )
{
   ProcessNodes(root,processRoot,true,callback);
   if (processRoot)
      callback(root);
}

public static void ProcessNodes( FrameworkElement root,
                                 bool processRoot, bool recursive,
                                 XProcessElementCallback callback )
{
   ProcessChildren(root,recursive,callback);
   if (processRoot)
      callback(root);
}

//--------------------------------------------------------------------------------------------------

public static void ProcessChildren(FrameworkElement root,XProcessElementCallback callback)
{
   ProcessChildren(root,true,callback);
}

public static void ProcessChildren( FrameworkElement root, bool recursive,
                                    XProcessElementCallback callback )
{
   foreach (var child in LogicalTreeHelper.GetChildren(root))
   {
      var element = child as FrameworkElement;

      if (element!=null)
      {
         if (recursive)
            ProcessChildren(element,callback);
         callback(element);
      }
   }
}

//==================================================================================================

public static void SetNodeValue(FrameworkElement root,DependencyProperty property,object value)
{
   SetNodeValue(root,true,property,value);
}

public static void SetNodeValue( FrameworkElement root, bool processRoot,
                                 DependencyProperty property, object value )
{
   SetNodeValue(root,processRoot,true,property,value);
}

public static void SetNodeValue( FrameworkElement root, bool processRoot, bool recursive,
                                 DependencyProperty property, object value )
{
   ProcessNodes
   (
      root, processRoot, recursive,
      delegate (FrameworkElement element)
      {
         element.SetValue(property,value);
      }
   );
}

//--------------------------------------------------------------------------------------------------

public static void SetChildrenValue( FrameworkElement root,
                                     DependencyProperty property,
                                     object value )
{
   SetChildrenValue(root,true,property,value);
}

public static void SetChildrenValue( FrameworkElement root, bool recursive,
                                     DependencyProperty property, object value )
{
   SetNodeValue(root,false,recursive,property,value);
}

//==================================================================================================

public static void ClearNodeValue(FrameworkElement root,DependencyProperty property)
{
   ClearNodeValue(root,true,property);
}

public static void ClearNodeValue( FrameworkElement root,
                                   bool processRoot,
                                   DependencyProperty property )
{
   ClearNodeValue(root,processRoot,true,property);
}

public static void ClearNodeValue( FrameworkElement root,
                                   bool processRoot, bool recursive,
                                   DependencyProperty property )
{
   ProcessNodes
   (
      root, processRoot, recursive,
      delegate (FrameworkElement element)
      {
         element.ClearValue(property);
      }
   );
}

//--------------------------------------------------------------------------------------------------

public static void ClearChildrenValue(FrameworkElement root,DependencyProperty property)
{
   ClearChildrenValue(root,true,property);
}

public static void ClearChildrenValue( FrameworkElement root,
                                       bool recursive,
                                       DependencyProperty property )
{
   ClearNodeValue(root,false,recursive,property);
}

//==================================================================================================

static XLogicalTreeHelper()
{
   Library.Access();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XLogicalTreeHelper
//**************************************************************************************************

} // XCursor
