﻿//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// XHelpers Solution Library (Freeware);  Author: Sergei Kitaev
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

using System;
using System.ComponentModel;

using System.Windows;
using System.Windows.Input;

namespace XHelpers {

//**************************************************************************************************
/// <summary>
/// </summary>
public static partial class XMessageBox {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

public static event Action
   BeforeShowHandler,
   AfterShowHandler;

//--------------------------------------------------------------------------------------------------

public static string CaptionFormatString
{
   get { return _ApplicationTag_; }
   set { _ApplicationTag_ = value; }
}

[DefaultValue(true)]
public static bool UseBeeperAccompaniment
{
   get { return _UseBeeperAccompaniment_; }
   set { _UseBeeperAccompaniment_ = value; }
}

//==================================================================================================

public static MessageBoxResult DefaultResultFromButton(MessageBoxButton button)
{
   switch (button)
   {
   case MessageBoxButton.OK         : return MessageBoxResult.OK;
   case MessageBoxButton.OKCancel   : return MessageBoxResult.OK;
   case MessageBoxButton.YesNo      : return MessageBoxResult.Yes;
   case MessageBoxButton.YesNoCancel: return MessageBoxResult.Yes;
   }
   return MessageBoxResult.None;
}

//==================================================================================================

static XMessageBox()
{
   Library.Access();
}

//--------------------------------------------------------------------------------------------------

static MessageBoxResult DoShowMessage( XSound sound, Window owner,
                                       string text, string caption,
                                       MessageBoxButton button, MessageBoxImage icon,
                                       MessageBoxResult defaultResult )
{
   XOps.NullSubstitution(ref owner,XApplication.AnchorWindow);

   if (BeforeShowHandler!=null)
      BeforeShowHandler();

   string
      title = CaptionFormatString != null ?
         string.Format(CaptionFormatString,caption) :
         caption;

   if (sound==XSounds.Default)
      if (icon==MessageBoxImage.Information)
         sound=XSounds.Information;
      else
      if (icon==MessageBoxImage.Error)
         sound=XSounds.Error;
      else
      if (icon==MessageBoxImage.Warning)
         sound=XSounds.Warning;
      else
      if (icon==MessageBoxImage.Exclamation)
         sound=XSounds.Exclamation;
      else
      if (icon==MessageBoxImage.Question)
         sound=XSounds.Question;
      else
         sound=XSounds.None;

   bool useSound = sound!=XSounds.None && !XSound.IsMute;

   Cursor savedCursor = XCursor.Current;

   if ( useSound && ( !XSound.IsInitialized ||
                      UseBeeperAccompaniment && XBeeper.IsPlaybackActive ) )
      XCursor.SetWait();

   XSound.Play( sound );

   XCursor.Reset();

   var result = owner!=null ?
      MessageBox.Show( owner, text, title,
                       button, icon, defaultResult ) :
      MessageBox.Show( text, title,
                       button, icon, defaultResult );

   if (useSound)
      XSound.Abort();

   XCursor.Override( savedCursor );

   if (AfterShowHandler!=null)
      AfterShowHandler();

   return result;
}

//==================================================================================================

static volatile string
   _ApplicationTag_;
static volatile bool
   _UseBeeperAccompaniment_ = true;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
} // XMessageBox
//**************************************************************************************************

} // XHelpers
