@echo off
rem ********************************************************************************
rem *** Command Line Utilities; Author: Sergei Kitaev ******************************
rem ********************************************************************************
if "%~0"=="$KEYGEN.cmd" goto START

if not "%$cmd_plain%"=="1" cls
echo 
echo  Public/Secret KeyPair Generation Utility (based on "SN"-utility) 
echo 
goto START

:SYNTAX
if "%~0"=="$KEYGEN.cmd" goto INCORRECT_NESTED_CALL
echo.
echo One parameter is required: ^<key name^>.
echo.
echo  Examples: 
echo.
echo $KeyGen "XSoft"
echo.
echo  This will produce the following files: Ŀ
echo                                                                              
echo  XSoft.KeyPair.snk    key pair in binary form                              
echo  XSoft.PublicKey.snk  public key in binary form                            
echo  XSoft.PublicKey.csv  public key in text form (decimal, comma separated)   
echo  XSoft.PublicKey.txt  public key and its token in text form (hexadecimal)  
echo                                                                              
echo 
echo.
echo *** Use "?", "/?", "-?" or no parameters to see this help. ***
goto FINISH

:START
setlocal
set errorlevel=0
set errorlevel=

if "%cmdextversion%"=="" goto INCORRECT_CMDEXT_VERSION
if not cmdextversion 2   goto INCORRECT_CMDEXT_VERSION

if [%1]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%2]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%3]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%4]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%5]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%6]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%7]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%8]==[""] goto EMPTY_PARAMS_FORBIDDEN
if [%9]==[""] goto EMPTY_PARAMS_FORBIDDEN

if not [%2]==[] goto NOT_SYNTAX_PARAMS
if "%~1"==""    goto SYNTAX
if "%~1"=="?"   goto SYNTAX
if "%~1"=="/?"  goto SYNTAX
if "%~1"=="-?"  goto SYNTAX
:NOT_SYNTAX_PARAMS

if not [%2]==[] goto INCORRECT_PARAMS

set KeyName=%~1

set KeyPairFile_SNK=%KeyName%.KeyPair.snk
set PublicKeyFile_SNK=%KeyName%.PublicKey.snk
set PublicKeyFile_CSV=%KeyName%.PublicKey.csv
set PublicKeyFile_TXT=%KeyName%.PublicKey.txt

if "%~0"=="$KEYGEN.cmd" goto CHECK_FILES

echo.
echo 
echo  CONFIRM SIGNATURE GENERATION:  ("Ctrl+C"  break off) 
echo.
echo  Keypair file        : %KeyPairFile_SNK%
echo  Public key file     : %PublicKeyFile_SNK%
echo  Public key CSV-file : %PublicKeyFile_CSV%
echo  Public key text file: %PublicKeyFile_TXT%
echo.
echo 
echo.
pause

:CHECK_FILES
if exist "%KeyPairFile_SNK%"   goto ALREADY_EXISTS
if exist "%PublicKeyFile_SNK%" goto ALREADY_EXISTS
if exist "%PublicKeyFile_CSV%" goto ALREADY_EXISTS
if exist "%PublicKeyFile_TXT%" goto ALREADY_EXISTS

echo.
echo 
echo  KEY FILES GENERATION: 

echo.
echo === Generating key files (.snk, .csv, .txt): ===

call :EXEC_CMD SN -q -k "%KeyPairFile_SNK%"
if %errorlevel% neq 0 goto FAILURE

call :EXEC_CMD SN -q -p "%KeyPairFile_SNK%" "%PublicKeyFile_SNK%"
if %errorlevel% neq 0 goto FAILURE

call :EXEC_CMD SN -q -o "%KeyPairFile_SNK%" "%PublicKeyFile_CSV%"
if %errorlevel% neq 0 goto FAILURE

echo.
echo  sn -q -tp "%PublicKeyFile_SNK%" ^> "%PublicKeyFile_TXT%" ...
sn -q -tp "%PublicKeyFile_SNK%" > "%PublicKeyFile_TXT%"
if %errorlevel% neq 0 goto FAILURE

echo.
echo 

echo.
echo === Making key files readonly: ===

call :EXEC_CMD attrib +r "%KeyPairFile_SNK%"
if %errorlevel% neq 0 goto FAILURE

call :EXEC_CMD attrib +r "%PublicKeyFile_SNK%"
if %errorlevel% neq 0 goto FAILURE

call :EXEC_CMD attrib +r "%PublicKeyFile_CSV%"
if %errorlevel% neq 0 goto FAILURE

call :EXEC_CMD attrib +r "%PublicKeyFile_TXT%"
if %errorlevel% neq 0 goto FAILURE

:SUCCESS
echo.
echo  KEYGEN  SUCCESS 
echo 
goto FINISH

:FAILURE
if "%~0"=="$KEYGEN.cmd" goto ERROR
echo.
echo  ## KEYGEN  FAILURE 
echo 
goto ERROR

:ALREADY_EXISTS
echo.
echo ## Some of the target files already exist.
echo ## First, manualy delete all of them, if you wish to regenerate key pair.
goto ERROR

:INCORRECT_PARAMS
echo.
echo ## Incorrect parameters.
echo ## One parameter expected: key name (base name).
goto ERROR

rem ================================================================================

:EXEC_CMD
echo.
echo  %* ...
echo.
%*
goto :EOF

rem ================================================================================

:INCORRECT_CMDEXT_VERSION
echo.
echo ## Incorrect Version of CMD-Extensions. Version 2 or later is required.
goto ERROR

:INCORRECT_NESTED_CALL
echo.
echo ## Incorrect nested call.
goto ERROR

:EMPTY_PARAMS_FORBIDDEN
echo.
echo ## Empty parameters ("") are forbidden.
goto ERROR

:ERROR
if "%~0"=="$KEYGEN.cmd" goto SET_ERROR
echo 
if not "%$cmd_plain%"=="1" pause

:SET_ERROR
endlocal
set errorlevel=1
goto :EOF

:FINISH
endlocal
set errorlevel=0
set errorlevel=
if "%~0"=="$KEYGEN.cmd" goto :EOF
echo.
if not "%$cmd_plain%"=="1" pause
goto :EOF

rem ================================================================================
